// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.clustering.matrix;

import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import cern.colt.matrix.linalg.SingularValueDecomposition;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.datamanager.kernel.Entity;

import org.datamanager.passiveentityvalue.WordFrequencyMapEntityValue;


/**
 * A basic DocumentTermMatrix implementation.
 */
public class DocumentTermMatrixImpl implements DocumentTermMatrix {
    /** How long a word must be to be added to the matrix. */
    public static final int MINIMUM_WORD_LENGTH = 3;

    /** How fast we grow rows when the matrix fills up. */
    public static final int ROW_GROWTH_MULTIPLIER = 2;

    /** How fast we grow columns when the matrix fills up. */
    public static final int COLUMN_GROWTH_MULTIPLIER = 2;

    // FIXME: this is just a prototype to test the principle
    // it should be in a helper class or a filter strategy
    private static final String[] STOP_WORDS = {
        "the", "when", "then", "http", "and", "from", "that", "for", "you", 
        "with", "edu", "com", "have", "about", "are", "not", "what", "date", 
        "mon", "tue", "wed", "thu", "fri", "sat", "sun", "www", "list", "there", 
        "can", "will", "some", "all", "like", "they", "was", "talk", "text", 
        "new", "net"
    };
    private static final Set stopWords = createStopWordMap(STOP_WORDS);

    /** Maximum number of terms we will index. */
    public static final int MAXIMUM_TERMS_INDEXED = 2000;

    // FIXME: these needn't (shouldn't) be static!

    /** How many rows this matrix has. */
    private static int numberOfRows = 100;

    /** How many columns this matrix has. */
    private static int numberOfColumns = 10;

    /** The only instance of the DocumentTermMatrixImpl.  It's a singleton. */
    private static final DocumentTermMatrixImpl INSTANCE = 
            new DocumentTermMatrixImpl();

    /** Provides alegbraic operations. */
    private static Algebra algebra = new Algebra();

    /**
     * A mapping of each document in the matrix to its corresponding index along
     * the x axis of the matrix.
     */
    private static Map documentIndexMap = new HashMap();

    /**
     * A mapping of each term in the matrix to its corresponding index along the
     * y axis of the matrix.
     */
    private static Map termIndexMap = new HashMap();

    /** Factory for constructing 2D matrices of double precision cells. */
    private static DoubleFactory2D doubleFactory2DSparse;
    private static DoubleFactory2D doubleFactory2DDense;

    /** The actual document x term matrix. */
    private static DoubleMatrix2D documentByTermMatrix;

    /** The actual document x document matrix. */
    private static DoubleMatrix2D documentByDocumentMatrix;

    /** The total number of documents in the matrix. */
    private int numberOfDocuments = 0;

    /** The total number of terms in the matrix. */
    private int numberOfTerms = 0;

    /** 
     * A flag for testing if the matrix needs to be updated.
     */
    private boolean documentsAdded = true;

    /**
     * Creates a new, empty DocumentTermMatrixImpl.
     */
    private DocumentTermMatrixImpl() {
        doubleFactory2DSparse = DoubleFactory2D.sparse;
        doubleFactory2DDense = DoubleFactory2D.dense;
        documentByTermMatrix = doubleFactory2DSparse.make(numberOfRows, 
                                                          numberOfColumns);
    }

    //FIXME: Xi Rao this is not implemented
    public ClusterableMatrix toClusterableMatrix() {
        throw new UnsupportedOperationException();
    }

    /**
     * Adds words from the provided WordFrequencyMapEntityValue to the document
     * term matrix.
     */
    public synchronized void addWords(Entity entity, 
                                      WordFrequencyMapEntityValue newWords) {
        Iterator wordIterator = newWords.getWordSet().iterator();

        while (wordIterator.hasNext()) {
            String word = (String) wordIterator.next();

            if (wordIsAcceptable(word)) {
                setTermFrequencyInDocument(entity, word, 
                                           (int) newWords.getFrequencyOf(word));
            }
        }

        setDocumentsAdded();
    }

    /**
     * Determines if this word is an acceptable word
     * to be added to the term list.
     *
     * FIXME: this is copied from the GSquared strategy
     * FIXME: this should do more complicated things than
     * length.
     */
    private boolean wordIsAcceptable(String word) {
        return (word.length() >= MINIMUM_WORD_LENGTH && 
               !wordIsStopWord(word));
    }

    /**
     * Compares the given word with a stop word list.
     *
     * FIXME: this should use a global stop word list or
     * similar mechanism.
     */
    private boolean wordIsStopWord(String word) {
        return stopWords.contains(word);
    }

    /**
     * Sets the frequency of the provided word to the provided frequency, adding
     * the word and entity to the matrix if necessary.
     *
     * FIXME: fail silently to prevent memory overflow when there 
     * are too many things to add.
     *
     */
    private void setTermFrequencyInDocument(Entity entity, String word, 
                                            int frequency) {
        addDocument(entity);

        if (numberOfTerms < MAXIMUM_TERMS_INDEXED) {
            addTerm(word);

            int entityIndex = indexFor(entity);
            int wordIndex = indexFor(word);


            // matrix is rows: terms columns: documents
            documentByTermMatrix.set(wordIndex, entityIndex, 
                                     frequency + 
                                     documentByTermMatrix.get(wordIndex, 
                                                              entityIndex));
        }
    }

    /**
     * Adds the given document to the matrix if it is not already there.
     */
    private void addDocument(Entity entity) {
        if (numberOfDocuments >= numberOfColumns) {
            growColumns();
        }

        if (indexFor(entity) < 0) {
            documentIndexMap.put(entity, new Integer(numberOfDocuments++));
        }
    }

    /**
     * Adds the given term to the matrix if it is not already there.
     *
     * FIXME: we silently fail to add a term if we have indexed enough
     * alread
     */
    private void addTerm(String term) {
        if (numberOfTerms >= numberOfRows) {
            growRows();
        }

        if (indexFor(term) < 0) {
            termIndexMap.put(term, new Integer(numberOfTerms++));
        }
    }

    /**
     * Returns the index along the x axis for the given document, or -1 if the
     * entity is not in the matrix at all.
     */
    private int indexFor(Entity entity) {
        return indexFor(entity, documentIndexMap);
    }

    /**
     * Returns the index along the y axis for the given term in the
     * documentByTermMatrix, or -1 if the term is not in the matrix at all.
     */
    private int indexFor(String term) {
        return indexFor(term, termIndexMap);
    }

    /**
     * Looks up the given key in the given map and, if it is found, returns the
     * integer value found therein.  If it's not found, returns 0.
     */
    private int indexFor(Object key, Map map) {
        Integer indexObject = (Integer) map.get(key);

        if (indexObject == null) {
            return -1;
        }

        return indexObject.intValue();
    }

    /**
     * Increases the number of rows in the matrix by allocating a new larger
     * matrix and copying.
     */
    private void growRows() {
        numberOfRows = numberOfRows * ROW_GROWTH_MULTIPLIER;

        copyMatrixIntoNewMatrix();
    }

    /**
     * Increases the number of columns in the matrix by allocating a new larger
     * matrix and copying.
     */
    private void growColumns() {
        numberOfColumns = numberOfColumns * COLUMN_GROWTH_MULTIPLIER;

        copyMatrixIntoNewMatrix();
    }

    /**
     * Allocates a new matrix (the new sizes should already have been placed in
     * numberOfColumns and numberOfRows), then copies the current
     * documentByTermMatrix into it, then makes that new matrix the current
     * documentByTermMatrix.
     */
    private void copyMatrixIntoNewMatrix() {
        DoubleMatrix2D newDocumentByTermMatrix = doubleFactory2DSparse.make(
                                                         numberOfRows, 
                                                         numberOfColumns);

        for (int row = 0; row < documentByTermMatrix.rows(); ++row)
            for (int column = 0;
                 column < documentByTermMatrix.columns();
                 ++column)
                newDocumentByTermMatrix.set(row, column, 
                                            documentByTermMatrix.get(row, 
                                                                     column));

        documentByTermMatrix = newDocumentByTermMatrix;
    }

    /**
     * Creates a docment by document matrix representing similarity
     * distances by creating the Singular Value Decomposition of
     * the document by term matrix. 
     *
     * FIXME: this is incomplete. 
     * See http://ella.slis.indiana.edu/~katy/L697/code/lsa.html
     */
    public void makeDocumentByDocumentMatrix() {
        System.err.println("making the docdoc matrix");

        // FIXME: this line causes an out of memory error
        SingularValueDecomposition singularValueDecomposition = 
                new SingularValueDecomposition(documentByTermMatrix);
        int singularValueRank = Math.max(singularValueDecomposition.rank() - 2, 1);

        System.err.println("number of terms: " + numberOfTerms);
        System.err.println("number of docs: " + numberOfDocuments);
        System.err.println("sv rank: " + singularValueRank);

        //DoubleMatrix2D reducedS = doubleFactory2DSparse.make(singularValueRank,singularValueRank);
        DoubleMatrix2D reducedS = singularValueDecomposition.getS()
                                                            .viewPart(0, 0, 
                                                                      singularValueRank - 1, 
                                                                      singularValueRank - 1);

        //DoubleMatrix2D uMatrixXReducedS = algebra.mult(singularValueDecomposition.getV().viewPart(0, 0, numberOfTerms, singularValueRank - 1), reducedS);
        DoubleMatrix2D uMatrixXReducedS = algebra.mult(
                                                  singularValueDecomposition.getV()
                                                                            .viewPart(0, 0, 
                                                                                      numberOfDocuments, 
                                                                                      singularValueRank - 1), 
                                                  reducedS);

        DoubleMatrix2D normalizedMatrix = doubleFactory2DSparse.make(
                                                  uMatrixXReducedS.rows(), 
                                                  uMatrixXReducedS.columns());
        normalizedMatrix = matrixNormalize(uMatrixXReducedS);

        DoubleMatrix2D transposedNormalizedMatrix = doubleFactory2DSparse.make(
                                                            normalizedMatrix.columns(), 
                                                            normalizedMatrix.rows());
        transposedNormalizedMatrix = normalizedMatrix.viewDice();

        documentByDocumentMatrix = doubleFactory2DDense.make(numberOfDocuments, 
                                                             numberOfDocuments);


        // FIXME: this causes an out of memory
        documentByDocumentMatrix = algebra.mult(normalizedMatrix, 
                                                transposedNormalizedMatrix);
    }

    /**
     * Normalizes the given matrix.
     *
     * FIXME: This math is way confusing. I'm sure there
     * must be a way to do this in colt that I'm missing.
     * Probably can do this by reference too.
     */
    private DoubleMatrix2D matrixNormalize(DoubleMatrix2D matrix) {
        for (int row = 0; row < matrix.rows(); row++) {
            double length = 0;

            for (int column = 0; column < matrix.columns(); column++) {
                length = length + 
                         (matrix.get(row, column) * matrix.get(row, column));
            }

            length = Math.sqrt(length);

            for (int column = 0; column < matrix.columns(); column++) {
                matrix.set(row, column, (matrix.get(row, column) / length));
            }
        }

        return matrix;
    }

    /**
     * Gets the singleton instance of this DocumentTermMatrix.
     */
    public static DocumentTermMatrixImpl getDocumentTermMatrix() {
        return INSTANCE;
    }

    /**
     * Gets the document x term matrix.
     */
    protected static DoubleMatrix2D getMatrix() {
        return documentByTermMatrix;
    }

    /**
     * Returns whether or not any documents have been added since we last
     * constructed the documentByDocumentMatrix.
     */
    private boolean documentsHaveBeenAdded() {
        return documentsAdded;
    }

    /**
     * Indicates that documents have been added since we last built the
     * documentByDocumentMatrix.
     */
    private void setDocumentsAdded() {
        documentsAdded = true;
    }

    /**
     * Indicates that documents have not been added since we last built the
     * documentByDocumentMatrix.
     */
    private void setDocumentsNotAdded() {
        documentsAdded = false;
    }

    /**
     * Gets the similarity between two entities by doing
     * a LSA look up for documents documents clustering matrix.
     */
    public double getSimilarityBetween(Entity entityOne, Entity entityTwo) {
        double similarity = 0;

        if (documentsHaveBeenAdded()) {
            makeDocumentByDocumentMatrix();
            setDocumentsNotAdded();
        }

        // FIXME: sometimes an entity is not found in the document list, how can that be?
        if ((indexFor(entityOne) >= 0) && (indexFor(entityTwo) >= 0)) {
            similarity = documentByDocumentMatrix.get(indexFor(entityOne), 
                                                      indexFor(entityTwo));
        }

        return similarity;
    }

    /**
     * Creates a Set which is the list of stop words
     * that are ignored in comparisons.
     */
    private static Set createStopWordMap(String[] words) {
        Set stopWords = new HashSet();

        for (int i = 0; i < words.length; i++) {
            stopWords.add(words[i]);
        }

        return stopWords;
    }
}

/*
 *
 * $Log: DocumentTermMatrixImpl.java,v $
 * Revision 1.16  2002/12/20 20:52:18  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.15  2002/12/20 04:33:41  cjdent
 *
 * Adjustments to the DocumentTermMatrixImpl class to more effectively
 * deal with memory issues, word frequency generation and otherwise
 * work.
 *
 * It was still getting out of memory errors so I changed the limiting
 * around so that it will take the first 2000 terms, but keep on adding
 * documents. How it was before, if you set a limit on one axis the other
 * other axis was stopped there too.
 *
 * Added support for stop words and word length filtering, stolen from
 * the G-squared stuff. All done to create a smaller list of words.
 *
 * Did some nasty dancing with the math that creates the
 * DocumentXDocumentMatrix to make sure that it is calculating the
 * right things. When we changed from a vector to a map for indexing
 * the other matrix, this stuff got confused.
 *
 * Two reasons I worked on this:
 *
 * - i need a better picture
 * - this is the one applied thing that we've done that is going
 *   to be immediately useful for me elsewhere in life, so I figured
 *   my as well not, time only goes away, so may as well use it
 *
 * More in other commits.
 *
 * Revision 1.14  2002/12/19 04:35:41  mliggett
 * codebase-style, plus
 *
 * Changed from using a List to a HashMap for storing mappings from documents and
 * terms to the indices for them on the matrix.
 *
 * Revision 1.13  2002/12/19 04:09:31  mliggett
 * Replaced doubleSize() with growRows() and growColumns(), which are both
 * enabled.
 *
 * Factored out how we store the mapping between documents/words and their
 * indices on the matrix axes.  I'm going to change this from using a List to
 * a HashMap, because otherwise this lookup is going to get very slow very
 * fast (we currently call indexOf() all the time on a Vector).
 *
 * Revision 1.12  2002/12/19 03:37:26  mliggett
 * Added javadoc for the interface.
 *
 * Revision 1.11  2002/12/19 03:12:30  mliggett
 * General refactoring, renaming, documenting and commenting of
 * DocumentTermMatrixImpl.  Also removed one unit test that wasn't really a unit
 * test.  The thing it was testing will probably reemerge in different guise.
 *
 * Doing this to get back into the groove, so I can get some more mental traction
 * in the developer docs.
 *
 *
 */
