// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.clustering.strategy.bayes;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.datamanager.clustering.strategy.ClusteringStrategy;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;

import org.datamanager.passiveentityvalue.WordFrequencyMapEntityValue;

import org.datamanager.tools.Toolbox;


/**
 * This class represents a naive Bayesian strategy for clustering Emails.
 * See <a href="http://sourceforge.net/docman/display_doc.php?docid=13334&group_id=63137">
 * http://sourceforge.net/docman/display_doc.php?docid=13334&group_id=63137</a>
 * for algorithm details.
 * @author Team Helium
 * @version $Revision: 1.4 $
 */
public class NaiveBayesStrategy implements ClusteringStrategy {
    public final static double DEFAULT_THRESHOLD = 0.7;
    private final static NaiveBayesStrategy instance = new NaiveBayesStrategy();
    private double threshold = DEFAULT_THRESHOLD;

    private NaiveBayesStrategy() {
    }

    // Hmm, so for NaiveBayes this is a bucket of 2... that isn't too hot.
    public double getSimilarityBetween(Entity a, Entity b) {
        try {
            // FIXME:  need to figure out where to place these hard-coded
            // 'names' for Entities and attributes, like "Word-Frequency".
            // Right now they're getting stored in the EntityValue itself?
            WordFrequencyMapEntityValue bucketA = (WordFrequencyMapEntityValue) Toolbox.getFirstAttributeNamed(a, 
                                                                                                               WordFrequencyMapEntityValue.ATTRIBUTE_NAME)
                                                                                       .getValue();
            WordFrequencyMapEntityValue bucketB = (WordFrequencyMapEntityValue) Toolbox.getFirstAttributeNamed(b, 
                                                                                                               WordFrequencyMapEntityValue.ATTRIBUTE_NAME)
                                                                                       .getValue();

            return getProbabilityForSet(bucketA.getWordSet(), bucketB);
        } catch (DataManagerException exception) {
            // FIXME: Bad monkey exception
            exception.printStackTrace();
            throw new RuntimeException(exception);
        }
    }

    /**
     * Satisfying interface.
     */
    public void addEntity(Entity entity) {
    }

    /**
     * Calculates the probability P(E|Bi), which represents the probability
     * that, for a given bucket (wordFrequencyMap), a given set of Words
     * appear in that bucket.
     */
    private double getProbabilityForSet(final Set wordSet, 
                                        final WordFrequencyMapEntityValue bucket) {
        double probability = 1;
        Iterator iterator = wordSet.iterator();

        // This gives us P(E|Bi) = P(E1|Bi) * P(E2|Bi) * ... * P(En|Bi) 
        // where n is the number of distinct words in the set.
        while (iterator.hasNext()) {
            probability *= getProbabilityForWord((String) iterator.next(), 
                                                 bucket);
        }


        // Now we need to calculate P(Bi) for this bucket
        probability *= getProbabilityForBucket(bucket);

        return probability;
    }

    /** 
     * Calculates the probability of a given bucket (the probability of any
     * email being in this bucket, represented as a Map).
     * P(Bi)
     */
    private double getProbabilityForBucket(final WordFrequencyMapEntityValue bucket) {
        return bucket.getNumberOfUniqueWords() / bucket.getTotalNumberOfWords();
    }

    /**
     * Calculates the probability that a given word appears in a given bucket
     * (buckets are represented as Maps of word frequencies right now).
     */
    private double getProbabilityForWord(String word, 
                                         WordFrequencyMapEntityValue bucket) {
        // Starts out at 1 because of the multiplication series - if we use 0,
        // then we kill the whole calculation instantly.  *snap*
        double probability = 1;

        if (bucket.contains(word)) {
            probability = bucket.getFrequencyOf(word) / bucket.getTotalNumberOfWords();
        }

        return probability;
    }

    public final static NaiveBayesStrategy getInstance() {
        return instance;
    }

    public void setThreshold(double threshold) {
        this.threshold = threshold;
    }

    public double getThreshold() {
        return this.threshold;
    }
}

/*
 * $Log: NaiveBayesStrategy.java,v $
 * Revision 1.4  2002/12/20 20:52:18  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.3  2002/12/16 10:48:46  cjdent
 *
 * Adjusted Xi's initial stab at lsa clustering to get it
 * into a more working state:
 *
 * * changed to using a sparse matrix for the document by term matrix
 * * stopped filling the matrix at the start
 * * did what appears to be the proper matrix manipulations to get
 *   a correct document by document similarity matrix
 * * Adds an addEntity() method to the ClusterStrategy method so
 *   firing off the building of the matrix or whatever else needs
 *   to be done can be the result of adding an entity to the strategy
 *
 * This is a very expensive implementation. It updates the documentXterm
 * matrix every time a document is added and then updates the
 * documentXdocument matrix. However it does not update the
 * documentXdocument matrix every time you query for info (it was
 * doing that for a while).
 *
 * The matrixNormalize() method is a heavy handed way of doing
 * someting that I'm pretty sure Colt will do for us, if I knew
 * how to ask.
 *
 * In makeDocumentByDocumentMatrix the rank() method used on
 * the SVD returns the number of non negligible values in
 * the array of singular values. This does not mean the entire
 * list is the best set of values to use for creating the reduced
 * matrix. This value can probably be shrunk by a fair
 * amount. Choosing the right value is the magic part. There's
 * probably some bogosness in this section of the code.
 *
 * Revision 1.2  2002/12/07 15:17:43  alllee
 * Changed to retrofit with WordFrequencyMapEntityValue refactoring -
 *
 * Math is still a bit squirrelly I think.  Need to go over it again somewhat in
 * my otherwise infinite spare time.  Documentation is very bad.
 *
 */
