// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.constraint;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.ContainsEntityEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.AbstractEventConstraint;
import org.datamanager.kernel.Entity;


/**
 * This EventConstraint describes a subset of Entity-containing events that
 * are described by a given EntityConstraint.  It selects all Events marked
 * with a ContainsEntityEvent type and then applies the EntityConstraint to
 * those Events.  
 *
 * @author Team Helium
 * @version $Revision: 1.4 $
 */
public class EventContainsEntityConstraint extends AbstractEventConstraint {
    /**
     * This constraint is used to acquire only those Events that actually
     * contain Entities.  
     */
    private static final EventConstraint containsEntityEventConstraint = 
            new EventTypeIsConstraint(ContainsEntityEvent.class);

    /**
     * The constraint to filter out all newly added Entities
     * stored in an AddEntityPoolEvent.
     */
    private EntityConstraint entityConstraint;

    /**
     * Create a new EventConstainsEntityConstraint to check
     * for Entities that satisfy the given EntityConstraint.
     *
     * @param constraint An EntityConstraint to use when filtering
     *                   newly added entities.
     */
    public EventContainsEntityConstraint(EntityConstraint constraint) {
        this.entityConstraint = constraint;
    }

    /**
     * Returns true if and only if <code>event</code> is a
     * ContainsEntityEvent and the Entity in the event satisfies the
     * constraint passed into the constructor of this constraint. 
     *
     * @param event An event to check
     *
     * @return Whether the event is an AddEntityPoolEvent and the
     *         new Entity in the event satisfies the constraint
     *         given in the constructor of this constraint.
     */
    protected boolean describes(DataManagerEvent event)
                         throws DataManagerException {
        if (containsEntityEventConstraint.accepts(event)) {
            Entity entity = ((ContainsEntityEvent) event).getEntity();

            return entityConstraint.accepts(entity);
        } else {
            return false;
        }
    }
}