// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Removed Debug.assert calls in constructor and replaced with
// IllegalArgumentExceptions.
///////////////////////////////////////////////////////////////////////////////

/**
 * EventClassIsConstraint.java
 *
 * Author: Matthew Farrellee
 */
package org.datamanager.constraint;

import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.AbstractEventConstraint;

import org.datamanager.tools.Debug;


/**
 * Checks to see if the type of a <code>DataManagerEvent</code> is valid or
 * not.  Validity is tested by checking if the <code>Class</code> passed into
 * the constructor is the same as or is a superinterface/superclass of the
 * <code>Class</code> of a defendant <code>DataManagerEvent</code>.
 *
 * @author Matthew Farrellee
 */
public class EventTypeIsConstraint extends AbstractEventConstraint {
    /**
     * The target type.
     */
    private Class target;

    /**
     * Creates a new EventTypeIsConstraint that will match
     * all DataManagerEvents that are of the given type,
     * this includes subtypes.
     *
     * @param target The type to target.
     * @throws IllegalArgumentException if target is null or
     *        target is not a DataManagerEvent class.
     */
    public EventTypeIsConstraint(Class target) {
        if (target == null) {
            throw new IllegalArgumentException("Given a null class.");
        }

        if (!DataManagerEvent.class.isAssignableFrom(target)) {
            throw new IllegalArgumentException(
                    "Given class is not a DataManagerEvent.");
        }

        this.target = target;
    }

    /**
     * Checks to see if the given DataManagerEvent is of
     * the type given in the constructor of this class.
     *
     * @param defendant A DataManagerEvent to check.
     *
     * @return Whether the given event is of the right type.
     */
    protected boolean describes(DataManagerEvent defendant)
                         throws DataManagerException {
        if (target == null) {
            return false;
        } else {
            return target.isAssignableFrom(defendant.getClass());
        }
    }
}

/*
 *
 * $Log: EventTypeIsConstraint.java,v $
 * Revision 1.7  2002/12/20 20:52:19  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.6  2002/11/17 08:41:37  alllee
 * Made a sweepin codebase refactoring (hope it's alright with everyone) that
 * added a new Event marker interface, ContainsEntityEvent.  Previously,
 * EventContainsEntityConstraint automatically limited its search space to just
 * those Entities that were newly added to the pool (via a EventTypeIsConstraint
 * over the AddEntityPoolEvent).  I removed that hard-coded limitation (which was
 * causing me grief with the EntityDisplayRequestEvent).  This means that any
 * future usage of EventContainsEntityConstraint _must_ specify that they are
 * only interested in AddEntityPoolEvents also with an AndEventConstraint.  A few
 * of the Simpletons that were depending on the AddEntityPoolEvent have also been
 * changed to reflect this.
 *
 * Revision 1.5  2002/10/25 05:14:41  cjdent
 *
 * The constructor was doing a backwards test using
 * Class.isAssignableFrom() when testing to see if a passed event
 * implements DataManagerEvent.
 *
 * This may have been a rather serious bug but could someone please
 * double-check what we've done to be sure this is correct, as this
 * may have been there for a while?
 *
 *
 */
