// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.constraint;

import java.util.ArrayList;
import java.util.List;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.NoneEntityConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.AbstractEntityConstraint;
import org.datamanager.kernel.Entity;


/**
 * Selects entities out of the Pool whose Attribute Entities may satisfy the
 * constraint as well.  It isn't very efficient, though that's a product of
 * the Attribute Entity attachment schema.  Perhaps the future holds an
 * MoreSophisticatedVerySimpleEntity plug-in for self-reordering B+ tree
 * attribute attachments.
 *
 * @author Allen Lee
 * @author Bob George
 * @version $Revision: 1.7 $
 */
public class RecursiveOrEntityConstraint extends AbstractEntityConstraint {
    /**
     * Represents the EntityConstraint to be applied to an Entity and all of
     * its attributes.
     */
    private EntityConstraint constraint;

    public RecursiveOrEntityConstraint(EntityConstraint constraint) {
        this.constraint = constraint;
    }

    /**
     * Returns true if the defendant Entity or any of its attributes is
     * accept()-ed by the EntityConstraint constructor parameter.
     * Circular attribute relationships are properly handled so long as this
     * constraint is not re-used.
     *
     * @param defendant The Entity to be examined.
     * @return a boolean specifying whether or not the Entity defendant or any
     * of its attributes is accepted by the EntityConstraint passed into the
     * constructor.
     */
    protected boolean describes(final Entity defendant)
                         throws DataManagerException {
        return recursiveDescribes(defendant, new ArrayList());
    }

    private boolean recursiveDescribes(final Entity defendant, 
                                       List visitedEntities)
                                throws DataManagerException {
        boolean hasBeenVisited = visitedEntities.contains(defendant);

        // This is redundant, as booleans are initialized as false.
        boolean isAcceptable = false;

        // Visits this entity iff it has not been visited yet.
        if (!hasBeenVisited) {
            visitedEntities.add(defendant);

            isAcceptable = constraint.accepts(defendant);

            // if this Entity defendant is not acceptable we examine its
            // attributes.
            if (!isAcceptable) {
                Entity[] attributes = defendant.getAttributes(
                                              NoneEntityConstraint.getInstance());

                // recursively examines the defendant's attributes.
                for (int i = 0; (i < attributes.length) && (!isAcceptable); i++) {
                    isAcceptable = constraint.accepts(attributes[i]) || 
                                   recursiveDescribes(attributes[i], 
                                                      visitedEntities);
                }
            }
        }

        return isAcceptable;
    }
} // RecursiveOrEntityConstraint
