// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.DataManagerWorker;
import org.datamanager.kernel.Entity;

import org.datamanager.tools.Debug;


/**
 * This class is an abstract EntityConstraint. It implements the
 * EntityConstraint interface located in org.datamanager.constraint,
 * and provides a public abstract describes() methods, which is exactly
 * the same as the public abstract accepts() method in EntityConstraint.
 * Every EntityConstraint should ideally extend this class instead of
 * implementing EntityConstraint themselves. By doing so the constraint
 * will gain extra power while working with Entities in the kernel.
 * For example, a when a constraint is working on an Entity, calling
 * methods such as getValue() and getAttributes(), it should not
 * cause DataManagerEvents to be fired. If the constraint is a
 * subclass of AbstractEntityConstraint then no events will be fired,
 * otherwise events will.
 *
 * If a constraint has already been written to implement EntityConstraint
 * it can easily be modified to use this class instead simply by changing
 * implements EntityConstraint to extends AbstractEntityConstraint and
 * renaming the accepts() method to describes().
 *
 * @author Matthew Farrellee
 *
 * @see org.datamanager.constraint.EntityConstraint
 * @see org.datamanager.kernel.Entity
 */
public abstract class AbstractEntityConstraint implements EntityConstraint {
    public static final String DEBUG_ACCEPTS_MESSAGE = 
            "Non-kernel thread attempted to invoke accepts(...) on Entity ";

    /**
     * The accepts() method is required by the EntityConstraint interface.
     * AbstractEntityConstraint creates a final implementation of this method
     * (no subclasses may override).
     * Eventually this accepts() method calls the abstract describes()
     * method that should be implemented by subclasses.  Makes the
     * DataManagerWorker thread invoking the accepts non-event generating for
     * the duration of the describes() call.
     *
     * @param entity An entity to check.
     *
     * @return Whether the entity is acceptable or not.
     */
    public final boolean accepts(Entity entity) throws DataManagerException {
        // If the entity is null, then it cannot be matched.
        boolean isConstraintValid = false;

        if (entity != null) {
            // Makes the DataManagerWorker non event-generating for the
            // duration of the describes call.
            // If the thread is not an DataManagerWorker
            // there should be a DataManagerException.
            try {
                // Note: Normally this code could be considered having
                // a bug since if two threads were executing in this
                // code at once the following could happen:
                // ThreadA gets oldValue = eventGenerating = true
                // ThreadA sets eventGenerating = false
                // ThreadB gets oldValue = eventGenerating = false
                // ThreadA sets eventGenerating = oldValue = true
                // ThreadB sets eventGenerating = oldValue = false
                // The end result would be a thread with invalid state.
                // However, since eventGenerating is specific to threads
                // ThreadA and ThreadB would not actually be sharing the
                // same generateEvents state and therefore this is not
                // a problem, but if for some reason in the future
                // the generateEvents state is not associated with
                // threads this code could become a problem.
                // If I recall correctly, this was presented as a solution
                DataManagerWorker thread = (DataManagerWorker) Thread.currentThread();

                // caches the invoking DataManagerWorker's event-generating
                // state.
                boolean oldValue = thread.isEventGenerating();

                thread.generateEvents(false);

                isConstraintValid = describes(entity);


                // Restores the invoking DataManagerWorker's event generating
                // state.
                thread.generateEvents(oldValue);
            } catch (ClassCastException exception) {
                // This should really never happen. If it has
                // then it means that someone is running a thread
                // of their own that is not valid.
                // The thread should really be killed at this point,
                // but maybe it would be better just to report the
                // rogue thread to some thread manager.
                Debug.writeLogMessage(DEBUG_ACCEPTS_MESSAGE + entity + 
                                      "\n\t Thread was: " + 
                                      Thread.currentThread());

                // TODO: Destroy thread in some thread-safe manner.
                // in some manner such as:
                // DataManager.FunkUp(Thread.currentThread());
            }
        }

        // This constraint will just return false if the thread
        // is invalid.
        return isConstraintValid;
    }

    /**
     * The method required by this class. This method has the
     * exact same semantics as the interface EntityConstraint's
     * accepts() method.
     *
     * @param entity An entity to check.
     *
     * @return Whether the entity is acceptable or not.
     */
    protected abstract boolean describes(Entity entity)
                                  throws DataManagerException;
}