// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.exception.DataManagerException;


/**
 * Entities that proxy for and lazily cache the information of other Entities.
 * The implicit assumption is that these underlying Entities will not be
 * modified through any other mechanism but through their enclosing
 * CachingProxyEntitys.
 */
public class CachingProxyEntity extends Entity {
    Entity proxiedEntity;

    // FIXME:mml: this CachingProxyEntity is designed to conserve space, yet we
    // use 7 different boolean variables, each probably taking up 16 bits or
    // whatever.  Is this stupid?  Would some sort of bitvector (properly hidden
    // behind the accessor methods below, of course) be smarter?
    private boolean nameIsCached = false;
    private String name;
    private boolean valueIsDirty = false;
    private boolean valueIsCached = false;
    private EntityValue value;
    private boolean attributesAreDirty = false;
    private boolean attributesAreCached = false;
    private List attributes;
    private boolean basesAreDirty = false;
    private boolean basesAreCached = false;
    private List bases;

    /**
     * Creates a new CachingProxyEntity proxying for the given Entity.
     */
    protected CachingProxyEntity(Entity proxiedEntity) {
        super();

        this.proxiedEntity = proxiedEntity;
    }

    /**
     * Flags the cached EntityValue as dirty.
     */
    private void setValueDirty() {
        valueIsDirty = true;
    }

    /**
     * Flags the cached EntityValue as clean.
     */
    private void setValueClean() {
        valueIsDirty = false;
    }

    /**
     * Returns whether or not the cached EntityValue is dirty.
     */
    private boolean valueIsDirty() {
        return valueIsDirty;
    }

    /**
     * Flags the cached attribute list as dirty.
     */
    private void setAttributesDirty() {
        attributesAreDirty = true;
    }

    /**
     * Flags the cached attribute list as clean.
     */
    private void setAttributesClean() {
        attributesAreDirty = false;
    }

    /**
     * Returns whether or not the cached attribute list is dirty.
     */
    private boolean attributesAreDirty() {
        return attributesAreDirty;
    }

    /**
     * Flags the cached base list as dirty.
     */
    private void setBasesDirty() {
        basesAreDirty = true;
    }

    /**
     * Flags the cached base list as clean.
     */
    private void setBasesClean() {
        basesAreDirty = false;
    }

    /**
     * Returns whether or not the cached base list is dirty.
     */
    private boolean basesAreDirty() {
        return basesAreDirty;
    }

    /**
     * Writes any locally changed data back to the proxied Entity.
     */
    public void writeBack() throws DataManagerException {
        if (valueIsDirty()) {
            proxiedEntity.setValue(value);
            setValueClean();
        }

        if (attributesAreDirty()) {
            proxiedEntity.setAttributes(
                    (Identifier[]) attributes.toArray(
                              new Identifier[attributes.size()]));
            setAttributesClean();
        }

        if (basesAreDirty()) {
            proxiedEntity.setBases(
                    (Identifier[]) bases.toArray(new Identifier[bases.size()]));
            setBasesClean();
        }
    }

    public void finalize() throws DataManagerException {
        writeBack();
    }

    public void attach(Entity attribute) throws DataManagerException {
        throw new UnsupportedOperationException(
                "attach() was called on CachingProxyEntity, which does not support this method!");
    }

    public void detach(Entity attribute) throws DataManagerException {
        throw new UnsupportedOperationException(
                "detach() was called on CachingProxyEntity, which does not support this method!");
    }

    public void attach(Identifier attribute) throws DataManagerException {
        ensureAttributesAreCached();
        attributes.add(attribute);
        setAttributesDirty();
    }

    protected void setAttributes(Identifier[] attributeIdentifiers)
                          throws DataManagerException {
        attributes = makeIdentifierList(attributeIdentifiers);
        attributesAreCached = true;
        setAttributesDirty();
    }

    protected void addBase(Identifier base) throws DataManagerException {
        ensureBasesAreCached();
        bases.add(base);
        setBasesDirty();
    }

    protected void setBases(Identifier[] baseIdentifiers)
                     throws DataManagerException {
        bases = makeIdentifierList(baseIdentifiers);
        basesAreCached = true;
        setBasesDirty();
    }

    public void detach(Identifier attribute) throws DataManagerException {
        ensureAttributesAreCached();
        attributes.remove(attribute);
        setAttributesDirty();
    }

    protected void removeBase(Identifier base) throws DataManagerException {
        ensureBasesAreCached();
        bases.remove(base);
        setBasesDirty();
    }

    public String getName() throws DataManagerException {
        if (!nameIsCached) {
            name = proxiedEntity.getName();
        }

        return name;
    }

    public EntityValue getValue() throws DataManagerException {
        if (!valueIsCached) {
            value = proxiedEntity.getValue();
        }

        return value;
    }

    public void setValue(EntityValue newValue) throws DataManagerException {
        value = newValue;
        valueIsCached = true;
        setValueDirty();
    }

    /**
     * Converts an array of <code>Entity</code>s to a List of
     * <code>Entity</code>s
     */
    private List entityArrayToList(Entity[] entityArray) {
        List entityList = new ArrayList();

        for (int entityIndex = 0;
             entityIndex < entityArray.length;
             ++entityIndex)
            entityList.add(entityArray[entityIndex]);

        return entityList;
    }

    /**
     * Ensures that the base list from the proxied Entity has been cached in
     * this CachingProxyEntity.
     */
    private void ensureBasesAreCached() throws DataManagerException {
        if (!basesAreCached) {
            bases = entityArrayToList(proxiedEntity.getBases(
                                              EntityConstraint.NONE));
            basesAreCached = true;
        }
    }

    /**
     * Ensures that the attribute list from the proxied Entity has been cached
     * in this CachingProxyEntity.
     */
    private void ensureAttributesAreCached() throws DataManagerException {
        if (!attributesAreCached) {
            attributes = entityArrayToList(proxiedEntity.getAttributes(
                                                   EntityConstraint.NONE));
            attributesAreCached = true;
        }
    }

    public Entity[] getAttributes(EntityConstraint constraint)
                           throws DataManagerException {
        ensureAttributesAreCached();

        return chooseEntities(constraint, attributes);
    }

    public Entity[] getBases(EntityConstraint constraint)
                      throws DataManagerException {
        ensureBasesAreCached();

        return chooseEntities(constraint, bases);
    }

    public void subscribe(EventHandler listener, EventConstraint constraint)
                   throws DataManagerException {
        throw new UnsupportedOperationException(
                "subscribe() was called on CachingProxyEntity, which does not support this method!");
    }

    public void unsubscribe(EventHandler handler) throws DataManagerException {
        throw new UnsupportedOperationException(
                "unsubscribe() was called on CachingProxyEntity, which does not support this method!");
    }

    public String toString() {
        try {
            return "Entity[" + "[name: " + getName() + "]" + "[value: " + 
                   getValue() + "]" + "]";
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }
}

/*
 *
 * $Log: CachingProxyEntity.java,v $
 * Revision 1.5  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.4  2002/12/10 04:53:04  mliggett
 * Added setAttributes() and setBases() as protected members of Entity.
 * These are used by CachingProxyEntity so that it can write back entire
 * changed lists of attributes and bases.  Thus endeth the work on
 * CachingProxyEntityS.  Except that despite the fact that they pass the unit
 * tests, there seems to be a problem somewhere in getAttributes() which
 * causes it to always return 0 attributes.  Something about how the
 * constraints are applied to the attached EntityS or something.  Unclear at
 * the moment, but just try to run with this as your default EntityManager and
 * you'll see what I mean.  No Simpletons come up because
 * SimpletonLoaderProcess always gets zero SimpletonEntityValueS back.
 *
 * Also added makeIdentifierList helper method to Entity to enable
 * setAttributes() and setBases() implementations.
 *
 * Revision 1.3  2002/12/08 04:37:39  mliggett
 * Added caching, local changes to the cached values, dirty bits and write
 * back of locally modified values to CachingProxyEntity.  writeBack() is
 * currently not called anywhere but from CachingProxyEntity's finalizer,
 * but that will change in the next iteration.
 *
 * Also, writeBack() currently only writes back modified EntityValue.  If
 * the bases or attributes have changed, that's never reflected in the proxied
 * Entity.  I'll fix that in a moment, too.
 *
 * Revision 1.2  2002/12/06 11:54:53  alllee
 * Yet more codebase style
 *
 * Revision 1.1  2002/12/05 06:06:54  mliggett
 * This version just proxies the calls directly to the underlying Entity and
 * does no caching, dirty bits or anything.  That's next.  But:
 *
 * - Modified CachingProxyEntityManager so that it now creates
 *   CachingProxyEntityS and handles all other calls itself.  The
 *   proxiedEntityManager is now only used for createEntity().
 *
 * - Modified EntityProxy constructor to also take an EntityManager so that
 *   EntityProxyS will work properly when multiple EntityManagerS are running
 *   together in the system.
 *
 * - As a result, had to make a few other changes elsewhere in the system.
 *
 *
 */
