// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import java.util.Map;
import java.util.WeakHashMap;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;


/**
 * An EntityManager which provides <code>CachingProxyEntity</code>s.
 *
 * @see org.datamanager.kernel.CachingProxyEntity
 */
class CachingProxyEntityManager extends EntityManager {
    /** The only instance of CachingProxyEntityManager. */
    private static CachingProxyEntityManager instance;

    /** The EntityManager for which we proxy. */
    private EntityManager proxiedEntityManager;
    private Map entityToIdentifierMap = new WeakHashMap();
    private Map identifierToEntityMap = new WeakHashMap();
    private Map identifierToProxyMap = new WeakHashMap();

    /**
     * Constructs a new CachingProxyEntityManager which proxies for the
     * JispEntityManager.
     */
    private CachingProxyEntityManager() throws DataManagerException {
        // FIXME: make this configurable
        proxiedEntityManager = JispEntityManager.getInstance();
    }

    public static EntityManager getInstance() throws DataManagerException {
        if (null == instance) {
            instance = new CachingProxyEntityManager();
        }

        return instance;
    }

    public EntityProxy createEntity(String name) {
        try {
            EntityProxy proxiedEntityProxy = proxiedEntityManager.createEntity(
                                                     name);
            Entity cachingProxyEntity = new CachingProxyEntity(
                                                proxiedEntityProxy.getProxiedEntity());
            EntityProxy ourProxy = new EntityProxy(
                                           proxiedEntityProxy.getIdentifier(), 
                                           this);

            entityToIdentifierMap.put(cachingProxyEntity, 
                                      proxiedEntityProxy.getIdentifier());
            identifierToEntityMap.put(proxiedEntityProxy.getIdentifier(), 
                                      cachingProxyEntity);
            identifierToProxyMap.put(proxiedEntityProxy.getIdentifier(), 
                                     proxiedEntityProxy);

            return ourProxy;
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }

    public EntityProxy findProxy(Identifier identifier)
                          throws NoSuchEntityException {
        if (!identifierToProxyMap.containsKey(identifier)) {
            throw new NoSuchEntityException("identifier '" + identifier + 
                                            "' not found");
        }

        return (EntityProxy) identifierToProxyMap.get(identifier);
    }

    public Entity findEntity(Identifier identifier) throws NoSuchEntityException {
        if (!identifierToEntityMap.containsKey(identifier)) {
            throw new NoSuchEntityException("identifier '" + identifier + 
                                            "' not found");
        }

        return (Entity) identifierToEntityMap.get(identifier);
    }

    public Identifier findIdentifier(EntityProxy entityProxy) {
        return entityProxy.getIdentifier();
    }

    public Identifier findIdentifier(Entity entity)
                              throws NoSuchIdentifierException {
        if (entity instanceof EntityProxy) {
            return findIdentifier((EntityProxy) entity);
        }

        if (!entityToIdentifierMap.containsKey(entity)) {
            throw new NoSuchIdentifierException("entity '" + entity + 
                                                "' not found");
        }

        return (Identifier) entityToIdentifierMap.get(entity);
    }

    public void persist() {
        proxiedEntityManager.persist();
    }

    public Pool getPool() {
        return proxiedEntityManager.getPool();
    }
}