// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Fixed a few typos/misspelings in comments.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// Fixed a typo in a comment.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// Added getKernelWindow() and changed to pass SimpletonList from
// SimpletonLoaderProcess to KernelWindow.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Changed to use KernelGui instead of KernelWindow.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-12 nab
// - added startPoolPersistingThread() method to persist the pool and entities
//   on a regular interval and call the method in main()
// - add a constant SAVE_INTERVAL_IN_MINUTES for the thread persist interval
// - removed the System.runFinalizersOnExit(true) call
///////////////////////////////////////////////////////////////////////////////
// 2002-10-14 cam/cjd
// 1. Turned the persistence by 1 minute intervals back on.
// 2. Made the persist call be to the EntityManager, not the pool.
///////////////////////////////////////////////////////////////////////////////

/**
 * DataManager.java
 */
package org.datamanager.kernel;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.PrintStream;

import java.net.URI;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import org.datamanager.event.DisposeGuiRequestEvent;
import org.datamanager.event.ExportGuiRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.DataManagerWorker;
import org.datamanager.kernel.KernelGUISimpleton;
import org.datamanager.kernel.SimpletonLoaderProcess;

import org.datamanager.tools.Debug;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * This is the startup code for the datamanager. When invoked this
 * class will load a KernelGui that can be used to run
 * Simpletons and UserInterfaces that are located in the startup
 * directory.
 *
 * @author Matthew Farrellee
 */
public class DataManager implements Runnable {
    /** The startup data XML file's name.  */
    public static final String STARTUP_DATA = "knownspace_config.xml";

    /** The interval between persistence writes. */
    private static final int SAVE_INTERVAL_IN_MINUTES = 1;

    /** The name of the ThreadGroup used by all created threads. */
    private static final String DATAMANAGER_THREADGROUP_NAME = 
            "DataManagerThreadGroup";

    /**
     * Private default constructor.
     */
    private DataManager() throws DataManagerException {
        final EntityManager entityManager = EntityManager.getInstance();

        DataManagerWorker shutdownWorker = new DataManagerWorker(new Simpleton() {
            public void process() {
                entityManager.persist();
            }
        });

        Runtime.getRuntime().addShutdownHook(shutdownWorker);
    }

    /**
     * Creates a FileInputStream that is the Startup Data File.
     */
    private InputStream getStartupDataFile() {
        InputStream fileStream = null;

        ClassLoader classLoader = this.getClass().getClassLoader();
        fileStream = classLoader.getResourceAsStream(STARTUP_DATA);

        if (fileStream == null) {
            try {
                fileStream = new FileInputStream(STARTUP_DATA);
            } catch (FileNotFoundException exception) {
                Debug.writeLogMessage("Unable to get configuration file: " + 
                                      exception.getMessage());
                JOptionPane.showMessageDialog(null, 
                                              "Unable to load configuration file: " + 
                                              exception.getMessage(), 
                                              "Unable to load configuration file.", 
                                              JOptionPane.ERROR_MESSAGE);
                System.exit(1);
            }
        }

        return fileStream;
    }

    /**
     * Starts the SimpletonLoaderProcess.
     */
    protected void startSimpletonLoader() throws DataManagerException {
        // This process loads all Simpletons.
        SimpletonLoaderProcess loader = new SimpletonLoaderProcess(
                                                getStartupDataFile());

        // Tries to set the priority.  Being unsuccessful currently does
        // nothing but prints a stack trace (forgets about the failure).  This
        // should probably be fixed.
        try {
            loader.setPriority(Simpleton.Priority.LOW_PRIORITY);
        } catch (DataManagerException exception) {
            exception.printStackTrace();
        }

        loader.start();
    }

    /**
     * This will start up a thread that will run every one minute.  This thread
     * will persist the pool, and all of its entities.
     */
    private static void startPoolPersistingThread() {
        new Thread(new Runnable() {
            public void run() {
                while (true) {
                    try {
                        Debug.writeLogMessage("*** Sleeping.");
                        Thread.sleep(1000 * 60 * SAVE_INTERVAL_IN_MINUTES);
                        Debug.writeLogMessage("*** starting to persist pool.");
                        EntityManager.getInstance().persist();
                        Debug.writeLogMessage("*** finished persisting pool.");
                    } catch (Exception exception) {
                        Debug.writeLogMessage("persisting the EntityManager" + " and Pool failed " + 
                                                  exception.getMessage());
                    }
                }
            }
        }, "KnownSpace persistence").start();
    }

    /**
     * Main method. Currently it does not take any parameters.
     * Creates a new DataManager and initializes it.
     */
    public static void main(String[] input) throws DataManagerException {
        // Sets up the debugger.
        Debug.setDebugger(true);


        // Produce messages.
        Debug.setMessenger(true);

        new DataManager().startWithNewThreadGroup();
    }

    /**
     * Starts this DataManager in a new Thread that is part of 
     * a new DataManagerThreadGroup. All child threads of this
     * will be in the same DataManagerThreadGroup.
     */
    private void startWithNewThreadGroup() {
        ThreadGroup threadGroup = new DataManagerThreadGroup(
                                          DATAMANAGER_THREADGROUP_NAME);
        Thread thread = new Thread(threadGroup, this);
        thread.start();
    }

    public void run() {
        startPoolPersistingThread();

        try {
            startSimpletonLoader();
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }

    /**
     * Inner class extending ThreadGroup used for trapping
     * uncaught exceptions.
     * FIXME: Wow, this doesn't do anything, but at least it
     * is here. In the future this should do something more
     * intelligent.
     */
    public class DataManagerThreadGroup extends ThreadGroup {
        public DataManagerThreadGroup() {
            this("DataManagerThreadGroup");
        }

        public DataManagerThreadGroup(String name) {
            super(name);
        }

        /**
         * Shows a message dialog announcing the uncaught
         * exception.
         *
         * FIXME: This should provide some intelligent mappings to Events that
         * should be fired to correct the situation.
         */
        public void uncaughtException(Thread thread, Throwable throwable) {
            final JOptionPane messagePane = new JOptionPane(
                                                    "Uncaught exception: " + 
                                                    throwable + 
                                                    "\n\nCaught from thread: " + 
                                                    thread.getName(), 
                                                    JOptionPane.ERROR_MESSAGE);
            final JDialog dialog = messagePane.createDialog(null, 
                                                            "Uncaught Exception.");
            dialog.setModal(false);
            dialog.setVisible(true);
            throwable.printStackTrace();
        }
    }
}

/*
 *
 * $Log: DataManager.java,v $
 * Revision 1.35  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.34  2002/12/05 03:18:49  alllee
 * Changed uncaughtException JOptionPane message dialog to be non-modal -
 * avoiding the ghost of a deadlock bug..
 *
 * Revision 1.33  2002/11/21 02:58:25  mliggett
 * - EntityManager.getInstance() now loads the proper EntityManager class
 *   using reflection and the system property
 *   org.datamanager.entity_manager.class.  It defaults to
 *   VerySimpleEntityManager.
 *
 * - As a result of all the horrible things that could go wrong during
 *   reflection procedure, getInstance() now throws DataManagerException, and
 *   hence so do a few other methods now, too.
 *
 * - Added direct calls to removeEntityFromMemory() before every call to
 *   loadIntoMemory() to ensure that the hashtable never grows above the
 *   specified size.  This will be refactored in the next revision.
 *
 * - SuaveEntityManagerTester's test was exploded into four different tests.
 *   Also, it now sets and unsets the org.datamanager.entity_manager.class
 *   property before and after each test run, so that it knows it's testing
 *   the right class.
 *
 * Revision 1.32  2002/11/14 07:30:47  mliggett
 * Added printStackTrace() to uncaughtException()
 *
 * Revision 1.31  2002/11/12 06:22:22  cjdent
 *
 * Adding a first implementation a ThreadGroup.uncaughtException() handler.
 * This will eventually allow us to do something intelligent with such things.
 * At the moment we just throw up a JOptionPane. See the FIXMEs for more.
 *
 * The change makes DataManager the parent thread in a ThreadGroup named
 * DataManagerThreadGroup. All other threads created afterwards will be
 * within that group, by default. DataManager has an inner class that
 * extends ThreadGroup and implements uncaughtException().
 *
 * Much of DataManager.main() was moved to other methods so the inner class
 * can be used.
 *
 * DataManager extends Runnable so ensure its parent status in the ThreadGroup.
 *
 * We're not entirely sure this is the right way to do this, so refactor or
 * complain at will.
 *
 * ThreadMonitorSimpleton was modified to show the ThreadGroup name. We expect
 * this to be helpful.
 *
 * Revision 1.30  2002/11/07 06:11:42  mliggett
 * Added a dialog box that pops up when the kernel can't find the
 * knownspace_config.xml file.
 *
 * Revision 1.29  2002/11/05 03:14:04  cjdent
 * This commits is a significant change to the way persistence works.
 *
 * It still uses Serializable but now the way the file is created, found,
 * and used it changed.
 *
 * This change is being made to make persistence under Web Start work
 * correctly as well as provide a more reasonable default. It will cause
 * the ant clean target in build.xml to not clean persistence. That will
 * need to be cleaned up, it may break the build depending on how
 * cruisecontrol likes to be. All of this may go away when we get a more
 * integrated persistence mechanism.
 *
 * persist() in VerySimpleEntityManager and EntityManager no longer takes a
 * file. The method is expected to get the file itself. In
 * VerySimpleEntityManager this is done with the getPerisistenceFile().
 *
 * DataManager, KernelGUISimpleton, Pool were changed to reflect the new
 * signature on persist().
 *
 * The method for finding the file needs to be refactored, it is crufty. This
 * is marked with a FIXME. Method goes like this:
 *
 * use a directory called .helium in the user.home property. create the dir
 * if necessary and possible
 *
 * if that fails use a directory called .helium in the user.dir property (cwd).
 * create the dir if necessary and possible
 *
 * if neither of those work, set the file to the user.home/.helium location
 * and let things fail.
 *
 * There is currently a System.err.println in the code on purpose so I can
 * do some debugging from within in Web Start (which presents some debugging
 * challenges).
 *
 * Potential problems with this: what if you persist in the secondary location
 * but then on your next session the primary location becomes available. We
 * can chase that tail for absolutely forever so I don't worry about it here.
 *
 * Revision 1.28  2002/11/04 16:48:54  mafarrel
 * ran ant codebase-style...
 *
 * Revision 1.27  2002/11/04 05:43:18  alllee
 * Added the addShutdownHook() to the constructor for DataManager.  Uses a
 * DataManagerWorker to persist() the EntityManager and it's Pool.
 * This commit message would be a lot more insightful and harrowing except for
 * the fact that CDent is waiting on our CVS lock.
 * -baa
 *
 * Revision 1.26  2002/11/03 22:25:29  mliggett
 * Added a thread name to the persistence thread.
 *
 * Revision 1.25  2002/11/03 17:53:54  mafarrel
 * Ran codebase-style...
 *
 * Revision 1.24  2002/11/03 07:22:01  mafarrel
 * Turned the KernelGui into a Simpleton, and in the process added a
 * "RunOnStartup" tag to Simpleton and UserInterface tags in the
 * configuration file. The schema should be changed as well! The new
 * KernelGUISimpleton is currently the only Simpleton that is
 * RunOnStartup.
 *
 * The KernelGui has become the KernelGUISimpleton, and, instead of
 * getting a list of Simpletons directly from the SimpletonLoaderProcess,
 * the SimpletonLoaderProcess adds the loadable Simpletons to the Pool,
 * indirectly through the SimpletonList, so they can later be read by the
 * KernelGUISimpleton to present the user with a number of Simpletons
 * that they can load.
 *
 * Also, the KernelGUISimpleton no longer uses clone() to create new
 * instances of an already running Simpleton. Instead, it uses the
 * Simpleton's class and creates a new instance with the newInstance()
 * method. The KernelGUISimpleton still needs to place Simpletons in
 * their proper lists, i.e. as User Interfaces or normal Simpletons.
 *
 * A bug was fixed in the ValueClassIsConstraint, which should have some
 * unit tests. Also, this should probably be renamed to be consistent
 * with the EntityTypeIsConstraint.
 *
 * The changes in some files were simply because they were manually
 * creating Simpleton.Information objects and needed to provide default
 * values for two new parameters given to Information's constructor.
 *
 * Revision 1.23  2002/10/31 20:54:58  mafarrel
 * Fixed the SecurityManager problem. This
 * System.setSecurityManager(null) has been removed from DataManager.java
 * and <security><all-permissions/></security> has been added to the jaf
 * and javamail jnlp files. According to the JNLP Spec v1.0.1 section
 * 4.7, "If [an extension] points to a component extension (i.e., a JNLP
 * file with a component-desc element), then the resources described in
 * the resources element in that JNLP file become part of the
 * application's resources. The included resources will have the
 * permissions specified in the component extension." That is, the
 * security (aka permissions) set in the Helium.jnlp are not inherited by
 * its extensions.
 *
 * Revision 1.22  2002/10/31 15:52:45  mafarrel
 * Added System.setSecurityManager(null) to reset the security manager
 * that is being set by things like Java WebStart. This allows Helium to
 * make arbitrary network connections. However, this is not a complete
 * fix. We should find another workaround.
 *
 * Revision 1.21  2002/10/31 05:28:20  cjdent
 *
 * DataManager has been modified to first try to use ClassLoader.getResourceAsStream()
 * to get the config file. This is required for use with Web Start. If this
 * fails it falls back to loading as a FileInputStream.
 *
 * In either case, what now gets passed to SimpletonLoaderProcess for XML
 * processing is an InputStream and not a File as it was before.
 *
 * The configuration file loading in DataManager can throw an exception that right
 * now just causes an error message and the system to exit. Later this ought
 * to do something for real like show a dialog box to hunt for the file.
 *
 * Revision 1.20  2002/10/29 06:39:31  cjdent
 *
 * stopped automatically starting the PoolViewer. We have lots of other tools that
 * do the same thing, or will soon.
 *
 * Revision 1.19  2002/10/28 15:52:00  mafarrel
 * Reformatted with 'ant codebase-style'
 *
 * Revision 1.18  2002/10/28 06:03:41  mafarrel
 * Fixed a bug where the SimpletonLoaderProcess was having it's priority set AFTER it was started and therefore sometimes after it had finished processing. This behavior resulted in NullPointereExceptions from java.lang.Thread.setPriority()
 *
 * Revision 1.17  2002/10/28 02:42:59  mliggett
 * As a further aid to debugging and examination, the PoolViewer window is now
 * started by default.
 *
 * Revision 1.16  2002/10/25 05:15:43  cjdent
 *
 * Ensured that Debug.writeLogMessage() outputs the messages we
 * generate. Handy during these times of change.
 *
 *
 */
