// For licensing, see: http://helium.knownspace.org/license.html
//
//Entity.java
//Created: 09/22/1999 14:40
//
//Modified (10/05/2002): maa
//1) Corrected the class description to indicate that it is not providing many
//method implementations, if fact it is only providing two - one for the create()
//and one for the empty constructor.
//2) Clarified the documentation to indicate that the empty constructor in the
//Entity abstract class is there to ensure that all subclasses comply with the
//JavaBean specification and are therefore Serializable.
//3) Cleaned up the JavaDoc for the create() method.
//4) Remove the Debug.assert() call in the create() method.  Since that call was
//doing parameter checking for a public method, we did not replace this call with a
//Java 1.4 assertion.  The Java 1.4 assertions documentation
//says that assertions should not be used for such checks.
//See: http://java.sun.com/j2se/1.4/docs/guide/lang/assert.html#usage
//5) Cleaned up the JavaDoc for the abstract methods.  Move description of the
//usage of addBase(), removeBase(), attach(), and detach() methods to the class
//description section as it was referencing Entity's collaborations.  We need to
//revisit this description once we are more familiar with the way things work.
//Also need to revisit the descriptions for these four methods, once we are more
//familiar with the term "base."
//6) Moved class name and create date into this class description, rather than
//having them in a separate comment above.
//
//Modified (10/06/2002): maa
//1) Modified create() method that we had previously just removed Debug.assert()
//from to actually throw an IllegalArgumentException if the param is null.
//2) Added a EntityTester to test Entity collaborators  (more specifically Entity, VerySimpleEntity, and EntityProxy)
//3) Wrote test for attach() and detach().  Could not write tests for subscribe() and unsubscribe()
//without writing an implementation of EntityHandler.  Could not write tests for getValue() and
//setValue() without writing an implementation of EntityValue.
//4) Extended testAttach() and testDetach() to test all attach() and detach() methods.  attach()
//and detach() are overloaded methods.
//5) Added tests for addBases() and removeBases(). Also, extended the functionality of the testAttach()
//method to test that we are not allowed to attach an Entity as an attribute that has already been added
//as a base.
//6) Added tests for getAttributes(), getBases(), and a combined test for both
//getValue() and setValue().
//7) Added test for IndentifierManager/SimpleIdentifier's id generation.
//
//Modified (10/08/2002): maa
//1) Logged everywhere that we are throwing an IllegalArgumentException.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// Changed @return javadoc to not name the thing that is returned.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.EntityValue;

import org.datamanager.tools.Debug;


/**
 * The major workhorse of the DataManager, Entity is an abstract class which
 * only implements the create() method and the default, no args constructor.
 * The addBase(), removeBase(), attach(), and detach() methods are needed to help
 * uncouple the implementation of Entity storage and the kernel API. They are used
 * inside the kernel by the EntityProxy. No one outside of the kernel should
 * extend Entity so it is mostly safe to have these. Actually,
 * since no one outside of the kernel can see Identifier it should
 * be impossible to extend Entity outside of the kernel now.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
public abstract class Entity implements EventGenerator, Serializable {
    /**
     * A constructor existing only for the sake of subclasses.
     */
    protected Entity() {
        super();
    }

    /**
     * Create a new Entity with the given name. Note: Names do
     * not have to be unique.
     * @param name The name of the new Entity.
     * @return A new Entity.
     * @throws DataManagerException
     */
    public synchronized static Entity create(String name)
                                      throws DataManagerException {
        if (name == null) {
            Debug.writeLogMessage("Trying to create Entity with null name!");
            throw new IllegalArgumentException(
                    "Trying to create Entity with null name!");
        }

        return EntityManager.getInstance().createEntity(name);
    }

    /**
     * Attach the given Entity as an attribute.
     *
     * @param attribute The Entity to attach.
     * @throws DataManagerException
     */
    public abstract void attach(Entity attribute) throws DataManagerException;

    /**
     * Detach the given Entity.
     *
     * @param attribute The Entity to detach.
     * @throws DataManagerException
     */
    public abstract void detach(Entity attribute) throws DataManagerException;

    /**
     * Request the Entity's name.
     *
     * @return The Entity's name.
     * @throws DataManagerException
     */
    public abstract String getName() throws DataManagerException;

    /**
     * Request the Entity's value.
     *
     * @return The Entity's value.
     * @throws DataManagerException
     */
    public abstract EntityValue getValue() throws DataManagerException;

    /**
     * Request a change of the Entity's value.
     *
     * @param value The new EntityValue that you wish to give the Entity.
     * @throws DataManagerException
     */
    public abstract void setValue(EntityValue value) throws DataManagerException;

    /**
     * Request a constrained set of all entities linked to the Entity.
     *
     * @param constraint A constraint with which to filter attributes.
     * @return An array of entities that are attributes of
     * the Entity and satisfy the given constraint.
     * @throws DataManagerException
     */
    public abstract Entity[] getAttributes(EntityConstraint constraint)
                                    throws DataManagerException;

    /**
     * Sets the list of all attributes on this Entity.
     */
    protected abstract void setAttributes(Identifier[] attributeIdentifiers)
                                   throws DataManagerException;

    /**
     * Request a constrained set of all entities of which the Entity
     * is an attribute.
     *
     * @param constraint The EntityConstraint that you wish to apply to get the
     * bases of the Entity.
     * @return An array of entities that are bases of
     * the Entity and satisfy the given constraint.
     * @throws DataManagerException
     */
    public abstract Entity[] getBases(EntityConstraint constraint)
                               throws DataManagerException;

    /**
     * Sets the list of all bases on this Entity.
     */
    protected abstract void setBases(Identifier[] baseIdentifiers)
                              throws DataManagerException;

    /**
     * Subscribe to the Entity for DataManagerEvents satisfying the
     * given EventConstraint.
     * @param listener The EventHandler that will receive the events.
     * @param constraint The constraint used to filter events.
     * @throws DataManagerException
     */
    public abstract void subscribe(EventHandler listener, 
                                   EventConstraint constraint)
                            throws DataManagerException;

    /**
     * Remove all event subscriptions for which the EventHandler is
     * subscribed.
     *
     * @param handler The EventHandler to unsubscribe.
     * @throws DataManagerException
     */
    public abstract void unsubscribe(EventHandler handler)
                              throws DataManagerException;

    /**
     * A toString() method that overrides the one in Object.
     *
     * @return A String representation of this entity.
     */

    /* FIXME:xirao:mattL:chris: we need to think about if we can get rid of the toString() method
       the current problem is that the EntityProxy's toString() method will call
       getProxyEntity().toString(), but the getProxyEntity() could throw out an Exception. It is
       totally unreasonable to throw out an exception from toString() method
    */
    public abstract String toString();

    /**
      * Add an Entity to the list of bases.  This should only be called when
      * the base is adding the EntityEntity as an attribute, that is, this
      * should only be called by attach().
      *
      * @param identifier The Identifier of the Entity that is a new base.
      * @throws DataManagerException
      */
    protected abstract void addBase(Identifier identifier)
                             throws DataManagerException;

    /**
     * Remove an Entity from the list of bases.  This should only be called
     * by the base as it removes the Entity as an attribute (which it does
     * automatically), that is, this should only be called by detach().
     *
     * @param identifier The Identifier of the Entity that will no longer
     * be a base.
     * @throws DataManagerException
     */
    protected abstract void removeBase(Identifier identifier)
                                throws DataManagerException;

    /**
     * Attach the Entity with the given Identifier as an attribute.
     *
     * @param identifier The Identifier of the Entity to attach.
     * @throws DataManagerException
     */
    protected abstract void attach(Identifier identifier)
                            throws DataManagerException;

    /**
     * Detach the Entity with the given Identifier.
     *
     * @param identifier The Identifier of the Entity to detach.
     * @throws DataManagerException
     */
    protected abstract void detach(Identifier identifier)
                            throws DataManagerException;

    /**
     * Returns an array of all the Entities in the given List that satisfy the
     * given Constraint.
     *
     * @param constraint An EntityConstraint to filter the entities with.
     * @param candidateEntityIdentifiers A List of Identifiers for candidate Entitys
     *                                   to chose from.
     * @return An array of EntityProxys that were identified in the
     *         candidateEntityIdentifiers List and were accepted by the
     *         EntityConstraint.
     * @throws DataManagerException
     */
    protected Entity[] chooseEntities(EntityConstraint constraint, 
                                      List candidateEntityIdentifiers)
                               throws DataManagerException {
        List selectedEntityProxies = new Vector();

        for (int i = 0; i < candidateEntityIdentifiers.size(); i++) {
            if (constraintAcceptsEntity(constraint, 
                                        (Identifier) candidateEntityIdentifiers.get(i))) {
                selectedEntityProxies.add(EntityManager.getInstance()
                                                       .findProxy((Identifier) candidateEntityIdentifiers.get(i)));
            }
        }

        return getEntities(selectedEntityProxies);
    }

    /**
     * Turns a List of Entities into an Array of Entities.
     *
     * @param entityList A List of Entities to turn into an array.
     * @return entityArray An array of Entities.
     */
    protected Entity[] getEntities(List entityList) {
        if (entityList == null) {
            Debug.writeLogMessage(
                    "Trying to turn null List of Entitys into Entity[]!");
            throw new IllegalArgumentException(
                    "Trying to turn null List of Entitys into Entity[]!");
        }

        Entity[] entityArray = new Entity[entityList.size()];

        return (Entity[]) entityList.toArray(entityArray);
    }

    /**
     * Returns true if the EntityConstraint accepts the Entity associated with
     * the Identifier; false if not.
     *
     * @param constraint An EntityConstraint to test the Entity associated with
     *                  the Identifier against.
     * @param entityIdentifier An Identifier for the Entity that we wish to test.
     * @return A boolean indicating whether or not the EntityConstraint accepts
     *         the Entity associated with the Identifier.
     * @throws DataManagerException
     */
    protected boolean constraintAcceptsEntity(EntityConstraint constraint, 
                                              Identifier entityIdentifier)
                                       throws DataManagerException {
        return constraint.accepts(EntityManager.getInstance()
                                               .findProxy(entityIdentifier));
    }

    /**
     * Creates a list of Identifiers from the given array of Identifiers.
     */
    protected List makeIdentifierList(Identifier[] identifierArray) {
        List identifierList = new ArrayList();

        for (int identifierIndex = 0;
             identifierIndex < identifierArray.length;
             ++identifierIndex)
            identifierList.add(identifierArray[identifierIndex]);

        return identifierList;
    }
}