// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
//EntityManager.java
//Created: 09/28/1999 08:24
///////////////////////////////////////////////////////////////////////////////
//Modified (10/06/2002): maa
//1) Cleaned up JavaDoc to adhere to standards and eliminate conflicts.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// Cleaned up javadoc a little more.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 baa
// Added initial integration with persistence layer and modified some
// javadocs.  Added abstract persist(File file) and default-implemented
// restore(File file) methods.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 cam/cjd
// Turned on serializable style of persistence, rather than XML,
// as that works
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.File;
import java.io.Serializable;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.Properties;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.kernel.Persistable;
import org.datamanager.kernel.Pool;

import org.datamanager.tools.Debug;


/**
 * The EntityManager is responsible for maintaining lookup tables between
 * the system IDs, the EntityProxies, and the actually entities they proxy
 * for.  Ideally, an EntityManager should also cache unused Entities to
 * disk and perform other such optomizations.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
abstract class EntityManager implements Serializable {
    /**
     * The class name of the default implementation.
     */
    protected static final String DEFAULT_IMPLEMENTATION_CLASS_NAME = 
            "org.datamanager.kernel.VerySimpleEntityManager";

    /**
     * The name of the system property that will be checked for
     * an alternative default listener agent class name.
     */
    protected static final String IMPLEMENTATION_PROPERTY_NAME = 
            "org.datamanager.kernel.entity_manager.class";

    /**
     * Our instance of an Entity Manager.
     */
    private static EntityManager entityManager = null;

    /**
     * Requests the instance of the EntityManager.
     *
     * @return The instance of the EntityManager.
     *
     * @throws DataManagerException
     *         If creation of the EntityManager failed. The actual
     *         reason for the failure can be retrieved with the
     *         getMessage() method.
     */
    static EntityManager getInstance() throws DataManagerException {
        if (entityManager == null) {
            entityManager = createEntityManagerInstance();
        }

        return entityManager;
    }

    /**
     * Causes EntityManager to forget the previously cached instance of
     * EntityManager so that the next call will necessarily generate a new
     * EntityManager instance.  This can be used if you wish to modify the value
     * of the system property EntityManager.IMPLEMENTATION_PROPERTY_NAME at
     * runtime.
     */
    static void forgetInstance() {
        entityManager = null;
    }

    private static EntityManager createEntityManagerInstance()
        throws DataManagerException {
        EntityManager entityManager = null;

        // Look for the default EntityManager's class name.
        String entityManagerClassName = System.getProperties()
                                              .getProperty(IMPLEMENTATION_PROPERTY_NAME, 
                                                           DEFAULT_IMPLEMENTATION_CLASS_NAME);

        try {
            // Try to load the EntityManager's class.
            Class entityManagerClass = Class.forName(entityManagerClassName);

            // Try to get the class's getInstance() static method.
            Method getInstanceMethod = entityManagerClass.getDeclaredMethod(
                                               "getInstance", new Class[0]);


            // Get the EntityManager.
            entityManager = (EntityManager) getInstanceMethod.invoke(null, 
                                                                     new Class[0]);
        } catch (NoSuchMethodException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        } catch (ClassCastException exception) {
            throw new DataManagerException(exception);
        } catch (IllegalAccessException exception) {
            throw new DataManagerException(exception);
        } catch (InvocationTargetException exception) {
            throw new DataManagerException(exception);
        } catch (SecurityException exception) {
            throw new DataManagerException(exception);
        }

        // This really should not happen. All errors should have been
        // reported in one of the exceptions above.
        if (entityManager == null) {
            throw new DataManagerException(
                    "Failed to create EntityManager from: " + 
                    entityManagerClassName);
        }

        return entityManager;
    }

    /**
     * Creates a new Entity with the given name.
     *
     * @param String The name of the new Entity.
     * @return A new EntityProxy pointing to a new Entity with the given name.
     */
    protected abstract EntityProxy createEntity(String name);

    /**
     * Finds the EntityProxy that has the given identifier.
     *
     * @param Identifier The Identifier for which to find the EntityProxy.
     * @return The EntityProxy with the given Identifier.
     */
    protected abstract EntityProxy findProxy(Identifier identifier)
                                      throws NoSuchEntityException;

    /**
     * Finds the Entity that has the given identifier.
     *
     * @param Identifier The Identifier for which to find the Entity.
     * @return The Entity with the given Identifier.
     */
    protected abstract Entity findEntity(Identifier identifier)
                                  throws NoSuchEntityException;

    /**
     * Finds the Identifier that identifies the given EntityProxy.
     *
     * @param EntityProxy The EntityProxy for which to find the Identifier.
     * @return The Identifier for the given EntityProxy.
     */
    protected abstract Identifier findIdentifier(EntityProxy entityProxy);

    /**
     * Finds the Identifier that identifies the given entity.
     *
     * @param Entity The Entity for which to find the Identifier.
     * @return The Identifier for the given entity.
     */
    protected abstract Identifier findIdentifier(Entity entity)
                                          throws NoSuchIdentifierException;

    /**
     * Persists this EntityManager.
     *
     */
    protected abstract void persist();

    /**
     * Restores the EntityManager specified by the given inputFile
     *
     * @param File inputFile from which an EntityManager is to be restored.
     * @return EntityManager restored from the file
     */
    protected static EntityManager restore(File inputFile) {
        return (EntityManager) Persistable.PERSISTER.restore(inputFile);
    }

    /** Gets the Pool associated with this EntityManager. */
    protected abstract Pool getPool();
}