// For licensing, see: http://helium.knownspace.org/license.html
//
//EntityProxy.java
//
//Modified (10/06/2002): maa
//1) Cleaned up JavaDoc to adhere to standards and eliminate conflicts.
//2) Is there any way that we can separate the entity proxy stuff out a bit from entity?
//3) Cleaned up equals method (redundant null check after instanceof check).
//4) Used replace temp with query on entity temp variable in attach() and detach() methods.
//5) Changed the addBase(Identifier), removeBase(Identifier), attach(Identifier), and
//detach(Identifier).  Replaced Debug.assert() with an UnsupportedOperationException.
//6) Changed loadNewAssistant to only loadNewAssistant if assistant instace variable is null.
//This allows callers of this method to not have to worry about checking for null.
//7) Removed long comment about the ?special?order of method calls in subscribe() method and
//just called loadNewAssistant.
//
//Modified (10/08/2002): maa
//1) Replaced Debug.assert()s for null params with IllegalArgumentExceptions.
//2) Logged everywhere that we are throwing an IllegalArgumentException.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// 1. Changed @return javadoc to not name the thing that is returned.
// 2. Adjusted tone and information in javadoc for style and clarity.
// 3. Removed unnecessary cast to EntityProxy in addBase(EntityProxy).
// 4. Adjusted removeBase(Entity) so that it takes an EntityProxy instead to
//    mirror addBase.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import org.datamanager.activeentityvalue.ActiveEntityValue;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.event.AttachLinkEvent;
import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.DetachLinkEvent;
import org.datamanager.event.GetValueEvent;
import org.datamanager.event.SetValueEvent;
import org.datamanager.event.SubscribeEventGeneratorEvent;
import org.datamanager.event.UnsubscribeEventGeneratorEvent;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.EventGeneratorAssistant;
import org.datamanager.kernel.Identifier;

import org.datamanager.passiveentityvalue.NullEntityValue;

import org.datamanager.tools.Debug;


/**
 * EntityProxies are what the Kernel deals out to Simpletons requesting
 * Entities.  They have an Identifier which they feed to the EntityManager
 * to delegate their methods to an appropriate Entity implementation for
 * the current object store.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
class EntityProxy extends Entity {
    /** The Identifier of the Entity that is being proxied. */
    private Identifier identifier;

    /** The EntityManager of the Entity that is being proxied. */
    private EntityManager entityManager;

    /** Delegate all subscription issues to this assistant. */
    private transient EventGeneratorAssistant assistant;

    /**
     * Creates an EntityProxy that proxies for the Entity with the given Identifier.
     * @param identifier The Identifier of the Entity for which
     *                   the EntityProxy will be a proxy.
     */
    protected EntityProxy(Identifier identifier, EntityManager entityManager) {
        this.identifier = identifier;
        this.entityManager = entityManager;
    }

    /**
     * Gets the Identifier of the Entity that is being
     * proxied.
     *
     * @return The Identifier of the Entity that is being
     *         proxied.
     */
    protected Identifier getIdentifier() {
        return identifier;
    }

    /**
     * Gets the EntityManager of the Entity that is being proxied.
     */
    protected EntityManager getEntityManager() {
        return entityManager;
    }

    /**
     * Gets the Entity that this EntityProxy points at.
     *
     * @return The Entity the EntityProxy points at.
     */
    protected Entity getProxiedEntity() throws DataManagerException {
        return getEntityManager().findEntity(identifier);
    }

    /**
     * Loads a new EventGeneratorAssistant for this EntityProxy.
     * The EventGeneratorAssistant is transient, so this
     * method is used to reload the EventGeneratorAssistant
     * when needed.
     */
    protected void loadNewAssistant() {
        if (assistant == null) {
            assistant = new EventGeneratorAssistant();
        }
    }

    /**
     * Fires an event from this EntityProxy.
     *
     * @param event The event to be fired.
     */
    protected void fireEvent(DataManagerEvent event) {
        if (event == null) {
            Debug.writeLogMessage("Firing a null event!");
            throw new IllegalArgumentException("Firing a null event!");
        }

        loadNewAssistant();
        assistant.fireEvent(event);
    }

    /**
     * Adds an Entity to the list of bases.  This should only be called from
     * the attach() method of the <code>base</code>.  That is, when it has just
     * added this EntityProxy as an attribute of itself.
     *
     * @param base The Entity that is a new base.
     * @throws DataManagerException
     */
    protected void addBase(EntityProxy base) throws DataManagerException {
        if (base == null) {
            Debug.writeLogMessage("Adding a null base!");
            throw new IllegalArgumentException("Adding a null base!");
        }

        getProxiedEntity().addBase(base.getIdentifier());
        fireEvent(new AttachLinkEvent((EventGenerator) this, base));
    }

    /**
     * Remove an Entity from the list of bases.  This should only be called
     * by the base as it removes the Entity as an attribute (which it does
     * automatically), that is, this should only be called by detach().
     *
     * @param base The Entity that will no longer be a base.
     * @throws DataManagerException
     */
    protected void removeBase(EntityProxy base) throws DataManagerException {
        if (base == null) {
            Debug.writeLogMessage("Removing a null base!");
            throw new IllegalArgumentException("Removing a null base!");
        }

        getProxiedEntity().removeBase(base.getIdentifier());
        fireEvent(new DetachLinkEvent((EventGenerator) this, base));
    }

    /**
     * A hashCode() method to override the one in Object.
     * The hashcode of an EntityProxy is the hashCode of the Identifier for the
     * Entity for which it is a proxy.
     *
     * @return The EntityProxy's hashCode.
     */
    public int hashCode() {
        return identifier.hashCode();
    }

    /**
     * Equals() method that overrides the one in Object.
     * Determines equality based first on whether the object passed in is an EntityProxy
     * and second on whether the Entitys that this and the object passed in are
     * proxying for have the same Identifier.
     *
     * @param object An object to test and see if it is
     *               the same as the EntityProxy.
     * @return A boolean indicating whether the EntityProxy and object are
     *         the same.
     */
    public boolean equals(Object object) {
        boolean equals = false;

        if ((object instanceof EntityProxy) && 
                identifier.equals(((EntityProxy) object).getIdentifier())) {
            equals = true;
        }

        return equals;
    }

    // The following methods are required by the Entity interface.

    /**
     * Attaches the given Entity as an attribute.  Relies on a private method
     * call to make a link back from the attribute.
     *
     * @param attribute The Entity to attach as an attribute.
     * @throws DataManagerException
     */
    public void attach(Entity attribute) throws DataManagerException {
        if (attribute == null) {
            Debug.writeLogMessage("Attaching a null attribute!");
            throw new IllegalArgumentException("Attaching a null attribute!");
        }

        getProxiedEntity().attach(((EntityProxy) attribute).getIdentifier());
        fireEvent(new AttachLinkEvent((EventGenerator) this, attribute));
        ((EntityProxy) attribute).addBase(this);
    }

    /**
     * Detaches an attribute. Relies on a private method
     * call to remove the link back from the attribute.
     *
     * @param attribute The Entity to detach.
     * @throws DataManagerException
     */
    public void detach(Entity attribute) throws DataManagerException {
        if (attribute == null) {
            Debug.writeLogMessage("Detaching a null attribute!");
            throw new IllegalArgumentException("Detaching a null attribute!");
        }

        getProxiedEntity().detach(((EntityProxy) attribute).getIdentifier());
        fireEvent(new DetachLinkEvent((EventGenerator) this, attribute));
        ((EntityProxy) attribute).removeBase(this);
    }

    /**
     * Gets the name of the Entity being proxied.
     *
     * @return The name of the Entity being proxied.
     * @throws DataManagerException
     */
    public String getName() throws DataManagerException {
        return getProxiedEntity().getName();
    }

    /**
     * Gets the value of the Entity being proxied.
     *
     * @return The value of the Entity being proxied.
     * @throws DataManagerException
     */
    public EntityValue getValue() throws DataManagerException {
        fireEvent(new GetValueEvent((EventGenerator) this));

        EntityValue result = getProxiedEntity().getValue();

        if (result instanceof ActiveEntityValue) {
            return ((ActiveEntityValue) result).getEntityValue();
        } else {
            return result;
        }
    }

    /**
     * Sets the value of the Entity being proxied.
     *
     * @param value The Entity being proxied's new value.
     * @throws DataManagerException
     */
    public void setValue(EntityValue value) throws DataManagerException {
        EntityValue oldValue = getProxiedEntity().getValue();
        EntityValue newValue;

        if (value == null) {
            newValue = NullEntityValue.getInstance();
        } else {
            newValue = value;
        }

        getProxiedEntity().setValue(newValue);

        fireEvent(new SetValueEvent((EventGenerator) this, oldValue, newValue));
    }

    /**
     * Gets a constrained array of all attribute entities of the Entity
     * being proxied.
     *
     * @param constraint The constraint used to select attributes.
     * @return An array of attributes that fit the constraint.
     * @throws DataManagerException
     */
    public Entity[] getAttributes(EntityConstraint constraint)
                           throws DataManagerException {
        return getProxiedEntity().getAttributes(constraint);
    }

    protected void setAttributes(Identifier[] attributeIdentifiers)
                          throws DataManagerException {
        throw new UnsupportedOperationException();
    }

    /**
     * Gets a constrained array of all entities for which the Entity
     * being proxied is an attribute.
     *
     * @param constraint The constraint used to select bases.
     * @return An array of bases that fit the constraint.
     * @throws DataManagerException
     */
    public Entity[] getBases(EntityConstraint constraint)
                      throws DataManagerException {
        return getProxiedEntity().getBases(constraint);
    }

    protected void setBases(Identifier[] baseIdentifiers)
                     throws DataManagerException {
        throw new UnsupportedOperationException();
    }

    /**
     * Subscribes the given EventHandler to the Entity being proxied
     * for DataManagerEvents that satisfy the event constraint.
     *
     * @param listener The EventHandler that will receive events.
     * @param constraint The constraint used to filter events.
     * @throws DataManagerException
     */
    public void subscribe(EventHandler listener, EventConstraint constraint)
                   throws DataManagerException {
        loadNewAssistant();
        assistant.addEventHandler(listener, constraint);
        fireEvent(new SubscribeEventGeneratorEvent((EventGenerator) this, 
                                                   listener, constraint));
    }

    /**
     * Removes all event subscriptions the Simpleton has requested
     * from the Entity being proxied.
     *
     * @param handler The EventHandler to unsubscribe.
     * @throws DataManagerException
     */
    public void unsubscribe(EventHandler handler) throws DataManagerException {
        loadNewAssistant();
        assistant.removeEventHandler(handler);
        fireEvent(new UnsubscribeEventGeneratorEvent((EventGenerator) this, 
                                                     handler));
    }

    /**
     * Always throws UnsupportedOperationException. Method only exists because
     * this class extends Entity.
     */
    protected void addBase(Identifier identifier) throws DataManagerException {
        throw new UnsupportedOperationException(
                "addBase(Identifier) was called on EntityProxy, which does not support this method!");
    }

    /**
     * Always throws UnsupportedOperationException. Method only exists because
     * this class extends Entity.
     */
    protected void removeBase(Identifier identifier) throws DataManagerException {
        throw new UnsupportedOperationException(
                "removeBase(Identifier) was called on EntityProxy, which does not support this method!");
    }

    /**
     * Always throws UnsupportedOperationException. Method only exists because
     * this class extends Entity.
     */
    public void attach(Identifier identifier) throws DataManagerException {
        throw new UnsupportedOperationException(
                "attach(Identifier) was called on EntityProxy, which does not support this method!");
    }

    /**
     * Always throws UnsupportedOperationException. Method only exists because
     * this class extends Entity.
     */
    public void detach(Identifier identifier) throws DataManagerException {
        throw new UnsupportedOperationException(
                "detach(Identifier) was called on EntityProxy, which does not support this method!");
    }

    public String toString() {
        try {
            return getProxiedEntity().toString();
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }
}