// For licensing, see: http://helium.knownspace.org/license.html
//
//IdentifierManager.java
//Created: 11/19/1999 16:44
//
//Modified (10/06/2002): maa
//1) Cleaned up JavaDoc to adhere to standards and eliminate conflicts.
//2) Cleaned up equals method (redundant null check after instanceof check).
//3) Used replace temp with query to eliminate temp index variable in SimpleIdentifier constructor.
//4) Renamed identifier and extra instance variables of SimpleIdentifier to dateSection and longSection respectively.
//5) Renamed format temp variable in generateIdentifier() to formattedDateString for clarity.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 cam
// 1. Cleaned up JavaDoc.
// 2. Further simplified the SimpleIdentifier.equals() method so no temp
//    variables or conditionals are needed.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.Serializable;

import org.datamanager.kernel.Identifier;


/**
 * The IdentifierManager class is a source of unique Identifiers.
 *
 * @author Benjamin Keil
 * @author Matthew Farrellee
 */
class IdentifierManager implements Serializable {
    /** The only instance of the IdentifierManager class. */
    private static IdentifierManager instance = new IdentifierManager();

    /**
     * The default constructor. It is private because IdentifierManager
     * is a singleton.
     */
    private IdentifierManager() {
    }

    /**
     * Returns the Singleton instance of the IdentifierManager.
     *
     * @return The instance of IdentifierManager.
     */
    public static IdentifierManager getInstance() {
        return instance;
    }

    /**
     * Returns the next available unique identifier.
     *
     * @return A new identifier.
     */
    public static Identifier getIdentifier() {
        return SimpleIdentifier.generateIdentifier();
    }

    /**
     * The SimpleIdentifier is an inner class that is responsible for generating unique
     * Identifiers, using a combination of the date and a long.
     *
     * @author Benjamin Keil
     * @author Matthew Farrellee
     */
    private static class SimpleIdentifier implements Identifier {
        /** The last Identifier that was created. */
        protected static SimpleIdentifier lastIdentifier = 
                new SimpleIdentifier(0, 0);

        /** The long value of the Identifier. */
        protected final long dateSection;

        /** A number to help distinguish between Identifiers. */
        protected final long longSection;

        /**
         * Constructs a unique Identifier.
         *
         * @param dateSection A long version of the date.
         * @param longSection The current incremented value of the long numeric that ensures
         *                    that Indentifiers are unique.
         */
        public SimpleIdentifier(long dateSection, long longSection) {
            this.dateSection = dateSection;
            this.longSection = longSection;
        }

        /**
         * Gets the long Date part of this Identifier.
         *
         * @return The long Date part of the Identifier
         */
        protected long getDateSection() {
            return this.dateSection;
        }

        /**
         * Gets the long part of this Identifier.
         *
         * @return The long part of the Indentifier.
         */
        protected long getLongSection() {
            return this.longSection;
        }

        public int hashCode() {
            return (int) (getDateSection() ^ getLongSection());
        }

        /**
         * Checks to see if the given Object is the same as this Identifier.
         *
         * @param object A candidate for equality.
         * @return A boolean indicating whether the given object and the Identifer are
         *         the same.
         */
        public boolean equals(Object object) {
            return (object instanceof SimpleIdentifier) && 
                   (((SimpleIdentifier) object).getDateSection() == this.getDateSection()) && 
                   (((SimpleIdentifier) object).getLongSection() == this.getLongSection());
        }

        /**
         * Gets a String representation of this Identifier.
         *
         * @return A String representation of the Identifier.
         */
        public String toString() {
            return dateSection + ":" + longSection;
        }

        /**
         * Generates a unique Identifier.
         *
         * @return A new, unique Identifier.
         */
        protected static Identifier generateIdentifier() {
            long date = System.currentTimeMillis();

            long longSection = 0;

            if (lastIdentifier.getDateSection() == date) {
                longSection = lastIdentifier.getLongSection() + 1;
            }

            SimpleIdentifier identifier = new SimpleIdentifier(date, 
                                                               longSection);

            lastIdentifier = identifier;

            return identifier;
        }
    }
}

/*
 * $Log: IdentifierManager.java,v $
 * Revision 1.12  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.11  2002/11/30 11:07:30  alllee
 * *** empty log message ***
 *
 * Revision 1.10  2002/11/26 08:04:54  cjdent
 *
 * Changed structure of identifier from a string made up two longs
 * to just two longs and then adjusting the equals() and hashcode()
 * methods accordingly.
 *
 * This moves the SimpleIdentifier way down the profilers list. If
 * we wish to improve things a bit we can probably avoid two
 * method calls in equals by using the object variables instead
 * of getters but there seem to be other fish to fry at the moment.
 *
 * Revision 1.9  2002/11/24 21:06:00  cjdent
 *
 * Got rid of using new Date.getTime() and now simply use
 * System.currentTimeMillis() which is basically the same
 * thing without needing to go to the trouble of creating
 * a new object.
 *
 * This is based on Xi's suggestion to reuse
 * ComparableIdentifier. I didn't do that bacause
 * of the different interfaces.
 *
 *
 */
