// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import com.coyotegulch.jisp.HashIndex;
import com.coyotegulch.jisp.IndexedObjectDatabase;
import com.coyotegulch.jisp.KeyObject;

import java.io.IOException;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.List;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.kernel.Entity;

import org.datamanager.passiveentityvalue.NullEntityValue;

import org.datamanager.tools.Debug;


/**
 * An Entity implementation that is always stored on disk in a Jisp database.
 *
 * @see org.datamanager.kernel.JispEntityManager
 * @version $Revision: 1.4 $
 */
class JispEntity extends Entity {
    private JispIdentifier identifier;

    /**
     * Creates an Entity with the given name.
     */
    protected JispEntity(String name, JispIdentifier identifier)
                  throws DataManagerException, IOException, 
                         ClassNotFoundException {
        this.identifier = identifier;

        getEntityManager().getNameDatabase().write(getKeyObjects(), name);
    }

    private JispEntityManager getEntityManager() throws DataManagerException {
        return (JispEntityManager) JispEntityManager.getInstance();
    }

    private KeyObject[] getKeyObjects() {
        KeyObject[] keyObjects = { identifier };

        return keyObjects;
    }

    /**
     * Attaches the given Entity as an attribute of this Entity.
     */
    public void attach(Entity attribute) throws DataManagerException {
        throw new UnsupportedOperationException(
                "attach() was called on JispEntity, which does not support this method!");
    }

    /**
     * Returns this Entity's identifier.
     */
    JispIdentifier getIdentifier() {
        return identifier;
    }

    /**
     * Detaches the given Entity as an attribute of this Entity.
     */
    public void detach(Entity attribute) throws DataManagerException {
        throw new UnsupportedOperationException(
                "detach() was called on JispEntity, which does not support this method!");
    }

    /**
     * Attaches the Entity associated with the given Identifier as an attribute
     * of this Entity.
     */
    public void attach(Identifier attribute) throws DataManagerException {
        Identifier[] attributes;
        Identifier[] newAttributes;

        try {
            attributes = (Identifier[]) getEntityManager()
                                            .getAttributesDatabase()
                                            .read(identifier, 
                                                  getEntityManager()
                                                      .getIdentifierAttributesIndex());

            if (attributes != null) {
                newAttributes = new Identifier[attributes.length + 1];
                System.arraycopy(attributes, 0, newAttributes, 0, 
                                 attributes.length);
            } else {
                newAttributes = new Identifier[1];
                attributes = new Identifier[0];
            }

            newAttributes[attributes.length] = attribute;
            getEntityManager().getAttributesDatabase()
                .write(getKeyObjects(), newAttributes);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    /**
     * Adds the Entity identified by the given Identifier to the list of bases.
     * This should only be called from the attach() method of the
     * <code>base</code>.  That is, when it has just added this EntityProxy as
     * an attribute of itself.
     */
    protected void addBase(Identifier base) {
        Identifier[] bases;
        Identifier[] newBases;

        try {
            bases = (Identifier[]) getEntityManager().getBasesDatabase()
                                       .read(identifier, 
                                             getEntityManager()
                                                 .getIdentifierBasesIndex());

            if (bases != null) {
                newBases = new Identifier[bases.length + 1];
                System.arraycopy(bases, 0, newBases, 0, bases.length);
            } else {
                newBases = new Identifier[1];
                bases = new Identifier[0];
            }

            newBases[bases.length] = base;
            getEntityManager().getBasesDatabase()
                .write(getKeyObjects(), newBases);
        } catch (IOException exception) {
            // FIXME:mml: In both these cases, we'd rather throw a
            // DataManagerException, but this method is not declared to throw
            // them.  Should it be?
            throw new RuntimeException(exception);
        } catch (ClassNotFoundException exception) {
            throw new RuntimeException(exception);
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }

    /**
     * Detaches the Entity identified by the given Identifier from this Entity.
     */
    public void detach(Identifier attribute) throws DataManagerException {
        Identifier[] attributes;
        Identifier[] newAttributes;

        try {
            attributes = (Identifier[]) getEntityManager()
                                            .getAttributesDatabase()
                                            .read(identifier, 
                                                  getEntityManager()
                                                      .getIdentifierAttributesIndex());

            if ((attributes != null) && (attributes.length > 0)) {
                newAttributes = new Identifier[attributes.length - 1];

                int newIndex = 0;

                for (int attributeIndex = 0;
                     attributeIndex < attributes.length;
                     ++attributeIndex)
                    if (!attributes[attributeIndex].equals(attribute)) {
                        newAttributes[newIndex++] = attributes[attributeIndex];
                    }
            } else {
                newAttributes = null;
            }

            getEntityManager().getAttributesDatabase()
                .write(getKeyObjects(), newAttributes);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    /**
     * Removes an Entity from this Entity's list of bases.  This should only be called
     * by the base as it removes this Entity as an attribute (which it does
     * automatically), that is, you should probably *NEVER* write a call to
     * this method explicitly.
     */
    protected void removeBase(Identifier base) {
        Identifier[] bases;
        Identifier[] newBases;

        try {
            bases = (Identifier[]) getEntityManager().getBasesDatabase()
                                       .read(identifier, 
                                             getEntityManager()
                                                 .getIdentifierBasesIndex());

            if ((bases != null) && (bases.length > 0)) {
                newBases = new Identifier[bases.length - 1];

                int newIndex = 0;

                for (int baseIndex = 0; baseIndex < bases.length; ++baseIndex)
                    if (!bases[baseIndex].equals(base)) {
                        newBases[newIndex++] = bases[baseIndex];
                    }
            } else {
                newBases = null;
            }

            getEntityManager().getBasesDatabase()
                .write(getKeyObjects(), newBases);
        } catch (IOException exception) {
            // FIXME:mml: In both these cases, we'd rather throw a
            // DataManagerException, but this method is not declared to throw
            // them.  Should it be?
            throw new RuntimeException(exception);
        } catch (ClassNotFoundException exception) {
            throw new RuntimeException(exception);
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }

    /**
     * Requests this Entity's name.
     */
    public String getName() throws DataManagerException {
        String name;

        try {
            name = (String) getEntityManager().getNameDatabase()
                                .read(identifier, 
                                      getEntityManager()
                                          .getIdentifierNameIndex());
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }

        if (null == name) {
            throw new NoSuchIdentifierException();
        }

        return name;
    }

    /**
     * Requests this Entity's value.
     */
    public EntityValue getValue() throws DataManagerException {
        EntityValue entityValue;

        try {
            entityValue = (EntityValue) getEntityManager()
                                            .getEntityValueDatabase()
                                            .read(identifier, 
                                                  getEntityManager()
                                                      .getIdentifierEntityValueIndex());
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }

        if (null == entityValue) {
            entityValue = NullEntityValue.getInstance();
        }

        return entityValue;
    }

    /**
     * Sets this Entity's value.
     */
    public void setValue(EntityValue newValue) throws DataManagerException {
        if (newValue == null) {
            newValue = NullEntityValue.getInstance();
        }

        try {
            getEntityManager().getEntityValueDatabase()
                .write(getKeyObjects(), newValue);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    /**
     * Gets a constrained array of all entities linked to this Entity.
     */
    public Entity[] getAttributes(EntityConstraint constraint)
                           throws DataManagerException {
        try {
            List matchingEntityProxies = new ArrayList();

            Identifier[] attributeIdentifiers = (Identifier[]) getEntityManager()
                                                                   .getAttributesDatabase()
                                                                   .read(identifier, 
                                                                         getEntityManager()
                                                                             .getIdentifierAttributesIndex());

            if (attributeIdentifiers == null) {
                attributeIdentifiers = new Identifier[0];
            }

            for (int attributeIndex = 0;
                 attributeIndex < attributeIdentifiers.length;
                 ++attributeIndex) {
                if (constraint.accepts(getEntityManager()
                                           .findProxy(attributeIdentifiers[attributeIndex]))) {
                    matchingEntityProxies.add(getEntityManager()
                                                  .findProxy(attributeIdentifiers[attributeIndex]));
                }
            }

            return entityListToEntityArray(matchingEntityProxies);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    protected void setAttributes(Identifier[] attributeIdentifiers)
                          throws DataManagerException {
        try {
            getEntityManager().getAttributesDatabase()
                .write(getKeyObjects(), attributeIdentifiers);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    private Entity[] entityListToEntityArray(List entityList) {
        Entity[] entityArray = new Entity[entityList.size()];

        return (Entity[]) entityList.toArray(entityArray);
    }

    /**
     * Gets a constrained array of all entities of which this Entity is an
     * attribute.
     */
    public Entity[] getBases(EntityConstraint constraint)
                      throws DataManagerException {
        try {
            List matchingEntityProxies = new ArrayList();

            Identifier[] baseIdentifiers = (Identifier[]) getEntityManager()
                                                              .getBasesDatabase()
                                                              .read(identifier, 
                                                                    getEntityManager()
                                                                        .getIdentifierBasesIndex());

            if (baseIdentifiers == null) {
                baseIdentifiers = new Identifier[0];
            }

            for (int baseIndex = 0;
                 baseIndex < baseIdentifiers.length;
                 ++baseIndex) {
                if (constraint.accepts(getEntityManager()
                                           .findProxy(baseIdentifiers[baseIndex]))) {
                    matchingEntityProxies.add(getEntityManager()
                                                  .findProxy(baseIdentifiers[baseIndex]));
                }
            }

            return entityListToEntityArray(matchingEntityProxies);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    protected void setBases(Identifier[] baseIdentifiers)
                     throws DataManagerException {
        try {
            getEntityManager().getBasesDatabase()
                .write(getKeyObjects(), baseIdentifiers);
        } catch (IOException exception) {
            throw new DataManagerException(exception);
        } catch (ClassNotFoundException exception) {
            throw new DataManagerException(exception);
        }
    }

    /**
     * Always throws UnsupportedOperationException, because this functionality is handled in EntityProxy.
     */
    public void subscribe(EventHandler listener, EventConstraint constraint)
                   throws DataManagerException {
        throw new UnsupportedOperationException(
                "subscribe() was called on JispEntity, which does not support this method!");
    }

    /**
     * Always throws UnsupportedOperationException, because this functionality is handled in EntityProxy.
     */
    public void unsubscribe(EventHandler handler) throws DataManagerException {
        throw new UnsupportedOperationException(
                "unsubscribe() was called on JispEntity, which does not support this method!");
    }

    public String toString() {
        try {
            return "Entity[" + "[name: " + getName() + "]" + "[value: " + 
                   getValue() + "]" + "]";
        } catch (DataManagerException exception) {
            throw new RuntimeException(exception);
        }
    }
}

/*
 * 
 * $Log: JispEntity.java,v $
 * Revision 1.4  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.3  2002/12/10 04:53:04  mliggett
 * Added setAttributes() and setBases() as protected members of Entity.
 * These are used by CachingProxyEntity so that it can write back entire
 * changed lists of attributes and bases.  Thus endeth the work on
 * CachingProxyEntityS.  Except that despite the fact that they pass the unit
 * tests, there seems to be a problem somewhere in getAttributes() which
 * causes it to always return 0 attributes.  Something about how the
 * constraints are applied to the attached EntityS or something.  Unclear at
 * the moment, but just try to run with this as your default EntityManager and
 * you'll see what I mean.  No Simpletons come up because
 * SimpletonLoaderProcess always gets zero SimpletonEntityValueS back.
 *
 * Also added makeIdentifierList helper method to Entity to enable
 * setAttributes() and setBases() implementations.
 *
 * Revision 1.2  2002/12/05 10:53:59  alllee
 * Some general hygiene changes - (swapped version/revision cvs keyword javadocin
 * JispEntity and removed some useless code in Pool).
 *
 * Revision 1.1  2002/12/03 05:04:30  mliggett
 * First stab at pure, jisp-only storage.  Obviously needs refactoring (which
 * will shorten the longer source files substantially) and a
 * looksee at the javadoc.  Separate message to be sent fully
 * explaining things, but these entities keep NO data in main memory.  All
 * four pieces of Entity data (name, value, attributes and bases) are kept in
 * separate indexed jisp databases and the contents are loaded on demand.
 *
 * Includes several unit tests.  Caching EntityManager to be checked in
 * next.
 *
 *
 */
