// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import com.coyotegulch.jisp.HashIndex;
import com.coyotegulch.jisp.IndexedObjectDatabase;
import com.coyotegulch.jisp.KeyObject;
import com.coyotegulch.jisp.ObjectIndex;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;

import java.util.Map;
import java.util.Vector;
import java.util.WeakHashMap;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.kernel.Persistable;

import org.datamanager.tools.Debug;


/**
 * An EntityManager that creates <code>JispEntity</code>s, which in turn have
 * their data stored in Jisp.
 *
 * @see org.datamanager.kernel.JispEntity
 */
class JispEntityManager extends EntityManager implements Serializable {
    public static final String PERSISTENCE_FILE_NAME = 
            "JispEntityManager.persist";

    // FIXME:mml: These files are just created in the current directory right
    // now.  They should be dropped into the home directory.
    public static final String NAME_DATABASE_FILE = "name.database";
    public static final String IDENTIFIER_NAME_INDEX_FILE = 
            "identifier.name.hashtable";
    public static final String ENTITY_VALUE_DATABASE_FILE = 
            "entityvalue.database";
    public static final String IDENTIFIER_ENTITY_VALUE_INDEX_FILE = 
            "identifier.entityvalue.hashtable";
    public static final String ATTRIBUTES_DATABASE_FILE = "attributes.database";
    public static final String IDENTIFIER_ATTRIBUTES_INDEX_FILE = 
            "identifier.attributes.hashtable";
    public static final String BASES_DATABASE_FILE = "bases.database";
    public static final String IDENTIFIER_BASES_INDEX_FILE = 
            "identifier.bases.hashtable";

    // FIXME:xirao: Jisp database can associate with hash-index, but we need pre-define
    // the length of hash-index when we create it. When Hash-index is used up. We can not
    // create entity. This is a huge limitation for Jisp database. Something, we deinitely
    // need to look into.
    // We may follow this scheme to fix it later: when the pre-define hash-index is reach,
    // we doule its value, we copy the old hash-index into new hash-index
    private static final int HASH_INDEX_LENGTH = 80000;

    /**
     * Number of buckets to be used in the hash table.  This should be prime.
     *
     * FIXME: don't hardcode this!
     */
    private static final int NUMBER_OF_HASH_BUCKETS = 104729;
    private static KeyObject[] keyObject = new KeyObject[2];

    /** The only instance of the JispEntityManager. */
    private static EntityManager instance = null;

    /** Persistence File. */
    private static File persistenceFile;

    // count the number of entity created. It assoicate with FIXME for jisp hash-index issue
    private int entityCounter = 0;

    /** The only instance of the Pool.  Made non-static for compliance with the
     * standard Serializable interface.  Plus, the future holds multiple Pools
     * for us.
     */
    protected Pool defaultPool;

    /** A handle for the IdentifierManager's instance. */
    protected JispIdentifierManager identifierManager = 
            JispIdentifierManager.getInstance();
    private IndexedObjectDatabase nameDatabase;
    private HashIndex identifierNameIndex;
    private IndexedObjectDatabase entityValueDatabase;
    private HashIndex identifierEntityValueIndex;
    private IndexedObjectDatabase attributesDatabase;
    private HashIndex identifierAttributesIndex;
    private IndexedObjectDatabase basesDatabase;
    private HashIndex identifierBasesIndex;
    private Map entityToIdentifierMap = new WeakHashMap();
    private Map identifierToEntityMap = new WeakHashMap();
    private Map identifierToProxyMap = new WeakHashMap();

    /** We subscribe to every EntityProxy we create with this constraint. */

    /*
     * FIXME: this belongs (eventually) in the caching EntityManager.
    private EventConstraint entityModifiedConstraint =
        new OrEventConstraint(
                new OrEventConstraint(new EventTypeIsConstraint(AttachLinkEvent.class),
                                      new EventTypeIsConstraint(DetachLinkEvent.class)),
                new EventTypeIsConstraint(SetValueEvent.class));
    */

    /**
     * Creates a new JispEntityManager with no mappings.
     */
    private JispEntityManager() throws IOException, ClassNotFoundException {
        bootPool();
        connectToDatabase();
        defaultPool = Pool.createDefaultPool();
    }

    IndexedObjectDatabase getNameDatabase() {
        return nameDatabase;
    }

    ObjectIndex getIdentifierNameIndex() {
        return identifierNameIndex;
    }

    IndexedObjectDatabase getEntityValueDatabase() {
        return entityValueDatabase;
    }

    ObjectIndex getIdentifierEntityValueIndex() {
        return identifierEntityValueIndex;
    }

    IndexedObjectDatabase getAttributesDatabase() {
        return attributesDatabase;
    }

    ObjectIndex getIdentifierAttributesIndex() {
        return identifierAttributesIndex;
    }

    IndexedObjectDatabase getBasesDatabase() {
        return basesDatabase;
    }

    ObjectIndex getIdentifierBasesIndex() {
        return identifierBasesIndex;
    }

    /**
     * Connects this JispEntityManager to its underlying database.  If the files
     * are not already there, it creates them.
     */
    private void connectToDatabase() throws IOException, ClassNotFoundException {
        File file = new File(NAME_DATABASE_FILE);

        if (file.exists()) {
            nameDatabase = connectToExistingDatabase(NAME_DATABASE_FILE);
            identifierNameIndex = attachExistingIndex(nameDatabase, 
                                                      IDENTIFIER_NAME_INDEX_FILE);

            entityValueDatabase = connectToExistingDatabase(
                                          ENTITY_VALUE_DATABASE_FILE);
            identifierEntityValueIndex = attachExistingIndex(
                                                 entityValueDatabase, 
                                                 IDENTIFIER_ENTITY_VALUE_INDEX_FILE);

            attributesDatabase = connectToExistingDatabase(
                                         ATTRIBUTES_DATABASE_FILE);
            identifierAttributesIndex = attachExistingIndex(attributesDatabase, 
                                                            IDENTIFIER_ATTRIBUTES_INDEX_FILE);

            basesDatabase = connectToExistingDatabase(BASES_DATABASE_FILE);
            identifierBasesIndex = attachExistingIndex(basesDatabase, 
                                                       IDENTIFIER_BASES_INDEX_FILE);
        } else {
            nameDatabase = createDatabase(NAME_DATABASE_FILE);
            identifierNameIndex = attachNewIndex(nameDatabase, 
                                                 IDENTIFIER_NAME_INDEX_FILE);

            entityValueDatabase = createDatabase(ENTITY_VALUE_DATABASE_FILE);
            identifierEntityValueIndex = attachNewIndex(entityValueDatabase, 
                                                        IDENTIFIER_ENTITY_VALUE_INDEX_FILE);

            attributesDatabase = createDatabase(ATTRIBUTES_DATABASE_FILE);
            identifierAttributesIndex = attachNewIndex(attributesDatabase, 
                                                       IDENTIFIER_ATTRIBUTES_INDEX_FILE);

            basesDatabase = createDatabase(BASES_DATABASE_FILE);
            identifierBasesIndex = attachNewIndex(basesDatabase, 
                                                  IDENTIFIER_BASES_INDEX_FILE);
        }
    }

    private IndexedObjectDatabase createDatabase(String filename)
                                          throws IOException {
        return new IndexedObjectDatabase(filename, true);
    }

    private HashIndex attachNewIndex(IndexedObjectDatabase database, 
                                     String indexFilename)
                              throws IOException, ClassNotFoundException {
        // Create the hash table with HASH_BUCKET buckets, HASH_INDEX_LENGTH
        // maximum entries, and keys of type JispIdentifier.
        HashIndex hashIndex = new HashIndex(indexFilename, 
                                            NUMBER_OF_HASH_BUCKETS, 
                                            HASH_INDEX_LENGTH, 
                                            ((JispIdentifier) JispIdentifier.generateIdentifier()).makeNullKey());
        database.attachIndex(hashIndex);

        return hashIndex;
    }

    private IndexedObjectDatabase connectToExistingDatabase(String databaseFilename)
                                                     throws IOException {
        return new IndexedObjectDatabase(databaseFilename, false);
    }

    private HashIndex attachExistingIndex(IndexedObjectDatabase database, 
                                          String indexFilename)
                                   throws IOException, ClassNotFoundException {
        HashIndex hashIndex = new HashIndex(indexFilename);
        database.attachIndex(hashIndex);

        return hashIndex;
    }

    /**
     * Creates new IndexedObjectDatabase and HashIndex with the given
     * parameters.
     */
    private void createDatabase(IndexedObjectDatabase database, 
                                String databaseFilename, HashIndex hashIndex, 
                                String hashIndexFilename)
                         throws IOException, ClassNotFoundException {
        database = new IndexedObjectDatabase(databaseFilename, true);


        // Create the hash table with HASH_BUCKET buckets, HASH_INDEX_LENGTH
        // maximum entries, and keys of type JispIdentifier.
        hashIndex = new HashIndex(hashIndexFilename, NUMBER_OF_HASH_BUCKETS, 
                                  HASH_INDEX_LENGTH, 
                                  ((JispIdentifier) JispIdentifier.generateIdentifier()).makeNullKey());

        database.attachIndex(hashIndex);
    }

    /**
     * Requests the only instance of the JispEntityManager.  The
     * non-null existence of the instance is guaranteed by loadEntityManager.
     *
     * @return The instance of the JispEntityManager.
     */
    protected static EntityManager getInstance() throws DataManagerException {
        if (instance == null) {
            try {
                instance = new JispEntityManager();
            } catch (IOException exception) {
                throw new DataManagerException(exception);
            } catch (ClassNotFoundException exception) {
                throw new DataManagerException(exception);
            }
        }

        return instance;
    }

    /**
     * Tries to restore the Pool from the persistence file.  Failing that, just
     * creates a new Pool.
     */
    private void bootPool() {
        if (defaultPool == null) {
            if (!restorePool()) {
                defaultPool = Pool.createDefaultPool();
            }
        }
    }

    /**
     * This method attempts to restore the Pool from persistence.  Returns true
     * if it succeeds, false otherwise.
     */
    private boolean restorePool() {
        if (getPersistenceFile().exists()) {
            // restore returns null if it fails.
            // FIXME:mml: should probably not be calling Persistable.PERSISTER
            // directly
            defaultPool = (Pool) Persistable.PERSISTER.restore(
                                  getPersistenceFile());
        }

        return defaultPool != null;
    }

    /**
     * Returns the File that is the persistence file
     */
    private static File getPersistenceFile() {
        if (persistenceFile == null) {
            persistenceFile = createPersistenceFile();
        }

        return persistenceFile;
    }

    /**
     * Creates the File object that is the persistence file.  Creates the
     * directory it lives in if necessary. First tries the user's home directory
     * and then the current working directory.
     */
    private static File createPersistenceFile() {
        String persistenceDirHomeName = System.getProperty("user.home") + 
                                        File.separator + 
                                        Persistable.PERSISTENCE_FOLDER;

        String persistenceDirCWDName = System.getProperty("user.dir") + 
                                       File.separator + 
                                       Persistable.PERSISTENCE_FOLDER;

        File persistenceDirHomeFile = new File(persistenceDirHomeName);
        File persistenceDirCWDFile = new File(persistenceDirCWDName);


        // Set a reasonable default
        persistenceFile = new File(persistenceDirHomeName + File.separator + 
                                   PERSISTENCE_FILE_NAME);

        // FIXME: the following hairy mess attempts to find a location for
        // persistence first looking in the users home directory, then looking
        // in the current working directory.
        if (persistenceDirHomeFile.exists()) {
            if (persistenceDirHomeFile.isDirectory()) {
                return persistenceFile;
            }
        } else if (persistenceDirHomeFile.mkdir()) {
            return persistenceFile;
        }

        Debug.writeLogMessage("Persistence directory " + 
                              persistenceDirHomeName + 
                              " exists as file not a directory. Trying " + 
                              persistenceDirCWDName);

        if (persistenceDirCWDFile.exists()) {
            if (persistenceDirCWDFile.isDirectory()) {
                return new File(persistenceDirCWDName + File.separator + 
                                PERSISTENCE_FILE_NAME);
            }
        } else if (persistenceDirCWDFile.mkdir()) {
            return new File(persistenceDirCWDName + File.separator + 
                            PERSISTENCE_FILE_NAME);
        }

        Debug.writeLogMessage("Unable to persist in " + 
                              persistenceDirHomeName + " or " + 
                              persistenceDirCWDName + 
                              " defaulting to failure.");

        return persistenceFile;
    }

    /**
     * Requests the default Pool.
     *
     * @return The default pool.
     */
    protected Pool getPool() {
        return defaultPool;
    }

    protected synchronized EntityProxy createEntity(String name) {
        if (entityCounter++ > this.HASH_INDEX_LENGTH) {
            // FIXME: We should grow the file dynamically instead of throwing an
            // exception.
            throw new RuntimeException("database is full!");
        }

        JispIdentifier identifier = identifierManager.getIdentifier();
        JispEntity entity;

        try {
            entity = new JispEntity(name, identifier);
        } catch (DataManagerException exception) {
            // FIXME: usual bogus exception handling
            throw new RuntimeException(exception);
        } catch (IOException exception) {
            // FIXME: usual bogus exception handling
            throw new RuntimeException(exception);
        } catch (ClassNotFoundException exception) {
            // FIXME: usual bogus exception handling
            throw new RuntimeException(exception);
        }


        // XXX: these mappings are not currently persisted... how should they
        // be?
        entityToIdentifierMap.put(entity, identifier);
        identifierToEntityMap.put(identifier, entity);

        EntityProxy entityProxy = new EntityProxy(identifier, this);

        identifierToProxyMap.put(identifier, entityProxy);

        return entityProxy;
    }

    protected Identifier findIdentifier(EntityProxy proxy) {
        if (proxy == null) {
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null EntityProxy!");
        }

        return proxy.getIdentifier();
    }

    protected Identifier findIdentifier(Entity entity)
                                 throws NoSuchIdentifierException {
        if (entity instanceof EntityProxy) {
            return findIdentifier((EntityProxy) entity);
        }

        return ((JispEntity) entity).getIdentifier();
    }

    protected EntityProxy findProxy(Identifier identifier)
                             throws NoSuchEntityException {
        EntityProxy entity = (EntityProxy) identifierToProxyMap.get(identifier);

        if (null == entity) {
            throw new NoSuchEntityException();
        }

        return entity;
    }

    protected Entity findEntity(Identifier identifier)
                         throws NoSuchEntityException {
        Entity entity = (Entity) identifierToEntityMap.get(identifier);

        if (null == entity) {
            throw new NoSuchEntityException();
        }

        return entity;
    }

    /**
     * Persists ONLY the pool to the persistence file.
     */
    protected void persist() {
        Persistable.PERSISTER.persist(getPool(), getPersistenceFile());
    }
}

/*
 *
 * $Log: JispEntityManager.java,v $
 * Revision 1.4  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.3  2002/12/05 06:06:54  mliggett
 * This version just proxies the calls directly to the underlying Entity and
 * does no caching, dirty bits or anything.  That's next.  But:
 *
 * - Modified CachingProxyEntityManager so that it now creates
 *   CachingProxyEntityS and handles all other calls itself.  The
 *   proxiedEntityManager is now only used for createEntity().
 *
 * - Modified EntityProxy constructor to also take an EntityManager so that
 *   EntityProxyS will work properly when multiple EntityManagerS are running
 *   together in the system.
 *
 * - As a result, had to make a few other changes elsewhere in the system.
 *
 * Revision 1.2  2002/12/05 01:23:34  mliggett
 * Added javadoc to top of class.
 *
 * Revision 1.1  2002/12/03 05:04:30  mliggett
 * First stab at pure, jisp-only storage.  Obviously needs refactoring (which
 * will shorten the longer source files substantially) and a
 * looksee at the javadoc.  Separate message to be sent fully
 * explaining things, but these entities keep NO data in main memory.  All
 * four pieces of Entity data (name, value, attributes and bases) are kept in
 * separate indexed jisp databases and the contents are loaded on demand.
 *
 * Includes several unit tests.  Caching EntityManager to be checked in
 * next.
 *
 *
 */
