// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import com.coyotegulch.jisp.KeyObject;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.Serializable;

import org.datamanager.kernel.Identifier;


/**
 * A class of Identifiers for use with JispEntityManager.  They are KeyObjects
 * (for jisp) and also implement Identifier.
 */
public class JispIdentifier extends KeyObject implements Identifier {
    /** The last Identifier that was created. */
    protected static JispIdentifier lastIdentifier = new JispIdentifier(0, 0);

    /** The long value of the Identifier. */
    protected long dateSection;

    /** A number to help distinguish between Identifiers. */
    protected long longSection;

    /**
     * Constructs a unique Identifier.
     *
     * @param dateSection A long version of the date.
     * @param longSection The current incremented value of the long numeric that ensures
     *                    that Indentifiers are unique.
     */
    public JispIdentifier(long dateSection, long longSection) {
        this.dateSection = dateSection;
        this.longSection = longSection;
    }

    /**
     * Public constructor so serialization will work.
     */
    public JispIdentifier() {
        dateSection = -1;
        longSection = -1;
    }

    public KeyObject makeNullKey() {
        return new JispIdentifier(-1, -1);
    }

    /**
     * Gets the long Date part of this Identifier.
     */
    protected long getDateSection() {
        return this.dateSection;
    }

    /**
     * Gets the long part of this Identifier.
     */
    protected long getLongSection() {
        return this.longSection;
    }

    public int hashCode() {
        // This must be a positive number to work around a bug in jisp.
        // -mml 2002-11-30
        return Math.abs((int) (getDateSection() ^ getLongSection()));
    }

    /**
     * Checks to see if the given Object is the same as this Identifier.
     */
    public boolean equals(Object object) {
        return (object instanceof JispIdentifier) && 
               (((JispIdentifier) object).getDateSection() == this.getDateSection()) && 
               (((JispIdentifier) object).getLongSection() == this.getLongSection());
    }

    /**
     * Gets a String representation of this Identifier.
     *
     * @return A String representation of the Identifier.
     */
    public String toString() {
        return dateSection + ":" + longSection;
    }

    public int compareTo(KeyObject keyObject) {
        if (!(keyObject instanceof JispIdentifier)) {
            return KeyObject.KEY_ERROR;
        }

        JispIdentifier jispIdentifier = (JispIdentifier) keyObject;

        if (dateSection < jispIdentifier.dateSection) {
            return KeyObject.KEY_LESS;
        } else if (dateSection > jispIdentifier.dateSection) {
            return KeyObject.KEY_MORE;
        } else if (longSection < jispIdentifier.longSection) {
            return KeyObject.KEY_LESS;
        } else if (longSection > jispIdentifier.longSection) {
            return KeyObject.KEY_MORE;
        }

        // If we get this far, the objects are identical.
        return KeyObject.KEY_EQUAL;
    }

    /**
     * Generates a unique Identifier.
     */
    protected static JispIdentifier generateIdentifier() {
        long date = System.currentTimeMillis();

        long longSection = 0;

        if (lastIdentifier.getDateSection() == date) {
            longSection = lastIdentifier.getLongSection() + 1;
        }

        JispIdentifier identifier = new JispIdentifier(date, longSection);

        lastIdentifier = identifier;

        return identifier;
    }

    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeLong(dateSection);
        out.writeLong(longSection);
    }

    public void readExternal(ObjectInput in) throws IOException, 
                                                    ClassNotFoundException {
        dateSection = in.readLong();
        longSection = in.readLong();
    }
}

/*
 *
 * $Log: JispIdentifier.java,v $
 * Revision 1.2  2002/12/20 20:52:31  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.1  2002/12/03 05:04:30  mliggett
 * First stab at pure, jisp-only storage.  Obviously needs refactoring (which
 * will shorten the longer source files substantially) and a
 * looksee at the javadoc.  Separate message to be sent fully
 * explaining things, but these entities keep NO data in main memory.  All
 * four pieces of Entity data (name, value, attributes and bases) are kept in
 * separate indexed jisp databases and the contents are loaded on demand.
 *
 * Includes several unit tests.  Caching EntityManager to be checked in
 * next.
 *
 *
 */
