// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// made to implement Observer and added setSimpletonList() (so that we can
// watch a SimpletonList), changed Vectors to Lists, added a couple
// package-protection getters to facilitate unit tests.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Class renamed from KernelWindow to KernelGui.  Changed to have a JFrame
// rather than be a JFrame.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Made update() synchronized so that the state of our internal lists does not
// get modified in the middle of an update, should the SimpletonList call
// update() on us in the middle of an update() call.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// 2 changes:
// 1. Factored out cut and paste nonsense in mouse-click listeners attached to
//    the Simpleton JLists and JButtons.  The common code is now in
//    startSimpletonsAtIndices().
// 2. Now, when you start a Simpleton from the GUI, it is clone()d and the
//    clone is start()d.  This means you can have as many copies as you like of
//    a given Simpleton.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Added params to javadocs and changed direct member reference to
// simpletonList to query method.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-12 nab
// - added persistPoolAndExit() method which should be called instead of
//   System.exit().  This will persist the pool and its entities as well
//   as exit the system.  This is called from the window listener and the
//   close button
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 baa
// renamed persistPoolAndExit() method to persistAndExit().  Changed same
// method to invoke persist(File) on the VerySimpleEntityManager instead of
// on the Pool.  This may be better served by having Pools delegate their
// persist() method to an EntityManager similar to the Pool's delegation of
// getDefaultPool().
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import java.util.List;
import java.util.Vector;

import javax.swing.*;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.ValueClassIsConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.LoadableSimpletonEntityValue;

import org.datamanager.tools.Debug;


/**
 * The overall interface for the Hydrogen version of the
 * DataManager. This is more or less a quick and dirty
 * overall controller for the system. It allows for the
 * selection and execution of Simpletons.
 */
public class KernelGUISimpleton extends Simpleton {
    /** The title that will be displayed on the main GUI frame. */
    private static final String FRAME_TITLE = "KnownSpace Controller";

    /** The list of interface Simpletons this KernelGui displays. */
    private List interfaceObjects = new Vector();

    /** The list of non-interface Simpletons this KernelGui displays. */
    private List simpletonObjects = new Vector();

    // The different components in this simple GUI.
    private JFrame frame;
    private GridBagLayout gridBagLayout = new GridBagLayout();
    private GridBagConstraints constraints = new GridBagConstraints();
    private JButton closeButton = new JButton("Shutdown");
    private JButton startSimpletonButton = new JButton("Start Simpletons");
    private JButton startInterfaceButton = new JButton("Start UserInterfaces");
    private DefaultListModel interfaceNames = new DefaultListModel();
    private DefaultListModel simpletonNames = new DefaultListModel();
    private JList interfaceJList = new JList(interfaceNames);
    private JList simpletonJList = new JList(simpletonNames);
    private JScrollPane interfacePane = new JScrollPane(interfaceJList);
    private JScrollPane simpletonPane = new JScrollPane(simpletonJList);

    /**
     * Creates a new KernelGui monitoring the given SimpletonList and
     * initializes all the GUI components on it.
     * Currently kernel access only.
     */
    public KernelGUISimpleton() {
    }

    public void process() {
        frame = new JFrame(FRAME_TITLE);
        makeGui(frame.getContentPane());
        setupComponents();
        display();

        // FIXME: This should be done dynamically instead of just once!
        try {
            Pool pool = Pool.getDefaultPool();
            Entity[] simpletonListEntities = pool.search(
                                                     SimpletonList.SIMPLETON_LIST_CONSTRAINT);

            if (simpletonListEntities.length != 0) {
                EntityConstraint attributeSelector = new ValueClassIsConstraint(
                                                             LoadableSimpletonEntityValue.class);
                Entity[] simpletonEntities = simpletonListEntities[0].getAttributes(
                                                     attributeSelector);

                for (int index = 0; index < simpletonEntities.length; index++) {
                    add((LoadableSimpletonEntityValue) simpletonEntities[index].getValue());
                }
            }
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Cannot load Simpletons from Pool: " + 
                                  exception.getMessage());
            exception.printStackTrace();
        }
    }

    /**
     * Creates the GUI; places all the components into the GUI's
     * container.
     *
     * @param container The container to place all the GUI
     *                  components on.
     */
    private void makeGui(Container container) {
        container.setLayout(gridBagLayout);

        constraints.weightx = 1;
        constraints.weighty = 1;
        constraints.gridy = 1;
        constraints.gridx = 2;
        constraints.gridwidth = 1;

        container.add(startInterfaceButton, constraints);

        constraints.gridx = 3;
        constraints.gridwidth = GridBagConstraints.REMAINDER;

        container.add(startSimpletonButton, constraints);

        constraints.gridy = 3;
        constraints.gridx = 2;
        constraints.gridwidth = 1;
        constraints.fill = GridBagConstraints.BOTH;

        container.add(interfacePane, constraints);

        constraints.gridx = 4;
        constraints.gridwidth = 1;

        container.add(simpletonPane, constraints);

        constraints.fill = GridBagConstraints.NONE;
        constraints.gridy = 6;
        constraints.gridx = 2;
        constraints.gridwidth = GridBagConstraints.REMAINDER;

        container.add(closeButton, constraints);
    }

    /**
     * Starts the Simpleton at each given index in the given list.
     */
    private void startSimpletonsAtIndices(List objects, int[] indices) {
        for (int i = 0; i < indices.length; i++) {
            try {
                System.out.println("started: " + objects.get(indices[i]));

                LoadableSimpletonEntityValue loadableSimpleton = 
                        (LoadableSimpletonEntityValue) objects.get(indices[i]);

                SimpletonLoaderProcess.loadSimpleton(loadableSimpleton, 
                                                     getClass()
                                                         .getClassLoader())
                      .start();
            } catch (IllegalAccessException exception) {
                exception.printStackTrace();
            } catch (InstantiationException exception) {
                exception.printStackTrace();
            } catch (MalformedURLException exception) {
                exception.printStackTrace();
            } catch (ClassNotFoundException exception) {
                exception.printStackTrace();
            }
        }
    }

    /**
     * This will persist the EntityManager (and, consequently, the Pool) and
     * exit the system.  Should be called from methods in the GUI that
     * would have called System.exit()
     */
    private void persistAndExit() {
        try {
            EntityManager.getInstance().persist();
        } catch (Exception exception) {
            Debug.writeLogMessage(
                    "persisting the EntityManager and Pool failed" + 
                    exception.getMessage());
        }

        System.exit(0);
    }

    /**
     * Initializes the GUI components. This method adds action
     * listeners, window listeners, etc. so the GUI's
     * components are functional.
     */
    private void setupComponents() {
        interfaceJList.setCellRenderer(new ToolTipCellRenderer());
        simpletonJList.setCellRenderer(new ToolTipCellRenderer());

        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                persistAndExit();
            }
        });

        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                persistAndExit();
            }
        });

        startSimpletonButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                int[] items = simpletonJList.getSelectedIndices();
                startSimpletonsAtIndices(simpletonObjects, items);
            }
        });

        startInterfaceButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                int[] items = interfaceJList.getSelectedIndices();
                startSimpletonsAtIndices(interfaceObjects, items);
            }
        });

        simpletonJList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent event) {
                if (event.getClickCount() == 2) {
                    int[] items = simpletonJList.getSelectedIndices();
                    startSimpletonsAtIndices(simpletonObjects, items);
                }
            }
        });

        interfaceJList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent event) {
                if (event.getClickCount() == 2) {
                    int[] items = interfaceJList.getSelectedIndices();
                    startSimpletonsAtIndices(interfaceObjects, items);
                }
            }
        });
    }

    /**
     * Clears the lists that contain Simpleton information.
     */
    private void clearSimpletonLists() {
        interfaceObjects.clear();
        simpletonObjects.clear();
        interfaceNames.clear();
        simpletonNames.clear();
    }

    /**
     * Adds a simpleton or interface component to the DataManager.
     *
     * @param loadableSimpleton A simpleton that can be run.
     */
    private void add(LoadableSimpletonEntityValue loadableSimpleton) {
        Information information = loadableSimpleton.getInformation();

        if (information.isUserInterface()) {
            interfaceObjects.add(loadableSimpleton);
            interfaceNames.addElement(information);
        } else {
            simpletonObjects.add(loadableSimpleton);
            simpletonNames.addElement(information);
        }
    }

    /**
     * Displays this GUI on the screen.
     * @param none
     */
    public void display() {
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * A ListCellRenderer for a JList that displays the short description of
     * each Simpleton in a tooltip.
     */
    class ToolTipCellRenderer extends JLabel implements ListCellRenderer {
        ToolTipCellRenderer() {
            setOpaque(true);
        }

        /**
         * Renders the cell.
         *
         * @param list The list to render.
         * @param value The value to display.
         * @param index The cell index in the list.
         * @param isSelected Is the cell selected.
         * @param cellHasFocus The list and the cell have focus.
         */
        public Component getListCellRendererComponent(JList list, Object value, 
                                                      int index, 
                                                      boolean isSelected, 
                                                      boolean cellHasFocus) {
            Simpleton.Information info = (Simpleton.Information) value;
            String text = info.getName();
            String toolTip = info.getShortDescription();

            setText(text);
            setToolTipText(toolTip);

            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }

            setEnabled(list.isEnabled());
            setFont(list.getFont());

            return this;
        }
    }
}