// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import java.util.Comparator;
import java.util.Hashtable;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import org.datamanager.exception.DataManagerException;

import org.datamanager.tools.Debug;


/**
 * This class will cooperate with SuaveEntityManger to manager the object in the memory
 * of the knernel. It can provide the lookup capability for Entity, Proxy, Identifier.
 * It also can find the oldest object reside in the memory
 *
 * @author xi rao & bob
 */
public class KernelMemory {
    /** The maximum number of Entities cached in memory at a given time. */
    public static final int HASHTABLE_LENGTH = 200;

    /** A mapping from VerySimpleEntities to Identifiers. */
    protected Map entityToIdentifier;

    /** A mapping from Identifiers to VerySimpleEntities. */
    protected Map identifierToEntity;

    /** A mapping from Identifiers to EntityProxies. */
    protected Map identifierToProxy;
    protected TreeMap timeStampToIdentifier;
    protected Map IdentifierToTimeStamp;
    private int entityCounter = 0;

    public KernelMemory() {
        entityToIdentifier = new Hashtable();
        identifierToEntity = new Hashtable();
        identifierToProxy = new Hashtable();

        timeStampToIdentifier = new TreeMap(new Comparator() {
            public int compare(Object a, Object b) {
                ComparableIdentifier item1 = (ComparableIdentifier) a;
                ComparableIdentifier item2 = (ComparableIdentifier) b;

                return item1.compareTo(item2);
            }
        });

        IdentifierToTimeStamp = new Hashtable();
    }

    public boolean isFull() {
        if (entityCounter < HASHTABLE_LENGTH) {
            return false;
        }

        return true;
    }

    public synchronized void loadIntoMemory(Identifier identifier, 
                                            Entity entity, 
                                            EntityProxy entityProxy)
                                     throws Exception {
        if (entityCounter > HASHTABLE_LENGTH) {
            throw new Exception("no memory exception");
        }

        entityToIdentifier.put(entity, identifier);
        identifierToEntity.put(identifier, entity);
        identifierToProxy.put(identifier, entityProxy);
        entityCounter++;

        ComparableIdentifier timeStampInstance = new ComparableIdentifier();

        timeStampToIdentifier.put(timeStampInstance, identifier);
        IdentifierToTimeStamp.put(identifier, timeStampInstance);
    }

    public Identifier findOldestIdentifier() {
        return (Identifier) timeStampToIdentifier.get(
                       timeStampToIdentifier.firstKey());
    }

    public synchronized Entity removeFromMemory(Identifier identifier) {
        entityCounter--;

        ComparableIdentifier timeStampInstance = (ComparableIdentifier) IdentifierToTimeStamp.remove(
                                                         identifier);

        timeStampToIdentifier.remove(timeStampInstance);
        identifierToProxy.remove(identifier);

        Entity entity = (Entity) identifierToEntity.remove(identifier);

        entityToIdentifier.remove(entity);

        return entity;
    }

    public int currentSize() {
        return entityCounter;
    }

    public int maxSize() {
        return HASHTABLE_LENGTH;
    }

    public Identifier findIdentifier(Entity entity) {
        if (entity == null) {
            Debug.writeLogMessage(
                    "Trying to find Identifier for a null Entity!");
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null Entity!");
        }

        return (Identifier) entityToIdentifier.get(entity);
    }

    protected EntityProxy findProxy(Identifier identifier) {
        if (identifier == null) {
            Debug.writeLogMessage(
                    "Trying to find EntityProxy for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find EntityProxy for a null Identifier!");
        }

        return (EntityProxy) identifierToProxy.get(identifier);
    }

    protected Entity findEntity(Identifier identifier) {
        if (identifier == null) {
            Debug.writeLogMessage(
                    "Trying to find Entity for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find Entity for a null Identifier!");
        }

        return (Entity) identifierToEntity.get(identifier);
    }
}