// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import com.coyotegulch.jisp.BTreeIndex;
import com.coyotegulch.jisp.BTreeObjectIterator;
import com.coyotegulch.jisp.HashIndex;
import com.coyotegulch.jisp.IndexedObjectDatabase;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.tools.Debug;


public class MemoryManager {
    protected KernelMemory kernelMemory = new KernelMemory();

    public MemoryManager() {
    }

    /**
     * Finds the Identifier for the given Entity.
     *
     * @param entity The Entity for which to find an Identifier.
     * @return The Indentifier for the specified Entity.
     */
    protected Identifier findIdentifier(Entity entity)
                                 throws NoSuchIdentifierException {
        if (entity == null) {
            Debug.writeLogMessage(
                    "Trying to find Identifier for a null Entity!");
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null Entity!");
        }

        Identifier identifier = (Identifier) kernelMemory.findIdentifier(entity);

        if (identifier != null) {
            return identifier;
        }

        identifier = this.findFromStorage(entity);

        if (identifier != null) {
            return identifier;
        } else {
            throw new NoSuchIdentifierException("No Such Identifier in Pool");
        }
    }

    /**
     * Finds the EntityProxy for the given Identifier.
     *
     * @param identifier The Identitifer for which to find an EntityProxy.
     * @return The EntityProxy associated with the given Identifier.
     */
    protected EntityProxy findProxy(Identifier identifier)
                             throws NoSuchEntityException {
        if (identifier == null) {
            Debug.writeLogMessage(
                    "Trying to find EntityProxy for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find EntityProxy for a null Identifier!");
        }

        EntityProxy entityProxy = (EntityProxy) kernelMemory.findProxy(
                                          identifier);
        Entity entity;

        if (entityProxy != null) {
            return entityProxy;
        }

        entity = (Entity) this.findFromStorage(identifier);

        if (entity != null) {
            try {
                entityProxy = new EntityProxy(identifier, 
                                              SuaveEntityManager.getInstance());
            } catch (DataManagerException exception) {
                throw new RuntimeException(exception);
            }

            try {
                // FIXME:mml:cut and paste code
                if (kernelMemory.isFull()) {
                    removeEntityFromMemory();
                }


                // end cut and paste
                kernelMemory.loadIntoMemory(identifier, entity, entityProxy);
            } catch (Exception exception) {
                exception.printStackTrace();
            }

            return entityProxy;
        } else {
            throw new NoSuchEntityException("No Such Entity in Pool");
        }
    }

    /**
     * Finds the Entity associated with the given Identifier.
     *
     * @param identifier The Identifier for which to find an Entity.
     * @return The Entity associated with the Identifier.
     */
    protected Entity findEntity(Identifier identifier)
                         throws NoSuchEntityException {
        if (identifier == null) {
            Debug.writeLogMessage(
                    "Trying to find Entity for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find Entity for a null Identifier!");
        }

        Entity entity = (Entity) kernelMemory.findEntity(identifier);

        if (entity != null) {
            return entity;
        }

        entity = (Entity) findFromStorage(identifier);

        if (entity != null) {
            try {
                EntityProxy entityProxy = new EntityProxy(identifier, 
                                                          SuaveEntityManager.getInstance());

                // FIXME:mml:cut and paste code
                if (kernelMemory.isFull()) {
                    removeEntityFromMemory();
                }


                // end cut and paste
                kernelMemory.loadIntoMemory(identifier, entity, entityProxy);
            } catch (DataManagerException exception) {
                throw new RuntimeException(exception);
            } catch (Exception exception) {
                exception.printStackTrace();
            }

            return entity;
        } else {
            throw new NoSuchEntityException("No such Entity in Pool");
        }
    }

    /**
     * Loads the given triple into the KernelMemory.
     */
    protected void loadEntityToMemory(Identifier identifier, 
                                      VerySimpleEntity entity, 
                                      EntityProxy entityProxy)
                               throws Exception {
        // FIXME:mml:cut and paste code
        if (kernelMemory.isFull()) {
            removeEntityFromMemory();
        }


        // end cut nad paste
        kernelMemory.loadIntoMemory(identifier, entity, entityProxy);
    }

    /**
     * Removes the oldest entity from the KernelMemory.
     */
    protected void removeEntityFromMemory() throws DataManagerException {
        Identifier firstIdentifier = kernelMemory.findOldestIdentifier();
        Entity removedOldestEntity = kernelMemory.removeFromMemory(
                                             firstIdentifier);
        persistEntity(firstIdentifier, removedOldestEntity);
    }

    protected void persistEntity(Identifier identifier, Entity entity)
                          throws DataManagerException {
        ((SuaveEntityManager) SuaveEntityManager.getInstance()).persistEntity(
                identifier, entity);
    }

    protected void readPersistedEntity() {
    }

    //return entity from disk storage or null
    protected Entity findFromStorage(Identifier identifier) {
        try {
            IndexedObjectDatabase database = new IndexedObjectDatabase(
                                                     SuaveEntityManager.PERSISTENCE_DATABASE_NAME, 
                                                     false);
            HashIndex indexIdentifier = new HashIndex(
                                                SuaveEntityManager.POOL_IDENTIFIER_INDEX);
            database.attachIndex(indexIdentifier);

            StringKey80 key = new StringKey80(identifier.toString());
            Entity entity = ((DBRecord) database.read(key, indexIdentifier)).getEntity();

            return entity;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    protected Identifier findFromStorage(Entity entity) {
        try {
            IndexedObjectDatabase database = new IndexedObjectDatabase(
                                                     SuaveEntityManager.PERSISTENCE_DATABASE_NAME, 
                                                     false);
            BTreeIndex timeStampIndex = new BTreeIndex(
                                                SuaveEntityManager.POOL_TIMESTAMP_INDEX);
            database.attachIndex(timeStampIndex);

            BTreeObjectIterator iterator = database.createIterator(
                                                   timeStampIndex);

            while (iterator.moveNext()) {
                DBRecord dbRecord = (DBRecord) iterator.getObject();
                Entity searchEntity = dbRecord.getEntity();

                if (searchEntity.equals(entity)) {
                    dbRecord.getIdentifier();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }
}