// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Added param/return to javadocs and updated some javadocs.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// File created.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Changed from Vector to ArrayList, since it's more lightweight and we're now
// handling synchronization on our own.
//
// Modified so that concurrent access from multiple threads is no longer an
// issue.  This means two things:
//
// 1. All methods that access the underlying List are synchronized.
// 2. iterator() now returns an Iterator over a temporary copy of the underlying
//    list, so there is no risk of either of the following:
//
//    a. Callers modifying the underlying list through method calls on the
//       Iterator.
//    b. The Iterator becoming invalid because the list has been manipulated by
//       other threads.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.ValueClassIsConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.passiveentityvalue.*;

import org.datamanager.tools.Debug;


/**
 * A SimpletonList is an abstract way to maintain an Entity that
 * holds all the Simpletons that can be run in the system.
 *
 * The format for the simpleton list is an Entity that has
 * the name "simpleton-list" and attributes that contain the
 * classes of each Simpleton that can be loaded.
 *
 * @author Chris Dent
 * @author Matt Liggett
 * @author Matthew Farrellee
 */
public class SimpletonList extends Observable {
    /** The name of the simpleton list Entity */
    public static final String SIMPLETON_LIST_ENTITY_NAME = "simpleton-list";

    /** The EntityConstraint used to locate the simpleton list Entity */
    public static final EntityConstraint SIMPLETON_LIST_CONSTRAINT = 
            new NameEqualsConstraint(SIMPLETON_LIST_ENTITY_NAME);

    /** The Entity that stores the simpletons */
    private Entity simpletonListEntity;

    /**
     * Create a new SimpletonList. This will retrieve the Entity that
     * will hold the simpleton-list. If it does not exist it will be
     * created, and if it does exist initialize it (by clearing it).
     */
    public SimpletonList(Pool pool) throws DataManagerException {
        Entity[] entities = pool.search(SIMPLETON_LIST_CONSTRAINT);

        if (entities.length == 0) {
            Debug.writeLogMessage(
                    "Did not find a simpleton list Entity, creating one.");
            this.simpletonListEntity = Entity.create(SIMPLETON_LIST_ENTITY_NAME);
            pool.add(this.simpletonListEntity);
        } else if (entities.length == 1) {
            Debug.writeLogMessage("Found one simpleton list Entity, using it.");
            clear(entities[0]);
            this.simpletonListEntity = entities[0];
        } else {
            Debug.writeLogMessage(
                    "There is more than one simpleton list Entity, using the first one found.");
            clear(entities[0]);
            this.simpletonListEntity = entities[0];
        }
    }

    /**
     * Clear the given simpleton list Entity.
     */
    private void clear(Entity entity) throws DataManagerException {
        EntityConstraint attributeSelector = new ValueClassIsConstraint(
                                                     LoadableSimpletonEntityValue.class);
        Entity[] attributes = entity.getAttributes(attributeSelector);

        for (int index = 0; index < attributes.length; index++) {
            entity.detach(attributes[index]);
        }
    }

    /**
     * Adds a LoadableSimpletonEntityValue as an attribute of the simpleton
     * list Entity.
     *
     * @param information The Simpleton.Information of the simpleton being
     *                    added to the list.
     */
    public synchronized void add(Simpleton.Information information)
                          throws DataManagerException {
        Entity simpletonEntity = Entity.create(information.getName());
        simpletonEntity.setValue(new LoadableSimpletonEntityValue(information));
        simpletonListEntity.attach(simpletonEntity);
        simpletonListEntity.attach(simpletonEntity);
    }

    /**
     * Get the count of currently available simpletons.
     */
    public int count() throws DataManagerException {
        return getSimpletonEntities().length;
    }

    /**
     * Get the Entity that is being used for storage of the simpletons.
     */
    public Entity getSimpletonListEntity() {
        return simpletonListEntity;
    }

    /**
     * Get a iterator for the simpleton list.
     */
    public Iterator iterator() throws DataManagerException {
        LoadableSimpletonEntityValue[] simpletonEntities = 
                getSimpletonEntities();

        return Arrays.asList(simpletonEntities).iterator();
    }

    /**
     * Get all of the LoadableSimpletonEntityValues stored on the simpleton
     * list Entity.
     */
    protected LoadableSimpletonEntityValue[] getSimpletonEntities()
        throws DataManagerException {
        EntityConstraint attributeSelector = new ValueClassIsConstraint(
                                                     LoadableSimpletonEntityValue.class);
        Entity[] entities = simpletonListEntity.getAttributes(attributeSelector);

        LoadableSimpletonEntityValue[] simpletonEntities = 
                new LoadableSimpletonEntityValue[entities.length];

        for (int index = 0; index < entities.length; index++) {
            simpletonEntities[index] = (LoadableSimpletonEntityValue) entities[index].getValue();
        }

        return simpletonEntities;
    }
}