// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Tidied javadocs/fixed some spelling mistakes.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// No longer has direct knowledge of the KernelWindow.  Instead, we export a
// SimpletonList which the KernelWindow registers as an Observer of.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Removed unneeded KernelWindow import line.
// Corrected the doc comment above load().
// Improved the class doc comment.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Extracted configureDocumentBuilderFactory() method.
// Corrected or refined javadoc comments.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Changed Debug.assert() calls to Debug.writeLogMessage() to enable
// java 1.4 assertions elsewhere.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.IOException;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Simpleton;
import org.datamanager.kernel.SimpletonList;

import org.datamanager.passiveentityvalue.LoadableSimpletonEntityValue;

import org.datamanager.tools.Debug;
import org.datamanager.tools.InformationParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;


/**
 * Looks in a given XML file and loads and instantiates all Simpletons found in
 * the file.  The Simpletons are then added to the SimpletonList created by
 * this SimpletonLoaderProcess.
 *
 * @author Matthew Farrellee
 */
class SimpletonLoaderProcess extends Simpleton {
    private static final String className = "org.datamanager.kernel.SimpletonLoaderProcess";
    private static final String name = "SimpletonLoaderProcess";
    private static final String shortDescription = "Loads Simpleton class files.";
    private static final String longDescription = "Loads Simpleton class files from a directory.";
    private static final String version = "Hydrogen";
    private static final String[] authors = { "Matthew Farrellee" };

    /** InputStream to read StartupData from. */
    private InputStream startupDataFile;

    /** Lists of the simpletons and interfaces we load. */
    private SimpletonList simpletonList;

    /**
     * Creates a SimpletonLoaderProcess given a InputStream that points
     * to a StartupData XML file.  Currently Kernel access only.
     *
     * @param startupDataFile An InputStream that should be an XML StartupData file.
     */
    SimpletonLoaderProcess(InputStream startupDataFile) {
        this.startupDataFile = startupDataFile;

        setInformation(
                new Simpleton.Information(className, name, shortDescription, 
                                          longDescription, version, authors, 
                                          false, false, null));
    }

    /**
     * Retrieves the list of Simpletons.  Currently Kernel access only.
     */
    SimpletonList getSimpletonList() {
        return simpletonList;
    }

    /**
     * Starts the SimpletonLoaderProcess, which parses the
     * StartupData file.  Extracts all the Simpletons from the
     * StartupData file, tries to create an instance of each of those
     * Simpletons, and then calls the load() method on each new instance via
     * the parse(Document document) method.
     * @see #parse(Document document)
     */
    public void process() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        configureDocumentBuilderFactory(factory);

        try {
            simpletonList = new SimpletonList(Pool.getDefaultPool());

            // Create the DOM parser from the factory.
            // Can throw: ParserConfigurationException
            DocumentBuilder builder = factory.newDocumentBuilder();

            // Parse the XML file into a DOM tree.
            // Can throw: SAXException, IOException
            Document document = builder.parse(startupDataFile);


            // Parse the DOM tree.
            parse(document);

            Iterator iterator = simpletonList.iterator();
            Simpleton simpleton;

            while (iterator.hasNext()) {
                try {
                    LoadableSimpletonEntityValue loadableSimpleton = 
                            (LoadableSimpletonEntityValue) iterator.next();
                    Simpleton.Information information = 
                            loadableSimpleton.getInformation();

                    if (information.runOnStartup()) {
                        simpleton = loadSimpleton(loadableSimpleton, 
                                                  getClass().getClassLoader());
                        simpleton.start();
                        Debug.writeLogMessage("Loaded: " + simpleton);
                    }
                } catch (ClassNotFoundException exception) {
                    exception.printStackTrace();
                } catch (IllegalAccessException exception) {
                    exception.printStackTrace();
                } catch (InstantiationException exception) {
                    exception.printStackTrace();
                }
            }
        } catch (ParserConfigurationException configurationException) {
            configurationException.printStackTrace();
        } catch (SAXException saxException) {
            saxException.printStackTrace();
        } catch (IOException ioException) {
            ioException.printStackTrace();
        } catch (DataManagerException exception) {
            exception.printStackTrace();
        }
    }

    /**
     * Configures the document builder factory with the required settings.
     */
    private void configureDocumentBuilderFactory(DocumentBuilderFactory factory) {
        // Enable support for XML namespaces.
        factory.setNamespaceAware(true);


        // Enable validation for XML documents.
        factory.setValidating(true);

        factory.setIgnoringComments(true);


        // Enable elimination of whitespace in element content.
        // FIXME: This does not work!!!
        factory.setIgnoringElementContentWhitespace(true);


        // Enable conversion of CDATA nodes to Text nodes appending
        // to adjacent text nodes.
        factory.setCoalescing(true);

        factory.setExpandEntityReferences(true);
    }

    /**
     * Parses the Document representing the configuration file. Creates the
     * Simpleton and UserInterfaces specified by the file.
     *
     * @param document The Document representation of the configuration file.
     */
    protected void parse(Document document) throws DataManagerException {
        Element root = document.getDocumentElement();

        NodeList simpletonList = root.getElementsByTagName("Simpleton");
        NodeList userInterfaceList = root.getElementsByTagName("Userinterface");

        for (int index = 0; index < simpletonList.getLength(); index++) {
            parseSimpleton(simpletonList.item(index));
        }

        for (int index = 0; index < userInterfaceList.getLength(); index++) {
            parseSimpleton(userInterfaceList.item(index));
        }
    }

    /**
     * Parses a Node in the configuration file representing a Simpleton.
     * Instantiates the Simpleton.
     *
     * @param simpleton Node within the XML configuration file representin' a
     * Simpleton
     */
    protected void parseSimpleton(Node simpleton) throws DataManagerException {
        Class simpletonClass;
        Object loadedObject;
        Simpleton loadedSimpleton;

        Simpleton.Information information = InformationParser.parseInformation(
                                                    simpleton);

        // If the information is incomplete parseInformation returns NONE
        if (!information.equals(Information.NONE)) {
            load(information);
        }
    }

    /**
     * Loads the given simpleton into the SimpletonList.
     */
    protected void load(Simpleton.Information information)
                 throws DataManagerException {
        simpletonList.add(information);
    }

    // FIXME: Should this code go into some sort of toolbox?
    public static Simpleton loadSimpleton(LoadableSimpletonEntityValue loadableSimpleton, 
                                          ClassLoader currentClassLoader)
                                   throws ClassNotFoundException, 
                                          IllegalAccessException, 
                                          InstantiationException, 
                                          MalformedURLException {
        Simpleton.Information information = loadableSimpleton.getInformation();

        String className = information.getClassName();

        // FIXME: This code uses try/catch for flow control, is there a better way?
        Class simpletonClass = null;

        try {
            simpletonClass = Class.forName(className, true, currentClassLoader);
        } catch (ClassNotFoundException exception) {
            URL[] location = { new URL(information.getClassLocation()) };
            URLClassLoader classLoader = new URLClassLoader(location);
            simpletonClass = Class.forName(className, true, classLoader);
        }

        // FIXME: Check to see if the simpletonClass is an interface.
        Simpleton simpleton = (Simpleton) simpletonClass.newInstance();
        simpleton.setInformation(information);

        return simpleton;
    }
}