// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import com.coyotegulch.jisp.BTreeIndex;
import com.coyotegulch.jisp.HashIndex;
import com.coyotegulch.jisp.IndexedObjectDatabase;
import com.coyotegulch.jisp.KeyObject;
import com.coyotegulch.jisp.LongKey;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;

import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.kernel.Persistable;

import org.datamanager.tools.Debug;


/**
 * This is a very simple implementation of an EntityManager.  It holds mappings
 * between EntityProxies, Identifiers, and VerySimpleEntityValues.  It should be
 * made persistable by the KernelGui's holds all entities in memory and
 * serializes them to disk when the program is shutdown. Future implementations
 * could do some sort of caching to allow for more entities to be in the system
 * with less RAM required.
 */
class SuaveEntityManager extends EntityManager implements Serializable {
    /** String representation of persistence destination (relative path) */
    public static final String PERSISTENCE_FILE_NAME = 
            "SuaveEntityManager.persist";

    // FIXME:mml: These pool.* files are just created in the current directory
    // right now.  They should be dropped into the same place as
    // SuaveEntityManager.persist: ~/.helium or the like.
    public static final String PERSISTENCE_DATABASE_NAME = "pool.database";

    // For doing search identifier in the disk storage
    public static final String POOL_IDENTIFIER_INDEX = "pool.identifer.hash";

    //Now, we need a btree index to get the iterator and go through the whole
    public static final String POOL_TIMESTAMP_INDEX = "pool.timestamp.btree";

    // FIXME:xirao: Jisp database can associate with hash-index, but we need pre-define
    // the length of hash-index when we create it. When Hash-index is used up. We can not
    // create entity. This is a huge limitation for Jisp database. Something, we deinitely
    // need to look into.
    // We may follow this scheme to fix it later: when the pre-define hash-index is reach,
    // we doule its value, we copy the old hash-index into new hash-index
    private static final int HASH_INDEX_LENGTH = 80000;

    /**
     * Number of buckets to be used in the hash table.  This should be prime.
     *
     * FIXME: don't hardcode this!
     */
    private static final int HASH_BUCKET = 104729;
    private static KeyObject[] keyObject = new KeyObject[2];

    /** The only instance of the SuaveEntityManager. */
    private static EntityManager instance = null;

    /** Persistence File. */
    private static File persistenceFile;

    // count the number of entity created. It assoicate with FIXME for jisp hash-index issue
    private int entityCounter = 0;

    /** The only instance of the Pool.  Made non-static for compliance with the
     * standard Serializable interface.  Plus, the future holds multiple Pools
     * for us.
     */
    protected Pool defaultPool;

    /** A handle for the IdentifierManager's instance. */
    protected IdentifierManager identifierManager = IdentifierManager.getInstance();
    private transient IndexedObjectDatabase database;
    private transient MemoryManager memoryManager = new MemoryManager();

    /**
     * Creates a new SuaveEntityManager with no mappings.
     */
    private SuaveEntityManager() {
        bootPool();

        try {
            File file = new File(PERSISTENCE_DATABASE_NAME);

            if (file.exists()) {
                database = new IndexedObjectDatabase(PERSISTENCE_DATABASE_NAME, 
                                                     false);
                database.attachIndex(new HashIndex(POOL_IDENTIFIER_INDEX));
                database.attachIndex(new BTreeIndex(POOL_TIMESTAMP_INDEX));
            } else {
                database = new IndexedObjectDatabase(PERSISTENCE_DATABASE_NAME, 
                                                     true);

                HashIndex identifierHashIndex = new HashIndex(
                                                        POOL_IDENTIFIER_INDEX, 
                                                        HASH_BUCKET, 
                                                        HASH_INDEX_LENGTH, 
                                                        new StringKey80());

                BTreeIndex timeStampIndex = new BTreeIndex(POOL_TIMESTAMP_INDEX, 5, 
                                                           new LongKey(), false);
                database.attachIndex(identifierHashIndex);
                database.attachIndex(timeStampIndex);
            }
        } catch (Exception everything) {
            everything.printStackTrace();
        }


        // createDefaultPool may initialize the Singleton Pool via its own
        // persisted XML file or instantiate a new Pool if the persisted file
        // is not found (or some other restoration error occurs).
        defaultPool = Pool.createDefaultPool();

        //assert defaultPool != null;
    }

    /**
     * Requests the only instance of the SuaveEntityManager.  The
     * non-null existence of the instance is guaranteed by loadEntityManager.
     *
     * @return The instance of the SuaveEntityManager.
     */
    protected static EntityManager getInstance() throws DataManagerException {
        if (instance == null) {
            instance = new SuaveEntityManager();
        }

        return instance;
    }

    /**
     * Tries to restore the Pool from the persistence file.  Failing that, just
     * creates a new Pool.
     */
    private void bootPool() {
        if (defaultPool == null) {
            if (!restorePool()) {
                defaultPool = Pool.createDefaultPool();
            }
        }
    }

    /**
     * This method attempts to restore the Pool from persistence.  Returns true
     * if it succeeds, false otherwise.
     */
    private boolean restorePool() {
        if (getPersistenceFile().exists()) {
            // restore returns null if it fails.
            // FIXME:mml: should probably not be calling Persistable.PERSISTER
            // directly
            defaultPool = (Pool) Persistable.PERSISTER.restore(
                                  getPersistenceFile());
        }

        return defaultPool != null;
    }

    /**
     * Returns the File that is the persistence file
     */
    private static File getPersistenceFile() {
        if (persistenceFile == null) {
            persistenceFile = createPersistenceFile();
        }

        return persistenceFile;
    }

    /**
     * Creates the File object that is the persistence file.  Creates the
     * directory it lives in if necessary. First tries the user's home directory
     * and then the current working directory.
     */
    private static File createPersistenceFile() {
        String persistenceDirHomeName = System.getProperty("user.home") + 
                                        File.separator + 
                                        Persistable.PERSISTENCE_FOLDER;

        String persistenceDirCWDName = System.getProperty("user.dir") + 
                                       File.separator + 
                                       Persistable.PERSISTENCE_FOLDER;

        File persistenceDirHomeFile = new File(persistenceDirHomeName);
        File persistenceDirCWDFile = new File(persistenceDirCWDName);


        // Set a reasonable default
        persistenceFile = new File(persistenceDirHomeName + File.separator + 
                                   PERSISTENCE_FILE_NAME);

        // FIXME: the following hairy mess attempts to find a location for
        // persistence first looking in the users home directory, then looking
        // in the current working directory.
        if (persistenceDirHomeFile.exists()) {
            if (persistenceDirHomeFile.isDirectory()) {
                return persistenceFile;
            }
        } else if (persistenceDirHomeFile.mkdir()) {
            return persistenceFile;
        }

        Debug.writeLogMessage("Persistence directory " + 
                              persistenceDirHomeName + 
                              " exists as file not a directory. Trying " + 
                              persistenceDirCWDName);

        if (persistenceDirCWDFile.exists()) {
            if (persistenceDirCWDFile.isDirectory()) {
                return new File(persistenceDirCWDName + File.separator + 
                                PERSISTENCE_FILE_NAME);
            }
        } else if (persistenceDirCWDFile.mkdir()) {
            return new File(persistenceDirCWDName + File.separator + 
                            PERSISTENCE_FILE_NAME);
        }

        Debug.writeLogMessage("Unable to persist in " + 
                              persistenceDirHomeName + " or " + 
                              persistenceDirCWDName + 
                              " defaulting to failure.");

        return persistenceFile;
    }

    /**
     * Requests the default Pool.
     *
     * @return The default pool.
     */
    protected Pool getPool() {
        return defaultPool;
    }

    /**
     * Writes the given (Identifier, Entity) pair to the jisp database.
     */
    protected synchronized void persistEntity(Identifier identifier, 
                                              Entity entity) {
        try {
            keyObject[0] = new StringKey80(identifier.toString());
            keyObject[1] = new LongKey(System.currentTimeMillis());

            DBRecord dbRecord = new DBRecord(entity, identifier);
            database.write(keyObject, dbRecord);
        } catch (Exception everything) {
            everything.printStackTrace();
        }
    }

    protected synchronized EntityProxy createEntity(String name) {
        VerySimpleEntity entity = new VerySimpleEntity(name);
        Identifier identifier = identifierManager.getIdentifier();
        EntityProxy entityProxy = new EntityProxy(identifier, this);

        if (entityCounter <= this.HASH_INDEX_LENGTH) {
            // XXX:mml: shouldn't I be able to comment this out?
            // yes!
            //persistEntity(identifier, entity);
            try {
                memoryManager.loadEntityToMemory(identifier, entity, 
                                                 entityProxy);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            //FIXME: throw out the Exception
        }

        this.entityCounter++;

        return entityProxy;
    }

    protected Identifier findIdentifier(EntityProxy proxy) {
        if (proxy == null) {
            Debug.writeLogMessage(
                    "Trying to find Identifier for a null EntityProxy!");
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null EntityProxy!");
        }

        return proxy.getIdentifier();
    }

    protected Identifier findIdentifier(Entity entity)
                                 throws NoSuchIdentifierException {
        return memoryManager.findIdentifier(entity);
    }

    protected EntityProxy findProxy(Identifier identifier)
                             throws NoSuchEntityException {
        return memoryManager.findProxy(identifier);
    }

    protected Entity findEntity(Identifier identifier)
                         throws NoSuchEntityException {
        return memoryManager.findEntity(identifier);
    }

    /**
     * Persists ONLY the pool to the persistence file.
     *
     * FIXME: This should also force a write of the cached Entities back to the
     * jisp database.
     */
    protected void persist() {
        Persistable.PERSISTER.persist(getPool(), getPersistenceFile());
    }
}