// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import java.awt.BorderLayout;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.WindowConstants;
import javax.swing.table.AbstractTableModel;


/**
 * Provides a way to monitor the running threads in KnownSpace.
 * FIXME: Should also probably recurse into ThreadGroups contained
 * within our ThreadGroup.
 *
 * @author Matt Liggett
 */
public class ThreadMonitorSimpleton extends Simpleton {
    /**
     * How long in milliseconds to sleep between updates of the thread
     * display.
     */
    public static final int SLEEP_INTERVAL = 5000;

    /** A display table of all threads we know about. */
    private JTable threadTable;

    /** A scrollpane for the threadList. */
    private JScrollPane scrollPane;

    /** This Simpleton's toplevel graphical element. */
    private JFrame frame;

    /** A model for the table of threads. */
    private ThreadTableModel threadTableModel;

    /**
     * Returns an array of all active threads in our current thread
     * group.
     */
    private Thread[] getActiveThreads() {
        ThreadGroup threadGroup = Thread.currentThread().getThreadGroup();

        int activeCount = threadGroup.activeCount();
        Thread[] threads = new Thread[activeCount];
        threadGroup.enumerate(threads);

        return threads;
    }

    /**
     * Throws the frame up and occasionally polls the system for
     * thread status.
     */
    public void process() {
        makeFrame();
        frame.setVisible(true);

        while (true) {
            threadTableModel.setThreads(getActiveThreads());

            try {
                Thread.currentThread().sleep(SLEEP_INTERVAL);
            } catch (InterruptedException exception) {
                // do nothing
            }
        }
    }

    /**
     * Creates the JFrame and other components that go into it;
     * configures layout.
     */
    private void makeFrame() {
        frame = new JFrame(getInformation().getName());
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.setSize(400, 600);

        threadTableModel = new ThreadTableModel();
        threadTable = new JTable(threadTableModel);
        scrollPane = new JScrollPane(threadTable);

        frame.getContentPane().add(scrollPane, BorderLayout.CENTER);
        frame.getContentPane().add(new JLabel("Threads"), BorderLayout.NORTH);
    }

    /**
     * A TableModel for displaying information about the system
     * threads.
     */
    public class ThreadTableModel extends AbstractTableModel {
        /** The number of columns our table has. */
        private final int NUMBER_OF_COLUMNS = 8;

        /** The names of the columns. */
        private final String[] COLUMN_NAMES = {
            "Type", "GroupName", "Name", "Priority", "Daemon?", "Alive?", 
            "EventGenerating?", "KILL!"
        };
        private final int TYPE_COLUMN = 0;
        private final int GROUP_NAME_COLUMN = 1;
        private final int NAME_COLUMN = 2;
        private final int PRIORITY_COLUMN = 3;
        private final int DAEMON_COLUMN = 4;
        private final int ALIVE_COLUMN = 5;
        private final int EVENT_COLUMN = 6;
        private final int KILL_COLUMN = 7;

        /** The list of all threads we know about. */
        private List threadList = new ArrayList();

        /** Sets the list of threads to display. */
        public void setThreads(Thread[] threads) {
            threadList = Arrays.asList(threads);
            fireTableDataChanged();
        }

        /**
         * Returns the thread at the given index in the internal
         * threadList.
         */
        private Thread getThreadAt(int index) {
            return (Thread) threadList.get(index);
        }

        public int getColumnCount() {
            return NUMBER_OF_COLUMNS;
        }

        public int getRowCount() {
            return threadList.size();
        }

        /**
         * Returns a string representing the type of the given thread.
         */
        private String getThreadType(Thread thread) {
            if (thread instanceof DataManagerWorker) {
                return "DMW";
            }

            return "Normal";
        }

        /**
         * Returns the name of the given thread.
         */
        private String getThreadName(Thread thread) {
            return thread.getName();
        }

        /**
         * Returns the name of the given threads threadgroup.
         */
        private String getThreadGroupName(Thread thread) {
            ThreadGroup group = thread.getThreadGroup();

            if (group != null) {
                return group.getName();
            }

            return "NULL group";
        }

        /**
         * Returns the priority of the given thread.
         */
        private Integer getPriority(Thread thread) {
            return new Integer(thread.getPriority());
        }

        /**
         * Returns true if the given thread is marked a daemon thread;
         * false otherwise.
         */
        private Boolean getIsDaemon(Thread thread) {
            return new Boolean(thread.isDaemon());
        }

        /**
         * Returns true if the given thread is alive; false otherwise.
         */
        private Boolean getIsAlive(Thread thread) {
            return new Boolean(thread.isAlive());
        }

        /**
         * Returns true if the given thread is an event-generating
         * DataManagerWorker, false if it's a non-event-generating
         * DataManagerWorker, and "N/A" otherwise.
         */
        private String getIsEventGenerating(Thread thread) {
            if (thread instanceof DataManagerWorker) {
                return new Boolean(
                               ((DataManagerWorker) thread).isEventGenerating()).toString();
            }

            return "N/A";
        }

        /**
         * Placeholder for a function to produce a JButton that, when
         * pressed, will kill the given thread.  Right now just
         * returns a rousing battle cry.
         */
        public String getKillButton(Thread thread) {
            return "KILL!";
        }

        public String getColumnName(int columnIndex) {
            return COLUMN_NAMES[columnIndex];
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Thread thread = getThreadAt(rowIndex);

            if (thread == null) {
                return "NULL";
            } else {
                switch (columnIndex) {
                case TYPE_COLUMN:
                    return getThreadType(thread);

                case GROUP_NAME_COLUMN:
                    return getThreadGroupName(thread);

                case NAME_COLUMN:
                    return getThreadName(thread);

                case PRIORITY_COLUMN:
                    return getPriority(thread);

                case DAEMON_COLUMN:
                    return getIsDaemon(thread);

                case ALIVE_COLUMN:
                    return getIsAlive(thread);

                case EVENT_COLUMN:
                    return getIsEventGenerating(thread);

                case KILL_COLUMN:
                    return getKillButton(thread);
                }
            }

            // This line should never be reached.
            return null;
        }
    }
}