// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.passiveentityvalue;

import javax.mail.Flags;

import org.datamanager.passiveentityvalue.PassiveEntityValue;

import org.datamanager.tools.Debug;


/*
 * Author: Manoj Shanbhag (mshanbha@cs.indiana.edu)
 * Group : Networks
 * Date  : Nov 28, 1998
 * Modified: December 18, 1999, gregory j. e. rawlins
 */

/**
 * Objects of this class store various email message flags
 * and can be stored in Entities as they are EntityValues.
 */
public class EmailMessageFlagsEntityValue implements PassiveEntityValue {
    private boolean messageHasBeenAnswered;
    private boolean messageHasBeenDeleted;
    private boolean messageIsADraft;
    private boolean messageIsNewAndUnread;
    private boolean messageIsRead;
    private int hashCode;

    /**
    Initialize a new EmailMessageFlagsEntityValue
    with the Flag values of the given Flags object.
    **/
    public EmailMessageFlagsEntityValue(Flags flags) {
        messageHasBeenAnswered = false;
        messageHasBeenDeleted = false;
        messageIsADraft = false;
        messageIsNewAndUnread = false;
        messageIsRead = false;

        setAllFlags(flags);

        hashCode = flags.hashCode(); //NOTE: quick hack to generate one
    }

    /**
    Set all known private email message flags.
    **/
    private void setAllFlags(Flags flags) {
        Flags.Flag[] flagArray = flags.getSystemFlags();

        for (int i = 0; i < flagArray.length; i++)
            setFlag(flagArray[i]);
    }

    /**
    Set an email message flag.
    **/
    public void setFlag(Flags.Flag flag) {
        if (flag == Flags.Flag.ANSWERED) {
            messageHasBeenAnswered = true;
        } else if (flag == Flags.Flag.DELETED) {
            messageHasBeenDeleted = true;
        } else if (flag == Flags.Flag.DRAFT) {
            messageIsADraft = true;
        } else if (flag == Flags.Flag.RECENT) {
            messageIsNewAndUnread = true;
        } else if (flag == Flags.Flag.SEEN) {
            messageIsRead = true;
        }
    }

    /**
    Report whether an email message flag is set or not.
    **/
    public boolean getFlag(Flags.Flag flag) {
        if (flag == Flags.Flag.ANSWERED) {
            return messageHasBeenAnswered;
        } else if (flag == Flags.Flag.DELETED) {
            return messageHasBeenDeleted;
        } else if (flag == Flags.Flag.DRAFT) {
            return messageIsADraft;
        } else if (flag == Flags.Flag.RECENT) {
            return messageIsNewAndUnread;
        } else if (flag == Flags.Flag.SEEN) {
            return messageIsRead;
        } else {
            Debug.writeLogMessage("Unknown flag requested");

            return false;
        }
    }

    /**
    Report whether this EmailMessageEntityValue has the same value
    as a given Object.
    **/
    public boolean equals(Object object) {
        //make sure the given object isn't the same as this object
        //else, make sure it's non-null and that its class is the same
        if (this == object) {
            return true;
        } else if ((object == null) || (getClass() != object.getClass())) {
            return false;
        }

        EmailMessageFlagsEntityValue other = (EmailMessageFlagsEntityValue) object;

        boolean equality = (this.messageHasBeenAnswered == other.messageHasBeenAnswered) && 
                           (this.messageHasBeenDeleted == other.messageHasBeenDeleted) && 
                           (this.messageIsADraft == other.messageIsADraft) && 
                           (this.messageIsNewAndUnread == other.messageIsNewAndUnread) && 
                           (this.messageIsRead == other.messageIsRead);

        return equality;
    }

    public int hashCode() {
        return hashCode;
    }

    /**
    Report a string representation of a set of email message flags.
    **/
    public String toString() {
        return "MessageHasBeenAnswered = " + messageHasBeenAnswered + "\n" + 
               "MessageHasBeenDeleted = " + messageHasBeenDeleted + "\n" + 
               "MessageIsADraft = " + messageIsADraft + "\n" + 
               "MessageIsNewAndUnread = " + messageIsNewAndUnread + "\n" + 
               "MessageIsRead = " + messageIsRead;
    }
}