// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.passiveentityvalue;

import org.datamanager.passiveentityvalue.PasswordEntityValue;


/**
 * This class represents the various JavaMail email account configuration
 * settings for a user's email account.  Encapsulates a String username, 
 * PasswordEntityValue, String host machine name, String access protocol, 
 * String mailbox name, String from address and String SMTP Host.
 *
 * @author Team Helium
 * @version $Revision: 1.8 $
 *
 */
public class EmailParametersEntityValue implements PassiveEntityValue {
    //  The parameters of an email account.
    private final String username;
    private final PasswordEntityValue passwordEntityValue;
    private final String hostMachineName;
    private final String accessProtocolName;
    private final String mailboxName;
    private final String fromAddress;
    private final String SMTPHost;

    //  FIXME:  the String values might want to be separate
    //  EntityValues, like UsernameEntityValue, AccessProtocolEntityValue,
    //  etc.
    public EmailParametersEntityValue(String username, 
                                      PasswordEntityValue passwordEntityValue, 
                                      String hostMachineName, 
                                      String accessProtocolName, 
                                      String mailboxName, String fromAddress, 
                                      String SMTPHost) {
        this.username = username;
        this.passwordEntityValue = passwordEntityValue;
        this.hostMachineName = hostMachineName;
        this.accessProtocolName = accessProtocolName;
        this.mailboxName = mailboxName;
        this.fromAddress = fromAddress;
        this.SMTPHost = SMTPHost;
    }

    /**
     * Returns the String username encapsulated by this
     * EmailParametersEntityValue.
     *
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * Returns the PasswordEntityValue encapsulated by this
     * EmailParametersEntityValue.  The PasswordEntityValue _should_ take care
     * of authenticating anyone attempting to access its password field.  
     */
    public PasswordEntityValue getPasswordEntityValue() {
        return this.passwordEntityValue;
    }

    /**
     * Returns a String representation of the host machine to which this
     * EmailParametersEntityValue can connect.
     */
    public String getHostMachineName() {
        return this.hostMachineName;
    }

    /**
     * Returns a String representation of the access protocol ("imap", "imaps",
     * "pop3", etc.) to be used when connecting to the host machine.
     */
    public String getAccessProtocolName() {
        return this.accessProtocolName;
    }

    /**
     * Returns the name of the mailbox data store to be opened after
     * connecting to the host machine.  A mailbox name of "*" signifies all
     * mailboxes at the top level of the email data store.
     */
    public String getMailboxName() {
        return this.mailboxName;
    }

    /**
     * Returns the address the user would like outgoing email to be sent as
     * from.
     */
    public String getFromAddress() {
        return this.fromAddress;
    }

    /**
     * Returns the SMTP host to which outgoing mail is delivered.
     */
    public String getSMTPHost() {
        return this.SMTPHost;
    }

    /**
     * Compares the specified Object with this EmailParametersEntityValue for
     * equality.  Returns true if and only if the specified Object is an
     * instance of a EmailParametersEntityValue and all fields of the
     * EmailParametersEntityValue satisfy <code>equals()</code> with all
     * fields of this EmailParametersEntityValue.  It is expected that all
     * member fields for <code>EmailParametersEntityValue</code>-s override
     * Object's <code>equals()</code> method.
     */
    public boolean equals(Object object) {
        boolean isEqual = (object != null) && 
                          (object instanceof EmailParametersEntityValue);

        if (isEqual) {
            EmailParametersEntityValue temp = (EmailParametersEntityValue) object;
            isEqual = (object == this) || 
                      (getUsername().equals(temp.getUsername()) && 
                      getPasswordEntityValue()
                          .equals(temp.getPasswordEntityValue()) && 
                      getHostMachineName().equals(temp.getHostMachineName()) && 
                      getAccessProtocolName()
                          .equals(temp.getAccessProtocolName()) && 
                      getMailboxName().equals(temp.getMailboxName()) && 
                      getFromAddress().equals(temp.getFromAddress()) && 
                      getSMTPHost().equals(temp.getSMTPHost()));
        }

        return isEqual;
    }

    /**
     * Returns the <code>int hashCode()</code> of this EmailParametersEntityValue.
     * FIXME: this doesn't include from address and stmp host, should it?
     */
    public int hashCode() {
        return (getUsername().hashCode() & 
               getPasswordEntityValue().hashCode()) ^ 
               getMailboxName().hashCode() ^ 
               (getHostMachineName().hashCode() & 
               getAccessProtocolName().hashCode());
    }
}

/*
 *
 * $Log: EmailParametersEntityValue.java,v $
 * Revision 1.8  2002/12/20 20:52:35  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.7  2002/11/22 03:41:51  alllee
 * Cosmetic change to not capitalize the B in Box.  Much as I love to capitalize
 * on Box, Mailbox is one word and hence the getter for the mail box name should
 * be getMailboxName(), not getMailBoxName().  Anal retentiveness! Yaaaay!
 *
 * Revision 1.6  2002/11/20 00:25:41  cjdent
 *
 *
 * EmailParametersEntityValue and EmailParameters
 *
 * Addition of SMTPHost field for the outgoing SMTP server
 * Updated the equals method in EmailParametersEntityValue to include the
 *   smtp and from address info. Was this right?
 *
 * Added an SMTP host to the EmailConfiguratorSimpleton
 *
 * SMTPSendSimpleton
 *  subscribes to the EmailParametersEnteredCollectorEvent and fires the
 *  EmailParametersRequestEvent to get SMTP host information.
 *
 *  FIXME: What happens if it is unable to get the info for some reason when
 *  it tries to make a delivery. I've set up a default, which is _not_ a good
 *  long term solution. This is integrated with the overall question of
 *  how to deal with SMTP errors.
 *
 * Rest is more bogus codebase-style
 *
 * Revision 1.5  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 *
 */
