// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.passiveentityvalue;

import java.io.UnsupportedEncodingException;

import java.lang.Comparable;

import java.util.Locale;

import org.datamanager.passiveentityvalue.PassiveEntityValue;


/**
 * This EntityValue wraps the String class, providing most of the methods
 * available to an actual Java String through a PassiveEntityValue type.
 *
 * Original author - Benjamin Keil
 *
 * @author Team Helium 
 * @version $Revision: 1.8 $
 */
public class StringEntityValue implements PassiveEntityValue, Comparable {
    public final static String NULL_STRING = "";
    private String value;

    /**
     * Initializes a newly created String object so that it represents an
     * empty character sequence.
     */
    public StringEntityValue() {
        // Changed from this.value = new String() because there is no reason
        // not to use the canonical version of "" (other than wasted
        // allocation);
        setValue(NULL_STRING);
    }

    /**
     * Construct a new String by converting the specified array of bytes using
     * the platform's default character encoding.
     */
    public StringEntityValue(byte[] bytes) {
        setValue(new String(bytes));
    }

    /**
     * Construct a new String by converting the specified subarray of bytes
     * using the platform's default character encoding.
     */
    public StringEntityValue(byte[] bytes, int offset, int length) {
        setValue(new String(bytes, offset, length));
    }

    /**
     * Construct a new String by converting the specified subarray of bytes
     * using the specified character encoding.
     */
    public StringEntityValue(byte[] bytes, int offset, int length, String enc)
                      throws UnsupportedEncodingException {
        setValue(new String(bytes, offset, length, enc));
    }

    /**
     * Construct a new String by converting the specified array of bytes using
     * the specified character encoding.
     */
    public StringEntityValue(byte[] bytes, String enc)
                      throws UnsupportedEncodingException {
        setValue(new String(bytes, enc));
    }

    /**
     * Allocates a new String so that it represents the sequence of characters
     * currently contained in the character array argument.
     */
    public StringEntityValue(char[] value) {
        setValue(new String(value));
    }

    /**
     * Allocates a new String that contains characters from a subarray of the
     * character array argument.
     */
    public StringEntityValue(char[] value, int offset, int count) {
        setValue(new String(value, offset, count));
    }

    /**
     * Initializes a newly created String object so that it represents the same
     * sequence of characters as the argument; in other words, the newly
     * created string is a copy of the argument string.
     */
    public StringEntityValue(String value) {
        setValue(value);
    }

    /**
     * Allocates a new string that contains the sequence of characters
     * currently contained in the string buffer argument.
     */
    public StringEntityValue(StringBuffer buffer) {
        setValue(buffer.toString());
    }

    /**
     * Returns the character at the specified index.
     */
    public char charAt(int index) {
        return value.charAt(index);
    }

    /**
     * Compares this String to another Object.
     */
    public int compareTo(Object o) {
        if (o instanceof StringEntityValue) {
            return value.compareTo(((StringEntityValue) o).intern());
        } else {
            return 0;
        }
    }

    /**
     * Compares two strings lexicographically.
     */
    public int compareTo(String anotherString) {
        return value.compareTo(anotherString);
    }

    /**
     * Compares two strings lexicographically, ignoring case considerations.
     */
    public int compareToIgnoreCase(String str) {
        return value.compareToIgnoreCase(str);
    }

    /**
     * Concatenates the specified string to the end of this string.
     */
    public String concat(String str) {
        return value.concat(str);
    }

    /**
     * Tests if this string ends with the specified suffix.
     */
    public boolean endsWith(String suffix) {
        return value.endsWith(suffix);
    }

    /**
     * Compares this string to the specified object.
     */
    public boolean equals(Object anObject) {
        if (anObject instanceof StringEntityValue) {
            return value.equals(((StringEntityValue) anObject).value);
        }

        return value.equals(anObject);
    }

    /**
     * Compares this String to another String, ignoring case considerations.
     */
    public boolean equalsIgnoreCase(String anotherString) {
        return value.equalsIgnoreCase(anotherString);
    }

    /**
     * Convert this String into bytes according to the platform's default
     *  character encoding, storing the result into a new byte array.
     */
    public byte[] getBytes() {
        return value.getBytes();
    }

    /**
     * Convert this String into bytes according to the specified character
     * encoding, storing the result into a new byte array.
     */
    public byte[] getBytes(String enc) throws UnsupportedEncodingException {
        return value.getBytes(enc);
    }

    /**
     * Copies characters from this string into the destination character array.
     */
    public void getChars(int srcBegin, int srcEnd, char[] dst, int dstBegin) {
        value.getChars(srcBegin, srcEnd, dst, dstBegin);
    }

    /**
     * Returns a hashcode for this string.
     */
    public int hashCode() {
        return value.hashCode();
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified character.
     */
    public int indexOf(int ch) {
        return value.indexOf(ch);
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified character, starting the search at the specified index.
     */
    public int indexOf(int ch, int fromIndex) {
        return value.indexOf(ch, fromIndex);
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified substring.
     */
    public int indexOf(String str) {
        return value.indexOf(str);
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified substring, starting at the specified index.
     */
    public int indexOf(String str, int fromIndex) {
        return value.indexOf(str, fromIndex);
    }

    /**
     * Returns a canonical representation for the string object.
     */
    public String intern() {
        return value.intern();
    }

    /**
     * Returns the index within this string of the last occurrence of the
     * specified character.
     */
    public int lastIndexOf(int ch) {
        return value.lastIndexOf(ch);
    }

    /**
     * Returns the index within this string of the last occurrence of the
     * specified character, searching backward starting at the specified index.
     */
    public int lastIndexOf(int ch, int fromIndex) {
        return value.lastIndexOf(ch, fromIndex);
    }

    /**
     * Returns the index within this string of the rightmost occurrence of the
     * specified substring.
     */
    public int lastIndexOf(String str) {
        return value.lastIndexOf(str);
    }

    /**
     * Returns the index within this string of the last occurrence of the
     * specified substring.
     */
    public int lastIndexOf(String str, int fromIndex) {
        return value.lastIndexOf(str, fromIndex);
    }

    /**
     * Returns the length of this string.
     */
    public int length() {
        return value.length();
    }

    /**
     * Tests if two string regions are equal.
     */
    public boolean regionMatches(boolean ignoreCase, int toffset, String other, 
                                 int ooffset, int len) {
        return value.regionMatches(ignoreCase, toffset, other, ooffset, len);
    }

    /**
     * Tests if two string regions are equal.
     */
    public boolean regionMatches(int toffset, String other, int ooffset, 
                                 int len) {
        return value.regionMatches(toffset, other, ooffset, len);
    }

    /**
     * Returns a new string resulting from replacing all occurrences of
     * oldChar in this string with newChar.
     */
    public String replace(char oldChar, char newChar) {
        return value.replace(oldChar, newChar);
    }

    /**
     * Tests if this string starts with the specified prefix.
     */
    public boolean startsWith(String prefix) {
        return value.startsWith(prefix);
    }

    /**
     * Tests if this string starts with the specified prefix beginning a
     * specified index.
     */
    public boolean startsWith(String prefix, int toffset) {
        return value.startsWith(prefix, toffset);
    }

    /**
     * Returns a new string that is a substring of this string.
     */
    public String substring(int beginIndex) {
        return value.substring(beginIndex);
    }

    /**
     * Returns a new string that is a substring of this string.
     */
    public String substring(int beginIndex, int endIndex) {
        return value.substring(beginIndex, endIndex);
    }

    /**
     * Converts this string to a new character array.
     */
    public char[] toCharArray() {
        return value.toCharArray();
    }

    /**
     * Converts all of the characters in this String to lower case using the
     * rules of the default locale, which is returned by Locale.getDefault .
     */
    public String toLowerCase() {
        return value.toLowerCase();
    }

    /**
     * Converts all of the characters in this String to lower case using the
     * rules of the given Locale.
     */
    public String toLowerCase(Locale locale) {
        return value.toLowerCase(locale);
    }

    /**
     * A copy of the String wrapped is returned.
     */
    public String toString() {
        return value;
    }

    /**
     * Converts all of the characters in this String to upper case using the
     * rules of the default locale, which is returned by Locale.getDefault.
     */
    public String toUpperCase() {
        return value.toUpperCase();
    }

    /**
     * Converts all of the characters in this String to upper case using the
     * rules of the given locale.
     */
    public String toUpperCase(Locale locale) {
        return value.toUpperCase(locale);
    }

    /**
     * Removes white space from both ends of this string.
     */
    public String trim() {
        return value.trim();
    }

    private final void setValue(String value) {
        if (null == value) {
            this.value = NULL_STRING;
        } else {
            this.value = value;
        }
    }
}

/*
 * $Log: StringEntityValue.java,v $
 * Revision 1.8  2002/12/20 20:52:35  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.7  2002/12/06 11:55:03  alllee
 * Even more codebase style
 *
 * Revision 1.6  2002/12/06 10:18:57  alllee
 * Modified StringEntityValue to not construct new String references for the
 * constructors that take a String|StringBuffer, and to return a null string, not
 * null itself.  This was done to try and fix a NullPointerException in the new
 * Touchgraph clustering visualization guy, but that's not the problem.  A fix
 * for that is coming soon.  Also added a setValue(...) method for
 * StringEntityValue to have a single point of assignment for the private String
 * value (it would have been tediously painful to try and do null checks for
 * every constructor, plus, this makes it easier to strip that out if we decide
 * to not return "" instead of null).
 *
 */
