// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.passiveentityvalue;

import java.io.Serializable;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.datamanager.passiveentityvalue.PassiveEntityValue;


/**
 * This class maintains a Map of words mapped to their numerical frequencies
 * of occurrence within a given Document.  
 *
 * @author Team Helium
 * @version $Revision: 1.8 $
 *
 */
public class WordFrequencyMapEntityValue implements PassiveEntityValue {
    /** Delimiters to be used when tokenizing up a String corpus. */
    public final static String DELIMITERS = ".,'\\/\";:?!\n\t\r~!@%^&*()_-+={}[]<> ,       ";

    /** 
     * This constant represents the attribute name that should be used for
     * the Entity whose EntityValue is this WordFrequencyMapEntityValue.
     */
    public final static String ATTRIBUTE_NAME = "Word-Frequency";
    private int totalNumberOfWords;
    private final Map wordFrequencyMap;
    private final String corpus;

    public WordFrequencyMapEntityValue(String corpus) {
        this.corpus = corpus;
        this.wordFrequencyMap = new HashMap();
        generateWordFrequencies(this.corpus);
    }

    /**
     * Returns the hashCode() of the String corpus.
     */
    public int hashCode() {
        return corpus.hashCode();
    }

    /**
     * Returns true if the object in question is a WordFrequencyMapEntityValue
     * and their underlying Maps and corpusi are also .equals() to each other.
     */
    public boolean equals(Object object) {
        // First check for pointer equality
        boolean isEqual = (this == object);

        if (!isEqual) {
            isEqual = ((object != null) && 
                      (object instanceof WordFrequencyMapEntityValue) && 
                      ((WordFrequencyMapEntityValue) object).getCorpus()
                                                          .equals(this.corpus));
        }

        return isEqual;
    }

    public Map getWordFrequencyMap() {
        return Collections.unmodifiableMap(this.wordFrequencyMap);
    }

    public Map getSynchronizedWordFrequencyMap() {
        return Collections.synchronizedMap(getWordFrequencyMap());
    }

    public Set getWordSet() {
        return Collections.unmodifiableSet(this.wordFrequencyMap.keySet());
    }

    public boolean contains(String word) {
        return this.wordFrequencyMap.containsKey(word);
    }

    public double getFrequencyOf(String word) {
        double numberOfOccurrences = 0;

        if (this.contains(word)) {
            numberOfOccurrences = ((Number) this.wordFrequencyMap.get(word)).doubleValue();
        }

        return numberOfOccurrences;
    }

    public double getNumberOfUniqueWords() {
        return getWordSet().size();
    }

    /**
     * Returns an double representing the total number of words in this word
     * frequency map.  Not the total number of unique words in the map, the
     * total number of all words in the map.  A double is used because a lot
     * of these values are used for calculating probabilities and what not and
     * forcing the explicit cast on the application side is a pain.
     */
    public double getTotalNumberOfWords() {
        return (double) this.totalNumberOfWords;
    }

    private String getCorpus() {
        return this.corpus;
    }

    private void generateWordFrequencies(final String corpus) {
        final StringTokenizer tokenizer = new StringTokenizer(corpus, 
                                                              DELIMITERS);
        String token;
        totalNumberOfWords = 0;

        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken().toLowerCase();

            if (wordFrequencyMap.containsKey(token)) {
                wordFrequencyMap.put(token, 
                                     IntegerPool.getInteger(1 + ((Integer) wordFrequencyMap.get(
                                                                         token)).intValue()));
            } else {
                wordFrequencyMap.put(token, IntegerPool.getInteger(1));
            }

            totalNumberOfWords++;
        }
    }

    /**
     * IntegerPool is used to avoid allocating new Integers constantly to keep
     * inside our WordFrequencyMap.  A cheap premature optimization.
     */
    private static class IntegerPool {
        private final static int DEFAULT_SIZE = 32;
        private static Integer[] integerPool;

        static {
            integerPool = new Integer[DEFAULT_SIZE];

            for (int i = integerPool.length; (--i) >= 0;) {
                integerPool[i] = new Integer(i);
            }
        }

        private IntegerPool() {
        }

        public final static Integer getInteger(final int index) {
            if (index >= integerPool.length) {
                Integer[] newIntegerPool = new Integer[index + DEFAULT_SIZE];
                System.arraycopy(integerPool, 0, newIntegerPool, 0, 
                                 integerPool.length);

                for (int i = newIntegerPool.length;
                     (--i) >= integerPool.length;) {
                    newIntegerPool[i] = new Integer(i);
                }

                integerPool = newIntegerPool;
            }

            return integerPool[index];
        }
    } // IntegerPool
} // WordFrequencyMapEntityValue
/*
 * $Log: WordFrequencyMapEntityValue.java,v $
 * Revision 1.8  2002/12/20 20:52:36  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.7  2002/12/09 03:25:29  cjdent
 *
 * lower-cased the token before adding it to the map
 *
 * added a few delimeters, found by inspecting some generated maps
 *
 * Revision 1.6  2002/12/07 15:14:56  alllee
 * Got rid of the magic marker WordTotal crap and moved the total number of words
 * into the class itself as a queryable method getTotalNumberOfWords().
 * Refactored a lot to push intelligence down from NaiveBayesStrategy into this
 * EntityValue itself - instead of having Strategies know, deep down, that this
 * EntityValue has a Map of Strings->Integers, there are methods to query the
 * total number of words, number of unique words, and new getters for the
 * frequency of a given word. The underlying Map and stuff are still available
 * but are now unmodifiable via Collections.unmodifiableMap. Reflected changes in
 * other classes followin shortly..
 *
 * Revision 1.5  2002/12/06 19:29:11  alllee
 * Added a magic value that marks the total number of words (all words in the
 * corpus) within the published Map of word frequencies... This is hardly a good
 * solution, probably a Wrapped Map class would be more ideal.
 *
 * Revision 1.4  2002/12/06 14:04:30  alllee
 * Putting the attribute name we are currently using for WordFrequencyMap
 * Entity attributes as a global constant in here, for lack of a better place,
 * and to avoid hard coding everything under the sun.
 *
 * Revision 1.3  2002/12/06 11:55:04  alllee
 * Even more codebase style
 *
 * Revision 1.2  2002/12/06 10:18:58  alllee
 * Modified StringEntityValue to not construct new String references for the
 * constructors that take a String|StringBuffer, and to return a null string, not
 * null itself.  This was done to try and fix a NullPointerException in the new
 * Touchgraph clustering visualization guy, but that's not the problem.  A fix
 * for that is coming soon.  Also added a setValue(...) method for
 * StringEntityValue to have a single point of assignment for the private String
 * value (it would have been tediously painful to try and do null checks for
 * every constructor, plus, this makes it easier to strip that out if we decide
 * to not return "" instead of null).
 *
 */
