// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;

import java.net.URL;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;

import org.datamanager.event.EmailMetadataSearchEvent;
import org.datamanager.event.LaunchBrowserWithURLEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;

import org.datamanager.simpleton.EmailMessageComponent;
import org.datamanager.simpleton.EmailMessageEntityHelper;

import org.datamanager.tools.Toolbox;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Provides a rudimentary layout and display of an email message.
 *
 * @version $Revision: 1.16 $
 */
public class BasicEmailMessageComponent implements EmailMessageComponent {
    /** Font used for header entries. */
    private static final Font HEADER_FONT = new Font("Courier", Font.BOLD, 12);

    /** Font used for body text. */
    private static final Font BODY_FONT = new Font("Courier", Font.PLAIN, 12);

    // Gui components.
    private JEditorPane content;
    private JLabel fromLabel;
    private JLabel toLabel;
    private JLabel dateLabel;
    private JLabel subjectLabel;
    private JLabel ccLabel;
    private JTextField fromTextField;
    private JTextField toTextField;
    private JTextField dateTextField;
    private JTextField subjectTextField;
    private JTextField ccTextField;

    /** Whether or not we should allow the user to edit the message. */
    private boolean isEditable;

    /** The panel which holds the email message. */
    private JPanel panel;

    /** Helper class to make accessing email attributes easier. */
    private EmailMessageEntityHelper messageHelper;

    /**
     * Creates a new BasicEmailMessageComponent with the given message and
     * editability.
     */
    public BasicEmailMessageComponent(Entity message, boolean isEditable) {
        panel = new JPanel();

        setEditable(isEditable);
        setMessage(message);
    }

    /**
     * Sets whether this component may be edited by the user.
     */
    public void setEditable(boolean isEditable) {
        this.isEditable = isEditable;
    }

    /**
     * Tests whether this component may be edited by the user.
     */
    public boolean isEditable() {
        return this.isEditable;
    }

    /**
     * Sets the message held by this component to the email
     * message represented by the provided Entity.
     */
    public void setMessage(Entity message) {
        try {
            messageHelper = new EmailMessageEntityHelper(message);
            layoutPanel();
        } catch (DataManagerException exception) {
            // FIXME: bogus exception handling
            throw new RuntimeException(exception);
        }
    }

    /**
     * Resets the Entity attributes to the content
     * of the GUI components.
     */
    public void recomposeMessageEntityFromFields() throws DataManagerException {
        messageHelper.setBody(content.getText());
        messageHelper.setFromAddress(fromTextField.getText());
        messageHelper.setToAddresses(toTextField.getText());
        messageHelper.setCcAddresses(ccTextField.getText());
        messageHelper.setSubject(subjectTextField.getText());
    }

    /**
     * Fires the LaunchBrowserWithURLEvent with the proper URL.
     */
    private void fireLaunchBrowserWithURLEvent(URL url) {
        try {
            KernelProxy.getInstance()
                       .addEvent(new LaunchBrowserWithURLEvent(
                                         KernelProxy.getInstance()
                                                    .getEventGenerator(), url));
        } catch (DataManagerException exception) {
            // FIXME: we need a better mechanism for dealing with these
            throw new RuntimeException(exception);
        }
    }

    /**
     * Creates and lays out all of the graphical elements to display the message
     * in the JPanel.
     *
     * FIXME: This method could probably still be shortened some.
     */
    private void layoutPanel() throws DataManagerException {
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);

        LayoutHelper layoutHelper = new LayoutHelper(gridBagConstraints, panel);

        fromLabel = new JLabel(EmailMessageEntityHelper.MAIL_FROM + ":");
        layoutHelper.add(fromLabel, GridBagConstraints.NONE, 0, 0, 0.0, 1);

        fromTextField = new JTextField(messageHelper.getFromAddress(), 80);
        fromTextField.setFont(HEADER_FONT);
        fromTextField.setEditable(this.isEditable());
        layoutHelper.add(fromTextField, GridBagConstraints.HORIZONTAL, 1, 0, 
                         1.0, 5);

        toLabel = new JLabel(EmailMessageEntityHelper.MAIL_TO + ":");
        layoutHelper.add(toLabel, GridBagConstraints.NONE, 0, 1, 0.0, 1);

        toTextField = new JTextField(messageHelper.getToAddresses(), 80);
        toTextField.setFont(HEADER_FONT);
        toTextField.setEditable(this.isEditable());
        layoutHelper.add(toTextField, GridBagConstraints.HORIZONTAL, 1, 1, 1.0, 5);

        ccLabel = new JLabel(EmailMessageEntityHelper.MAIL_CC + ":");
        layoutHelper.add(ccLabel, GridBagConstraints.NONE, 0, 2, 0.0, 1);

        ccTextField = new JTextField(messageHelper.getCcAddresses(), 80);
        ccTextField.setFont(HEADER_FONT);
        ccTextField.setEditable(this.isEditable());
        layoutHelper.add(ccTextField, GridBagConstraints.HORIZONTAL, 1, 2, 1.0, 5);

        dateLabel = new JLabel(EmailMessageEntityHelper.MAIL_DATE + ":");
        layoutHelper.add(dateLabel, GridBagConstraints.NONE, 0, 3, 0.0, 1);

        dateTextField = new JTextField(messageHelper.getDateString(), 80);
        dateTextField.setFont(HEADER_FONT);
        dateTextField.setEditable(this.isEditable());
        layoutHelper.add(dateTextField, GridBagConstraints.HORIZONTAL, 1, 3, 
                         1.0, 5);

        subjectLabel = new JLabel(EmailMessageEntityHelper.MAIL_SUBJECT + 
                                  ":");
        layoutHelper.add(subjectLabel, GridBagConstraints.NONE, 0, 4, 0.0, 1);

        subjectTextField = new JTextField(messageHelper.getSubject(), 80);
        subjectTextField.setFont(HEADER_FONT);
        subjectTextField.setEditable(this.isEditable());
        layoutHelper.add(subjectTextField, GridBagConstraints.HORIZONTAL, 1, 4, 
                         1.0, 5);

        // FIXME: this is a silly way to determine
        // if we are doing a reply or a display.
        if (isEditable()) {
            content = new JEditorPane();
            content.setDocument(new WordWrappingDocument(70));
            content.setFont(BODY_FONT);
            content.setContentType("text/plain");
            content.setText(getBodyAsText());
            content.setEditable(true);
        } else {
            content = new JEditorPane("text/html", getBodyAsHtml());
            content.setEditable(false);
            content.addHyperlinkListener(new HyperlinkListener() {
                public void hyperlinkUpdate(HyperlinkEvent event) {
                    if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                        fireLaunchBrowserWithURLEvent(event.getURL());
                    }
                }
            });
        }

        JScrollPane scrollPane = new JScrollPane(content);
        layoutHelper.add(scrollPane, GridBagConstraints.BOTH, 0, 5, 1.0, 1.0, 6, 4);
    }

    /**
     * Retrieves the EmailMessageEntityHelper used by this component.
     */
    public EmailMessageEntityHelper getEmailMessageEntityHelper() {
        return this.messageHelper;
    }

    /**
     * Retrieves the Entity representing the email message this
     * component is displaying.
     */
    public Entity getMessage() {
        return messageHelper.getMessage();
    }

    public Component getComponent() {
        return panel;
    }

    public Component getToComponent() {
        return toTextField;
    }

    public Component getCcComponent() {
        return ccTextField;
    }

    public Component getFromComponent() {
        return fromTextField;
    }

    public Component getSubjectComponent() {
        return subjectTextField;
    }

    public Component getDateComponent() {
        return dateTextField;
    }

    /**
     * Returns a text version of the body of the message.
     *
     * FIXME: doesn't do any checking of okayness at this
     * stage. This is because we only handle text/plain
     * anyway.
     */
    private String getBodyAsText() throws DataManagerException {
        return messageHelper.getBody();
    }

    /**
     * Returns an HTML version of the body of the message.
     */
    private String getBodyAsHtml() throws DataManagerException {
        String body = messageHelper.getBody();

        if (messageHelper.getContentType().equals("text/html")) {
            // No change needed.
        } else if (messageHelper.getContentType().equals("text/plain")) {
            body = Toolbox.textToPreHtml(body);
        } else {
            body = "Don't know how to format content type: '" + 
                   messageHelper.getContentType() + "'";
        }

        return body;
    }

    /**
     * Provides a styled Document that does hard wrapping at
     * a certain column.
     *
     * FIXME: under certain circumstances the most recent
     * linefeed gets lost and the math falls apart.
     */
    class WordWrappingDocument extends DefaultStyledDocument {
        /** The column at which we want to do our wrapping. */
        private int wrapColumn;

        /**
         * Counter of current linefeed location.
         * FIXME: we can do this better with complicated math...
         */
        private int lineFeedOffset = 0;

        public WordWrappingDocument(int wrapColumn) {
            this.wrapColumn = wrapColumn;
        }

        /**
         * Inserts the provided string into the document at the given offset,
         * inserting linefeeds where necessary.
         *
         * FIXME: the ifs have become messy.
         */
        public void insertString(int offset, String insertedString, 
                                 AttributeSet attributes)
                          throws BadLocationException {
            int indexOfLineFeed = insertedString.lastIndexOf('\n');

            if (indexOfLineFeed >= 0) {
                lineFeedOffset = offset + indexOfLineFeed;
            }

            // only do linefeeds during typing not pastes
            if ((insertedString.length() == 1) && 
                    ((offset + insertedString.length()) > (wrapColumn + lineFeedOffset))) {
                insertLineFeed(offset, insertedString, attributes);
            } else {
                super.insertString(offset, insertedString, attributes);
            }
        }

        /**
         * Inserts a linefeed into this Document by finding the most recent
         * space in the text and replacing it with a linefeed.
         */
        private void insertLineFeed(int offset, String insertedString, 
                                    AttributeSet attributes)
                             throws BadLocationException {
            String thisLine = getText(lineFeedOffset, offset - 
                                      lineFeedOffset);
            int spaceOffset = thisLine.lastIndexOf(' ');

            if (spaceOffset >= 0) {
                remove(lineFeedOffset + spaceOffset, 1);


                // FIXME: probaby shouldn't be using the provided attributes here
                super.insertString(lineFeedOffset + spaceOffset, "\n", 
                                   attributes);
                lineFeedOffset = lineFeedOffset + spaceOffset;
            }

            super.insertString(offset, insertedString, attributes);
        }
    }
}

/*
 *
 * $Log: BasicEmailMessageComponent.java,v $
 * Revision 1.16  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.15  2002/12/20 06:45:42  cjdent
 *
 * Simple cleanups to javadoc.
 *
 * Also took advantage of public static strings in EmailMessageEntityHelper
 * for header values.
 *
 * Revision 1.14  2002/12/06 11:33:02  alllee
 * Now that EmailMessageEntityHelper throws a DataManagerException from both of
 * its constructors, we have to try/catch or re-throw in all these classes that
 * utilize EmailMessageEntityHelper.
 *
 * Revision 1.13  2002/11/26 06:48:59  alllee
 * Some more dead-code elimination from EmailConfiguratorSimpleton (removed
 * boxAdd) and fixed the reply bug where a class-level instance variable was
 * getting clobbered in EmailDisplaySimpleton each time a new
 * EntityDisplayRequest was being handled.  The solution to this is to have
 * fresh, lexically scoped email messages being handed out each time a new
 * EntityDisplayRequest comes in.  More refactoring to come w/
 * EmailMessageComponent and friends..
 *
 * Revision 1.12  2002/11/25 06:56:12  cjdent
 *
 * codebase style to spark a website-deploy
 *
 * Revision 1.11  2002/11/24 22:12:53  cjdent
 *
 * This appears to clear bug
 *
 *   http://lazlo.bloomington.in.us:5224/helium/issue62
 *
 * The problem was with the WordWrappingDocument used. It was
 * choking on input in the original message that was over a
 * certain line length. It appears. What I did was make the
 * wrapping only come into play when typing a single
 * character, not when doing a long insert. This was my
 * orginal intent anyway.
 *
 * So now a few more messages show up for replying.
 *
 * Revision 1.10  2002/11/24 05:56:02  alllee
 * codebase-style/dead-code-elimination and the change to use
 * EmailMessageEntityHelper.EMAIL_ATTRIBUTE_(VALUE|NAME) when searching the pool
 * for email message entities.
 *
 * Revision 1.9  2002/11/24 03:59:59  xirao
 * xi rao
 * extract out the LayoutHelper class for re-use purpose
 *
 * Revision 1.8  2002/11/23 03:42:35  cjdent
 *
 * Added support for a WordWrappingDocument in
 * BasicEmailMessageComponent for use in replies.
 *
 * Utilizes a WordWrappingDocument inner class
 * that extends DefaultStyledDocument.
 *
 * The wrapping is acceptable for typed text but
 * does not work with pasted text. This may
 * actually be a good thing.
 *
 * The math done for the wrapping seems a bit
 * messy.
 *
 * BasicEmailMessageComponenet is too long and
 * should be refactored.
 *
 * The rest is codebase-style.
 *
 * Revision 1.7  2002/11/22 02:28:56  alllee
 * Made the header textfields (from/to/cc/subject) setEditable() to whatever the
 * parent component's isEditable() tag returns.  See
 * http://lazlo.bloomington.in.us:5224/helium/issue52
 *
 * Revision 1.6  2002/11/21 03:39:20  cjdent
 * These changes implement quoting and attribution in email replies. These
 * classes are getting long, so will need some refactoring. More notes on this
 * coming.
 *
 * EmailMessageEntityHelper has changed setDate() and getDate() to
 * setDateString() and getDateString(). getDateObject() and setDateObject()
 * have been added to retrieve the the EmailDateObject attribute.
 *
 * Revision 1.5  2002/11/19 06:15:36  cjdent
 *
 * Adding a recomposeMessageEntityFromFields().
 *
 * Resets the attributes on the Entity to the text that is contained in the
 * component fields by using the setters on the EmailMessageEntityHelper.
 *
 * Revision 1.4  2002/11/19 03:51:11  cjdent
 *
 * BasicEmailMessageComponent:
 *
 * 1) Changed DISPLAY_FONT to HEADER_FONT and added BODY_FONT so we have
 *    a monospace font in the reply.
 * 2) Added getBodyAsText() to go alongside getBodyAsHTML() for retreiving
 *    the content of a reply
 * 3) Did some silly dancing with isEditable() in the creation of the component
 *    to establish if this is a reply or not and choose getBodyAsText() or
 *    getBodyAsHTML() accordingly. There's a FIXME with this because we probably
 *    want to do this in a less hard to find in the code sort of way.
 *
 * BasicEmailReplySimpleton:
 *
 * 1) Added JButtons for Send and Cancel
 * 2) Added a JPanel to hold them in the south position of the border layout
 * 3) Added actions for send and cancel
 *    - Cancel disposes of the window. In the future, presumably, we'll want
 *      a BasicDraftEmailSelectorSimpleton to retrieve the Draft message
 *      Entities that are still out there in the pool.
 *    - Send does nothing but print out a little blurb to stderr. Cogitation
 *      is needed to determine what happens on a send. Presumably we need to
 *      gather all the content from the various fields and reset the values
 *      of the reply entity and then send out an SMTP event of some kind.
 *
 * Revision 1.3  2002/11/16 18:45:17  mliggett
 * Added methods to request the components that are handling To: From:, etc.
 * display to the EmailMessageComponent interface.
 *
 * Using those, extracted the metadata search code from
 * BasicEmailMessageComponent back into EmailDisplaySimpleton, so that
 * BasicEmailMessageComponent is more generic.
 *
 * Revision 1.2  2002/11/16 17:44:54  mliggett
 * Added javadoc.
 *
 *
 */
