// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.text.SimpleDateFormat;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeStringMatchConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.OrEventConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailMetadataSearchEvent;
import org.datamanager.event.EmailParametersEnteredCollectorEvent;
import org.datamanager.event.EmailParametersRequestEvent;
import org.datamanager.event.EmailReplyRequestEvent;
import org.datamanager.event.EntityDisplayRequestEvent;
import org.datamanager.event.SMTPSendRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;

import org.datamanager.passiveentityvalue.EmailParametersEntityValue;

import org.datamanager.simpleton.EmailParameters;

import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Provides an exportable JPanel in which a user may reply to an email
 * message.
 *
 * @version $Revision: 1.21 $
 */
public class BasicEmailReplySimpleton extends GuiSimpleton
    implements EventHandler {
    /**
     * Acceptable content-type for replying.
     * FIXME: we can do better than this.
     */
    private static final String USABLE_REPLY_CONTENT_TYPE = "text/plain";

    /** Date format string for attribution. */
    private static final String ATTRIBUTION_DATE_FORMAT = "EEE, MMM dd, yyyy";

    /** The user's preferences. */
    private EmailParameters emailParameters = null;

    /**
     * Runs this Simpleton. Subscribes to the Pool listening
     * for EntityDisplayRequestEvents that contain email messages.  
     *
     * FIXME: We need to deal with exception handling in a better fashion.
     */
    public void process() {
        try {
            subscribe();
            askForParameters();
        } catch (DataManagerException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        }
    }

    /**
     * Subscribes this Simpleton to the pool for EmailReplyRequestEventS and
     * EmailParametersRequestEventS.
     */
    private void subscribe() throws DataManagerException {
        Pool.getDefaultPool()
            .subscribe(this, 
                       new OrEventConstraint(
                               new EventTypeIsConstraint(
                                       EmailReplyRequestEvent.class), 
                               new EventTypeIsConstraint(
                                       EmailParametersEnteredCollectorEvent.class)));
    }

    /**
     * Signals that we would like some email parameters; specifically, we want
     * the user's address so we can set the From: field.
     */
    private void askForParameters() throws DataManagerException {
        Pool.getDefaultPool()
            .add(new EmailParametersRequestEvent(Pool.getDefaultPool()));
    }

    public void handleReply(EmailReplyRequestEvent event)
                     throws DataManagerException {
        makePanel(event.getEntity());
    }

    public void handleParameters(EmailParametersEnteredCollectorEvent event)
                          throws DataManagerException {
        emailParameters = new EmailParameters(
                                  (EmailParametersEntityValue) event.getEntity()
                                                                    .getValue());
    }

    public void handle(DataManagerEvent dataManagerEvent) {
        try {
            if (dataManagerEvent instanceof EmailReplyRequestEvent) {
                handleReply((EmailReplyRequestEvent) dataManagerEvent);
            } else if (dataManagerEvent instanceof EmailParametersEnteredCollectorEvent) {
                handleParameters(
                        (EmailParametersEnteredCollectorEvent) dataManagerEvent);
            }
        } catch (DataManagerException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        } catch (ClassCastException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        }
    }

    /**
     * Makes the GUI exportable panel for replying to a single message.
     */
    private void makePanel(Entity originalMessage) throws DataManagerException {
        final EmailMessageEntityHelper originalMessageHelper = 
                new EmailMessageEntityHelper(originalMessage);

        final EmailMessageEntityHelper replyHelper = createMessage(
                                                             originalMessageHelper);
        final JPanel panel = new JPanel(new BorderLayout());
        panel.setName(replyHelper.getSubject());

        final BasicEmailMessageComponent emailComponent = 
                new BasicEmailMessageComponent(replyHelper.getMessage(), true);

        panel.add(emailComponent.getComponent(), BorderLayout.CENTER);

        // Create and add the send and cancel buttons.
        JButton sendButton = new JButton(new AbstractAction("Send") {
            public void actionPerformed(ActionEvent event) {
                try {
                    emailComponent.recomposeMessageEntityFromFields();
                    fireSMTPSendRequestEvent(replyHelper.getMessage());
                    BasicEmailReplySimpleton.super.dispose(panel);
                } catch (DataManagerException exception) {
                    // FIXME: usual bogus exception handling
                    throw new RuntimeException(exception);
                }
            }
        });

        JButton cancelButton = new JButton(new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent event) {
                BasicEmailReplySimpleton.super.dispose(panel);
            }
        });

        JPanel buttonPanel = new JPanel();

        buttonPanel.add(cancelButton);
        buttonPanel.add(sendButton);

        panel.add(buttonPanel, BorderLayout.SOUTH);
        super.exportComponent(panel, new Rectangle(20, 50, 650, 500));
    }

    /**
     * Fires an SMTPSendRequestEvent with the provided
     * entity.
     */
    private void fireSMTPSendRequestEvent(Entity entity) {
        try {
            KernelProxy.getInstance()
                       .addEvent(new SMTPSendRequestEvent(
                                         KernelProxy.getInstance()
                                                    .getEventGenerator(), 
                                         entity));
        } catch (DataManagerException exception) {
            // FIXME: we need a better mechanism for dealing with these
            throw new RuntimeException(exception);
        }
    }

    /**
     * Returns the from address contained in the EmailParametersEntity.
     */
    private String getFrom() {
        if (emailParameters != null) {
            return emailParameters.getFromAddress();
        }

        return "";
    }

    /**
     * Creates a message for the user to begin editing and returns an
     * EmailMessageEntityHelper for that message.
     */
    private EmailMessageEntityHelper createMessage(EmailMessageEntityHelper originalMessageHelper)
                                            throws DataManagerException {
        EmailMessageEntityHelper messageHelper = new EmailMessageEntityHelper();


        // From: taken from user settings
        messageHelper.setFromAddress(getFrom());

        // An original message with no date object
        // is an empty message initiating a new message.
        // FIXME: dirty hack.  Might this have something to do with replies
        // that have no bodies? 
        if (originalMessageHelper.getDateObject() != null) {
            createReply(originalMessageHelper, messageHelper);
        } else {
            createNewMessage(messageHelper);
        }

        return messageHelper;
    }

    /**
     * Sets the body and other attributes of a new message 
     * based on the attributes of the original message to 
     * which a response is being made.
     */
    private void createReply(EmailMessageEntityHelper originalMessageHelper, 
                             EmailMessageEntityHelper replyHelper)
                      throws DataManagerException {
        // To: copied from original From:
        // Strings attributes are returned as empty strings rather 
        // than null so we have to remember that
        String toAddress = originalMessageHelper.getReplyToAddress();

        if (toAddress.length() == 0) {
            toAddress = originalMessageHelper.getFromAddress();
        }

        if (toAddress.length() > 0) {
            replyHelper.setToAddresses(toAddress);
        }

        replyHelper.setBody(processBody(originalMessageHelper));


        // Subject: generated from original Subject:
        // FIXME: should do some RE checking and that sort of thing
        replyHelper.setSubject(processSubject(
                                       originalMessageHelper.getSubject()));


        // In-Reply-To: copied from original Message-ID:
        replyHelper.setInReplyTo(originalMessageHelper.getMessageId());
    }

    /**
     * Ensures the body and subject of a new message have 
     * reasonable defaults to avoid problems elsewhere.
     */
    private void createNewMessage(EmailMessageEntityHelper messageHelper)
                           throws DataManagerException {
        messageHelper.setSubject("");
        messageHelper.setBody("");
    }

    /**
     * Processes a given subject string to make a proper
     * reply subject. 
     *
     * FIXME: should detect existing re and other things
     * along those lines.
     */
    private String processSubject(String subject) {
        return "RE: " + subject;
    }

    /**
     * Processes the original message body to determine if it is suitable
     * for a reply and create the body text.
     *
     * FIXME: should do some processing, including quoting, and 
     * attribution.
     */
    private String processBody(EmailMessageEntityHelper originalMessageHelper)
                        throws DataManagerException {
        String body = "";
        String contentType = originalMessageHelper.getContentType();
        String attribution = createAttribution(originalMessageHelper);

        if (contentType.equals(USABLE_REPLY_CONTENT_TYPE)) {
            body = attribution + quoteBody(originalMessageHelper.getBody());
        } else {
            body = attribution + "Unusable content type " + contentType + 
                   ".\n";
        }

        return body;
    }

    /**
     * Creates an attribution for this mail message using the date and from 
     * from the original message.
     *
     * FIXME: Should use Date Object and perhaps get wrapped.
     */
    private String createAttribution(EmailMessageEntityHelper originalMessageHelper)
                              throws DataManagerException {
        SimpleDateFormat simpleDate = new SimpleDateFormat(
                                              ATTRIBUTION_DATE_FORMAT);
        String dateString = simpleDate.format(
                                    originalMessageHelper.getDateObject());

        return "On " + dateString + " " + 
               originalMessageHelper.getFromAddress() + " wrote:\n";
    }

    /**
     * Takes a string representing the body of a
     * message and quotes it in traditional email style.
     *
     * FIXME: Doesn't care about line length, quoted printable, or
     * any of that stuff. Maybe that's good?
     */
    private String quoteBody(String body) {
        String quotedBody;

        // FIXME: may be issues with this regexp
        // multiline match with unix mode turned on this 
        // means end of line is only matching on \n. This
        // appears to be okay with email messages and 
        // preserves the \r\n that does exist on the end
        // of the lines.
        Pattern quotedPattern = Pattern.compile("(?md)^");
        Matcher quotedMatcher = quotedPattern.matcher(body);

        quotedBody = quotedMatcher.replaceAll("> ");

        return quotedBody;
    }
}

/*
 *
 * $Log: BasicEmailReplySimpleton.java,v $
 * Revision 1.21  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.20  2002/12/02 05:34:47  alllee
 * GuiSimpleton uses the exported Component's getName() as its title now to allow
 * for specific email reply display frames (which are all exported by a single
 * BasicEmailReplySimpleton) having titles specific to the email message's
 * subject being replied to.  It's hacky, and not well documented, yet.
 * Basically, if you wish to have a GuiSimpleton's exported component publish the
 * title to be used with its JInternalFrame representation within a GuiCollector,
 * you have to setName() the component-to-be-exported to the String title that
 * you wish to use.  This may not be optimal for future developers, but it works
 * for now, which I think is what we are aiming for while concentrating on Jisp
 * and clustering.
 *
 * Changed the class-level reference to the the KernelProxy Singleton in
 * EmailGuiCollectorSimpleton to a private final static variable for no apparent
 * reason other than possible hygiene.
 *
 * Revision 1.19  2002/11/30 11:22:14  alllee
 * Modified these events to reflect changes to fix the bug sitting at
 *
 * http://lazlo.bloomington.in.us:5224/helium/issue73
 *
 * ExportGuiRequestEvent no longer uses getIdentifier() in its toString method
 * because Identifiers were removed from GuiSimpletonInfo-s.  Identifiers ended
 * up becoming useless in the face of how difficult it would be to make them do
 * what they do - uniquely identify Gui Components so that dispose and export
 * requests dispatched by the GuiSimpleton were appropriately handled by the
 * GuiCollector.  This is now being done by passing around a pointer to the
 * original Component being exported and using the Component itself as a key in a
 * hashtable mapping to JInternalFrame-s.  This could become a potential FIXME if
 * it turns out that the native code implementation of hashCode() in Object is
 * causing lotsa collisions in the HashMap, and that those collisions are causing
 * performance problems, but that doesn't seem likely since there aren't that
 * many components being collected by the GuiCollector.  A job for ze Profiler
 * in any case.
 *
 * DisposeGuiRequestEvent no longer carries a reference to a GuiSimpletonInfo --
 * it has a reference to the Component that should be disposed instead so that
 * the EmailGuiCollectorSimpleton can retrieve the JInternalFrame that needs to
 * be disposed from its HashMap of Component keys and JInternalFrame values.
 *
 * The getTitle() method was added to the GuiSimpletonInfo so that the
 * JInternalFrame has some way to figure out a vague idea as to what to call a
 * JInternalFrame to be spawned (getName()/setName() on the Component being
 * passed around is another viable way of doing this).
 *
 * Revision 1.18  2002/11/26 06:48:59  alllee
 * Some more dead-code elimination from EmailConfiguratorSimpleton (removed
 * boxAdd) and fixed the reply bug where a class-level instance variable was
 * getting clobbered in EmailDisplaySimpleton each time a new
 * EntityDisplayRequest was being handled.  The solution to this is to have
 * fresh, lexically scoped email messages being handed out each time a new
 * EntityDisplayRequest comes in.  More refactoring to come w/
 * EmailMessageComponent and friends..
 *
 * Revision 1.17  2002/11/25 22:01:11  cjdent
 *
 * Changed to have slightly more reasonable defaults for the layout of these
 * guys. Not good defaults, just slightly more reasonable.
 *
 * Revision 1.16  2002/11/24 05:56:03  alllee
 * codebase-style/dead-code-elimination and the change to use
 * EmailMessageEntityHelper.EMAIL_ATTRIBUTE_(VALUE|NAME) when searching the pool
 * for email message entities.
 *
 * Revision 1.15  2002/11/23 05:44:10  cjdent
 *
 * Adds the EmailCompositionInitiatorSimpleton and makes the
 * change to BasicEmailReplysimpleton, EmailBundleSimpleton,
 * and EmailMessageEntityHelper to make it usable.
 *
 * Composition is initiate by using the "Componse New" button
 * provided by EmailCompositionInitiatorSimpleton. That
 * simpleton has an empty message Entity that it uses to
 * initiate the email Reply chain used for replies.
 *
 * BasicEmailReplySimpleton has been adjusted to notice when
 * an original message has no DateObject. Under those
 * circumstances it assume that it is creating a new clean
 * message.
 *
 * Some cleanups are requird in the SMTP send simpleton to avoid
 * some bogus headers.
 *
 * Cleanup needed badly in these simpletons.
 *
 * Revision 1.14  2002/11/23 03:50:49  cjdent
 *
 * removing no longer necessary System.err.println calls.
 *
 * Revision 1.13  2002/11/22 03:42:55  alllee
 * EmailConfiguratorSimpleton now fills in its text fields with the appropriate
 * persisted EmailParameters.
 *
 * Revision 1.12  2002/11/22 03:01:51  alllee
 * Oh yeah, that's the repercussion.  Not including the import statement.  Great.
 * Yeah.
 *
 * Revision 1.11  2002/11/22 03:00:00  alllee
 * Made BasicEmailReplySimpleton capable of disposing its export()-ed component.
 * Also changed the double subscribe to the Pool to a single OrEventConstraint
 * (this might have negative repercussions?).
 *
 * Revision 1.10  2002/11/21 10:41:41  alllee
 * Added ability to request that the GuiCollectorSimpleton collecting these
 * GuiSimpletonS dispose() the JInternalFrame windows into which these
 * GuiSimpletonS' export()-ed Components get crammed.
 *
 * Revision 1.9  2002/11/21 07:14:18  cjdent
 *
 * Sets the to of the reply message to the reply-to of the original message, or the
 * from is a reply to does not exist.
 *
 * Also changes panel.getParent().hide() to setVisible(false) to kill some warnings.
 *
 * Revision 1.8  2002/11/21 06:16:08  cjdent
 * full bore codebase style
 *
 * Revision 1.7  2002/11/21 05:43:13  alllee
 * Did I commit this earlier?  For some reason cvs status says that it's still
 * locally modified and cvs diff tells me that a bunch of stuff is still missing.
 * Same changes as earlier applied (exporting JPanel now).
 *
 * Revision 1.6  2002/11/21 03:39:21  cjdent
 * These changes implement quoting and attribution in email replies. These
 * classes are getting long, so will need some refactoring. More notes on this
 * coming.
 *
 * EmailMessageEntityHelper has changed setDate() and getDate() to
 * setDateString() and getDateString(). getDateObject() and setDateObject()
 * have been added to retrieve the the EmailDateObject attribute.
 *
 * Revision 1.5  2002/11/19 06:21:08  cjdent
 *
 * Several changes to faciliate replying.
 *
 * 1) Grants object level knowledge of the BasicEmailMessageComponent so
 *    it can be used from a few different places. (This may not actually
 *    be necessary, I added it and took it away several times while
 *    fiddling)
 * 2) Modified makeSendAction to accept an EmailMessageEntityHelper so
 *    the action has access to the Entity kept by the helper.
 * 3) That action recomposes the entity with the text from the fields and
 *    fires an SMTPSendRequestEvent
 *
 * FIXME rife
 *
 * Revision 1.4  2002/11/19 03:51:11  cjdent
 *
 * BasicEmailMessageComponent:
 *
 * 1) Changed DISPLAY_FONT to HEADER_FONT and added BODY_FONT so we have
 *    a monospace font in the reply.
 * 2) Added getBodyAsText() to go alongside getBodyAsHTML() for retreiving
 *    the content of a reply
 * 3) Did some silly dancing with isEditable() in the creation of the component
 *    to establish if this is a reply or not and choose getBodyAsText() or
 *    getBodyAsHTML() accordingly. There's a FIXME with this because we probably
 *    want to do this in a less hard to find in the code sort of way.
 *
 * BasicEmailReplySimpleton:
 *
 * 1) Added JButtons for Send and Cancel
 * 2) Added a JPanel to hold them in the south position of the border layout
 * 3) Added actions for send and cancel
 *    - Cancel disposes of the window. In the future, presumably, we'll want
 *      a BasicDraftEmailSelectorSimpleton to retrieve the Draft message
 *      Entities that are still out there in the pool.
 *    - Send does nothing but print out a little blurb to stderr. Cogitation
 *      is needed to determine what happens on a send. Presumably we need to
 *      gather all the content from the various fields and reset the values
 *      of the reply entity and then send out an SMTP event of some kind.
 *
 * Revision 1.3  2002/11/19 02:26:00  cjdent
 *
 * BasicEmailReplySimpleton updated to include the subject and the body of the
 * message in the created Reply entity. processSubject() and processBody()
 * were created to get the content of those attriutes on the original message
 * and doctor them to make a reasonable response.
 *
 * No doctoring is currently done. That's coming later.
 *
 * Added setBody() to EmailMessageEntityHelper.
 *
 * Revision 1.2  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.1  2002/11/18 02:38:37  mliggett
 * Added BasicEmailReplySimpleton, which currently only listens for the proper
 * event and pops up totally empty emails.  More functionality to come in just
 * a few.
 *
 * Also added this Simpleton to knownspace_config.xml and
 * EmailBundleSimpleton.
 *
 *
 */
