// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.Rectangle;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.SearchResultsFoundEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;

import org.datamanager.simpleton.TabularEmailDisplay;

import org.datamanager.tools.Debug;


/**
 * Provides a basic interface for viewing email search results.
 *
 */
public class BasicEmailSearchViewerSimpleton extends GuiSimpleton
    implements EventHandler {
    /** This Simpleton's toplevel graphical element. */
    private TabularEmailDisplay tabularEmailDisplay;

    /**
     * Subscribes to the pool to hear SearchResultsFoundEvents.
     */
    public void process() {
        try {
            EventConstraint eventConstraint = new EventTypeIsConstraint(
                                                      SearchResultsFoundEvent.class);
            Pool.getDefaultPool().subscribe(this, eventConstraint);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("error in kernel!");
            exception.printStackTrace();
        }
    }

    /**
     * Adds the email messages (specified inside the SearchResultsEntity)
     * to a TabularEmailDisplay.
     */
    public void handle(DataManagerEvent dataManagerEvent) {
        try {
            Entity searchResult = ((SearchResultsFoundEvent) dataManagerEvent).getEntity();

            tabularEmailDisplay = new TabularEmailDisplay(
                                          searchResult.getValue().toString());

            Entity[] attributes = searchResult.getAttributes(
                                          EntityConstraint.NONE);
            tabularEmailDisplay.setEmailTableModel(attributes);
            super.exportComponent(tabularEmailDisplay.getPanel(), 
                                  new Rectangle(50, 10, 350, 600));
        } catch (DataManagerException dataManagerException) {
            // FIXME: bogus exception handling
            Debug.writeLogMessage("kernel error");
            dataManagerException.printStackTrace();
            throw new RuntimeException(dataManagerException);
        } catch (ClassCastException exception) {
            // FIXME: debug purposes.
            throw new RuntimeException(exception);
        }
    }
}

/*
 * $Log: BasicEmailSearchViewerSimpleton.java,v $
 * Revision 1.13  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.12  2002/12/02 05:34:47  alllee
 * GuiSimpleton uses the exported Component's getName() as its title now to allow
 * for specific email reply display frames (which are all exported by a single
 * BasicEmailReplySimpleton) having titles specific to the email message's
 * subject being replied to.  It's hacky, and not well documented, yet.
 * Basically, if you wish to have a GuiSimpleton's exported component publish the
 * title to be used with its JInternalFrame representation within a GuiCollector,
 * you have to setName() the component-to-be-exported to the String title that
 * you wish to use.  This may not be optimal for future developers, but it works
 * for now, which I think is what we are aiming for while concentrating on Jisp
 * and clustering.
 *
 * Changed the class-level reference to the the KernelProxy Singleton in
 * EmailGuiCollectorSimpleton to a private final static variable for no apparent
 * reason other than possible hygiene.
 *
 * Revision 1.11  2002/11/30 11:22:15  alllee
 * Modified these events to reflect changes to fix the bug sitting at
 *
 * http://lazlo.bloomington.in.us:5224/helium/issue73
 *
 * ExportGuiRequestEvent no longer uses getIdentifier() in its toString method
 * because Identifiers were removed from GuiSimpletonInfo-s.  Identifiers ended
 * up becoming useless in the face of how difficult it would be to make them do
 * what they do - uniquely identify Gui Components so that dispose and export
 * requests dispatched by the GuiSimpleton were appropriately handled by the
 * GuiCollector.  This is now being done by passing around a pointer to the
 * original Component being exported and using the Component itself as a key in a
 * hashtable mapping to JInternalFrame-s.  This could become a potential FIXME if
 * it turns out that the native code implementation of hashCode() in Object is
 * causing lotsa collisions in the HashMap, and that those collisions are causing
 * performance problems, but that doesn't seem likely since there aren't that
 * many components being collected by the GuiCollector.  A job for ze Profiler
 * in any case.
 *
 * DisposeGuiRequestEvent no longer carries a reference to a GuiSimpletonInfo --
 * it has a reference to the Component that should be disposed instead so that
 * the EmailGuiCollectorSimpleton can retrieve the JInternalFrame that needs to
 * be disposed from its HashMap of Component keys and JInternalFrame values.
 *
 * The getTitle() method was added to the GuiSimpletonInfo so that the
 * JInternalFrame has some way to figure out a vague idea as to what to call a
 * JInternalFrame to be spawned (getName()/setName() on the Component being
 * passed around is another viable way of doing this).
 *
 * Revision 1.10  2002/11/26 06:48:59  alllee
 * Some more dead-code elimination from EmailConfiguratorSimpleton (removed
 * boxAdd) and fixed the reply bug where a class-level instance variable was
 * getting clobbered in EmailDisplaySimpleton each time a new
 * EntityDisplayRequest was being handled.  The solution to this is to have
 * fresh, lexically scoped email messages being handed out each time a new
 * EntityDisplayRequest comes in.  More refactoring to come w/
 * EmailMessageComponent and friends..
 *
 * Revision 1.9  2002/11/25 22:01:11  cjdent
 *
 * Changed to have slightly more reasonable defaults for the layout of these
 * guys. Not good defaults, just slightly more reasonable.
 *
 * Revision 1.8  2002/11/25 06:56:13  cjdent
 *
 * codebase style to spark a website-deploy
 *
 * Revision 1.7  2002/11/24 20:35:42  xirao
 *
 *         xi rao
 *         add a little for pre-layout all internal frame ( each component for email application )
 *
 * Revision 1.6  2002/11/21 04:34:55  alllee
 * These GUI components no longer display their own top-level Frame, and instead
 * publish a JPanel() ala GuiSimpleton's vile contract of
 * exportComponent(Component component).  These are currently collected by the
 * EmailGuiCollectorSimpleton's JDesktopPane, which still needs a lot of work.
 * Some residual issues include:
 *
 *     figuring out how to persist layout
 *     figuring out how to properly dispose() of a JPanel (should be easy)
 *     figuring out whatever else Xi&I forgot/neglected to figure out
 *
 * Revision 1.5  2002/11/21 03:29:22  alllee
 * First stab at SimpletonGooey compliance.  Lots of FIXMEs to deal with
 * (duplicate frame/panel functionality for now that needs to be removed and
 * other blougats).
 *
 * Revision 1.4  2002/11/18 02:14:41  mliggett
 * Changed setListModel() to setEmailTableModel().
 *
 * Revision 1.3  2002/11/17 08:46:51  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 * Revision 1.2  2002/11/07 07:52:16  cjdent
 *
 * Adjusted the SearchRequestEvent and the simpletons that use it so that
 * the query string is available to the BasicEmailSearchViewerSimpleton
 * to use in its display. I added the query as an EntityValue on the search results
 * entity that is passed in the SearchRequestEvent. Was going to store it
 * as an attribute but this would have broken some other things.
 *
 * This is not a particularly clean implementation but it does allow the windows
 * to make some more sense. should be cleaned up.
 *
 * Revision 1.1  2002/11/07 05:26:01  cjdent
 *
 * First check in of work to fix the bug mentioned at
 *
 *   http://lazlo.bloomington.in.us:5224/helium/issue10
 *
 * Tabular display of email messages is factored out to the TabularEmailDisplay
 * class that is used by BasicEmailViewerSimpleton and the new
 * BasicEmailSearchViewerSimpleton. These two classes are quite similar but
 * differ in the way they listen to the pool. By splitting them,
 * BasicEmailSearchViewer is able to use multiple TabularEmailDisplayS. These
 * do not collide with one another.
 *
 * See the FIXMEs in setListModel(): it is in the wrong package but we don't
 * have a clear picture of where such things should go. Also setListModel() is
 * cumbersome. It should be possible to just set the data in one lump. In fact
 * I've just noticed, with Matt's help, that there is continued use of Jlist here
 * that can go away.
 *
 * Adjusted the SearchCollectorSimpleton to ensure that BasicEmailSearchViewer is
 * started.
 *
 * Next step: adjusting the SearchRequestEvent so the query string is eventually
 * available for display and removing the Jlist cruft.
 *
 *
 */
