// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.Rectangle;

import javax.swing.JPanel;

import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeStringMatchConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;

import org.datamanager.simpleton.EmailMessageEntityHelper;
import org.datamanager.simpleton.TabularEmailDisplay;

import org.datamanager.tools.Debug;


/**
 * Provides a basic email viewing interface.
 *
 * @author Matt Liggett
 * @version $Revision: 1.27 $
 */
public class BasicEmailViewerSimpleton extends GuiSimpleton
    implements EventHandler {
    /**
     * Title used for the Viewer's display.
     */
    private static final String EMAIL_DISPLAY_FRAME_TITLE = 
            "Email Entities Display";

    /** This Simpleton's toplevel graphical element. */
    private TabularEmailDisplay tabularEmailDisplay;

    /**
     * Builds and displays the GUI, subscribes this Simpleton to events
     * indicating new email messages, and searches the Pool for any existing
     * email messages.
     */
    public void process() {
        tabularEmailDisplay = new TabularEmailDisplay(EMAIL_DISPLAY_FRAME_TITLE);
        super.exportComponent(tabularEmailDisplay.getPanel(), 
                              new Rectangle(0, 375, 345, 300));

        try {
            final EntityConstraint emailConstraint = new AttributeStringMatchConstraint(
                                                             EmailMessageEntityHelper.EMAIL_ATTRIBUTE_NAME, 
                                                             EmailMessageEntityHelper.EMAIL_ATTRIBUTE_VALUE);
            final EventConstraint isEmailAndNewEntityConstraint = 
                    new AndEventConstraint(
                            new EventContainsEntityConstraint(emailConstraint), 
                            new EventTypeIsConstraint(AddEntityPoolEvent.class));
            Pool.getDefaultPool()
                .subscribe(this, isEmailAndNewEntityConstraint);

            // suck in the list of existing email entities
            Entity[] entityArray = Pool.getDefaultPool()
                                       .search(emailConstraint);

            tabularEmailDisplay.setEmailTableModel(entityArray);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("error in kernel!");
            exception.printStackTrace();
        }
    }

    /**
     * Adds a new email message (specified inside the AddEntityPoolEvent) to
     * the list model.
     */
    public void handle(DataManagerEvent dataManagerEvent) {
        tabularEmailDisplay.addToEmailTableModel(
                ((AddEntityPoolEvent) dataManagerEvent).getEntity());
    }
}

/*
 * $Log: BasicEmailViewerSimpleton.java,v $
 * Revision 1.27  2002/12/11 08:39:21  cjdent
 *
 * Adjusted the default layout of the various GuiCollector components and the
 * collector itself to be a larger window with things more tightly
 * packed. For the sake of the demo and also because with touchgraph
 * in there we need more space.
 *
 * Revision 1.26  2002/12/02 05:34:47  alllee
 * GuiSimpleton uses the exported Component's getName() as its title now to allow
 * for specific email reply display frames (which are all exported by a single
 * BasicEmailReplySimpleton) having titles specific to the email message's
 * subject being replied to.  It's hacky, and not well documented, yet.
 * Basically, if you wish to have a GuiSimpleton's exported component publish the
 * title to be used with its JInternalFrame representation within a GuiCollector,
 * you have to setName() the component-to-be-exported to the String title that
 * you wish to use.  This may not be optimal for future developers, but it works
 * for now, which I think is what we are aiming for while concentrating on Jisp
 * and clustering.
 *
 * Changed the class-level reference to the the KernelProxy Singleton in
 * EmailGuiCollectorSimpleton to a private final static variable for no apparent
 * reason other than possible hygiene.
 *
 * Revision 1.25  2002/11/30 11:22:15  alllee
 * Modified these events to reflect changes to fix the bug sitting at
 *
 * http://lazlo.bloomington.in.us:5224/helium/issue73
 *
 * ExportGuiRequestEvent no longer uses getIdentifier() in its toString method
 * because Identifiers were removed from GuiSimpletonInfo-s.  Identifiers ended
 * up becoming useless in the face of how difficult it would be to make them do
 * what they do - uniquely identify Gui Components so that dispose and export
 * requests dispatched by the GuiSimpleton were appropriately handled by the
 * GuiCollector.  This is now being done by passing around a pointer to the
 * original Component being exported and using the Component itself as a key in a
 * hashtable mapping to JInternalFrame-s.  This could become a potential FIXME if
 * it turns out that the native code implementation of hashCode() in Object is
 * causing lotsa collisions in the HashMap, and that those collisions are causing
 * performance problems, but that doesn't seem likely since there aren't that
 * many components being collected by the GuiCollector.  A job for ze Profiler
 * in any case.
 *
 * DisposeGuiRequestEvent no longer carries a reference to a GuiSimpletonInfo --
 * it has a reference to the Component that should be disposed instead so that
 * the EmailGuiCollectorSimpleton can retrieve the JInternalFrame that needs to
 * be disposed from its HashMap of Component keys and JInternalFrame values.
 *
 * The getTitle() method was added to the GuiSimpletonInfo so that the
 * JInternalFrame has some way to figure out a vague idea as to what to call a
 * JInternalFrame to be spawned (getName()/setName() on the Component being
 * passed around is another viable way of doing this).
 *
 * Revision 1.24  2002/11/25 06:56:13  cjdent
 *
 * codebase style to spark a website-deploy
 *
 * Revision 1.23  2002/11/24 20:35:43  xirao
 *
 *         xi rao
 *         add a little for pre-layout all internal frame ( each component for email application )
 *
 * Revision 1.22  2002/11/24 05:56:03  alllee
 * codebase-style/dead-code-elimination and the change to use
 * EmailMessageEntityHelper.EMAIL_ATTRIBUTE_(VALUE|NAME) when searching the pool
 * for email message entities.
 *
 * Revision 1.21  2002/11/24 03:57:17  mliggett
 * Many changes that finally resulted in Suave working properly.  Turns out we
 * were never persisting the objects after creation.  So EntityValues and
 * attributes were missing on restore.
 *
 * - Added toString() back to Entity.  Chris thought this might help us solve
 *   the problem.  It wasn't the problem, but I left it in.
 *
 * - Changed a few comments.
 *
 * - Completely changed the way getInstance() works, so that it would not
 *   always return a new SuaveEntityManager.
 *
 * - Added a call to persistEntity() whenever we remove something from the
 *   KernelMemory.
 *
 * - Codebase-style changed a few other things.
 *
 * Revision 1.20  2002/11/24 01:48:57  xirao
 *         xi rao
 *         Now, the EmailGuiCollectorSimpletion can persist the layout of GUI component.
 *
 * Revision 1.19  2002/11/21 04:34:55  alllee
 * These GUI components no longer display their own top-level Frame, and instead
 * publish a JPanel() ala GuiSimpleton's vile contract of
 * exportComponent(Component component).  These are currently collected by the
 * EmailGuiCollectorSimpleton's JDesktopPane, which still needs a lot of work.
 * Some residual issues include:
 *
 *     figuring out how to persist layout
 *     figuring out how to properly dispose() of a JPanel (should be easy)
 *     figuring out whatever else Xi&I forgot/neglected to figure out
 *
 * Revision 1.18  2002/11/21 03:29:22  alllee
 * First stab at SimpletonGooey compliance.  Lots of FIXMEs to deal with
 * (duplicate frame/panel functionality for now that needs to be removed and
 * other blougats).
 *
 * Revision 1.17  2002/11/18 23:18:48  mliggett
 * Two minor changes to get the build to fire.
 *
 * Revision 1.16  2002/11/18 01:51:40  alllee
 * Minor name change from setListModel to setEmailTableModel.
 *
 * Revision 1.15  2002/11/18 01:47:13  alllee
 * Fixed some FIXMES:
 * - added a public setEmailList(List) method that is used by setListModel
 *   (actually, this introduces another fixme, as it should be
 *   setEmailTableModel... crap, another commit soon.)
 * - renamed addToListModel to addToEmailTableModel.
 * - noted that getHeaderValueAt(int) generates spurious "ERROR!" strings in the
 *   BEVS when emails have an empty subject field (legal).
 * - added lastRow() method to simplify the off-by-one deal with emailList.size()
 *   and fireTableRows(Insert|Updated|Deleted) invocations.
 *
 * Revision 1.14  2002/11/17 08:46:52  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 * Revision 1.13  2002/11/10 07:31:29  cjdent
 *
 * Setting the frame title to a reasonable something so that it can be
 * identified in documentation.
 *
 * Revision 1.12  2002/11/07 05:26:02  cjdent
 *
 * First check in of work to fix the bug mentioned at
 *
 *   http://lazlo.bloomington.in.us:5224/helium/issue10
 *
 * Tabular display of email messages is factored out to the TabularEmailDisplay
 * class that is used by BasicEmailViewerSimpleton and the new
 * BasicEmailSearchViewerSimpleton. These two classes are quite similar but
 * differ in the way they listen to the pool. By splitting them,
 * BasicEmailSearchViewer is able to use multiple TabularEmailDisplayS. These
 * do not collide with one another.
 *
 * See the FIXMEs in setListModel(): it is in the wrong package but we don't
 * have a clear picture of where such things should go. Also setListModel() is
 * cumbersome. It should be possible to just set the data in one lump. In fact
 * I've just noticed, with Matt's help, that there is continued use of Jlist here
 * that can go away.
 *
 * Adjusted the SearchCollectorSimpleton to ensure that BasicEmailSearchViewer is
 * started.
 *
 * Next step: adjusting the SearchRequestEvent so the query string is eventually
 * available for display and removing the Jlist cruft.
 *
 * Revision 1.11  2002/11/05 05:45:43  cjdent
 *
 * Changed SearchHandlerSimpleton so that it does not return SearchResultEntity-s
 * inside a SearchResultEntity. Much discussion as to the why as wherefores of this
 * in the chatlog here
 *
 * http://lazlo.bloomington.in.us/helium/chat/irclog-20021105.html#nid01
 *
 * as well as the previous day.
 *
 * This takes a lot of the cruft out of EmailMetadataSearchSimpleton and
 * BasicEmailViewerSimpleton that was trying to deal with spurious results.
 *
 * BEVS still adds entities to its data model in a funny way, so there is a FIXME
 * for that. Also see chat log for issue on BEVS spawning BEVS here:
 *
 * http://lazlo.bloomington.in.us/helium/chat/irclog-20021104.html#nid0162
 *
 * Revision 1.10  2002/11/04 16:48:55  mafarrel
 * ran ant codebase-style...
 *
 * Revision 1.9  2002/11/04 05:35:33  xirao
 * xi rao
 *
 * Revision 1.8  2002/11/04 05:20:45  xirao
 *         xi rao
 *         1. let BEVS subscribe the SearchResultFoundEvent
 *         2. Handle the SearchResultFoundEvent and build a new frame to show result
 *         ToDo:
 *         get rid of hard code for eliminate the entity have the name "SearchResultsEntity" by using compound constraint
 *
 * Revision 1.7  2002/11/03 22:28:58  mliggett
 * Fixed a few typos:  FOO_ROW is now FOO_COLUMN.
 *
 * Revision 1.6  2002/11/03 03:26:40  cjdent
 *
 * 1) Move the calls to construct DefaultListModel and EmailTableModel to
 *    makeFrame() to deal with problems with two versions of the same simpleton
 *    having the same table and list models. MattF's changes to SimpletonLoaderProcess
 *    will eventually address this, but this fixes it in here for now.
 *
 * 2) Took out the hard coded call to EmailDisplaySimpelton, we are creating a Bundle
 *    Simpleton for email retrieving and viewing.
 *
 * 3) Was in here to fix the problem with the new emails not being viewed in the list
 *    when added to the pool. Turns out this problems was in EmailRetrieverSimpleton.
 *    Explanation in that checkin.
 *
 * Revision 1.5  2002/11/01 04:34:09  xirao
 *         xi rao
 *         add some comment for hard code the links between the BasicEmailViewerSimpleton and EmailDisplaySimpleton
 *
 * Revision 1.4  2002/11/01 03:34:02  xirao
 *         xi rao
 *         add two line in Matt L's constructor. It will automatically load the EmailDisplaySimpleton.
 *         Currently, I aslo still keep the EmailDisplaySimpleton in the knownspace_config.xml for debug purpose.
 *
 * Revision 1.3  2002/10/31 04:42:30  mliggett
 * Made to use a JTable instead of a JList.  The file is riddled with FIXMEs at the
 * moment, but it does its thing.
 *
 * Revision 1.2  2002/10/31 02:04:25  mliggett
 * Now fires an EmailDisplayRequestEvent when you doubleclick on an email message.
 *
 * Revision 1.1  2002/10/29 04:56:51  mliggett
 * Only the framework for the email viewer Simpleton.  Right now it is little more
 * than a clone of EntityMakerSimpleton but with a different constraint.
 *
 */
