// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.SubscribeEventGeneratorEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.Debug;


/**
 * Starts the email application with all the required 
 * simpletons in the order required. Each successive
 * simpleton is only loaded after the first one has
 * subscribed to the Pool.
 *
 * Contains two static arrays: DEPENDENT_SIMPLETONS and 
 * INDEPENDENT_SIMPLETONS. The former are Simpletons
 * that subscribe to the pool. We wait until each
 * has subscribed until we launch the next. 
 * Independent Simpletons do not subscribed to
 * the pool. We launch them after all the others.
 *
 * FIXME: This causes compile time dependencies
 * but we are cool with that for the time being.
 *
 */
public class EmailBundleSimpleton extends Simpleton implements EventHandler {
    /**
     * An ordered list of the simpleton classes we want
     * to run that subscribe to the pool.
     */
    private static final Class[] DEPENDENT_SIMPLETONS = 
            {
        EmailGuiCollectorSimpleton.class, EmailConfiguratorSimpleton.class, 
        EmailParametersRequestHandlerSimpleton.class, 
        EmailRetrieverSimpleton.class, EmailMetadataSearchSimpleton.class, 
        SearchHandlerSimpleton.class, BasicEmailViewerSimpleton.class, 
        BasicEmailSearchViewerSimpleton.class, EmailDisplaySimpleton.class, 
        BasicEmailReplySimpleton.class, BrowserLauncherSimpleton.class, 
        SMTPSendSimpleton.class, TouchgraphEmailDisplaySimpleton.class, 
    };

    /**
     * An ordered list of the simpleton classes we 
     * want to run that do not subscribe to the pool.
     */
    private static final Class[] INDEPENDENT_SIMPLETONS = 
            { TextSearchSimpleton.class, EmailCompositionInitiatorSimpleton.class };

    /** Index to the list of arrays. */
    private int simpletonIndex = 0;

    /** Reference to the Pool. */
    private Pool pool;

    /**
     * Subscribes to the pool to hear SubscribeEventGeneratorEventS
     * and starts the first of the simpletons.
     */
    public void process() {
        subscribe();
        startSimpleton(DEPENDENT_SIMPLETONS, simpletonIndex);
    }

    /** 
     * Starts a simpleton from the simpletons array. If
     * the end of the array has been reached, unsubscribe from
     * the pool and start processing the independent Simpletons.
     */
    private void startSimpleton(Class[] simpletons, int index) {
        if (index < simpletons.length) {
            Class simpletonClass = simpletons[index];
            Simpleton simpleton;

            try {
                simpleton = (Simpleton) simpletonClass.newInstance();
            } catch (InstantiationException exception) {
                // FIXME: bogus exception handling
                Debug.writeLogMessage(
                        "Unable to instantiate a simpleton class: " + 
                        simpletonClass + " " + exception);
                throw new RuntimeException(exception);
            } catch (IllegalAccessException exception) {
                // FIXME: bogus exception handling
                Debug.writeLogMessage(
                        "Illegal Access Exception starting simpleton class: " + 
                        simpletonClass + " " + exception);
                throw new RuntimeException(exception);
            }

            Debug.writeLogMessage("Starting simpleton: " + simpleton);
            simpleton.start();
        } else {
            unsubscribe();
            processIndependentSimpletons();
        }
    }

    /**
     * Processes the independent simpletons.
     */
    private void processIndependentSimpletons() {
        for (int i = 0; i < INDEPENDENT_SIMPLETONS.length; i++) {
            startSimpleton(INDEPENDENT_SIMPLETONS, i);
        }
    }

    /**
     * Subscribes to the pool.
     */
    private void subscribe() {
        try {
            pool = Pool.getDefaultPool();
            pool.subscribe(this, 
                           new EventTypeIsConstraint(
                                   SubscribeEventGeneratorEvent.class));
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Unable to subcribe to pool: " + exception);
            throw new RuntimeException(exception);
        }
    }

    /**
     * Unsubscribes from the pool.
     */
    private void unsubscribe() {
        try {
            pool.unsubscribe(this);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Unable to unsubscribe from pool: " + 
                                  exception);
            throw new RuntimeException(exception);
        }
    }

    /**
     * Handles the SubscribeEventGeneratorEvent. If the
     * event has been caused by the expected simpleton, 
     * start the next one.
     */
    public void handle(DataManagerEvent event) {
        SubscribeEventGeneratorEvent subscribeEvent = (SubscribeEventGeneratorEvent) event;
        Class simpletonClass = subscribeEvent.getEventHandler().getClass();

        try {
            if (simpletonClass.equals(DEPENDENT_SIMPLETONS[simpletonIndex])) {
                startSimpleton(DEPENDENT_SIMPLETONS, ++simpletonIndex);
            }
        } catch (ArrayIndexOutOfBoundsException exception) {
            // we don't care, it just means we're hearing stuff we don't care about.
        }
    }
}