// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.ValueClassIsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailConfigurationRequestEvent;
import org.datamanager.event.EmailParametersEnteredCollectorEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.EmailParametersEntityValue;
import org.datamanager.passiveentityvalue.PasswordEntityValue;

import org.datamanager.tools.Debug;
import org.datamanager.tools.Toolbox;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Basic GUI Email configuration implementation that queries the user for
 * their email store username, password, host, folder, and access protocol
 * information.
 *
 * @author Team Helium
 * @version $Revision: 1.26 $
 */
public class EmailConfiguratorSimpleton extends GuiSimpleton
    implements EventHandler {
    private final static String HELP_URL_STRING = "http://helium.knownspace.org/user/manual.html#configuration";
    private final static int DEFAULT_NUM_COLS = 20;
    private final JPanel queryPanel = new JPanel(new BorderLayout());
    private JTextField usernameField;
    private JPasswordField passwordField;
    private JTextField hostMachineField;
    private JComboBox accessProtocolNameField;
    private JTextField mailboxNameField;
    private JTextField fromAddressField;
    private JTextField SMTPHostField;
    private JButton confirmButton;
    private JButton helpButton;
    private JButton configureButton;

    /**
     * Reference to the EmailParameters Entity that
     * contains configuration information.
     */
    Entity emailParametersEntity = null;

    public void process() {
        attachToPool();
        initializeGuiComponents();
    }

    private void attachToPool() {
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new EventTypeIsConstraint(
                                   EmailConfigurationRequestEvent.class));
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("unable to subscribe to Pool: " + 
                                  exception);
            exception.printStackTrace();

            // FIXME: Debug only
            throw new RuntimeException(exception);
        }
    }

    public void handle(DataManagerEvent event) {
        // trusting events to be of the appropriate type.
        promptUserForEmailParameters();
    }

    // FIXME: um, this is a bit ugly.  Thought about subscribing this
    // Simpleton to EmailParametersEnteredCollectorEventS, but those are also
    // generated by this Simpleton, and it'd end up handle()-ing itself.
    // Although some might not think that is a bad idea, me didn't think too
    // highly of it at the time.  So it ends up that the
    // EmailParametersRequestHandler AND this guy both search the pool looking
    // for the same thing.  Ah well.  Redundancy is consistency.
    private final void populateTextFields() {
        try {
            final Entity[] entities = Pool.getDefaultPool()
                                          .search(new ValueClassIsConstraint(
                                                          EmailParametersEntityValue.class));

            if (entities.length != 0) {
                emailParametersEntity = entities[0];

                EmailParametersEntityValue emailParametersEntityValue = 
                        (EmailParametersEntityValue) emailParametersEntity.getValue();
                usernameField.setText(emailParametersEntityValue.getUsername());
                passwordField.setText(emailParametersEntityValue.getPasswordEntityValue()
                                                                .getPassword());
                hostMachineField.setText(
                        emailParametersEntityValue.getHostMachineName());
                accessProtocolNameField.setSelectedItem(
                        emailParametersEntityValue.getAccessProtocolName());
                mailboxNameField.setText(
                        emailParametersEntityValue.getMailboxName());
                fromAddressField.setText(
                        emailParametersEntityValue.getFromAddress());
                SMTPHostField.setText(emailParametersEntityValue.getSMTPHost());
            }
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            throw new RuntimeException(exception);
        }
    }

    private void initializeGuiComponents() {
        usernameField = new JTextField(DEFAULT_NUM_COLS);
        passwordField = new JPasswordField(DEFAULT_NUM_COLS);
        hostMachineField = new JTextField(DEFAULT_NUM_COLS);
        accessProtocolNameField = new JComboBox(
                                          AccessProtocolNames.PROTOCOL_NAMES);
        mailboxNameField = new JTextField(DEFAULT_NUM_COLS);
        fromAddressField = new JTextField(DEFAULT_NUM_COLS);
        SMTPHostField = new JTextField(DEFAULT_NUM_COLS);

        populateTextFields();

        confirmButton = new JButton("Ok");
        configureButton = new JButton("Configure");
        helpButton = new JButton("Help");


        // Add event handling to the buttons.
        configureButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                queryPanel.setVisible(true);
            }
        });

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                parseEmailParameters();
                queryPanel.setVisible(false);
            }
        });

        helpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                Toolbox.launchURL(HELP_URL_STRING);
            }
        });

        JPanel configurePanel = new JPanel(new BorderLayout());
        JPanel centerPanel = new JPanel(new GridBagLayout());
        JPanel buttonPanel = new JPanel(new FlowLayout());
        configurePanel.add(centerPanel, BorderLayout.CENTER);
        configurePanel.add(buttonPanel, BorderLayout.SOUTH);
        configurePanel.setName("Email Configuration Options");

        buttonPanel.add(confirmButton);
        buttonPanel.add(helpButton);

        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        LayoutHelper layoutHelper = new LayoutHelper(gridBagConstraints, 
                                                     centerPanel);
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);

        layoutHelper.add(new JLabel("Username: "), GridBagConstraints.WEST, 0, 0, 
                         0.0, 1);
        layoutHelper.add(usernameField, GridBagConstraints.BOTH, 1, 0, 1.0, 3);
        layoutHelper.add(new JLabel("Password: "), GridBagConstraints.WEST, 0, 1, 
                         0.0, 1);
        layoutHelper.add(passwordField, GridBagConstraints.BOTH, 1, 1, 1.0, 3);
        layoutHelper.add(new JLabel("Host machine: "), GridBagConstraints.WEST, 0, 2, 
                         0.0, 1);
        layoutHelper.add(hostMachineField, GridBagConstraints.BOTH, 1, 2, 1.0, 3);
        layoutHelper.add(new JLabel("Access protocol: "), 
                         GridBagConstraints.WEST, 0, 3, 0.0, 1);
        layoutHelper.add(accessProtocolNameField, GridBagConstraints.BOTH, 1, 3, 
                         1.0, 1);
        layoutHelper.add(new JLabel("Mailbox name: "), GridBagConstraints.WEST, 0, 4, 
                         0.0, 1);
        layoutHelper.add(mailboxNameField, GridBagConstraints.BOTH, 1, 4, 1.0, 3);
        layoutHelper.add(new JLabel("Email address: "), GridBagConstraints.WEST, 0, 5, 
                         0.0, 1);
        layoutHelper.add(fromAddressField, GridBagConstraints.BOTH, 1, 5, 1.0, 3);
        layoutHelper.add(new JLabel("SMTP Host: "), GridBagConstraints.WEST, 0, 6, 
                         0.0, 1);
        layoutHelper.add(SMTPHostField, GridBagConstraints.BOTH, 1, 6, 1.0, 3);
        super.exportComponent(configurePanel, new Rectangle(0, 0, 345, 250));
    }

    /**
     * Queries user for Email Parameters (via JTextFields) to use when
     * connecting to a remote email inbox.
     */
    private void promptUserForEmailParameters() {
        queryPanel.setVisible(true);
    }

    /**
      * Parses the GUI input parameters and notifies the Pool.
      */
    private void parseEmailParameters() {
        // FIXME:  validate user information (incomplete information gets a
        // dialog box or something of the sort).
        final String username = usernameField.getText();
        final String password = new String(passwordField.getPassword());
        final String hostMachine = hostMachineField.getText();
        final String accessProtocolName = (String) accessProtocolNameField.getSelectedItem();
        final String mailboxName = mailboxNameField.getText();
        final String fromAddress = fromAddressField.getText();
        final String SMTPHost = SMTPHostField.getText();

        KernelProxy kernelProxy = KernelProxy.getInstance();

        if (emailParametersEntity == null) {
            emailParametersEntity = kernelProxy.createEntity(
                                            "EmailParametersEntity");
        }

        kernelProxy.setEntityValue(emailParametersEntity, 
                                   new EmailParametersEntityValue(username, 
                                                                  new PasswordEntityValue(
                                                                          password), 
                                                                  hostMachine, 
                                                                  accessProtocolName, 
                                                                  mailboxName, 
                                                                  fromAddress, 
                                                                  SMTPHost));

        try {
            kernelProxy.addEvent(
                    new EmailParametersEnteredCollectorEvent(
                            kernelProxy.getEventGenerator(), 
                            emailParametersEntity));
            kernelProxy.addEntity(emailParametersEntity);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage(
                    "Unable to store email configuration paramaters: " + 
                    exception);
            throw new RuntimeException(exception);
        }
    }

    // Private inner class for the access protocol names combo box.
    private static final class AccessProtocolNames {
        static final String[] PROTOCOL_NAMES = { "imap", "imaps" };
    }
} // EmailConfiguratorSimpleton
/*
 * $Log: EmailConfiguratorSimpleton.java,v $
 * Revision 1.26  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.25  2002/12/19 03:13:30  mliggett
 * codebase-style
 *
 * Revision 1.24  2002/12/18 06:07:05  cjdent
 *
 * Adds a launchURL helper method that fires the LaunchBrowserWithURLEvent
 *
 * Adjusted TextSearchSimpleton and EmailConfiguratorSimpleton to use
 * this for their Help button actions. Got rid of the Cancel buttons as
 * they are not used.
 *
 * Each Simpleton has, for the time being, a HELP_URL_STRING that for the
 * time being points to some stuff that I'm writing but is not yet
 * committed.
 *
 * Revision 1.23  2002/12/11 08:39:22  cjdent
 *
 * Adjusted the default layout of the various GuiCollector components and the
 * collector itself to be a larger window with things more tightly
 * packed. For the sake of the demo and also because with touchgraph
 * in there we need more space.
 *
 * Revision 1.22  2002/12/02 06:24:44  alllee
 * Removed spurious getTitle() declarations and some codebase-style..
 *
 * Revision 1.21  2002/11/30 11:22:15  alllee
 * Modified these events to reflect changes to fix the bug sitting at
 *
 * http://lazlo.bloomington.in.us:5224/helium/issue73
 *
 * ExportGuiRequestEvent no longer uses getIdentifier() in its toString method
 * because Identifiers were removed from GuiSimpletonInfo-s.  Identifiers ended
 * up becoming useless in the face of how difficult it would be to make them do
 * what they do - uniquely identify Gui Components so that dispose and export
 * requests dispatched by the GuiSimpleton were appropriately handled by the
 * GuiCollector.  This is now being done by passing around a pointer to the
 * original Component being exported and using the Component itself as a key in a
 * hashtable mapping to JInternalFrame-s.  This could become a potential FIXME if
 * it turns out that the native code implementation of hashCode() in Object is
 * causing lotsa collisions in the HashMap, and that those collisions are causing
 * performance problems, but that doesn't seem likely since there aren't that
 * many components being collected by the GuiCollector.  A job for ze Profiler
 * in any case.
 *
 * DisposeGuiRequestEvent no longer carries a reference to a GuiSimpletonInfo --
 * it has a reference to the Component that should be disposed instead so that
 * the EmailGuiCollectorSimpleton can retrieve the JInternalFrame that needs to
 * be disposed from its HashMap of Component keys and JInternalFrame values.
 *
 * The getTitle() method was added to the GuiSimpletonInfo so that the
 * JInternalFrame has some way to figure out a vague idea as to what to call a
 * JInternalFrame to be spawned (getName()/setName() on the Component being
 * passed around is another viable way of doing this).
 *
 * Revision 1.20  2002/11/26 06:48:59  alllee
 * Some more dead-code elimination from EmailConfiguratorSimpleton (removed
 * boxAdd) and fixed the reply bug where a class-level instance variable was
 * getting clobbered in EmailDisplaySimpleton each time a new
 * EntityDisplayRequest was being handled.  The solution to this is to have
 * fresh, lexically scoped email messages being handed out each time a new
 * EntityDisplayRequest comes in.  More refactoring to come w/
 * EmailMessageComponent and friends..
 *
 * Revision 1.19  2002/11/24 08:49:49  cjdent
 * codebase style
 *
 * Revision 1.18  2002/11/24 08:31:07  cjdent
 *
 * Addressing:
 *
 *   http://lazlo.bloomington.in.us:5224/helium/issue75
 *
 * Email configuration information was being saved multiple times
 * to new entities, but only the first entity was every being used.
 *
 * I changed things so that only one Entity is kept around in this
 * Simpleton's head and if it knows about it, instead of creating
 * another, it just changes the value on the one it knows about.
 *
 * Seems to work correctly.
 *
 * Revision 1.17  2002/11/24 05:56:11  alllee
 * codebase-style/dead-code-elimination and the change to use
 * EmailMessageEntityHelper.EMAIL_ATTRIBUTE_(VALUE|NAME) when searching the pool
 * for email message entities.
 *
 * Revision 1.16  2002/11/24 05:14:23  xirao
 *         xi rao
 *         rebuild EmailConfiguratorSimpletion layout .
 *
 * Revision 1.13  2002/11/24 03:45:36  xirao
 * xi rao
 * add some capability to let Gui simpletion can slef define the size/position at first time
 *
 * Revision 1.12  2002/11/23 04:33:49  cjdent
 *
 * Changed EmailConfiguratorSimpleton to use KernelProxy instead of a
 * simpleton that it creates itself for dealing with spurious threads.
 *
 * Fixed a meaningful comment in KernelProxy
 *
 * Revision 1.11  2002/11/22 03:42:55  alllee
 * EmailConfiguratorSimpleton now fills in its text fields with the appropriate
 * persisted EmailParameters.
 *
 * Revision 1.10  2002/11/21 08:28:24  alllee
 * The configure button no longer disappears with the rest of the components when
 * cancel/ok is clicked.
 *
 * Revision 1.9  2002/11/21 04:34:55  alllee
 * These GUI components no longer display their own top-level Frame, and instead
 * publish a JPanel() ala GuiSimpleton's vile contract of
 * exportComponent(Component component).  These are currently collected by the
 * EmailGuiCollectorSimpleton's JDesktopPane, which still needs a lot of work.
 * Some residual issues include:
 *
 *     figuring out how to persist layout
 *     figuring out how to properly dispose() of a JPanel (should be easy)
 *     figuring out whatever else Xi&I forgot/neglected to figure out
 *
 * Revision 1.8  2002/11/20 00:25:41  cjdent
 *
 *
 * EmailParametersEntityValue and EmailParameters
 *
 * Addition of SMTPHost field for the outgoing SMTP server
 * Updated the equals method in EmailParametersEntityValue to include the
 *   smtp and from address info. Was this right?
 *
 * Added an SMTP host to the EmailConfiguratorSimpleton
 *
 * SMTPSendSimpleton
 *  subscribes to the EmailParametersEnteredCollectorEvent and fires the
 *  EmailParametersRequestEvent to get SMTP host information.
 *
 *  FIXME: What happens if it is unable to get the info for some reason when
 *  it tries to make a delivery. I've set up a default, which is _not_ a good
 *  long term solution. This is integrated with the overall question of
 *  how to deal with SMTP errors.
 *
 * Rest is more bogus codebase-style
 *
 * Revision 1.7  2002/11/19 02:37:33  cjdent
 *
 * Changed a setSize() to pack() for the queryFrame so that the email
 * address text field doesn't disapper underneath the lower edge of the
 * window.
 *
 * Revision 1.6  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.5  2002/11/16 06:26:08  alllee
 * Fixed class-level javadocs and other comments/documentation.  Hygiene cleanup.
 * Still need to add more detailed eventflow description.
 *
 */
