// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeStringMatchConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailMetadataSearchEvent;
import org.datamanager.event.EmailReplyRequestEvent;
import org.datamanager.event.EntityDisplayRequestEvent;
import org.datamanager.event.LaunchBrowserWithURLEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.simpleton.EmailMessageEntityHelper;

import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Provides a GUI to view one single email. Supports text/plain content-type at
 * this time.
 *
 *
 * @author Team Helium
 */
public class EmailDisplaySimpleton extends GuiSimpleton
    implements EventHandler {
    /**
     * Runs this Simpleton. Subscribes to the Pool listening
     * for EntityDisplayRequestEvents that contain email messages.
     *
     * FIXME: We need to deal with exception handling in a better fashion.
     */
    public void process() {
        try {
            // Filter out all Events that aren't EntityDisplayRequestEvents
            final EventConstraint entityDisplayConstraint = 
                    new EventTypeIsConstraint(EntityDisplayRequestEvent.class);

            // Filter out all Events that don't contain an attribute with the
            // specified EMAIL_ATTRIBUTE_VALUE and EMAIL_ATTRIBUTE_NAME
            final EventConstraint emailConstraint = new EventContainsEntityConstraint(
                                                            new AttributeStringMatchConstraint(
                                                                    EmailMessageEntityHelper.EMAIL_ATTRIBUTE_NAME, 
                                                                    EmailMessageEntityHelper.EMAIL_ATTRIBUTE_VALUE));
            final EventConstraint isEmailAndNewEntityConstraint = 
                    new AndEventConstraint(entityDisplayConstraint, 
                                           emailConstraint);

            Pool.getDefaultPool()
                .subscribe(this, isEmailAndNewEntityConstraint);
        } catch (DataManagerException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        }
    }

    /**
     * The BasicEmailViewerSimpleton generates EntityDisplayRequestEvents with
     * the appropriate Email entities.  This method handles the aforementioned
     * events by displaying the email message in a new window.
     *
     * FIXME: We need to deal with exception handling in a better fashion.
     */
    public void handle(DataManagerEvent dataManagerEvent) {
        try {
            final Entity message = ((EntityDisplayRequestEvent) dataManagerEvent).getEntity();
            final EmailMessageEntityHelper messageHelper = 
                    new EmailMessageEntityHelper(message);
            makePanel(messageHelper);
        } catch (DataManagerException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        } catch (ClassCastException exception) {
            // FIXME: Usual bogus exception handling.
            throw new RuntimeException(exception);
        }
    }

    /**
     * Adds MouseListeners to the Components gotten from the
     * EmailMessageComponent to fire off metadata searches.
     */
    private void addListeners(EmailMessageComponent emailComponent, 
                              EmailMessageEntityHelper messageHelper)
                       throws DataManagerException {
        emailComponent.getFromComponent()
              .addMouseListener(createMouseAdapter(
                                        EmailMessageEntityHelper.MAIL_FROM, 
                                        messageHelper.getFromAddress()));
        emailComponent.getToComponent()
              .addMouseListener(createMouseAdapter(
                                        EmailMessageEntityHelper.MAIL_TO, 
                                        messageHelper.getToAddresses()));
        emailComponent.getCcComponent()
              .addMouseListener(createMouseAdapter(
                                        EmailMessageEntityHelper.MAIL_CC, 
                                        messageHelper.getCcAddresses()));
        emailComponent.getDateComponent()
              .addMouseListener(createMouseAdapter(
                                        EmailMessageEntityHelper.MAIL_DATE, 
                                        messageHelper.getDateString()));
        emailComponent.getSubjectComponent()
              .addMouseListener(createMouseAdapter(
                                        EmailMessageEntityHelper.MAIL_SUBJECT, 
                                        messageHelper.getSubject()));
    }

    /**
     * Makes the GUI frame for displaying a single message.
     */
    private void makePanel(final EmailMessageEntityHelper messageHelper)
                    throws DataManagerException {
        JPanel panel = new JPanel(new BorderLayout());


        // Currently, the name of the exported panel becomes the Title of the
        // JInternalFrame that the EmailGuiCollectorSimpleton creates for this
        // exported component.
        panel.setName(messageHelper.getSubject());

        EmailMessageComponent emailComponent = new BasicEmailMessageComponent(
                                                       messageHelper.getMessage(), 
                                                       false);

        panel.add(emailComponent.getComponent(), BorderLayout.CENTER);
        panel.add(new JButton(makeReplyAction(messageHelper)), 
                  BorderLayout.SOUTH);

        addListeners(emailComponent, messageHelper);

        super.exportComponent(panel, new Rectangle(0, 0, 675, 500));
    }

    /**
     * Creates an Action that fires an EmailReplyRequestEvent pointing to the
     * email message displayed by this EmailDisplaySimpleton.
     */
    private Action makeReplyAction(final EmailMessageEntityHelper messageHelper) {
        return new AbstractAction("Reply") {
            public void actionPerformed(ActionEvent event) {
                try {
                    KernelProxy.getInstance()
                               .addEvent(new EmailReplyRequestEvent(
                                                 KernelProxy.getInstance()
                                                            .getEventGenerator(), 
                                                 messageHelper.getMessage()));
                } catch (DataManagerException exception) {
                    // FIXME: usual bogus exception handling
                    throw new RuntimeException(exception);
                }
            }
        };
    }

    /**
     * Creates a MouseAdapter for use with a double click on a text field.
     */
    private MouseAdapter createMouseAdapter(final String header, 
                                            final String content) {
        return new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    fireMetadataSearchEvent(header, content);
                }
            }
        };
    }

    /**
     * Fires the MetadataSearchEvent with the proper attribute
     * name and attribute contents.
     */
    private void fireMetadataSearchEvent(String header, String content) {
        try {
            // FIXME:  using KernelProxy to deal with AWT thread adding a new
            // event.
            KernelProxy.getInstance()
                       .addEvent(new EmailMetadataSearchEvent(
                                         KernelProxy.getInstance()
                                                    .getEventGenerator(), 
                                         header, content));
        } catch (DataManagerException exception) {
            // FIXME: we need a better mechanism for dealing with these
            throw new RuntimeException(exception);
        }
    }
}

/*
 * $Log: EmailDisplaySimpleton.java,v $
 * Revision 1.40  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.39  2002/12/02 06:24:44  alllee
 * Removed spurious getTitle() declarations and some codebase-style..
 *
 * Revision 1.38  2002/11/30 11:22:15  alllee
 * Modified these events to reflect changes to fix the bug sitting at
 *
 * http://lazlo.bloomington.in.us:5224/helium/issue73
 *
 * ExportGuiRequestEvent no longer uses getIdentifier() in its toString method
 * because Identifiers were removed from GuiSimpletonInfo-s.  Identifiers ended
 * up becoming useless in the face of how difficult it would be to make them do
 * what they do - uniquely identify Gui Components so that dispose and export
 * requests dispatched by the GuiSimpleton were appropriately handled by the
 * GuiCollector.  This is now being done by passing around a pointer to the
 * original Component being exported and using the Component itself as a key in a
 * hashtable mapping to JInternalFrame-s.  This could become a potential FIXME if
 * it turns out that the native code implementation of hashCode() in Object is
 * causing lotsa collisions in the HashMap, and that those collisions are causing
 * performance problems, but that doesn't seem likely since there aren't that
 * many components being collected by the GuiCollector.  A job for ze Profiler
 * in any case.
 *
 * DisposeGuiRequestEvent no longer carries a reference to a GuiSimpletonInfo --
 * it has a reference to the Component that should be disposed instead so that
 * the EmailGuiCollectorSimpleton can retrieve the JInternalFrame that needs to
 * be disposed from its HashMap of Component keys and JInternalFrame values.
 *
 * The getTitle() method was added to the GuiSimpletonInfo so that the
 * JInternalFrame has some way to figure out a vague idea as to what to call a
 * JInternalFrame to be spawned (getName()/setName() on the Component being
 * passed around is another viable way of doing this).
 *
 * Revision 1.37  2002/11/26 06:48:59  alllee
 * Some more dead-code elimination from EmailConfiguratorSimpleton (removed
 * boxAdd) and fixed the reply bug where a class-level instance variable was
 * getting clobbered in EmailDisplaySimpleton each time a new
 * EntityDisplayRequest was being handled.  The solution to this is to have
 * fresh, lexically scoped email messages being handed out each time a new
 * EntityDisplayRequest comes in.  More refactoring to come w/
 * EmailMessageComponent and friends..
 *
 */
