// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.OrEventConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.DisposeGuiRequestEvent;
import org.datamanager.event.ExportGuiRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpletonInfo;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.GuiLayoutEntityValue;

import org.datamanager.tools.Debug;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Initial stab at EmailGuiCollectorSimpleton.  Work in progress.
 *
 * @author Team Helium
 * @version $Revision: 1.26 $
 */
public class EmailGuiCollectorSimpleton extends Simpleton
    implements EventHandler {
    private final static String LAYOUT_ENTITY_NAME = "GuiLayout";
    private final static String COMPONENT_DEFAULT_NAME = "default";
    private final static Rectangle DEFAULT_BOUND = new Rectangle(0, 0, 200, 200);
    private final static KernelProxy KERNEL_PROXY = KernelProxy.getInstance();
    private JFrame topLevelFrame;
    private JDesktopPane desktopPane;
    private final Map componentMap = new HashMap();
    private final Map layoutMap = new HashMap();

    public void process() {
        makeFrame();
        getPreviousLayout();
        attachToPool();
    }

    public void getPreviousLayout() {
        EntityConstraint searchQuery = new NameEqualsConstraint(
                                               LAYOUT_ENTITY_NAME);
        Entity[] layoutEntities;

        try {
            layoutEntities = Pool.getDefaultPool().search(searchQuery);

            for (int i = 0; i < layoutEntities.length; i++) {
                Entity layoutEntity = layoutEntities[i];
                GuiLayoutEntityValue guiLayoutValue = (GuiLayoutEntityValue) layoutEntity.getValue();

                if (!guiLayoutValue.getName()
                                   .equalsIgnoreCase(COMPONENT_DEFAULT_NAME)) {
                    layoutMap.put(guiLayoutValue.getName(), layoutEntity);
                }
            }
        } catch (ClassCastException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage(
                    "Incorrect event type handled by EmailGuiCollectorSimpleton");
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage("DataManagerException " + 
                                  exception.getMessage() + 
                                  " generated while handling SearchResults in EmailGuiCollectorSimpleton");
        }
    }

    public void handle(final DataManagerEvent event) {
        if (event instanceof ExportGuiRequestEvent) {
            handleExportGuiRequest((ExportGuiRequestEvent) event);
        } else if (event instanceof DisposeGuiRequestEvent) {
            handleDisposeGuiRequest((DisposeGuiRequestEvent) event);
        }
    }

    private final void handleDisposeGuiRequest(final DisposeGuiRequestEvent event) {
        Component componentToBeDisposed = ((DisposeGuiRequestEvent) event).getComponentToBeDisposed();

        if (componentMap.containsKey(componentToBeDisposed)) {
            ((JInternalFrame) componentMap.get(componentToBeDisposed)).dispose();
        }

        // FIXME: is ignoring the request if we don't have the GuiSimpleton's
        // exported component in our componentMap a good thing?
    }

    private synchronized final void handleExportGuiRequest(final ExportGuiRequestEvent event) {
        GuiSimpletonInfo guiSimpletonInfo = ((ExportGuiRequestEvent) event).getGuiSimpletonInfo();

        Component newComponent = guiSimpletonInfo.getComponent();

        JInternalFrame internalFrame = new JInternalFrame(
                                               newComponent.getName(), true, // resizable
                                               true, // closable
                                               true, // maximizable
                                               true // iconifiable
                                              );

        componentMap.put(guiSimpletonInfo.getComponent(), internalFrame);
        internalFrame.getContentPane().add(newComponent);

        // FIXME: use something other than the published title of the Gui
        // Simpleton to distinguish between specific layouts..  Actually this
        // might not be a bad thing -- specifically named gui components are
        // placed in specific places, generically named gui components (like
        // new composition initiator) are placed in the same place always..
        if (layoutMap.containsKey(guiSimpletonInfo.getTitle())) {
            Entity layoutEntity = (Entity) layoutMap.get(
                                          guiSimpletonInfo.getTitle());
            GuiLayoutEntityValue layoutValue = (GuiLayoutEntityValue) KERNEL_PROXY.getEntityValue(
                                                       layoutEntity);
            internalFrame.setBounds(layoutValue.getBound());
        } else {
            createDefaultLayout(guiSimpletonInfo, internalFrame);
        }

        internalFrame.setTitle(guiSimpletonInfo.getTitle());
        desktopPane.add(internalFrame);
        internalFrame.show();
    }

    /**
     * create a Entity which has a GuiLayoutEntityValue. The GuiLayoutEntityValue contain the default layout
     * keep the entity in the hash table, so that we can change its entity value at any time by look up
     * the table. The JInternalFrame also will be set the default layout etc.
     */
    private void createDefaultLayout(final GuiSimpletonInfo guiSimpletonInfo, 
                                     final JInternalFrame internalFrame) {
        Rectangle preferredBound = guiSimpletonInfo.getPreferredBound();

        if (null == preferredBound) {
            preferredBound = DEFAULT_BOUND;
        }

        GuiLayoutEntityValue layoutValue = new GuiLayoutEntityValue(
                                                   preferredBound, 
                                                   COMPONENT_DEFAULT_NAME);

        try {
            Entity entity = Entity.create(LAYOUT_ENTITY_NAME);
            KERNEL_PROXY.setEntityValue(entity, layoutValue);
            KERNEL_PROXY.addEntity(entity);
            layoutMap.put(guiSimpletonInfo.getTitle(), entity);
        } catch (DataManagerException exception) {
            exception.printStackTrace();
        }

        internalFrame.setBounds(layoutValue.getBound());
    }

    private final void attachToPool() {
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new OrEventConstraint(
                                   new EventTypeIsConstraint(
                                           ExportGuiRequestEvent.class), 
                                   new EventTypeIsConstraint(
                                           DisposeGuiRequestEvent.class)));
        } catch (DataManagerException exception) {
            // FIXME: usual reasons
            exception.printStackTrace();
            throw new RuntimeException(exception);
        }
    }

    private final void makeFrame() {
        topLevelFrame = new JFrame("Helium Email Application");
        desktopPane = new JDesktopPane();
        desktopPane.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

        JPanel buttonPanel = new JPanel(new FlowLayout());

        final JButton cascade = new JButton("Cascade");
        cascade.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                if (evt.getClickCount() == 1) {
                    cascadeWindows();
                }
            }
        });
        buttonPanel.add(cascade);

        final JButton savePersistence = new JButton("Save Interface");
        savePersistence.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                if (evt.getClickCount() == 1) {
                    persistInterface();
                }
            }
        });
        buttonPanel.add(savePersistence);

        topLevelFrame.getContentPane().add(desktopPane, BorderLayout.CENTER);
        topLevelFrame.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        topLevelFrame.setSize(1000, 750);
        topLevelFrame.setVisible(true);
    }

    private void persistInterface() {
        JInternalFrame[] frames = desktopPane.getAllFrames();

        for (int i = 0; i < frames.length; i++) {
            JInternalFrame internalFrame = frames[i];
            String componentTitle = internalFrame.getTitle();
            Rectangle newBound = internalFrame.getBounds();
            Entity entity = (Entity) layoutMap.get(componentTitle);
            GuiLayoutEntityValue layoutValue = (GuiLayoutEntityValue) KERNEL_PROXY.getEntityValue(
                                                       entity);
            layoutValue.setName(componentTitle);
            layoutValue.setBound(newBound);
        }
    }

    // liberally stolen from the web
    private void cascadeWindows() {
        JInternalFrame[] frames = desktopPane.getAllFrames();
        int x = 0;
        int y = 0;
        int frameDistance = 20; // 20 pixels between frames

        // count frames that aren't iconized
        int frameCount = 0;

        for (int i = 0; i < frames.length; i++) {
            if (!frames[i].isIcon()) {
                frameCount++;
            }
        }

        // width and height of instances which will fill available space
        int width = desktopPane.getWidth() - 
                    ((frameCount - 1) * frameDistance);
        int height = desktopPane.getHeight() - 
                     ((frameCount - 1) * frameDistance);

        for (int i = 0; i < frames.length; i++) {
            if (!frames[i].isIcon()) {
                try {
                    frames[i].setMaximum(false);
                    frames[i].reshape(x, y, width, height);
                    x += frameDistance;
                    y += frameDistance;

                    // wrap around at the desktop edge
                    if ((x + width) > desktopPane.getWidth()) {
                        x = 0;
                    }

                    if ((y + height) > desktopPane.getHeight()) {
                        y = 0;
                    }
                } catch (PropertyVetoException e) {
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        }
    }
}