// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;

import org.datamanager.passiveentityvalue.DateEntityValue;
import org.datamanager.passiveentityvalue.StringEntityValue;

import org.datamanager.tools.Toolbox;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Provides a convenient way to access the attributes of an email message represented
 * by an Entity.
 *
 * @author Team Helium
 * @version $Revision: 1.16 $
 */
public class EmailMessageEntityHelper {
    /**
     * Attribute name which, along with EMAIL_ATTRIBUTE_VALUE, we use to
     * identify Entities which are email.
     */
    public static final String EMAIL_ATTRIBUTE_NAME = "content-source";

    /**
     * Attribute name which, along with EMAIL_ATTRIBUTE_NAME, we use to
     * identify Entities which are email.
     */
    public static final String EMAIL_ATTRIBUTE_VALUE = "email";

    // Mail headers

    /** String value of the from mail header. */
    public static final String MAIL_FROM = "From";

    /** String value of the to mail header. */
    public static final String MAIL_TO = "To";

    /** String value of the cc mail header. */
    public static final String MAIL_CC = "Cc";

    /** String value of the reply-to mail header. */
    public static final String MAIL_REPLYTO = "Reply-To";

    /** String value of the date mail header. */
    public static final String MAIL_DATE = "Date";

    /** String value of the subject mail header. */
    public static final String MAIL_SUBJECT = "Subject";

    /** String value of the message-id mail header. */
    public static final String MAIL_MESSAGE_ID = "Message-ID";

    /** String value of the in-reply-to mail header. */
    public static final String MAIL_IN_REPLY_TO = "In-Reply-To";

    /** String value of the content-type mail header. */
    public static final String MAIL_CONTENT_TYPE = "Content-Type";

    /** String name of the DateEntityValue associated with this message. */
    public static final String MAIL_DATE_OBJECT = "EmailDateObject";

    /** String name of the Entity that has as its String value the body of this message. */
    public static final String MAIL_PART_IDENTIFIER = "Part0";
    public static final String MAIL_PREFERRED_CONTENT_TYPE = "text/plain";
    public static final String MAIL_HTML_CONTENT_TYPE = "text/html";
    private static final StringEntityValue NULL_STRING_ENTITY_VALUE = 
            new StringEntityValue("");

    /** The email message Entity that this EmailMessageEntityHelper wraps. */
    private Entity message;

    /** 
     * This attributeMap contains attribute String names mapped to their
     * appropriate EntityValues
     */
    private Map attributeMap = new HashMap();

    /**
     * Creates a new EmailMessageEntityHelper to access the data in the given
     * message.
     */
    public EmailMessageEntityHelper(final Entity message)
                             throws DataManagerException {
        this.message = message;

        // FIXME: currently adding all top-level attributes to the map,
        // and not recurring down any nested attributes.  This may or may
        // not be OK.
        Entity[] attributes = message.getAttributes(EntityConstraint.NONE);

        for (int i = 0; i < attributes.length; i++) {
            if (attributes[i] != null) {
                // FIXME:  dealing with this NullEntityValue crap should
                // be done at email retrieval time..?
                attributeMap.put(attributes[i].getName(), 
                                 (attributes[i].getValue() == null)
                                 ? NULL_STRING_ENTITY_VALUE
                                 : attributes[i].getValue());
            }
        }


        // FIXME: hack to deal with mail bodies... ugly.  
        attributeMap.put(MAIL_PART_IDENTIFIER, 
                         Toolbox.getFirstAttributeNamedIgnoringCase(message, 
                                                                    MAIL_PART_IDENTIFIER));
    }

    /**
     * Creates a new EmailMessageEntityHelper containing a new email message
     * entity with no content.
     */
    public EmailMessageEntityHelper() throws DataManagerException {
        this(Entity.create("new email message"));
    }

    /**
     * Returns the Object attribute value mapped to the String key passed in.
     */
    public Object get(String key) {
        if ((key == null) || !attributeMap.containsKey(key)) {
            return NULL_STRING_ENTITY_VALUE;
        }

        return attributeMap.get(key);
    }

    /**
     * Returns the string value of the first attribute of the message whose
     * name equals <code>name</code>, ignoring case, or defaultString if no
     * matching attribute is found.
     */
    private String getAttributeString(String name, String defaultString)
                               throws DataManagerException {
        String attributeStringValue = defaultString;

        //FIXME: special casing for email bodies is ugly (checking for
        //MAIL_PART_IDENTIFIER)
        if (attributeMap.containsKey(name)) {
            if (name.equals(MAIL_PART_IDENTIFIER)) {
                attributeStringValue = ((Entity) attributeMap.get(name)).getValue()
                                                                      .toString();
            } else {
                EntityValue attributeEntityValue = (EntityValue) attributeMap.get(
                                                           name);
                attributeStringValue = attributeEntityValue.toString();
            }
        }

        return attributeStringValue;
    }

    /**
     * Replaces the value of the attribute named <code>name</code> with the
     * value <code>value</code>.  If no such attribute yet exists, creates one.
     */
    public void replaceStringAttribute(String name, String value)
                                throws DataManagerException {
        Entity match = Toolbox.getFirstAttributeNamedIgnoringCase(message, name);

        StringEntityValue newValue = new StringEntityValue(value);

        KernelProxy kernelProxy = KernelProxy.getInstance();

        if (match != null) {
            // replaced explicit match.setValue(newValue) with the following
            // since AWT-Event threads were calling this guy from
            // BasicEmailReplySimpleton.
            kernelProxy.setEntityValue(match, newValue);


            // HashMap replaces the old mapped value with the newValue
            attributeMap.put(name, newValue);
        } else {
            Entity attribute = Entity.create(name);


            // Same reasons as above for replacing this call: 
            // attribute.setValue(newValue);
            kernelProxy.setEntityValue(attribute, newValue);
            kernelProxy.attachEntity(message, attribute);


            // message.attach(attribute);
            attributeMap.put(name, newValue);
        }
    }

    /**
     * Returns the address in the <code>From:</code> field of the message.
     */
    public String getFromAddress() throws DataManagerException {
        return getAttributeString(MAIL_FROM, "");
    }

    /**
     * Sets the from address of the message.
     */
    public void setFromAddress(String address) throws DataManagerException {
        replaceStringAttribute(MAIL_FROM, address);
    }

    /**
     * Returns the addresses in the <code>To:</code> field of the message.
     *
     * FIXME: This should probably return a List of addresses instead of just a
     * string.
     */
    public String getToAddresses() throws DataManagerException {
        return getAttributeString(MAIL_TO, "");
    }

    /**
     * Sets the to addresses of the message.
     */
    public void setToAddresses(String addresses) throws DataManagerException {
        replaceStringAttribute(MAIL_TO, addresses);
    }

    /**
     * Returns the addresses in the <code>Cc:</code> field of the message.
     *
     * FIXME: This should probably return a List of addresses instead of just a
     * string.
     */
    public String getCcAddresses() throws DataManagerException {
        return getAttributeString(MAIL_CC, "");
    }

    /**
     * Sets the cc addresses of the message.
     */
    public void setCcAddresses(String addresses) throws DataManagerException {
        replaceStringAttribute(MAIL_CC, addresses);
    }

    /**
     * Gets the reply-to address of the message.
     */
    public String getReplyToAddress() throws DataManagerException {
        return getAttributeString(MAIL_REPLYTO, "");
    }

    /**
     * Sets the reply to field of the message.
     */
    public void setReplyToAddress(String address) throws DataManagerException {
        replaceStringAttribute(MAIL_REPLYTO, address);
    }

    /**
     * Returns the string from the <code>Date:</code> field of the message.
     */
    public String getDateString() throws DataManagerException {
        return getAttributeString(MAIL_DATE, "");
    }

    /**
     * Sets the date field of the message.
     */
    public void setDateString(String date) throws DataManagerException {
        replaceStringAttribute(MAIL_DATE, date);
    }

    /**
     * Returns the Date object associated with this message.
     */
    public Date getDateObject() {
        Date date = null;

        DateEntityValue dateEntityValue = (DateEntityValue) attributeMap.get(
                                                  MAIL_DATE_OBJECT);

        if (dateEntityValue != null) {
            date = dateEntityValue.getDate();
        }

        return date;
    }

    /**
     * Sets the date object of the message.
     */
    public void setDateObject(Date date) throws DataManagerException {
        attributeMap.put(MAIL_DATE_OBJECT, date);
    }

    /**
     * Returns the subject of the message.
     */
    public String getSubject() throws DataManagerException {
        return getAttributeString(MAIL_SUBJECT, "");
    }

    /**
     * Sets the subject of the message.
     */
    public void setSubject(String subject) throws DataManagerException {
        replaceStringAttribute(MAIL_SUBJECT, subject);
    }

    /**
     * Gets the Message-ID of the message.
     */
    public String getMessageId() throws DataManagerException {
        return getAttributeString(MAIL_MESSAGE_ID, "");
    }

    /**
     * Sets the Message-ID of the message.
     */
    public void setMessageId(String messageId) throws DataManagerException {
        replaceStringAttribute(MAIL_MESSAGE_ID, messageId);
    }

    /**
     * Gets the In-Reply-To of the message.
     */
    public String getInReplyTo() throws DataManagerException {
        return getAttributeString(MAIL_IN_REPLY_TO, "");
    }

    /**
     * Sets the In-Reply-To of the message.
     */
    public void setInReplyTo(String messageId) throws DataManagerException {
        replaceStringAttribute(MAIL_IN_REPLY_TO, messageId);
    }

    /**
     * Returns the lowercased string from the <code>Content-Type:</code> field
     * of the message, or "text/plain" if no <code>Content-Type:</code> header
     * is found.  Anything following the initial content type declaration (e.g.
     * '; charset="iso-8859-1"'), and all leading and trailing whitespace is
     * stripped.
     */
    public String getContentType() throws DataManagerException {
        String contentType = getAttributeString(MAIL_CONTENT_TYPE, "text/plain");
        int semicolonIndex = contentType.indexOf(";");

        if (semicolonIndex > 0) {
            contentType = contentType.substring(0, semicolonIndex);
        }

        return contentType.trim().toLowerCase();
    }

    /**
     * Sets the content type of the message.
     */
    public void setContentType(String contentType) throws DataManagerException {
        replaceStringAttribute(MAIL_CONTENT_TYPE, contentType);
    }

    /**
     * Returns the first chunk of content in this message, regardless of MIME
     * type.
     */
    public String getBody() throws DataManagerException {
        return getAttributeString(MAIL_PART_IDENTIFIER, 
                                  "Trouble retrieving message content.\n");
    }

    /**
     * Sets the first chunk of content in this message (part0) to be the 
     * given String. 
     */
    public void setBody(String body) throws DataManagerException {
        replaceStringAttribute(MAIL_PART_IDENTIFIER, body);
    }

    /**
     * Returns the message this EmailMessageEntityHelper is helping with.
     */
    public Entity getMessage() {
        return this.message;
    }
}

/*
 *
 * $Log: EmailMessageEntityHelper.java,v $
 * Revision 1.16  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.15  2002/12/20 06:59:20  cjdent
 *
 * Slight cleanups. I'll codebase-style the whole lot here in a bit.
 *
 * Revision 1.14  2002/12/06 11:33:02  alllee
 * Now that EmailMessageEntityHelper throws a DataManagerException from both of
 * its constructors, we have to try/catch or re-throw in all these classes that
 * utilize EmailMessageEntityHelper.
 *
 * Revision 1.13  2002/12/06 10:47:18  alllee
 * For some reason, sending a reply to a message stopped working for me
 * (generated invalid thread in kernel messages) -- fixing that here.
 *
 * Revision 1.12  2002/11/24 05:53:27  alllee
 * Since this is our facade to the EmailMessageEntities themselves, made public
 * final Strings that contain the attribute name/values
 * ("content-source"/"email") and added references to these variables in the
 * other classes that use them.
 *
 * Revision 1.11  2002/11/23 05:44:11  cjdent
 *
 * Adds the EmailCompositionInitiatorSimpleton and makes the
 * change to BasicEmailReplysimpleton, EmailBundleSimpleton,
 * and EmailMessageEntityHelper to make it usable.
 *
 * Composition is initiate by using the "Componse New" button
 * provided by EmailCompositionInitiatorSimpleton. That
 * simpleton has an empty message Entity that it uses to
 * initiate the email Reply chain used for replies.
 *
 * BasicEmailReplySimpleton has been adjusted to notice when
 * an original message has no DateObject. Under those
 * circumstances it assume that it is creating a new clean
 * message.
 *
 * Some cleanups are requird in the SMTP send simpleton to avoid
 * some bogus headers.
 *
 * Cleanup needed badly in these simpletons.
 *
 * Revision 1.10  2002/11/21 07:12:39  cjdent
 * Adds support for getting and setting the Reply-To header of a message.
 *
 * Revision 1.9  2002/11/21 06:04:34  cjdent
 *
 * Removing a throws DataManagerException because that doesn't actually get
 * thrown.
 *
 * Revision 1.8  2002/11/21 03:39:21  cjdent
 * These changes implement quoting and attribution in email replies. These
 * classes are getting long, so will need some refactoring. More notes on this
 * coming.
 *
 * EmailMessageEntityHelper has changed setDate() and getDate() to
 * setDateString() and getDateString(). getDateObject() and setDateObject()
 * have been added to retrieve the the EmailDateObject attribute.
 *
 * Revision 1.7  2002/11/19 06:22:43  cjdent
 *
 * Adding a setContentType(String) method to complement the other setters
 * and getters. I don't think I ended up using this.
 *
 * Revision 1.6  2002/11/19 05:24:06  alllee
 * Added a backing Map that caches various portions of an email message Entity so
 * that the sluggish Toolbox attribute search doesn't need to happen.  This has
 * improved sorting speed tenfold.  The getAttributeString utilizes the HashMap
 * to return cached values (which are cached at EmailMessageEntityHelper
 * instantiation time).  Various new FIXMEs have been placed within the codebase,
 * and should get some attention, once some more big fockers have been crushed
 * like deep fried corn starch packing peanuts.
 *
 * TODO: try to get rid of underlying ugliness in special-casing with the body of
 * the message, and dealing with nulls in the EntityValue slot of an attribute.
 * I think this should be done at the email retrieval phase, but that's just me..
 *
 * Revision 1.5  2002/11/19 02:26:00  cjdent
 *
 * BasicEmailReplySimpleton updated to include the subject and the body of the
 * message in the created Reply entity. processSubject() and processBody()
 * were created to get the content of those attriutes on the original message
 * and doctor them to make a reasonable response.
 *
 * No doctoring is currently done. That's coming later.
 *
 * Added setBody() to EmailMessageEntityHelper.
 *
 * Revision 1.4  2002/11/18 03:18:56  mliggett
 * Added getters and setters for Message-Id and In-Reply-To, plus unit tests
 * for those.  Also a unit test for getMessage().
 *
 * Revision 1.3  2002/11/18 02:11:00  mliggett
 * Added setters for email header values and also a bunch more unit tests.
 *
 * This is in preparation for building reply messages.
 *
 * Revision 1.2  2002/11/17 01:19:18  alllee
 * EntityDisplayRequestEvent handling/firing instead of
 * EmailDisplayRequestEvents. The constraint used currently captures all
 * EntityDisplayRequestEvents, instead of just those Events that contain the
 * relevant Email entities (I didn't know enough about the structure of the
 * message Entity to play with this as of now, but will fix this later unless
 * someone else steps right up).  Added a getMessage() method to
 * EmailMessageEntityHelper so that EmailDisplaySimpleton doesn't need a
 * class-level reference to the email message Entity.
 *
 * Revision 1.1  2002/11/16 17:22:05  mliggett
 * Added EmailMessageEntityHelper factored out of EmailDisplaySimpleton.
 *
 * Also unit test.
 *
 *
 */
