// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.text.ParseException;

import java.util.Date;

import javax.mail.internet.MailDateFormat;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.DateValueRangeConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameContainsConstraint;
import org.datamanager.constraint.StringValueContainsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailMetadataSearchEvent;
import org.datamanager.event.SearchRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.Debug;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * This class handles the EmailMetadataSearchEvent. It will get the field/query
 * out from the EmailMatadataSearchEvent and build the SearchRequestEvent and
 * fire it.  The SearchHandlerSimpleton will handle the SearchRequestEVent and
 * Fire the SearchResultFoundEvent to the target.
 *
 * @author Xi Rao
 */
public class EmailMetadataSearchSimpleton extends Simpleton
    implements EventHandler {
    private Pool pool;
    private KernelProxy kernelProxy = KernelProxy.getInstance();

    /**
     * Runs this simpleton by subscribing it to the EmailMetadataSearchEvent.
     */
    public void process() {
        try {
            pool = Pool.getDefaultPool();

            EventConstraint eventConstraint = new EventTypeIsConstraint(
                                                      EmailMetadataSearchEvent.class);
            pool.subscribe(this, eventConstraint);
        } catch (Exception everything) {
            everything.printStackTrace();
        }
    }

    /**
     * Handles the EmailMetadataSearchEvent, get the field and query out to
     * build a complicated query to guarantee the search result correct.
     *
     * @param dataManagerEvent the EmailMetadataSearchEvent fire with
     *                         "field"(entity name) * and "query"(entity value)
     */
    public void handle(DataManagerEvent dataManagerEvent) {
        EmailMetadataSearchEvent emailMetadataSearchEvent = 
                (EmailMetadataSearchEvent) dataManagerEvent;
        String field = emailMetadataSearchEvent.getFieldName();
        String query = emailMetadataSearchEvent.getQueryString();

        // FIXME: This is inelegant. Suggests that metadata
        // searches need to be divided up into separate things.
        // We generalized to far and now we can't pass metadata about
        // our metadata searches along with the searches, such as
        // window of time, or portion of subject, or person identifier
        // instead of From header string.
        EntityConstraint metaSearchQuery;

        if (field.toLowerCase().equals("date")) {
            Date date;

            try {
                date = new MailDateFormat().parse(query);
            } catch (ParseException parseException) {
                // FIXME: what the hell are we doing with exception anyway?
                throw new RuntimeException(parseException);
            }

            // FIXME: this is inelegant too and does not support ranges
            // right now it just supports a 24 hour window surrounding
            // the query string.
            long min = date.getTime() - (1000 * 60 * 60 * 12);
            long max = min + (1000 * 60 * 60 * 24);
            Date minDate = new Date(min);
            Date maxDate = new Date(max);

            metaSearchQuery = new AndEntityConstraint(
                                      new NameContainsConstraint(field), 
                                      new DateValueRangeConstraint(minDate, 
                                                                   maxDate));
        } else {
            metaSearchQuery = new AndEntityConstraint(
                                      new NameContainsConstraint(field), 
                                      new StringValueContainsConstraint(query));
        }

        try {
            kernelProxy.addEvent(
                    new SearchRequestEvent(kernelProxy.getEventGenerator(), 
                                           metaSearchQuery, 
                                           field + ": " + query.toString()));
        } catch (DataManagerException dataManagerException) {
            Debug.writeLogMessage("Kernel error");
            dataManagerException.printStackTrace();
        }
    }
}