// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.ValueClassIsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailConfigurationRequestEvent;
import org.datamanager.event.EmailParametersEnteredCollectorEvent;
import org.datamanager.event.EmailParametersRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.EmailParametersEntityValue;
import org.datamanager.passiveentityvalue.PasswordEntityValue;

import org.datamanager.tools.Debug;


/**
 * This Simpleton is responsible for handling EmailParametersRequestEvent-s. 
 * When signaled with such an event, it first searches the Pool for existing
 * EmailParametersEntityValue-s.  If none are found, it prompts the user for
 * the email parameters by firing an EmailConfigurationRequestEvent into the
 * Pool's event channel.  Any interested EmailConfigurator-s handle()-ing
 * EmailConfigurationRequestEvents should prompt the user for new email
 * parameters and dump a EmailParametersEnteredCollectorEvent containing the
 * user's email parameters (as an EmailParametersEntityValue).
 *
 * @author Team Helium
 * @version $Revision: 1.6 $
 *
 */
public class EmailParametersRequestHandlerSimpleton extends Simpleton
    implements EventHandler {
    public void process() {
        attachToPool();
    }

    public void handle(DataManagerEvent event) {
        queryForEmailParameters();
    }

    /**
     * Things we need to do in this method:
     * 1. query the Pool for EmailParametersEntityValue-s
     * 2. If none exist, notify the EmailConfiguratorSimpleton and let it
     * handle the collection of EmailParameters
     * 3. If a persisted EmailParametersEntityValue exists, fire an
     * EmailParametersEnteredCollectorEvent containinig the
     * EmailParameterEntityValue (if multiple EmailParametersEntityValue-s
     * exist in the Pool, we, entirely arbitrarily, choose the first).
     *
     * FIXME:  Should we have some way to enforce a single
     * EmailParametersEntityValue in the pool at any given point in time?
     *
     */
    private void queryForEmailParameters() {
        try {
            final Entity[] emailParameterEntities = Pool.getDefaultPool()
                                                        .search(new ValueClassIsConstraint(
                                                                        EmailParametersEntityValue.class));

            if (emailParameterEntities.length == 0) {
                // We didn't find any EmailParametersEntity-s in the pool.
                // Query the user for email configuration info.
                notifyConfigurator();
            } else {
                addEmailParametersToPool(emailParameterEntities[0]);
            }
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Couldn't connect to Pool: " + exception);

            // FIXME: Debug exception only.
            throw new RuntimeException(exception);
        }
    }

    /**
     * Alerts any interested Simpletons that Email Configuration is requested
     * (user wants to enter new email parameters or change existing email
     * parameters).  
     */
    private void notifyConfigurator() {
        try {
            Pool.getDefaultPool()
                .add(new EmailConfigurationRequestEvent(Pool.getDefaultPool()));
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Unable to add event: " + exception);
            exception.printStackTrace();

            // FIXME: Debug only
            throw new RuntimeException(exception);
        }
    }

    /** 
     * Adds the given EmailParametersEntity to the Pool and fires an event
     * into the Pool letting any interested Requester know that the Entity
     * containing email parameters has been found. 
     */
    private void addEmailParametersToPool(Entity entity) {
        try {
            Pool.getDefaultPool()
                .add(new EmailParametersEnteredCollectorEvent(
                             Pool.getDefaultPool(), entity));
            Pool.getDefaultPool().add(entity);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage(
                    "Unable to notify pool of email parameters collector event: " + 
                    exception);

            // FIXME: Debug-thrown exception only
            throw new RuntimeException(exception);
        }
    }

    /** 
     * Subscribes to the pool, expressing interest in
     * EmailParametersRequestEvent-s.
     */
    private void attachToPool() {
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new EventTypeIsConstraint(
                                   EmailParametersRequestEvent.class));
        } catch (DataManagerException exception) {
            Debug.writeLogMessage(
                    "EmailParametersRequestHandler couldn't attach to pool: " + 
                    exception);

            // FIXME: For debug purposes only
            throw new RuntimeException(exception);
        }
    }
} // EmailParametersRequestHandlerSimpleton
/*
 * $Log: EmailParametersRequestHandlerSimpleton.java,v $
 * Revision 1.6  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.5  2002/11/16 06:29:07  alllee
 * Cleaned/added javadocs at class level and comments throughout the code.
 * Removed Swing-related import statements since this handler is no longer a GUI
 * simpleton.  Removed references to KernelProxy.  Hygienic commit.
 *
 */
