// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import java.security.Security;

import java.text.ParseException;

import java.util.Enumeration;
import java.util.Properties;

import javax.mail.*;
import javax.mail.internet.MailDateFormat;

import javax.net.ssl.SSLSocketFactory;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.AttributeExistsConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsIgnoringCaseConstraint;
import org.datamanager.constraint.ValueEqualsConstraint;

import org.datamanager.event.ContainsEntityEvent;
import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailParametersEnteredCollectorEvent;
import org.datamanager.event.EmailParametersRequestEvent;
import org.datamanager.event.StartedRetrievingEmailEvent;
import org.datamanager.event.StoppedRetrievingEmailEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.ByteArrayEntityValue;
import org.datamanager.passiveentityvalue.DateEntityValue;
import org.datamanager.passiveentityvalue.EmailParametersEntityValue;
import org.datamanager.passiveentityvalue.PassiveEntityValue;
import org.datamanager.passiveentityvalue.PasswordEntityValue;
import org.datamanager.passiveentityvalue.StringEntityValue;
import org.datamanager.passiveentityvalue.WordFrequencyMapEntityValue;

import org.datamanager.simpleton.EmailParameters;

import org.datamanager.tools.Debug;


/**
 * Retrieves some email from an IMAP server and creates corresponding Entities
 * for that email in the Pool.  When this Simpleton is started, it fires an
 * EmailParametersRequestEvent into the Pool.  Any interested handler of this
 * request event responds by firing an EmailParametersEnteredCollectorEvent
 * containing the various Email Configuration parameters needed to connect to
 * the email data store (username, password, host machine, mailbox name).
 * These parameters are encapsulated by an EmailParametersEntityValue.
 *
 * FIXME:
 * 1. document the structure of the created Entities.
 * 2. Refactor this class.  My god.  Talk about monolithic pieces of
 * ever-burgeoning panty-waist-line mungballs functionality.
 * 3. Fix bug with re-retrieval of email downloading triplicate copies.
 */
public class EmailRetrieverSimpleton extends Simpleton
    implements EventHandler {
    /** The next sequence number to be given to an email message. */
    private static volatile int nextSequenceNumber = 0;

    /**
     * The name of the attribute that will contain the DateEntityValue
     * for an email message.
     */
    private static final String DATE_ATTRIBUTE_NAME = "EmailDateObject";
    private volatile EmailParametersEntityValue currentEmailParameters;

    /** SSL factory used for generating JavaMail sockets. */
    final String SSL_FACTORY = "javax.net.ssl.SSLSocketFactory";

    public void process() {
        attachToPool();
        authenticate();
    }

    private void attachToPool() {
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new EventTypeIsConstraint(
                                   EmailParametersEnteredCollectorEvent.class));
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Unable to subscribe to pool: " + 
                                  exception);
            exception.printStackTrace();

            // FIXME: Debug only
            throw new RuntimeException(exception);
        }
    }

    private void authenticate() {
        try {
            // signals that this Simpleton would like some email parameters
            Pool.getDefaultPool()
                .add(new EmailParametersRequestEvent(Pool.getDefaultPool()));
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage(
                    "Exception caught while trying to fire event: " + 
                    exception.getMessage());
        }
    }

    public void handle(DataManagerEvent event) {
        try {
            Entity emailParametersEntity = ((ContainsEntityEvent) event).getEntity();

            EmailParametersEntityValue emailParametersEntityValue = 
                    (EmailParametersEntityValue) emailParametersEntity.getValue();

            // If we just received an event with email parameters that are the
            // exact same as our current email parameters (have to use
            // .equals, not pointer ==), then don't retrieve any more emails.
            // This should stop that damned dirty ape of a bug that kept
            // giving emails in triplicate.  BasicEmailReplySimpleton,
            // EmailRetrieverSimpleton, and SMTPSendSimpleton all send forth
            // EmailParametersRequestEvent-s, and that means that every time
            // the EmailParametersRequestHandlerSimpleton handles one of
            // those, it dumps a new EmailParametersEnteredCollectorEvent into
            // the Pool, which then gets handled by this guy.
            if ((currentEmailParameters == null) || 
                    (!currentEmailParameters.equals(emailParametersEntityValue))) {
                // cache the current email parameters entity value so that we
                // can compare it to incoming EmailParametersEntityValue
                currentEmailParameters = emailParametersEntityValue;

                EmailParameters freshEmailParameters = 
                        new EmailParameters(emailParametersEntityValue);


                // retrieve mail with the new set of parameters
                retrieveMail(freshEmailParameters);
            }
        } catch (ClassCastException exception) {
            // FIXME:  Exception handling is weak here.  At this point these
            // are/should be sanity checks, and for Debug information only.
            exception.printStackTrace();
            Debug.writeLogMessage("Invalid event or entity value type: " + 
                                  exception);
            throw new RuntimeException(exception);
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage(exception.toString());
            throw new RuntimeException(exception);
        }
    }

    /**
     * Configures SSL IMAP properties for use by JavaMail.
     */
    private void configPropertiesForSsl(Properties properties) {
        properties.setProperty("mail.imap.socketFactory.class", 
                               "org.datamanager.simpleton.DummySSLSocketFactory");
        properties.setProperty("mail.imap.socketFactory.fallback", "false");
        properties.setProperty("mail.imap.socketFactory.port", "993");
        properties.setProperty("mail.imap.port", "993");
        Security.setProperty("ssl.SocketFactory.provider", 
                             "org.datamanager.simpleton.DummySSLSocketFactory");
    }

    /**
     * Configures non-SSL IMAP properties for use by JavaMail.
     */
    private void configPropertiesForNonSsl(Properties properties) {
        properties.remove("mail.imap.socketFactory.class");
        properties.remove("mail.imap.socketFactory.fallback");
        properties.remove("mail.imap.socketFactory.port");
        properties.remove("mail.imap.port");
    }

    /**
     * Sets up JavaMail and connects to a message store.
     *
     * @return the connected message store
     */
    private Store getConnectedStore(EmailParameters emailParameters)
                             throws NoSuchProviderException, MessagingException {
        Properties properties = System.getProperties();
        Authenticator authenticator = null; //NOTE: no checking for now

        // FIXME: This is horrible. The solution is to find out how
        //        to install an IMAPS Provider in JavaMail.
        if (emailParameters.getAccessProtocolName().equals("imaps")) {
            configPropertiesForSsl(properties);
        } else {
            configPropertiesForNonSsl(properties);
        }

        Session session = Session.getDefaultInstance(properties, authenticator);
        Store store = null;

        if (emailParameters.getAccessProtocolName() == null) {
            // If no protocol is specified, the connection will proceed with
            // one specified in the mail.store.protocol property.
            store = session.getStore();
        } else {
            // FIXME: Create an IMAPS provider! And change "imap" to
            //        emailParameters.getAccessProtocolName() below.
            store = session.getStore("imap");
        }

        if ((emailParameters.getHostMachineName() == null) || 
                (emailParameters.getUsername() == null) || 
                (emailParameters.getPassword() == null)) {
            // If no machine, username or password are specified, the
            // connection will proceed with properties loaded from the
            // Properties object above.
            store.connect();
        } else {
            store.connect(emailParameters.getHostMachineName(), 
                          emailParameters.getUsername(), 
                          emailParameters.getPassword());
        }

        return store;
    }

    /**
     * Returns true if the given message has already been loaded into the Pool.  False otherwise.
     */
    private boolean isMessageAlreadyInPool(Message message)
                                    throws DataManagerException {
        Entity[] duplicates = Pool.getDefaultPool()
                                  .search(new AttributeExistsConstraint(
                                                  new AndEntityConstraint(
                                                          new NameEqualsIgnoringCaseConstraint(
                                                                  "Message-ID"), 
                                                          new ValueEqualsConstraint(
                                                                  new StringEntityValue(
                                                                          getMessageId(
                                                                                  message))))));

        return duplicates.length > 0;
    }

    /**
     * Adds a single Message to the pool as an Entity with assorted attributes
     * representing its headers, flags, attachments, etc.
     */
    private void addMessageToPool(Message message) throws DataManagerException, 
                                                          MessagingException, 
                                                          IOException {
        if (!isMessageAlreadyInPool(message)) {
            Entity entity = Entity.create(makeEntityName(message));


            //attach special marker attributes for interface viewers
            //to find email message entities
            attachStringAttribute("content-source", "email", entity);

            storePart(message, 0, entity);
        }
    }

    /**
     * Hints to the folder that we're going to want ENVELOPE and FLAGS
     * information.  Helper function for retrieveMessagesToPool.  Note this is
     * just an optimization.
     */
    private void prefetchEnvelopeAndFlags(Message[] messages, Folder folder)
                                   throws MessagingException {
        FetchProfile fetchProfile = new FetchProfile();
        fetchProfile.add(FetchProfile.Item.ENVELOPE);
        fetchProfile.add(FetchProfile.Item.FLAGS);
        folder.fetch(messages, fetchProfile);
    }

    /**
     * Retrieves all the messages from the specified folder into the Pool.
     */
    private void retrieveMessagesToPool(Folder folder)
                                 throws MessagingException, 
                                        DataManagerException, IOException {
        if (folder.getMessageCount() == 0) {
            // FIXME: Is this just for debugging, or do we need to do something
            // special here? -mml 2002-11-03
            Debug.writeLogMessage("Specified folder is empty");
        } else {
            Message[] messages = folder.getMessages();

            prefetchEnvelopeAndFlags(messages, folder);

            for (int messageIndex = 0;
                 messageIndex < messages.length;
                 messageIndex++)
                addMessageToPool(messages[messageIndex]);
        }
    }

    /**
     * Fires a StartedRetrievingEmailEvent into the Pool.
     */
    private void fireStartEvent(EmailParameters emailParameters)
                         throws DataManagerException {
        Pool.getDefaultPool()
            .add(new StartedRetrievingEmailEvent(Pool.getDefaultPool(), 
                                                 emailParameters.toString()));
    }

    /**
     * Fires a StoppedRetrievingEmailEvent into the Pool.
     */
    private void fireStopEvent(EmailParameters emailParameters)
                        throws DataManagerException {
        Pool.getDefaultPool()
            .add(new StoppedRetrievingEmailEvent(Pool.getDefaultPool(), 
                                                 emailParameters.toString()));
    }

    /**
     * Retrieves emails from a server and drops Entities representing them and
     * their attributes into the Pool.
     */
    private void retrieveMail(final EmailParameters emailParameters) {
        Store store = null;

        try {
            fireStartEvent(emailParameters);
            store = getConnectedStore(emailParameters);

            Folder mailbox = store.getDefaultFolder();

            if (mailbox == null) {
                //FIXME: this message should be in a window
                Debug.writeLogMessage("Could not find specified mailbox");

                return;
            }

            mailbox = mailbox.getFolder(emailParameters.getMailboxName());

            if (mailbox == null) {
                //FIXME: this message should be in a window
                Debug.writeLogMessage("Could not find default mailbox");

                return;
            }

            mailbox.open(Folder.READ_ONLY);
            retrieveMessagesToPool(mailbox);
            mailbox.close(false);
        } catch (AuthenticationFailedException exception) {
            Debug.writeLogMessage("Could not authenticate your information");

            //FIXME: should put retry code here
        } catch (NoSuchProviderException exception) {
            Debug.writeLogMessage("Could not find host machine");
            exception.printStackTrace();

            //FIXME: should put retry code here
        } catch (IOException exception) {
            Debug.writeLogMessage("Could not access stream message data");
            exception.printStackTrace();
        } catch (MessagingException exception) {
            Debug.writeLogMessage("oops. something unexpected happened");
            exception.printStackTrace();
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("bug in kernel!");
            exception.printStackTrace();
        } finally {
            try {
                fireStopEvent(emailParameters);
                store.close();
            } catch (MessagingException exception) {
                Debug.writeLogMessage("oops. something unexpected happened");
                exception.printStackTrace();
            } catch (DataManagerException exception) {
                Debug.writeLogMessage("bug in kernel!");
                exception.printStackTrace();
            }
        }
    }

    /**
     * Returns a sensible name for the entity that will contain the given
     * message.  Tries the Message-ID, but if that cannot be found, assigns a
     * number to it.
     */
    private String makeEntityName(Message message) {
        String entityName = getMessageId(message);

        if (null == entityName) {
            entityName = "malformed email" + getNextSequenceNumber();
        }

        return entityName;
    }

    /**
     * Returns the next integer in a sequence.  Right now, it just increments a
     * static variable, so the integers are not unique across sessions.
     * Eventually we may want a better source for the sequence that <i>is</i>
     * unique across sessions.
     */
    private int getNextSequenceNumber() {
        return nextSequenceNumber++;
    }

    /**
     * Returns the Message-ID of the given message, or null if unable to
     * retrieve it.
     */
    private String getMessageId(Message message) {
        try {
            String[] values = message.getHeader("Message-ID");

            if ((values != null) && (values.length > 0)) {
                return values[0];
            }
        } catch (MessagingException exception) {
            // do nothing; we just want to return null
        }

        return null;
    }

    /**
     * Stores the content of the Part. This involve handling "text/plain"
     * "multipart" and other content types. All parts of the message are
     * added as attributes named "partN" with "N" starting at 0.
     */
    private void storePart(Part part, int partNumber, Entity entity)
                    throws MessagingException, IOException, 
                           DataManagerException {
        // Write all the headers on the Part as attribute of the Entity.
        storeHeaders(part, entity);

        try {
            storeContent(part, partNumber, entity);

            Pool.getDefaultPool().add(entity);
        } catch (UnsupportedEncodingException exception) {
            // FIXME: we should be smarter here
            // This says, if the we can't deal with the encoding
            // don't add the message but carry on.
            Debug.writeLogMessage("Unable to store message part number " + 
                                  partNumber + ", unsupported encoding: " + 
                                  exception);
        }
    }

    /**
     * Store all the headers in the message on the entity as attributes.
     * The header's name will be the attribute's name and the header's
     * value will be the attribute's value.
     */
    private void storeHeaders(Part part, Entity entity)
                       throws MessagingException, IOException, 
                              DataManagerException {
        Enumeration headers = part.getAllHeaders();
        Header header;

        while (headers.hasMoreElements()) {
            header = (Header) headers.nextElement();
            attachStringAttribute(header.getName(), header.getValue(), entity);

            // FIXME: this seems rather inelegant
            if (header.getName().equalsIgnoreCase("date")) {
                attachDateAttribute(header.getValue(), entity);
            }
        }
    }

    private void storeContent(Part part, int partNumber, Entity entity)
                       throws MessagingException, IOException, 
                              DataManagerException {
        Object content = part.getContent();

        if (part.isMimeType("TEXT/*") && (content instanceof String)) {
            // Attach the frequency of words in a Map as an Attribute hanging
            // off of this email entity.
            attachWordFrequencyMapAttribute((String) content, entity);
            attachStringAttribute("part" + partNumber, (String) content, entity);
        } else {
            attachWordFrequencyMapAttribute("Unhandled content-type.", entity);

            if (part.isMimeType("MULTIPART/*") && 
                    (content instanceof Multipart)) {
                Multipart multipart = (Multipart) content;
                Entity partAttribute = Entity.create("part" + partNumber);
                entity.attach(partAttribute);

                for (int subPartNumber = 0;
                     subPartNumber < multipart.getCount();
                     subPartNumber++) {
                    storePart(multipart.getBodyPart(subPartNumber), 
                              subPartNumber, partAttribute);
                }
            } else if (part.isMimeType("MESSAGE/*") && 
                           (content instanceof Message)) {
                Entity partAttribute = Entity.create("part" + partNumber);
                entity.attach(partAttribute);
                storePart((Message) content, 0, partAttribute);
            } else {
                attachByteArrayAttribute("part" + partNumber, part, entity);
            }
        }
    }

    /**
     * Creates and attaches a new entity containing a Map of words found
     * within this email as keys mapped to the number of occurrences as
     * values.
     */
    private void attachWordFrequencyMapAttribute(String content, Entity entity)
                                          throws DataManagerException {
        attachAttribute(WordFrequencyMapEntityValue.ATTRIBUTE_NAME, 
                        new WordFrequencyMapEntityValue(content), entity);
    }

    /**
     * Create and attach a new entity containing a string as an attribute of
     * the current email message entity.
     */
    private void attachStringAttribute(String name, String string, 
                                       Entity entity)
                                throws DataManagerException {
        StringEntityValue value = new StringEntityValue(string);
        attachAttribute(name, value, entity);
    }

    /**
     * Parses a Date header string, turns it into a java.util.Date
     * and attaches is as an attribute on the current email messages
     * entity.
     */
    private void attachDateAttribute(String date, Entity entity)
                              throws DataManagerException {
        DateEntityValue dateEntityValue;

        try {
            dateEntityValue = new DateEntityValue(new MailDateFormat().parse(
                                                          date));
        } catch (ParseException parseException) {
            Debug.writeLogMessage("Unable to parse date string: " + date);
            throw new RuntimeException(parseException);
        }

        attachAttribute(DATE_ATTRIBUTE_NAME, dateEntityValue, entity);
    }

    private void attachByteArrayAttribute(String name, Part part, Entity entity)
                                   throws MessagingException, 
                                          DataManagerException, IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        part.writeTo(byteArrayOutputStream);

        byte[] byteArray = byteArrayOutputStream.toByteArray();
        attachAttribute(name, new ByteArrayEntityValue(byteArray), entity);
    }

    /**
     * Create and attach a new entity as an attribute of the current email
     * message entity.
     *
     * NOTE: currently there's no need to add() the attribute; so we simply
     * attach() it to a central email entity.
     **/
    private void attachAttribute(String name, PassiveEntityValue value, 
                                 Entity entity) throws DataManagerException {
        Entity attributeEntity = Entity.create(name);
        attributeEntity.setValue(value);
        entity.attach(attributeEntity);
    }
}

/*
 * $Log: EmailRetrieverSimpleton.java,v $
 * Revision 1.31  2002/12/20 20:52:38  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.30  2002/12/16 10:38:41  cjdent
 *
 * Removed LSA DocumentMatrix calculation calls from
 * TouchgraphEmailDisplaySimpleton and put it in the clustering
 * strategy, via TouchgraphEntityDisplayPanel
 *
 * Revision 1.29  2002/12/10 04:30:55  xirao
 * xi rao
 *         add DocumentTermMatrixBuilderSimpletion back with code-style run
 *
 * Revision 1.28  2002/12/10 03:23:27  xirao
 * xi rao
 * DocTermMatrixBuilderSimpletion will handle add Entity into pool events to see if the Entity has
 * the WordFrequencyMapEntityValue. If it has that entity value, it will add them into DocumentTermMatrix
 *
 * Revision 1.27  2002/12/08 04:37:39  mliggett
 * Added caching, local changes to the cached values, dirty bits and write
 * back of locally modified values to CachingProxyEntity.  writeBack() is
 * currently not called anywhere but from CachingProxyEntity's finalizer,
 * but that will change in the next iteration.
 *
 * Also, writeBack() currently only writes back modified EntityValue.  If
 * the bases or attributes have changed, that's never reflected in the proxied
 * Entity.  I'll fix that in a moment, too.
 *
 * Revision 1.26  2002/12/07 15:35:28  alllee
 * EmailRetrieverSimpleton now tags all incoming messages with a
 * WordFrequencyMapEntityValue (there is a bogus "Unhandled content-type." body
 * that gets used with it for non-text messages).  This seems to fix the
 * NullPointerExceptions, but we need to make sure those guys don't get included
 * in the analysis -
 *
 * The touchgraph stuff still needs more mothering with tweaking the probability
 * thresholds and multipliers and wahoo..  my clusters look like hairballs, which
 * is nice, but..
 *
 * Revision 1.25  2002/12/06 19:49:35  alllee
 * Added very rudimentary support for TouchgraphEntityDisplayPanel to use
 * NaiveBayesStrategy's getSimilarityBetween().. currently, similarities are
 * being drawn from email->email - support for email->email sets will be added
 * soon..
 *
 * FIXME:  still need to figure out usable threshold levels, add ability to
 * switch between ClusteringStrategy-s more easily, and fix the Edge factor
 *     (currently 100000 for no apparent reason).
 *
 * Revision 1.24  2002/12/06 14:30:51  alllee
 * Uses the WordFrequencyMapEntityValue ATTRIBUTE_NAME now.
 *
 * Revision 1.23  2002/12/06 12:35:27  alllee
 * Really fixed the bug now.  It turns out that since retrieveMail takes so damn
 * long, and handle() gets invoked and isn't synchronized, we need to assign
 * currentEmailParameters _before_ passing control off to retrieveEmail.  Seems
 * to work.
 *
 * Revision 1.22  2002/12/06 12:04:44  alllee
 * Fixed triplicate download bug in EmailRetrieverSimpleton.  The problem lay in
 * the fact that three Simpletons were firing EmailParametersRequestEvent-s, and
 * the EmailParameterRequestHandlerSimpleton was responsibly dumping
 * EmailParametersEnteredCollectorEvent-s into the Pool as it is wont to do for
 * each EmailParametersRequestEvent that it handled.
 *
 * The fix involves making EmailRetrieverSimpleton cache a copy of the current
 * EmailParametersEntityValue that it is using, and comparing it using .equals(..)
 * to the event-incoming EmailParametersEntityValue.  If the event's
 * EmailParametersEntityValue is the same as the current one, it doesn't connect
 * and try to retrieve mail.  Otherwise, it does.
 *
 * Revision 1.21  2002/12/05 12:29:08  alllee
 * Added attachment of WordFrequencyMapEntityValue attribute to the main email
 * message - see
 *
 * http://lazlo.bloomington.in.us/helium/chat/irclog-20021205.html#nid09
 *
 * for more details.
 *
 * Revision 1.20  2002/11/21 20:48:30  mafarrel
 * Removed reference to Sun's Security provider. The provider should be properly set in the jre/lib/security/java.security file distributed with the 1.4 JRE. If we discover a JRE that does not have this properly set we would not be able to guess the proper provider anyway! In such a case a SecurityException will surely be raised somewhere, and we can deal with notifying the user of a problem at that place.
 *
 * Revision 1.19  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.18  2002/11/17 08:46:53  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 * Revision 1.17  2002/11/16 06:31:04  alllee
 * Cleaned/fixed javadocs and comments.
 *
 * Revision 1.16  2002/11/16 05:11:34  cjdent
 *
 * Adjusted EmailRetrieverSimpleton to add an "EmailDateObject" attribute to
 * the email messages. Uses DateEntityValue. Adds a method called
 * attachDateAttribute() that is called from storeHeaders().
 *
 * This can be used later in constraints created for searching by date.
 *
 * The rest (if any) is codebase-style.
 *
 * Revision 1.15  2002/11/13 08:26:18  cjdent
 *
 * Codebase style.
 *
 * We need to decide if it is in or out. Most people don't seem to be
 * running codebase style before they commit.
 *
 * Revision 1.14  2002/11/11 01:58:26  mliggett
 * Added StartedRetrievingEmailEvent and StoppedRetrievingEmailEvent, and also unit
 * tests for both.
 *
 * Added code to fire these events inside EmailRetrieverSimpleton when we
 * start/stop retrieving email (duh).
 *
 * Revision 1.13  2002/11/10 08:02:41  mafarrel
 * Made it so you can switch between IMAP and IMAPS in the
 * EmailConfiguratorSimpleton. However, it is very ugly code and the
 * solution to this hackiness is to write an IMAPS Provider for JavaMail.
 *
 * Revision 1.12  2002/11/10 06:17:49  mafarrel
 * SSL now works, by default. However, only try IMAP as POP is not
 * implemented. The fix involved changing the
 * mail.imap.socketFactory.class property from
 * javax.net.ssl.SSLSocketFactory to
 * org.datamanager.simpleton.DummySSLSocketFactory.
 *
 * Revision 1.11  2002/11/08 05:58:33  xirao
 *  xi rao
 *  I try to add the ssl function for acess imap mail box.
 *  The code currently won't work well, so I comments the code inside onfigPropertiesForSsl() function.
 *
 * Revision 1.10  2002/11/08 04:18:33  cjdent
 *
 * Some spam has totally bad character set encoding. This was breaking
 * the retriever. It would just stop instead of riding through the
 * error. I've caught an UnsupportedEncodingException such that the message
 * does not get added to the pool, a debug message is written, and the next
 * message is retrieved. There will probably be a few other these.
 *
 * There's probably a better thing to do here: such as default to a certain type of
 * encoding. However, this stuff is _always_ spam so maybe this is a little
 * gift?
 *
 * Revision 1.9  2002/11/08 03:17:42  alllee
 * Refactored EmailParametersRequestHandlerSimpleton into two parts.  The EPRHS
 * still handles EmailParametersRequestEvents - when it receives them, however,
 * it just searches the Pool for an EmailParametersEntityValue and, if it doesn't
 * find a persisted EmailParametersEntityValue, fires a
 * EmailConfigurationRequestEvent (hopefully noticed by an
 * EmailConfiguratorSimpleton).  The EmailConfiguratorSimpleton is in charge of
 * querying the user for appropriate EmailParameters, and fires ye olde
 * EmailParametersEnteredCollectorEvent after it receives them.
 *
 * FIXME: This is getting messy.  Needs a lot better documentation.  We're
 * starting to get a lot of Simpletons onscreen and lots of Simpletons isn't very
 * Simple anymore.  Perhaps it's time for a CompositeSimpleton or some such?
 * Bundling is definitely needed (just for testing purposes even!)
 *
 * Revision 1.8  2002/11/07 11:00:16  alllee
 * Modified to acquiesce with new EmailParametersRequestHandlerSimpleton and play
 * nice and stuff.  Email configuration parameters are no longer hard-coded into
 * the class (using Events hierarchy now).  Minor name-change from MailParameters
 * to EmailParameters.
 *
 * When ERS starts up, it dumps an EmailParametersRequestEvent into the Pool, sez
 * somethin like "I'm lookin for me some EmailParameters".  This should get
 * handled by the EmailRequestHandlerSimpleton (which must obviously be loaded
 * first, and whose dependency chain I mistakenly reversed in the previous CVS
 * commit log).  I go fix that now.  What was Gregory saying about never coding
 * tired again?
 *
 * Revision 1.7  2002/11/04 18:35:39  mafarrel
 * codebase style and new format for ValueClassIsConstraintTester
 *
 * Revision 1.6  2002/11/04 16:48:55  mafarrel
 * ran ant codebase-style...
 *
 * Revision 1.5  2002/11/04 03:52:50  mliggett
 * Major Refactoring reporting for duty, Sir.
 *
 * - Added lots of comments.
 *
 * - Changed some NOTE:s to FIXMe:s
 *
 * - Broke up retrieveMail() into several new methods:
 *   - getConnectedStore()
 *   - addMessageToPool()
 *   - prefetchEnvelopeAndFlags()
 *   - retrieveMessagesToPool()
 *
 * - Added messageIsAlreadyInPool which doesn't exactly do what it says.  It only
 *   tests for a matching Message-Id, which could be the "same message" that
 *   arrived by a different path.  So, it might be slightly different but still get
 *   squashed.  Good enough approximation for now, but this needs improved.
 *   Perhaps a better way to do this is to dump these all into the Pool regardless,
 *   then provide some sort of filtering Constraint or else a Simpleton that flits
 *   around tagging email messages as being "almost the same" as one another.
 *   Maybe another one could then rank "almost the same" messages in order that
 *   most tools could pick the best specimen to display.  Yet another could notice
 *   messages that are truly identical and zap all but one.
 *
 * - Renamed NextSequenceNumber static variable to nextSequenceNumber.
 *
 * Revision 1.4  2002/11/03 17:53:55  mafarrel
 * Ran codebase-style...
 *
 * Revision 1.3  2002/11/03 03:37:09  cjdent
 *
 * 1) Gets lots of mail from INBOX now, instead of mail/nested
 * 2) The content-source attribute is now added to the email Entity
 *    before the Entity is added to the pool. This is necessary to make sure
 *    that BasicEmailViewer can identify a mail message when the Entity is
 *    added to the pool.
 * 3) A little bit of codebase-style
 *
 * Revision 1.2  2002/10/30 03:12:51  mliggett
 * Modified so that message Entities now have the Message-ID as their name by
 * default.  If their is any trouble getting their Message-ID, they get "malformed
 * email N" instead.  See getNextSequenceNumber() for changes to how N is assigned.
 *
 */
