// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import java.io.*;

import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import koala.dynamicjava.interpreter.Interpreter;
import koala.dynamicjava.interpreter.InterpreterException;
import koala.dynamicjava.interpreter.TreeInterpreter;

import koala.dynamicjava.parser.wrapper.JavaCCParserFactory;

import org.datamanager.kernel.Simpleton;


/**
 * Provides a Java interpreter in a window.
 */
public class InterpreterSimpleton extends Simpleton {
    /** Interpreter prompt. */
    public static final String INTERPRETER_PROMPT = "> ";

    /** This Simpleton's toplevel container. */
    private JFrame frame;

    /** The textArea for the interpreter to run in. */
    private final JTextArea textArea = new JTextArea(24, 80);

    /** The Java interpreter. */
    private final TreeInterpreter interpreter = new TreeInterpreter(
                                                        new JavaCCParserFactory());

    public void process() {
        makeFrame();
        frame.setVisible(true);
    }

    /**
     * Returns the text of the line which the cursor is currently on.
     */
    private String getCurrentLine(JTextArea textArea) {
        try {
            int dot = textArea.getCaret().getDot();
            int lineNumber = textArea.getLineOfOffset(dot);
            int start = textArea.getLineStartOffset(lineNumber);
            int end = textArea.getLineEndOffset(lineNumber);

            return (textArea.getText(start, end - start));
        } catch (BadLocationException exception) {
            // FIXME: do something smarter here
            exception.printStackTrace();
        }

        return "";
    }

    public String stripPrompt(String line) {
        return line.substring(INTERPRETER_PROMPT.length());
    }

    public void prompt() {
        textArea.append("\n" + INTERPRETER_PROMPT);
    }

    public void interpret(String line) {
        line = stripPrompt(line);

        try {
            Object result = interpreter.interpret(new StringReader(line), 
                                                  "buffer");

            if (result != null) {
                textArea.append(result.toString());
            }
        } catch (koala.dynamicjava.interpreter.InterpreterException exception) {
            // FIXME: provide a more informative message
            textArea.append("PARSE ERROR.");
            exception.printStackTrace();
        }

        prompt();
    }

    public boolean isInterpreterInput(String line) {
        return 0 == line.indexOf(INTERPRETER_PROMPT);
    }

    /**
     * Lays out the GUI elements.
     */
    private void makeFrame() {
        frame = new JFrame("Interpreter");

        frame.getContentPane().add(textArea, BorderLayout.CENTER);

        prompt();

        textArea.addKeyListener(new InterpreterKeyListener());

        frame.pack();
    }

    /**
     * Listens to a JTextArea for keypresses that indicate it should interpret
     * some text.
     */
    class InterpreterKeyListener extends KeyAdapter {
        public void keyPressed(KeyEvent event) {
            if (KeyEvent.VK_ENTER == event.getKeyCode()) {
                String line = getCurrentLine(textArea);

                if (isInterpreterInput(line)) {
                    interpret(line);
                }
            }
        }
    }
}

/*
 * $Log: InterpreterSimpleton.java,v $
 * Revision 1.15  2002/12/20 20:52:39  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.14  2002/11/21 02:46:25  mliggett
 * Removed commented code.
 *
 * Revision 1.13  2002/11/20 00:25:42  cjdent
 *
 *
 * EmailParametersEntityValue and EmailParameters
 *
 * Addition of SMTPHost field for the outgoing SMTP server
 * Updated the equals method in EmailParametersEntityValue to include the
 *   smtp and from address info. Was this right?
 *
 * Added an SMTP host to the EmailConfiguratorSimpleton
 *
 * SMTPSendSimpleton
 *  subscribes to the EmailParametersEnteredCollectorEvent and fires the
 *  EmailParametersRequestEvent to get SMTP host information.
 *
 *  FIXME: What happens if it is unable to get the info for some reason when
 *  it tries to make a delivery. I've set up a default, which is _not_ a good
 *  long term solution. This is integrated with the overall question of
 *  how to deal with SMTP errors.
 *
 * Rest is more bogus codebase-style
 *
 * Revision 1.12  2002/11/19 06:45:32  cjdent
 *
 * whole tree codebase style
 *
 * Revision 1.11  2002/11/18 05:09:33  xirao
 *         xi rao
 *         change the NoSuchEntityException, NoSuchIdentifierException to extends
 *         the DataManagerException. let find... method throw out the Exception.
 *
 * Revision 1.10  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.9  2002/11/18 02:38:37  mliggett
 * Added BasicEmailReplySimpleton, which currently only listens for the proper
 * event and pops up totally empty emails.  More functionality to come in just
 * a few.
 *
 * Also added this Simpleton to knownspace_config.xml and
 * EmailBundleSimpleton.
 *
 * Revision 1.8  2002/11/17 08:46:55  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 */
