// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.util.ArrayList;
import java.util.List;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.NotEntityConstraint;
import org.datamanager.constraint.RecursiveOrEntityConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.SearchRequestEvent;
import org.datamanager.event.SearchResultsFoundEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.StringEntityValue;

import org.datamanager.tools.Debug;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Simpleton that handles SearchRequestEvents, performing the requested searches and generating
 * SearchResultsFoundEvents.
 *
 * Created: Fri Nov  1 13:17:21 2002
 *
 * @author <a href="mailto:rwgeorge@cs.indiana.edu">Robert W. George</a>
 * @author <a href="mailto:alllee@cs.indiana.edu">Allen Lee</a>
 * @version $Revision: 1.11 $
 *
 */
public class SearchHandlerSimpleton extends Simpleton implements EventHandler {
    private KernelProxy kernelProxy = KernelProxy.getInstance();

    /**
     * Attaches this SearchHandlerSimpleton to the Pool
     */
    public void process() {
        attachToPool();
    }

    /**
     * Attaches this SearchHandlerSimpleton to the Pool
     * with a SearchRequestEvent.
     */
    private void attachToPool() {
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new EventTypeIsConstraint(SearchRequestEvent.class));
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage("DataManagerException caught by " + 
                                  "SearchHandlerSimpleton while attempting " + 
                                  "to attach to pool:" + 
                                  exception.getMessage());
        }
    }

    /**
     * Handle a DataManagerEvent that has been fired.  In this case 
     * a SearchRequestEvent.
     *
     * @param event The SearchRequestEvent to handle. 
     */
    public void handle(final DataManagerEvent event) {
        try {
            SearchRequestEvent searchRequest = (SearchRequestEvent) event;
            EntityConstraint exceptQuery = new NotEntityConstraint(
                                                   new NameEqualsConstraint(
                                                           "SearchResultsEntity"));
            EntityConstraint containQuery = new RecursiveOrEntityConstraint(
                                                    searchRequest.getConstraint());
            EntityConstraint searchQuery = new AndEntityConstraint(containQuery, 
                                                                   exceptQuery);
            Entity[] searchResults = Pool.getDefaultPool().search(searchQuery);

            Entity searchResultsEntity = Entity.create("SearchResultsEntity");

            StringEntityValue value = new StringEntityValue(
                                              searchRequest.getQuery());
            searchResultsEntity.setValue(value);

            for (int i = 0; i < searchResults.length; i++) {
                searchResultsEntity.attach(searchResults[i]);
            }

            kernelProxy.addEntity(searchResultsEntity);

            kernelProxy.addEvent(
                    new SearchResultsFoundEvent(kernelProxy.getEventGenerator(), 
                                                searchResultsEntity));
        } catch (ClassCastException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage(
                    "Incorrect event type handled by SearchHandlerSimpleton");
        } catch (DataManagerException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage("DataManagerException " + 
                                  exception.getMessage() + 
                                  " generated while handling SearchResults");
        }
    }
} // SearchHandlerSimpleton
