// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EntityDisplayRequestEvent;
import org.datamanager.event.SearchRequestEvent;
import org.datamanager.event.SearchResultsFoundEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.AbstractEntityConstraint;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.Debug;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Simpleton that views search results. Not currently used in the application.
 *
 * Created: Sat Nov  2 15:23:15 2002
 *
 * @author Team Helium
 * @version $Revision: 1.10 $
 */
public class SearchResultsViewerSimpleton extends Simpleton
    implements EventHandler {
    private final JList resultsList = new JList(new DefaultListModel());
    private final JFrame frame = new JFrame("Search Results Viewer Simpleton");

    public void process() {
        initializeGuiComponents();
        attachToPool();
    }

    public void handle(final DataManagerEvent event) {
        try {
            // The SearchResultsFoundEvent contains the SearchResultsEntity
            // which has the relevant search results attached to it as its
            // attributes.
            Entity searchResultsEntity = ((SearchResultsFoundEvent) event).getEntity();

            Entity[] searchResults = searchResultsEntity.getAttributes(
                                             EntityConstraint.NONE);
            synchronized (resultsList) {
                resultsList.setListData(searchResults);
            }

            resultsList.validate();
        } catch (DataManagerException exception) {
            exception.printStackTrace();
        } catch (ClassCastException exception) {
            exception.printStackTrace();
            Debug.writeLogMessage("Handled wrong event type: " + event);
        }
    }

    private void attachToPool() {
        // Subscribe to Pool, listening to any and all
        // SearchResultsFoundEvents.
        try {
            Pool.getDefaultPool()
                .subscribe(this, 
                           new EventTypeIsConstraint(
                                   SearchResultsFoundEvent.class));
        } catch (DataManagerException exception) {
            exception.printStackTrace();

            // FIXME: what do we do when we fail to subscribe to the pool?
        }
    }

    private void initializeGuiComponents() {
        final JPanel resultsPanel = new JPanel(new BorderLayout());
        final JScrollPane resultsScrollPane = new JScrollPane(resultsList);
        final Box sideButtonBox = new Box(BoxLayout.Y_AXIS);
        final JButton helpButton = new JButton("Help");
        final JButton closeButton = new JButton("Close");

        helpButton.setMnemonic('h');
        helpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                JOptionPane.showMessageDialog(frame, "Help!");
            }
        });

        closeButton.setMnemonic('c');
        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                frame.dispose();
            }
        });

        sideButtonBox.add(helpButton);
        sideButtonBox.add(closeButton);


        // Sets up JList mouse listener so that search results can be turned
        // into EntityDisplayRequestEvents..
        resultsList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                if (evt.getClickCount() == 2) {
                    int entityIndex = resultsList.locationToIndex(
                                              evt.getPoint());
                    synchronized (resultsList) {
                        Entity selectedEntity = (Entity) resultsList.getModel()
                                                                    .getElementAt(entityIndex);
                        fireDisplayEvent(selectedEntity);
                    }
                }
            }
        });

        resultsPanel.add(new JLabel("Search Results"), BorderLayout.NORTH);
        resultsPanel.add(resultsScrollPane, BorderLayout.CENTER);
        resultsPanel.add(sideButtonBox, BorderLayout.EAST);

        frame.getContentPane().add(resultsPanel);

        frame.setSize(237, 237);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.setVisible(true);
    }

    /** 
     * FIXME:  This method should eventually fire generic
     * EntityDisplayRequestEvents which should be handled by the appropriate
     * EntityDisplaySimpleton.  
     */
    private void fireDisplayEvent(Entity entity) {
        try {
            KernelProxy.getInstance()
                       .addEvent(new EntityDisplayRequestEvent(
                                         KernelProxy.getInstance()
                                                    .getEventGenerator(), 
                                         entity));
        } catch (DataManagerException exception) {
            exception.printStackTrace();

            // FIXME:  Exception handling is weak here.
        }
    }
} // SearchResultsViewerSimpleton
