// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.WindowConstants;
import javax.swing.table.AbstractTableModel;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.event.EntityDisplayRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;

import org.datamanager.passiveentityvalue.StringEntityValue;

import org.datamanager.simpleton.EmailMessageEntityHelper;

import org.datamanager.tools.Toolbox;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * A GUI that displays a list of email messages in a
 * tabular format. Factored out of BasicEmailViewerSimpleton.
 * FIXME: This is the wrong package location for this class,
 * we need to decide what the right one is.
 *
 * @author Team Helium
 * @version $Revision: 1.17 $
 */
public class TabularEmailDisplay {
    /** The String used as the based of the status JLabel. */
    private static String STATUS_STRING = " Email message";

    /** A display table of all email Entities we know about. */
    private JTable entityTable;

    /** A model for the table of email message characteristics (eg From, Subject, Date). */
    private EmailTableModel emailTableModel;

    /** A scrollpane for the entityList. */
    private JScrollPane entityScrollPane;

    /** JPanel graphical element. */
    private JPanel panel;

    /** The Jlabel used for satus info at the top of the table. */
    private JLabel status;

    /**
     * Constructs the TabularEmailDisplay, initializing the JPanel used.
     */
    public TabularEmailDisplay(final String name) {
        makePanel(name);
    }

    /** Returns a reference to this TabularEmailDisplay's JPanel. */
    public JPanel getPanel() {
        return panel;
    }

    /**
     * Fires an EntityDisplayRequestEvent containing the given (email) Entity.
     */
    private void fireDisplayMessageEvent(final Entity message)
                                  throws DataManagerException {
        KernelProxy.getInstance()
           .addEvent(new EntityDisplayRequestEvent(KernelProxy.getInstance()
                                                              .getEventGenerator(), 
                                                   message));
    }

    /**
     * Creates the JFrame and other components that go in it; configures
     * layout.
     */
    private void makePanel(final String name) {
        panel = new JPanel(new BorderLayout());

        emailTableModel = new EmailTableModel();
        entityTable = new JTable(emailTableModel);

        entityScrollPane = new JScrollPane(entityTable);
        panel.add(entityScrollPane, BorderLayout.CENTER);

        status = new JLabel(STATUS_STRING);
        panel.add(status, BorderLayout.NORTH);
        panel.setName(name);


        // FIXME: We also need to listen for a return keypress as well
        // as multiple selections. There are fancy ways to do this that
        // I (cdent) don't want to mess with right now. Xi claims to have
        // some insight.
        entityTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int index = entityTable.rowAtPoint(e.getPoint());

                    try {
                        EmailMessageEntityHelper emailEntityHelper = 
                                (EmailMessageEntityHelper) emailTableModel.getElementAt(
                                        index);
                        fireDisplayMessageEvent(
                                (Entity) emailEntityHelper.getMessage());
                    } catch (DataManagerException exception) {
                        // FIXME: Is there some smarter way to propagate or handle this exception?
                        throw new RuntimeException(exception);
                    }
                }
            }
        });

        entityTable.getTableHeader().addMouseListener(new MouseAdapter() {
            public void mouseClicked(final MouseEvent event) {
                if (event.getClickCount() == 1) {
                    int columnIndex = entityTable.getTableHeader()
                                                 .getColumnModel()
                                                 .getColumnIndexAtX(event.getX());
                    int viewColumnIndex = entityTable.convertColumnIndexToModel(
                                                  columnIndex);
                    emailTableModel.sort(viewColumnIndex);
                    emailTableModel.fireTableStructureChanged();
                }
            }
        });
    }

    /**
     * Sets the list and table models to the provided array of Entities.
     */
    public synchronized void setEmailTableModel(final Entity[] entities) {
        try {
            EmailMessageEntityHelper[] entityHelpers = 
                    new EmailMessageEntityHelper[entities.length];

            for (int i = 0; i < entities.length; i++) {
                entityHelpers[i] = new EmailMessageEntityHelper(entities[i]);
            }

            emailTableModel.setEmailList(Arrays.asList(entityHelpers));
        } catch (DataManagerException exception) {
            // FIXME: bad monkeyexception handling as per usual
            exception.printStackTrace();
            throw new RuntimeException(exception);
        }
    }

    /**
     * Adds an Entity to the EmailTableModel.
     */
    public synchronized void addToEmailTableModel(final Entity entity) {
        emailTableModel.addEmail(entity);
        updateStatusLabel();
    }

    /**
     * Updates the status JLabel to report the number of emails in this
     * list.
     */
    private void updateStatusLabel() {
        final int numberOfEmails = emailTableModel.getRowCount();
        final StringBuffer statusBuffer = new StringBuffer().append(
                                                  numberOfEmails)
                                                            .append(STATUS_STRING);

        if (numberOfEmails != 1) {
            statusBuffer.append("s");
        }

        status.setText(statusBuffer.toString());
    }

    /**
     * The table used to display the email Entities.
     */
    public class EmailTableModel extends AbstractTableModel {
        /** String names for the column fields of this table model. */
        private final String[] COLUMN_NAMES = { "From", "Subject", "Date" };

        /** Number of columns in this data model. */
        private final int NUMBER_OF_COLUMNS = COLUMN_NAMES.length;
        private final int FROM_COLUMN = 0;
        private final int SUBJECT_COLUMN = 1;
        private final int DATE_COLUMN = 2;
        private int currentSortColumn = FROM_COLUMN;
        private boolean isSortedAscending = true;

        /** The list of all emails we know about. */
        private List emailList = new ArrayList();

        public int getColumnCount() {
            return NUMBER_OF_COLUMNS;
        }

        public int getRowCount() {
            return emailList.size();
        }

        /**
         * Returns the object at row rowIndex.
         */
        public synchronized Object getElementAt(final int rowIndex) {
            return emailList.get(rowIndex);
        }

        /**
         * Initiates a sort of the tabular data. Date is a
         * special case. Calls stringSort() or dateSort().
         */
        public synchronized void sort(final int columnIndex) {
            if (columnIndex == currentSortColumn) {
                isSortedAscending = !isSortedAscending;
            } else {
                currentSortColumn = columnIndex;
            }

            if (columnIndex == DATE_COLUMN) {
                dateSort();
            } else {
                stringSort(columnIndex);
            }
        }

        private synchronized void stringSort(final int columnIndex) {
            Collections.sort(emailList, 
                             new Comparator() {
                public int compare(Object a, Object b) {
                    int comparison = 0;

                    if ((a instanceof EmailMessageEntityHelper) && (b instanceof EmailMessageEntityHelper)) {
                        EmailMessageEntityHelper aMessage = 
                                (EmailMessageEntityHelper) a;
                        EmailMessageEntityHelper bMessage = 
                                (EmailMessageEntityHelper) b;

                        comparison = ((StringEntityValue) aMessage.get(
                                              COLUMN_NAMES[columnIndex])).compareTo(
                                             (StringEntityValue) bMessage.get(
                                                     COLUMN_NAMES[columnIndex]));
                    }

                    return isSortedAscending ? comparison : -comparison;
                }
            });
        }

        private synchronized void dateSort() {
            Collections.sort(emailList, 
                             new Comparator() {
                public int compare(Object a, Object b) {
                    int comparison = 0;

                    if ((a instanceof EmailMessageEntityHelper) && (b instanceof EmailMessageEntityHelper)) {
                        EmailMessageEntityHelper aMessage = 
                                (EmailMessageEntityHelper) a;
                        EmailMessageEntityHelper bMessage = 
                                (EmailMessageEntityHelper) b;

                        comparison = (aMessage.getDateObject().compareTo(bMessage.getDateObject()));
                    }

                    return isSortedAscending ? comparison : -comparison;
                }
            });
        }

        public void setEmailList(List emailList) {
            this.emailList = new ArrayList(emailList);
            fireTableRowsInserted(0, lastRow());
            updateStatusLabel();
        }

        public Object getValueAt(final int rowIndex, final int columnIndex) {
            if (columnIndex > NUMBER_OF_COLUMNS) {
                // sanity check
                return null;
            }

            return getHeaderValueAt(COLUMN_NAMES[columnIndex], rowIndex);
        }

        public String getColumnName(final int columnIndex) {
            return COLUMN_NAMES[columnIndex];
        }

        /**
         * Adds the given Entity (which is expected to represent an email) to
         * the table.
         */
        public synchronized void addEmail(final Entity email) {
            try {
                emailList.add(new EmailMessageEntityHelper(email));
                fireTableRowsInserted(lastRow(), lastRow());
            } catch (DataManagerException exception) {
                // FIXME: bad monkey exception handling
                exception.printStackTrace();
                throw new RuntimeException(exception);
            }
        }

        private synchronized String getHeaderValueAt(final String headerName, 
                                                     final int rowIndex) {
            EmailMessageEntityHelper emailEntityHelper = 
                    (EmailMessageEntityHelper) emailList.get(rowIndex);

            EntityValue headerValue = (EntityValue) emailEntityHelper.get(
                                              headerName);

            if (null == headerValue) {
                return "EMPTY!";
            } else {
                return headerValue.toString();
            }
        }

        private final int lastRow() {
            return emailList.size() - 1;
        }
    }
}

/*
 * $Log: TabularEmailDisplay.java,v $
 * Revision 1.17  2002/12/20 20:52:39  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.16  2002/12/06 11:33:02  alllee
 * Now that EmailMessageEntityHelper throws a DataManagerException from both of
 * its constructors, we have to try/catch or re-throw in all these classes that
 * utilize EmailMessageEntityHelper.
 *
 * Revision 1.15  2002/12/02 05:34:48  alllee
 * GuiSimpleton uses the exported Component's getName() as its title now to allow
 * for specific email reply display frames (which are all exported by a single
 * BasicEmailReplySimpleton) having titles specific to the email message's
 * subject being replied to.  It's hacky, and not well documented, yet.
 * Basically, if you wish to have a GuiSimpleton's exported component publish the
 * title to be used with its JInternalFrame representation within a GuiCollector,
 * you have to setName() the component-to-be-exported to the String title that
 * you wish to use.  This may not be optimal for future developers, but it works
 * for now, which I think is what we are aiming for while concentrating on Jisp
 * and clustering.
 *
 * Changed the class-level reference to the the KernelProxy Singleton in
 * EmailGuiCollectorSimpleton to a private final static variable for no apparent
 * reason other than possible hygiene.
 *
 * Revision 1.14  2002/11/21 06:05:51  cjdent
 *
 * Adjusts sort() to call stringSort() or dateSort() depending on which
 * column is clicked. And then in dateSort() sorts on the Date object
 * rather than the date string.
 *
 * Revision 1.13  2002/11/21 04:31:08  alllee
 * No longer uses a top-level JFrame, instead, exports a JPanel() with the same
 * information.
 *
 * Revision 1.12  2002/11/21 03:29:22  alllee
 * First stab at SimpletonGooey compliance.  Lots of FIXMEs to deal with
 * (duplicate frame/panel functionality for now that needs to be removed and
 * other blougats).
 *
 * Revision 1.11  2002/11/19 05:28:15  alllee
 * Finished cleaning up most of the sort bugs that were plaguing us before.
 *
 * Speed is noticeably improved for multiple messages, support for
 * ascending/descending sort orders has been added, and that damn deadlock bug
 * with the uncaught exception freezing the entire GUI (but not the rest of
 * Knownspace, as the persistence thread is still running) has disappeared once
 * again into the inner bowels of somethingorother.  All is well once again, in
 * knownspace land.  Rejoice, my brethren, for tomorrow there shall be endless
 * more tasks to envision as trivial and take hours upon hours to complete :)
 *
 * TODO:
 * Add funky images to the column names so that we know which order a given
 * column name is sorted in.  Start out in some ascending/descending order?
 *
 * Order by Date, not by String representation of Date.
 *
 * Order by Subject, not by Re:: Re:: subject.
 *
 * I'm ready to pass this one on.
 *
 * Revision 1.10  2002/11/18 23:18:49  mliggett
 * Two minor changes to get the build to fire.
 *
 * Revision 1.9  2002/11/18 07:40:28  alllee
 * Added very basic sort() functionality to column headings in the
 * TabularEmailDisplay.  Problem:  very very slow.  More fixes in the future are
 * predictiated.
 *
 * Revision 1.8  2002/11/18 01:51:41  alllee
 * Minor name change from setListModel to setEmailTableModel.
 *
 * Revision 1.7  2002/11/18 01:47:13  alllee
 * Fixed some FIXMES:
 * - added a public setEmailList(List) method that is used by setListModel
 *   (actually, this introduces another fixme, as it should be
 *   setEmailTableModel... crap, another commit soon.)
 * - renamed addToListModel to addToEmailTableModel.
 * - noted that getHeaderValueAt(int) generates spurious "ERROR!" strings in the
 *   BEVS when emails have an empty subject field (legal).
 * - added lastRow() method to simplify the off-by-one deal with emailList.size()
 *   and fireTableRows(Insert|Updated|Deleted) invocations.
 *
 * Revision 1.6  2002/11/17 08:47:01  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 * Revision 1.5  2002/11/17 01:19:18  alllee
 * EntityDisplayRequestEvent handling/firing instead of
 * EmailDisplayRequestEvents. The constraint used currently captures all
 * EntityDisplayRequestEvents, instead of just those Events that contain the
 * relevant Email entities (I didn't know enough about the structure of the
 * message Entity to play with this as of now, but will fix this later unless
 * someone else steps right up).  Added a getMessage() method to
 * EmailMessageEntityHelper so that EmailDisplaySimpleton doesn't need a
 * class-level reference to the email message Entity.
 *
 * Revision 1.4  2002/11/14 06:10:55  cjdent
 *
 * Adjusted TabularEmailDisplay so that it announces in the top JLabel how many
 * emails it is currently holding. This is helpful little cognitive aid. I found
 * myself constantly wondering "how many of those are there" and now I know.
 *
 * Also codebase-style.
 *
 * Revision 1.3  2002/11/07 06:54:51  cjdent
 *
 * Committing this once again because I still didn't get the right version
 * in. This overwrites Xi's change but this is intentional (we talked about
 * it.)
 *
 *
 */
