// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.StringValueContainsConstraint;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.SearchRequestEvent;
import org.datamanager.event.SearchResultsFoundEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.GuiSimpleton;

import org.datamanager.tools.Debug;
import org.datamanager.tools.Toolbox;
import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * Search simpleton for text-based string searches of entities.
 * Displays a query box with a search button and fires SearchRequestEventS.
 *
 *
 * @author Team Helium
 * @version $Revision: 1.16 $
 *
 */
public class TextSearchSimpleton extends GuiSimpleton {
    private final static String QUERY_EMPTY_MESSAGE = "Please enter the keyword(s) you wish to use in the search.";
    private final static String HELP_URL_STRING = "http://helium.knownspace.org/user/manual.html#searching";

    /** This Simpleton's Kernel Proxy */
    private KernelProxy kernelProxy = KernelProxy.getInstance();

    /** The Simpleton's exportable canvas. */
    private JPanel panel;

    /** Search/Close/Help Buttons. */
    private JButton searchButton;

    /** Help context button. */
    private JButton helpButton;

    /** Search TextField. */
    private JTextField searchTextField;

    public void process() {
        initializeGuiComponents();
    }

    public void initializeGuiComponents() {
        // Local GUI containers
        Box sideButtonPanelBox = new Box(BoxLayout.X_AXIS);
        Box searchBox = new Box(BoxLayout.Y_AXIS);

        panel = new JPanel(new BorderLayout());
        panel.setName("Text Search Simpleton");

        helpButton = new JButton("Help");
        helpButton.setMnemonic('h');
        helpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                Toolbox.launchURL(HELP_URL_STRING);
            }
        });

        searchButton = new JButton("Search");
        searchButton.setMnemonic('s');
        searchButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                search();
            }
        });

        searchTextField = new JTextField();
        searchTextField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                search();
            }
        });

        sideButtonPanelBox.add(helpButton);

        searchBox.add(new JLabel("Search String:"));
        searchBox.add(searchTextField);

        panel.add(sideButtonPanelBox, BorderLayout.EAST);
        panel.add(searchBox, BorderLayout.NORTH);
        panel.add(searchButton, BorderLayout.SOUTH);
        super.exportComponent(panel, new Rectangle(0, 250, 200, 125));
    }

    /**
     * search() generates a SearchRequestEvent containing the search terms.
     * This event should be handled by a SearchHandlerSimpleton, which will
     * search the pool with the appropriate compound AndConstraint.  The
     * SearchHandlerSimpleton then generates a SearchResultsFoundEvent
     * containing the appropriate Entities.  This Event should be handled by a
     * SearchResultsDisplaySimpleton which will display the appropriate results.
     *
     */
    public void search() {
        final String query = searchTextField.getText();

        if (query.trim().length() == 0) {
            JOptionPane.showMessageDialog(panel, QUERY_EMPTY_MESSAGE);
        } else {
            EntityConstraint textConstraint = new StringValueContainsConstraint(
                                                      query);

            try {
                kernelProxy.addEvent(
                        new SearchRequestEvent(kernelProxy.getEventGenerator(), 
                                               textConstraint, 
                                               "Text Search: " + query));
            } catch (DataManagerException exception) {
                exception.printStackTrace();
                Debug.writeLogMessage("DataManagerException caught in " + 
                                      "TextSearchSimpleton while trying to query " + 
                                      query + ".");
            }
        }
    }

    public String getTitle() {
        return "Text Search Simpleton";
    }
} // TextSearchSimpleton
