// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.Rectangle;

import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeStringMatchConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.GuiSimpleton;
import org.datamanager.kernel.Pool;

import org.datamanager.passiveentityvalue.WordFrequencyMapEntityValue;

import org.datamanager.simpleton.TouchgraphEntityDisplayPanel;

import org.datamanager.tools.Debug;
import org.datamanager.tools.Toolbox;


/**
 * Provides a GuiSimpleton containing a TouchgraphEntityDisplayPanel
 * that displays TouchgraphEntityNodes. 
 *
 * Email message Entities are found for display by searching
 * the Pool for existing message Entities as well as listening
 * for AddEntityEvents that contain message Entities.
 *
 */
public class TouchgraphEmailDisplaySimpleton extends GuiSimpleton
    implements EventHandler {
    private TouchgraphEntityDisplayPanel entityDisplayPanel;

    /**
     * Subscribes to the pool to hear about new entities and
     * searches the pool for existing entities. Starts the
     * GuiComponent that is this GuiSimpleton.
     */
    public void process() {
        try {
            final EntityConstraint emailConstraint = new AttributeStringMatchConstraint(
                                                             EmailMessageEntityHelper.EMAIL_ATTRIBUTE_NAME, 
                                                             EmailMessageEntityHelper.EMAIL_ATTRIBUTE_VALUE);
            final EventConstraint isEmailAndNewEntityConstraint = 
                    new AndEventConstraint(
                            new EventContainsEntityConstraint(emailConstraint), 
                            new EventTypeIsConstraint(AddEntityPoolEvent.class));
            Pool.getDefaultPool()
                .subscribe(this, isEmailAndNewEntityConstraint);

            // suck in the list of existing email entities
            Entity[] entityArray = Pool.getDefaultPool()
                                       .search(emailConstraint);

            startDisplayPanel(entityArray);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("error in kernel!");
            exception.printStackTrace();
        }
    }

    /**
     * Adds the email message specified in the event
     * to a TouchgraphEmailDisplay.
     */
    public void handle(DataManagerEvent dataManagerEvent) {
        Entity entity = ((AddEntityPoolEvent) dataManagerEvent).getEntity();
        addEntityToPanel(entity);
    }

    /**
     * Creates and exports the TouchgraphEntityDisplayPanel
     * graphical Component. Adds entities to the display.
     */
    private void startDisplayPanel(Entity[] entities) {
        entityDisplayPanel = new TouchgraphEntityDisplayPanel();
        super.exportComponent(entityDisplayPanel, 
                              new Rectangle(345, 0, 655, 675));
        entityDisplayPanel.addEntities(entities);
    }

    /**
     * Adds a single entity to the TouchgraphEntityDisplayPanel.
     */
    private void addEntityToPanel(Entity entity) {
        entityDisplayPanel.addEntity(entity);
    }

    public String getTitle() {
        return "Touchgraph Viewer Simpleton";
    }
}

/*
 * $Log: TouchgraphEmailDisplaySimpleton.java,v $
 * Revision 1.8  2002/12/20 20:52:39  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.7  2002/12/20 08:29:50  cjdent
 *
 * Cleanups of the javadoc and import kind.
 *
 * Revision 1.6  2002/12/16 10:38:41  cjdent
 *
 * Removed LSA DocumentMatrix calculation calls from
 * TouchgraphEmailDisplaySimpleton and put it in the clustering
 * strategy, via TouchgraphEntityDisplayPanel
 *
 * Revision 1.5  2002/12/16 05:17:28  xirao
 * xi rao
 *         This is new strategy in processing. You probably will run out memory.
 *         The lsa is not implemented correctly. I am still working on it.
 *
 * Revision 1.4  2002/12/11 08:39:22  cjdent
 *
 * Adjusted the default layout of the various GuiCollector components and the
 * collector itself to be a larger window with things more tightly
 * packed. For the sake of the demo and also because with touchgraph
 * in there we need more space.
 *
 * Revision 1.3  2002/12/05 21:21:18  cjdent
 *
 * Updates to TouchgraphEmailDisplayPanel to use a very simple
 * method of pairwise document comparisons. Some sort of
 * completely wrong g-square meaure of variance with a bit of
 * bayesian probability is mixed in.
 *
 * THE MATH IS NOT RIGHT
 *
 * but it does do some interesting things.
 *
 * It's also not in the right place it should be in its own
 * class, perhaps out in the interfaces Allen is working on.
 *
 * I put it in here to test it out and cuz it is simple.
 *
 * The math creates a value. The closer to one it is, the more
 * similar the docs should be. I've set a very high threshold for
 * create edges.
 *
 * This needs some serious tweaking.
 *
 * In the process I bandaided the ConcurrentModificationException
 * problem by catching it. We don't care about it, so we can
 * just breeze by it. Messy.
 *
 * I'll continue working on the math in my perl prototype
 *
 * Revision 1.2  2002/12/05 07:20:01  cjdent
 *
 * Update to Touchgraph EmailDisplaySimpleton and EntityDisplayPanel
 * to support the features provided by EntityNavigateUI and
 * TouchgraphEntity Node and Edge.
 *
 * Major refactoring needed as this was done in a very
 * seat of the pants fashion.
 *
 * More in a forthcoming summary posting.
 *
 * Revision 1.1  2002/12/05 02:14:22  cjdent
 *
 * Adding a very rudimentary pass at including touchgraph support. This
 * doesn't do anything fancy yet. If you want to try it do this:
 *
 *   run helium
 *   run the EmailGui
 *   run Touchgraph Email Display Simpleton
 *
 * you'll get three little blips on the screen. That's it for now.
 *
 * TouchgraphEntityDisplayPanel extends a GLPanel in touchgraph because
 * of the way the navigation of the map is managed is rather tightly
 * coupled. This can be fixed eventually. I'm going for the get it
 * working quickly mode at the moment.
 *
 * Next will use real email entities rather than ones created for fakery.
 *
 *
 */
