// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import com.touchgraph.graphlayout.Edge;
import com.touchgraph.graphlayout.GLPanel;
import com.touchgraph.graphlayout.LocalityUtils;
import com.touchgraph.graphlayout.Node;
import com.touchgraph.graphlayout.TGException;
import com.touchgraph.graphlayout.TGLensSet;
import com.touchgraph.graphlayout.TGPanel;
import com.touchgraph.graphlayout.interaction.GLNavigateUI;
import com.touchgraph.graphlayout.interaction.HVScroll;
import com.touchgraph.graphlayout.interaction.LocalityScroll;
import com.touchgraph.graphlayout.interaction.RotateScroll;
import com.touchgraph.graphlayout.interaction.TGUIManager;
import com.touchgraph.graphlayout.interaction.ZoomScroll;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.lang.Math;

import java.util.ConcurrentModificationException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;

import org.datamanager.clustering.strategy.ClusteringStrategy;
import org.datamanager.clustering.strategy.bayes.NaiveBayesStrategy;
import org.datamanager.clustering.strategy.gsquared.GSquaredStrategy;
import org.datamanager.clustering.strategy.lsa.LsaStrategy;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;

import org.datamanager.passiveentityvalue.WordFrequencyMapEntityValue;

import org.datamanager.simpleton.TouchgraphEntityEdge;
import org.datamanager.simpleton.TouchgraphEntityNode;

import org.datamanager.tools.Debug;
import org.datamanager.tools.Toolbox;


/**
 * A class for displaying Entities with touchgraph.
 * Copies from GLPanel in the touchgraph distribution 
 * to create a visual display of entites. Uses a 
 * ClusteringStrategy to provide a way of calculating
 * associative edges between TouchgraphEntityNodes
 * that represent messages.
 *
 * FIXME: this copying business sucks, but the navigation
 * thingies that GLPanel provides needs a GLPanel passed
 * in. It's especially sucky since GLPanel does all sorts
 * of public method stuff that ought to be private. The 
 * extending is especially grumpifying because so much
 * of it is actually copying and copying of grungy 
 * stuff. In other words major refactoring desired.
 */
public class TouchgraphEntityDisplayPanel extends JPanel {
    /** The name of this panel for use by the GuiCollectorSimpleton. */
    private static final String TOUCHGRAPH_PANEL_NAME = 
            "Touchgraph Email Display";

    /** The color of the background of the panel. */
    private static final Color DEFAULT_BACKGROUND_COLOR = Color.black;
    public String zoomLabel = "Zoom"; // label for zoom menu item
    public String rotateLabel = "Rotate"; // label for rotate menu item
    public String localityLabel = "Locality"; // label for locality menu item
    private TGPanel tgPanel;
    private TGLensSet tgLensSet;
    private TGUIManager tgUIManager;
    private HVScroll hvScroll;
    private ZoomScroll zoomScroll;
    private LocalityScroll localityScroll;
    private RotateScroll rotateScroll;
    private Hashtable scrollBarHash;
    private JPopupMenu panelPopup;
    private Color defaultColor = Color.lightGray;
    private ClusteringStrategy clusteringStrategy;
    private JButton updateButton;

    /**
     * Creates a new panel and sets up the required scrollbars,
     * lenses, and panels.
     */
    public TouchgraphEntityDisplayPanel() {
        scrollBarHash = new Hashtable();
        tgLensSet = new TGLensSet();
        tgPanel = new TGPanel();
        hvScroll = new HVScroll(tgPanel, tgLensSet);
        zoomScroll = new ZoomScroll(tgPanel);
        rotateScroll = new RotateScroll(tgPanel);
        localityScroll = new LocalityScroll(tgPanel);

        initialize();

        clusteringStrategy = LsaStrategy.getInstance();
    }

    /**
     * Returns the node and edge manipulaton panel. Used
     * by the navigator UI.
     */
    public JPopupMenu getPopup() {
        return panelPopup;
    }

    /**
     * Builds up the panels, lenses and users interfaces.
     */
    private void initialize() {
        buildPanel();
        buildLens();
        tgPanel.setLensSet(tgLensSet);
        addUIs();


        //Select first node, so hiding works
        tgPanel.setSelect(tgPanel.getGES().getFirstNode());
        tgPanel.setBackColor(DEFAULT_BACKGROUND_COLOR);
        setLocalityRadius(LocalityUtils.INFINITE_LOCALITY_RADIUS);
        setName(TOUCHGRAPH_PANEL_NAME);
        setVisible(true);
    }

    /**
     * Builds the lens view of this panel.
     */
    public void buildLens() {
        tgLensSet.addLens(hvScroll.getLens());
        tgLensSet.addLens(zoomScroll.getLens());


        //tgLensSet.addLens(hyperScroll.getLens());
        tgLensSet.addLens(rotateScroll.getLens());
        tgLensSet.addLens(tgPanel.getAdjustOriginLens());
    }

    /**
     * Builds this graphical panel.
     */
    private void buildPanel() {
        final JScrollBar horizontalSB = hvScroll.getHorizontalSB();
        final JScrollBar verticalSB = hvScroll.getVerticalSB();
        final JScrollBar zoomSB = zoomScroll.getZoomSB();
        final JScrollBar rotateSB = rotateScroll.getRotateSB();
        final JScrollBar localitySB = localityScroll.getLocalitySB();

        setLayout(new BorderLayout());

        JPanel scrollPanel = new JPanel();
        scrollPanel.setBackground(defaultColor);
        scrollPanel.setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        final JPanel topPanel = new JPanel();
        topPanel.setBackground(defaultColor);
        topPanel.setLayout(new GridBagLayout());
        c.gridy = 0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.weightx = 0;
        c.insets = new Insets(0, 10, 0, 10);

        updateButton = new JButton("Update Layout");
        updateButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                updateEdges();
            }
        });

        topPanel.add(updateButton, c);

        c.insets = new Insets(0, 0, 0, 0);
        c.gridx = 1;
        c.weightx = 1;

        scrollBarHash.put(zoomLabel, zoomSB);
        scrollBarHash.put(rotateLabel, rotateSB);
        scrollBarHash.put(localityLabel, localitySB);

        JPanel scrollselect = scrollSelectPanel(
                                      new String[] {
            zoomLabel, rotateLabel, localityLabel
        });
        scrollselect.setBackground(defaultColor);
        topPanel.add(scrollselect, c);

        add(topPanel, BorderLayout.NORTH);

        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 1;
        c.weighty = 1;
        scrollPanel.add(tgPanel, c);

        c.gridx = 1;
        c.gridy = 1;
        c.weightx = 0;
        c.weighty = 0;
        scrollPanel.add(verticalSB, c);

        c.gridx = 0;
        c.gridy = 2;
        scrollPanel.add(horizontalSB, c);

        add(scrollPanel, BorderLayout.CENTER);

        panelPopup = new JPopupMenu();
        panelPopup.setBackground(defaultColor);

        JMenuItem menuItem = new JMenuItem("Toggle Controls");
        ActionListener toggleControlsAction = new ActionListener() {
            boolean controlsVisible = true;

            public void actionPerformed(ActionEvent e) {
                controlsVisible = !controlsVisible;
                horizontalSB.setVisible(controlsVisible);
                verticalSB.setVisible(controlsVisible);
                topPanel.setVisible(controlsVisible);
            }
        };

        menuItem.addActionListener(toggleControlsAction);
        panelPopup.add(menuItem);
    }

    /**
     * Adds the Toughgraph User Interfaces to the
     * Touchgraph User Interface Manager. We only have
     * one but it is possible to have many modes and
     * switch between them.
     */
    private void addUIs() {
        tgUIManager = new TGUIManager();

        EntityNavigateUI navigateUI = new EntityNavigateUI(this);
        tgUIManager.addUI(navigateUI, "Navigate");
        tgUIManager.activate("Navigate");
    }

    /**
     * Adds a provided array of Entities to this panel.
     */
    public void addEntities(Entity[] entities) {
        for (int i = 0; i < entities.length; i++) {
            addEntity(entities[i]);
        }
    }

    private ClusteringStrategy getClusteringStrategy() {
        return clusteringStrategy;
    }

    /**
     * Adds the provided single entity to this panel
     * and to the clustering strategy.
     */
    public synchronized void addEntity(Entity entity) {
        TouchgraphEntityNode entityNode;


        // FIXME: this will blowup on anything other than lsa
        getClusteringStrategy().addEntity(entity);

        try {
            entityNode = new TouchgraphEntityNode(entity);
            tgPanel.addNode(entityNode);
        } catch (TGException exception) {
            // FIXME: will we ever have excellent exception handling
            Debug.writeLogMessage("Exception adding Nodes to touchgraph: " + 
                                  exception);
            throw new RuntimeException(exception);
        } catch (DataManagerException exception) {
            Debug.writeLogMessage("Exception getting entity node: " + 
                                  exception);
            throw new RuntimeException(exception);
        }

        // do this on deman rather than every time
        //updateEdges(entityNode);
    }

    /**
     * Updates edges for all the nodes in the display. First
     * delete all the edges.
     */
    private synchronized void updateEdges() {
        // FIXME: this is causing concurrent modification problems

        /*
        Iterator edgeIterator = tgPanel.getAllEdges();
        // FIXME: it would be grand to have a null iterator
        if (edgeIterator != null) {
            while (edgeIterator.hasNext()) {
                tgPanel.deleteEdge((TouchgraphEntityEdge) edgeIterator.next());
            }
        }
        */
        Iterator nodeIterator = tgPanel.getAllNodes();

        if (nodeIterator != null) {
            while (nodeIterator.hasNext()) {
                updateEdges((TouchgraphEntityNode) nodeIterator.next());
            }
        }
    }

    /**
     * Updates the edges in the display to include
     * any connection between the provided node
     * and the rest of the nodes.
     *
     * FIXME: this is inefficient. We should only udpate
     * edges when it is worth our while.
     *
     * FIXME: we get ConcurrentModificationExceptions in
     * this because the list behind getAllNodes is fail-fast.
     * We don't care really, we just want to see the action.
     * I'm not sure what to do. Catching the exception seems
     * bogus -cdent
     */
    private void updateEdges(TouchgraphEntityNode entityNode) {
        Iterator nodeIterator = tgPanel.getAllNodes();

        while (nodeIterator.hasNext()) {
            TouchgraphEntityNode otherEntityNode = entityNode;

            try {
                otherEntityNode = (TouchgraphEntityNode) nodeIterator.next();
            } catch (ConcurrentModificationException exception) {
                // We don't care in this case
                exception.printStackTrace();
            }

            if (!entityNode.equals(otherEntityNode)) {
                if (tgPanel.findEdge(entityNode, otherEntityNode) == null) {
                    // FIXME: this will query the matrix
                    // eventually. Zero will be the default, I guess
                    double similarity = getClusteringStrategy()
                                            .getSimilarityBetween(entityNode.getEntity(), 
                                                                  otherEntityNode.getEntity());

                    //System.err.println("similarity is: " + similarity);
                    // FIXME, do threshold checking here
                    if (similarity >= getClusteringStrategy().getThreshold()) {
                        // FIXME: length calculation is seat of pants, not science.
                        double edgeLength = (1000 / similarity) * (100 / tgPanel.getNodeCount());
                        TouchgraphEntityEdge entityEdge = 
                                new TouchgraphEntityEdge(entityNode, 
                                                         otherEntityNode, 
                                                         (int) edgeLength);
                        tgPanel.addEdge(entityEdge);
                    }
                }
            }
        }
    }

    /**
     * Fakes a similarity between two entities value.
     *
     * Actually for the moment tries to do a sort of G-square
     * Bayesian comparison.
     *
     * FIXME: this is the not the place for this code. Move
     * out to Clustering strategies, as NaiveBayesStrategy is done.
     */
    private double getSimilarity(Entity entityOne, Entity entityTwo) {
        //return Math.random();
        WordFrequencyMapEntityValue mapEntityValueOne;
        WordFrequencyMapEntityValue mapEntityValueTwo;

        try {
            mapEntityValueOne = (WordFrequencyMapEntityValue) Toolbox.getFirstAttributeNamed(
                                                                      entityOne, 
                                                                      WordFrequencyMapEntityValue.ATTRIBUTE_NAME)
                                                                     .getValue();
            mapEntityValueTwo = (WordFrequencyMapEntityValue) Toolbox.getFirstAttributeNamed(
                                                                      entityTwo, 
                                                                      WordFrequencyMapEntityValue.ATTRIBUTE_NAME)
                                                                     .getValue();
        } catch (DataManagerException exception) {
            // could not the value, let's default to 0
            return 0.0;
        } catch (NullPointerException exception) {
            // FIXME: this catches the case where we have a
            // message with no frequency info, I think. not
            // good way to go
            return 0.0;
        }

        Map mapOne = mapEntityValueOne.getWordFrequencyMap();
        Map mapTwo = mapEntityValueTwo.getWordFrequencyMap();
        double wordCountOne = getWordCountFromMap(mapOne);
        double wordCountTwo = getWordCountFromMap(mapTwo);
        double totalWordCount = wordCountOne + wordCountTwo;

        double probability = 0;

        Set mapTwoSet = mapTwo.keySet();
        Iterator mapTwoIterator = mapTwoSet.iterator();

        while (mapTwoIterator.hasNext()) {
            Object obj = mapTwoIterator.next();

            if (obj instanceof String) {
                String word = (String) obj;
                Integer wordCountInteger = (Integer) mapOne.get(word);
                double wordCount = 0;

                if (wordCountInteger != null) {
                    wordCount = wordCountInteger.doubleValue();
                }

                if (wordCount > 0) {
                    double wordFrequencyOne = ((Integer) mapOne.get(word)).doubleValue();
                    double wordFrequencyTwo = ((Integer) mapTwo.get(word)).doubleValue();
                    probability += ((wordFrequencyOne / totalWordCount) * (Math.log((wordFrequencyOne / totalWordCount) / (wordFrequencyTwo / totalWordCount))));
                }
            }
        }

        // FIXME: cantankerous
        if (probability == 0) {
            return probability;
        }

        return 1 - Math.abs(probability);
    }

    private double getWordCountFromMap(Map wordMap) {
        Set wordSet = wordMap.keySet();
        Iterator wordIterator = wordSet.iterator();

        int sum = 0;

        while (wordIterator.hasNext()) {
            Object obj = wordIterator.next();

            if (obj instanceof String) {
                Integer wordCount = (Integer) wordMap.get((String) obj);
                sum += wordCount.doubleValue();
            }
        }

        return sum;
    }

    /** Return the TGPanel used with this GLPanel. */
    public TGPanel getTGPanel() {
        return tgPanel;
    }

    // navigation .................

    /** Return the HVScroll used with this GLPanel. */
    public HVScroll getHVScroll() {
        return hvScroll;
    }

    /** Sets the horizontal offset to p.x, and the vertical offset to p.y
      * given a Point <tt>p<tt>.
      */
    public void setOffset(Point p) {
        hvScroll.setOffset(p);
    }

    /** Return the horizontal and vertical offset position as a Point. */
    public Point getOffset() {
        return hvScroll.getOffset();
    }

    // rotation ...................

    /** Return the RotateScroll used with this GLPanel. */
    public RotateScroll getRotateScroll() {
        return rotateScroll;
    }

    /** Set the rotation angle of this GLPanel (allowable values between 0 to 359). */
    public void setRotationAngle(int angle) {
        rotateScroll.setRotationAngle(angle);
    }

    /** Return the rotation angle of this GLPanel. */
    public int getRotationAngle() {
        return rotateScroll.getRotationAngle();
    }

    // locality ...................

    /** Return the LocalityScroll used with this GLPanel. */
    public LocalityScroll getLocalityScroll() {
        return localityScroll;
    }

    /** Set the locality radius of this TGScrollPane
      * (allowable values between 0 to 4, or LocalityUtils.INFINITE_LOCALITY_RADIUS).
      */
    public void setLocalityRadius(int radius) {
        localityScroll.setLocalityRadius(radius);
    }

    /** Return the locality radius of this GLPanel. */
    public int getLocalityRadius() {
        return localityScroll.getLocalityRadius();
    }

    // zoom .......................

    /** Return the ZoomScroll used with this GLPanel. */
    public ZoomScroll getZoomScroll() {
        return zoomScroll;
    }

    /** Set the zoom value of this GLPanel (allowable values between -100 to 100). */
    public void setZoomValue(int zoomValue) {
        zoomScroll.setZoomValue(zoomValue);
    }

    /** Return the zoom value of this GLPanel. */
    public int getZoomValue() {
        return zoomScroll.getZoomValue();
    }

    // ....
    private JPanel scrollSelectPanel(String[] scrollBarNames) {
        final JComboBox scrollCombo = new JComboBox(scrollBarNames);
        scrollCombo.setBackground(defaultColor);
        scrollCombo.setPreferredSize(new Dimension(80, 20));
        scrollCombo.setSelectedIndex(0);

        final JScrollBar initialSB = (JScrollBar) scrollBarHash.get(
                                             scrollBarNames[0]);
        scrollCombo.addActionListener(new ActionListener() {
            JScrollBar currentSB = initialSB;

            public void actionPerformed(ActionEvent e) {
                JScrollBar selectedSB = (JScrollBar) scrollBarHash.get(
                                                (String) scrollCombo.getSelectedItem());

                if (currentSB != null) {
                    currentSB.setVisible(false);
                }

                if (selectedSB != null) {
                    selectedSB.setVisible(true);
                }

                currentSB = selectedSB;
            }
        });

        final JPanel sbp = new JPanel(new GridBagLayout());
        sbp.setBackground(defaultColor);

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        sbp.add(scrollCombo, c);
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1;
        c.insets = new Insets(0, 10, 0, 17);
        c.fill = GridBagConstraints.HORIZONTAL;

        for (int i = 0; i < scrollBarNames.length; i++) {
            JScrollBar sb = (JScrollBar) scrollBarHash.get(scrollBarNames[i]);

            if (sb == null) {
                continue;
            }

            if (i != 0) {
                sb.setVisible(false);
            }


            //sb.setMinimumSize(new Dimension(200,17));
            sbp.add(sb, c);
        }

        return sbp;
    }
}