// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import com.touchgraph.graphlayout.Node;
import com.touchgraph.graphlayout.TGPanel;

import java.awt.Color;
import java.awt.Graphics;

import java.util.Date;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;

import org.datamanager.passiveentityvalue.DateEntityValue;

import org.datamanager.tools.Toolbox;


/**
 * TouchgraphEntityNode is a touchgraph node representing
 * an Entity. This code is based on the WikiNode from
 * the Touchgraph WikiBrowser.
 *
 * The Node label is the Subject of the mail message.
 *
 * TouchgraphEntityNodes have dynamic color and size. 
 * The color is an indicator of how old the message the
 * node represents is compared to the current time. The
 * size is inversely proportional to the number of 
 * Nodes currently being shown.
 *
 * FIXME: that size thing is sort of meaningless chrome.
 */
public class TouchgraphEntityNode extends com.touchgraph.graphlayout.Node {
    /**
     * Number of days that make up the range in which 
     * nodes should have a color other than the default.
     */
    private static final int RANGE_OF_DAYS = 365;

    /** Red value of the newest possible email message. */
    private static final int WHITE_RED_VALUE = 255;

    /** Green value of the newest possible email message. */
    private static final int WHITE_GREEN_VALUE = 255;

    /** Blue value of the newest possible email message. */
    private static final int WHITE_BLUE_VALUE = 255;

    /** Red value of the messages older than RANGE_OF_DAYS. */
    private static final int BLUE_RED_VALUE = 0;

    /** Green value of the messages older than RANGE_OF_DAYS. */
    private static final int BLUE_GREEN_VALUE = 0;

    /** Blue value of the messages older than RANGE_OF_DAYS. */
    private static final int BLUE_BLUE_VALUE = 205;

    /** Default color for a node. */
    private static final Color DEFAULT_NODE_COLOR = new Color(BLUE_RED_VALUE, 
                                                              BLUE_GREEN_VALUE, 
                                                              BLUE_BLUE_VALUE);

    /** Minimum possible diameter of a node. */
    private static final int MIN_DIAMETER = 10;
    private int diameter = MIN_DIAMETER;
    private Entity entity;

    // FIXME: why are these here?
    int hintWidth = 300;
    int hintHeight = 300;

    public TouchgraphEntityNode(Entity entity) throws DataManagerException {
        super(entity.getName(), 
              (null == (Entity) Toolbox.getFirstAttributeNamed(entity, 
                                                               "Subject"))
              ? "No Subject Specified"
              : (String) ((Entity) Toolbox.getFirstAttributeNamed(entity, 
                                                                  "Subject")).getValue()
                                                                                   .toString());
        this.entity = entity;


        // set a reasonable color for node
        setColorByDate(RANGE_OF_DAYS);
    }

    /**
     * Sets the Entity represented by this node.
     */
    public void setEntity(Entity entity) {
        this.entity = entity;
    }

    /**
     * Retrieves the Entity represented by this node.
     */
    public Entity getEntity() {
        return entity;
    }

    /**
     * Sets the color of the node based on how old it 
     * is. If it is older than daysRange, the color
     * is set to a default.
     */
    private void setColorByDate(int daysRange) throws DataManagerException {
        setBackColor(DEFAULT_NODE_COLOR);

        final int yearSeconds = daysRange * 24 * 60 * 60;

        int redComponentValue;
        int greenComponentValue;
        int blueComponentValue;

        Date date = ((DateEntityValue) ((Entity) Toolbox.getFirstAttributeNamed(
                                                getEntity(), "EmailDateObject")).getValue()).getDate();

        if (date != null) {
            double now = (double) (System.currentTimeMillis() / 1000);
            double then = (double) (date.getTime() / 1000);

            if ((then + yearSeconds) < now) {
                redComponentValue = BLUE_RED_VALUE;
                greenComponentValue = BLUE_GREEN_VALUE;
                blueComponentValue = BLUE_BLUE_VALUE;
            } else {
                double oldness = now - then;


                // FIXME: look at the cutting and pasting!
                redComponentValue = WHITE_RED_VALUE - 
                                    (int) ((oldness / yearSeconds) * (WHITE_RED_VALUE - BLUE_RED_VALUE));
                greenComponentValue = WHITE_GREEN_VALUE - 
                                      (int) ((oldness / yearSeconds) * (WHITE_GREEN_VALUE - BLUE_GREEN_VALUE));
                blueComponentValue = WHITE_BLUE_VALUE - 
                                     (int) ((oldness / yearSeconds) * (WHITE_BLUE_VALUE - BLUE_BLUE_VALUE));
            }

            setBackColor(
                    new Color(redComponentValue, greenComponentValue, 
                              blueComponentValue));
        }
    }

    /**
     * Paints the Node. If the node has attached edges that
     * are hidden, a small label of the number of hidden
     * edges is displayed.
     */
    public void paint(Graphics g, TGPanel tgPanel) {
        if (!intersects(tgPanel.getSize())) {
            return;
        }

        paintNodeBody(g, tgPanel);

        if (visibleEdgeCount() < edgeCount()) {
            int ix = (int) drawx;
            int iy = (int) drawy;
            int h = getHeight();
            int w = getWidth();
            int tagX = (ix + ((w - 7) / 2)) - 2 + (w % 2);
            int tagY = iy - (h / 2) - 2;
            char character;
            int hiddenEdgeCount = edgeCount() - visibleEdgeCount();
            character = (hiddenEdgeCount < 9)
                        ? (char) ('0' + hiddenEdgeCount) : '*';
            paintSmallTag(g, tgPanel, tagX, tagY, Color.red, Color.white, 
                          character);
        }
    }

    /**
     * Paints the background of the Node.
     */
    public void paintNodeBody(Graphics g, TGPanel tgPanel) {
        setDiameter(tgPanel);

        int ix = (int) drawx;
        int iy = (int) drawy;

        int x = ix - (diameter / 2);
        int y = iy - (diameter / 2);

        g.setColor(getBackColor());

        g.fillOval(x, y, diameter, diameter);
        g.setColor(tgPanel.getBackground());
        g.drawOval(x, y, diameter, diameter);
    }

    /**
     * Calculates a diameter for the node based on the number
     * of nodes in the display.
     */
    private void setDiameter(TGPanel tgPanel) {
        int numberNodes = tgPanel.getNodeCount();
        int maxSize = (Math.min(tgPanel.getHeight(), tgPanel.getWidth())) / 2;
        diameter = Math.max(MIN_DIAMETER, maxSize / numberNodes);
    }

    /**
     * Returns the height of the node. 
     * This is required by the interface to support
     * things like mouseOver.
     */
    public int getHeight() {
        return diameter;
    }

    /**
     * Returns the width of the node. 
     * This is required by the interface to support
     * things like mouseOver.
     */
    public int getWidth() {
        return diameter;
    }
}