// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.datamanager.event.NewsGroupDownloadRequestEvent;
import org.datamanager.event.WebSearchRequestEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 *This class displays to the user several advanced search options.  At the
 *moment those advanced options are: Search the web, and new group search.
 *For these options the user must specify certain information.
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 * Modified to be a Simpleton that can be run to do web and newsgroup searches.
 * - Matthew Farrellee <mafarrel@cs.indiana.edu>
 */

//  methods:
//  ~~~~~~~~
//  public AdvancedSearchFrame()
//       This constructor puts together all of the elements of the frame
//  public AdvancedSearchFrame(String)
//       Specifies the String for the keywords JTextField
//  public void actionPerformed(ActionEvent)
//       This causes the searches to happen.
//  public void setWebFields()
//  public void setNewsFields()
//       These methods are set the fields either enabeled or not, depending on
//       whether web search or newsgroup download is selected.
//  public void execute()
//       This method adds the events to the pool (via KernelProxy),
//       which in turn
//       cause either a web search or a newsgroup download to occur.
//  public void exit()
//       This method makes sure that the frame is disposed of to free up
//        resources
//////////////////////////////////////////////////////////////////////////
public class WebAndNewsSearchSimpleton extends Simpleton
    implements ActionListener {
    public static final String DEFAULT_STRING = "";
    public static final int NUM_COLUMNS = 20;
    private JFrame frame;
    private JCheckBox news;
    private JCheckBox web;

    //for user input
    private JTextField keywords;

    //for user input
    private JTextField newsServer;

    //for user input
    private JTextField newsGroup;

    //select the search engine to do a websearch through
    private JComboBox searchEngines;
    private JButton okay;
    private JButton cancel;
    String[] displayEngines;
    String[] engines;

    //////////////////////////////////////////////////////////////

    /** This constructor uses the default String of "" to call the
     *  other constructor.
     */
    public WebAndNewsSearchSimpleton() {
    }

    public void process() {
        this.frame = new JFrame("Web and News Search");


        //setup the text fields.
        this.keywords = new JTextField(DEFAULT_STRING, NUM_COLUMNS);
        newsServer = new JTextField(NUM_COLUMNS);
        newsGroup = new JTextField(NUM_COLUMNS);


        //setup the JCheckBoxes
        news = new JCheckBox("Download News Group", false);
        web = new JCheckBox("Web Search", true);

        //setup the JComboBox with the search engines avaiable
        //I know this looks funny, but it would give me errors otherwise
        String[] displayEngines = {
            "GOOGLE", "DEJANEWS", "ALTAVISTA", "YAHOO", "EXCITE", "WEBCRAWLER"
        };
        this.displayEngines = displayEngines;

        String[] engines = {
            WebSearchRequestEvent.GOOGLE, WebSearchRequestEvent.DEJANEWS, 
            WebSearchRequestEvent.ALTAVISTA, WebSearchRequestEvent.YAHOO
        };


        //                WebSearchRequestEvent.EXCITE,
        //                WebSearchRequestEvent.WEBCRAWLER};
        this.engines = engines;
        searchEngines = new JComboBox(displayEngines);


        //setup the buttons
        okay = new JButton("Search");
        cancel = new JButton("Cancel");


        //setup the size, image, and layout
        frame.setSize(400, 200);

        String separator = System.getProperty("file.separator");
        frame.setIconImage(Toolkit.getDefaultToolkit()
                                  .getImage("org" + separator + 
                                                "datamanager" + separator + 
                                                "userinterface" + separator + 
                                                "cerulean" + separator + 
                                                "images" + separator + 
                                                "icon.gif"));
        frame.getContentPane().setLayout(new GridLayout(6, 2));


        //enable only the web search items
        newsServer.setEnabled(false);
        newsGroup.setEnabled(false);


        //add the items
        frame.getContentPane().add(web);
        frame.getContentPane().add(news);
        frame.getContentPane().add(new JLabel("Keywords"));
        frame.getContentPane().add(keywords);
        frame.getContentPane().add(new JLabel("Search Engines"));
        frame.getContentPane().add(searchEngines);
        frame.getContentPane().add(new JLabel("News Group"));
        frame.getContentPane().add(newsGroup);
        frame.getContentPane().add(new JLabel("News Server"));
        frame.getContentPane().add(newsServer);
        frame.getContentPane().add(okay);
        frame.getContentPane().add(cancel);


        //add the listeners
        web.addActionListener(this);
        news.addActionListener(this);
        okay.addActionListener(this);
        cancel.addActionListener(this);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent event) {
                Window window = event.getWindow();
                window.setVisible(false);

                // window.dispose();
            }
        });

        frame.setVisible(true);
    }

    ///////////////////////////////////////////////////////////////

    /**
     *From the ActionListener interface. Determines which button was
     *pushed and does the corresponing action.
     *@param event Fired when one of the buttons is pushed
     */
    public void actionPerformed(ActionEvent event) {
        if (event.getSource() == web) {
            setWebFields();
        } else if (event.getSource() == news) {
            setNewsFields();
        } else if (event.getSource() == okay) {
            execute();
        } else {
            frame.setVisible(false);
            frame.dispose();
        }
    }

    ///////////////////////////////////////////////////////////////

    /**
     *Sets the fields enabled based on the state of the Web Search
     *check box.
     */
    public void setWebFields() {
        keywords.setEnabled(web.isSelected());
        searchEngines.setEnabled(web.isSelected());
    }

    /**
     *Sets the fields enabled based on the state of the Newsgroup Download
     *check box.
     */
    public void setNewsFields() {
        newsServer.setEnabled(news.isSelected());
        newsGroup.setEnabled(news.isSelected());
    }

    //////////////////////////////////////////////////////////////

    /**
     *This method sends the events to the KernelProxy to be added
     * to the pool, so that either a web search or news group is
     *downloaded, or both.
     */
    public void execute() {
        //just exit if neither are selected.
        if (!web.isSelected() && !news.isSelected()) {
            exit();
        }

        KernelProxy proxy = KernelProxy.getInstance();

        if (web.isSelected()) {
            //we need to test if the keywords is empty
            if (keywords.getText().equals("")) {
                JOptionPane.showMessageDialog(frame, 
                                              "Please fill out the keywords , or select Cancel", 
                                              "Keywords not filled out...", 
                                              JOptionPane.ERROR_MESSAGE);

                //then we need them to go back and fill out the info
                return;
            }

            try {
                proxy.addEvent(
                        new WebSearchRequestEvent(proxy.getEventGenerator(), 
                                                  keywords.getText(), 
                                                  getSearchEngine()));
            } catch (DataManagerException exception) {
                JOptionPane.showMessageDialog(frame, exception.getMessage(), 
                                              "Error while processing input", 
                                              JOptionPane.ERROR_MESSAGE);

                return;
            }
        }

        if (news.isSelected()) {
            //check to see if the news group fields are blank
            if (newsGroup.getText().equals("") || 
                    newsServer.getText().equals("")) {
                JOptionPane.showMessageDialog(frame, 
                                              "Please fill out newgroup and server information, or select Cancel", 
                                              "Newgroup information incomplete...", 
                                              JOptionPane.ERROR_MESSAGE);

                //return for them to enter the needed info
                return;
            }

            try {
                proxy.addEvent(
                        new NewsGroupDownloadRequestEvent(
                                proxy.getEventGenerator(), newsGroup.getText(), 
                                newsServer.getText()));
            } catch (DataManagerException exception) {
                JOptionPane.showMessageDialog(frame, exception.getMessage(), 
                                              "Error while processing input", 
                                              JOptionPane.ERROR_MESSAGE);

                return;
            }
        }


        //if we get to here then then we have already done our searchs
        // so we need to exit the window.
        exit();
    }

    //////////////////////////////////////////////////////////////

    /**
     *Returns the search engine selected by the user.
     *@return The selected search engine.
     */
    public String getSearchEngine() {
        int index = searchEngines.getSelectedIndex();

        return engines[index];
    }

    //////////////////////////////////////////////////////////////
    public void exit() {
        frame.setVisible(false);

        // this.dispose();
    }
}