// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;
import java.io.StringReader;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import java.util.StringTokenizer;
import java.util.Vector;

import org.datamanager.tools.Debug;


public class WebConnection {
    // The value returned at the end of a Stream. Used in downloading.
    private static final int END_OF_STREAM = -1;
    public final static String URL_DELIMITERS = " \n\t\"";

    ////////////////////////////////////////////////////////////////////
    private WebConnection() {
    }

    ////////////////////////////////////////////////////////////////////
    public static WebConnection getConnection() {
        return new WebConnection();
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * This method takes a URL and will return a String that is
     * its contents.
     *
     * @param url The URL object to get the contents of.
     * @return The contents of the URL.
     */
    public String getContents(URL url) throws IOException {
        // Get the InputStream from the URL.
        InputStream inputStream = getInputStream(url);

        // Download the contents of the URL into a String.
        String contents = getString(inputStream);

        // Return that contents String.
        return contents;
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * This method will take a URL and get an InputStream from it.
     *
     * @param url The URL from which to get the InputStream.
     * @return InputStream An InputStream from the given URL.
     */
    protected InputStream getInputStream(URL url) throws IOException {
        if (url == null) {
            throw new IllegalArgumentException("Given a null URL!");
        }

        // openConnection() could throw an IOException
        URLConnection connection = url.openConnection();


        // FIXME
        // Set a user-agent that is something other than the default.
        // This is required for places like Google which deny access
        // from Java's default User-Agent!
        connection.setRequestProperty("User-Agent", "KS-H");

        // getInputStream() could throw an IOException or an
        // UnknownServiceException
        InputStream input = connection.getInputStream();

        // Return the InputStream.
        return input;
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * Take an InputStream and read its contents into a String.
     *
     * @param inputStream The Stream from which to get the String.
     * @return String A String representation of the InputStream.
     */
    protected String getString(InputStream inputStream)
                        throws IOException {
        if (inputStream == null) {
            throw new IllegalArgumentException("Given a null InputStream!");
        }

        StringBuffer theString;

        // Transform the InputStream into an InputStreamReader
        // and make a BufferedReader out of the InputStreamReader
        BufferedReader stream = new BufferedReader(
                                        new InputStreamReader(inputStream));


        // Initialize the StringBuffer, which will be used to
        // create the contents String.
        theString = new StringBuffer("");

        // read() gets one char from the stream. that char is then
        // appended to the StringBuffer.
        // read() can throw an IOException
        int character = stream.read();

        while (character != END_OF_STREAM) {
            theString.append((char) character);
            character = stream.read();
        }


        // close() can throw an IOException
        stream.close();

        // turn the StringBuffer into a String and return it
        return theString.toString();
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * Find all isolated URLs in the given string
     * and return them in a Vector
     */
    public Vector getURLs(String string) throws MalformedURLException {
        if (string == null) {
            throw new IllegalArgumentException("Given a null String!");
        }

        Vector urls = new Vector();
        URL url;
        String token = "";
        StringTokenizer tokenizer = new StringTokenizer(string, URL_DELIMITERS);

        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken();

            if (isURL(token)) {
                // When creating a new URL object it is possible
                // to have a MalformedURlException thrown. So, the
                // code to do this is contained in a try block.
                try {
                    // Create a new URL object with the url String
                    // from the stream. If this token is not actually
                    // a valid URL a MalformedURLException will be
                    // thrown and the token will be skipped.
                    url = new URL(token);


                    // Since the token was valid, add it as a URL
                    // to the urlList Vector.
                    urls.addElement(url);
                } catch (MalformedURLException badUrl) {
                    // Ignore the current token as it is not a
                    // valid URL.
                    //Debug.assert("Bad url: " + badUrl.getMessage());
                    throw badUrl;
                }
            }
        }

        return urls;
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * This method takes an InputStream, and parses out all the URLs
     * located within that InputStream.
     *
     * @param stream An InputStream out of which to parse URLs.
     * @return Vector A Vector of URL objects from the InputStream.
     */
    public Vector getURLs(InputStream stream) {
        if (stream == null) {
            throw new IllegalArgumentException("Given a null InputStream!");
        }

        Vector urlList = new Vector();

        // Make a StreamTokenizer out of an InputStream.
        InputStreamReader streamReader = new InputStreamReader(stream);
        BufferedReader bufferedReader = new BufferedReader(streamReader);
        StreamTokenizer tokenizer = new StreamTokenizer(bufferedReader);


        // States that the / character should never be part of a token.
        // The "/" is used in closing tags in HTML, ie: <a>anchor</a>
        tokenizer.ordinaryChar('/');


        // States that for something to be a String in the stream it
        // must be delimited by a '"' (quote mark).
        tokenizer.quoteChar('"');

        // While reading from a stream there is the possibility that
        // an IOException will be thrown. For this reason the following
        // code is contained within a try block.
        try {
            // Declare the url. It will be initialized many times within
            // the while loop below.
            URL url;

            // Declare and initialize the tokenType variable. This is used
            // to check and see if the token stored in the stream/sval is
            // a potential url.
            int tokenType = tokenizer.nextToken();

            // Declare the token. It will be initialized many times
            // within the while loop below.
            String token;

            // While the stream is not empty. That is the last token
            // read is not StreamTokenizer.TT_EOF.
            while (tokenType != StreamTokenizer.TT_EOF) {
                // If the tokenType states that the value of the current
                // token is a String process the potential url.
                if (tokenType == '"') {
                    // Get the potential url.
                    token = tokenizer.sval;

                    // Check to see if it is actually a URL.
                    // If so keep processing it.
                    if (isURL(token)) {
                        // When creating a new URL object it is possible
                        // to have a MalformedURlException thrown. So, the
                        // code to do this is contained in a try block.
                        try {
                            // Create a new URL object with the url String
                            // from the stream. If this token is not actually
                            // a valid URL a MalformedURLException will be
                            // thrown and the token will be skipped.
                            url = new URL(token);


                            // Since the token was valid, add it as a URL
                            // to the urlList Vector.
                            urlList.addElement(url);
                        } catch (MalformedURLException badUrl) {
                            // Ignore the current token as it is not a
                            // valid URL.
                            //Debug.assert("Bad url: " + badUrl.getMessage());
                            throw badUrl;
                        }
                    }
                }


                // Get the next tokenType from the stream.
                tokenType = tokenizer.nextToken();
            }
        } catch (IOException ioError) {
            // If an IOException is thrown from the stream there is not
            // much to do except to report it.
            Debug.writeLogMessage("IOException thrown in parseUrlsFrom(): " + 
                                  ioError.getMessage());
        }

        // Return the Vector of all urls from the content of the
        // entity that was just processed.
        return urlList;
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * Determine whether the given string looks like a url.
     *
     * @param string A String that might be a url.
     * @return boolean True if the String is a url, false otherwise.
     */
    protected boolean isURL(String string) {
        if (string == null) {
            throw new IllegalArgumentException("Given a null String!");
        }

        // Check to see if the String looks like a url.
        return ((string.startsWith("http") || string.endsWith("html") || 
                   string.endsWith("htm") || string.endsWith("/")) && 
               !(string.endsWith("gif") || string.endsWith("jpg") || 
                   string.endsWith("pdf")));
    }
}