// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.io.IOException;

import java.net.URL;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeExistsConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.NotEntityConstraint;
import org.datamanager.constraint.NotEventConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;
import org.datamanager.kernel.Simpleton.Information;

import org.datamanager.passiveentityvalue.StringEntityValue;

import org.datamanager.simpleton.WebConnection;

import org.datamanager.tools.Debug;


/**
This Simpleton has the job of downloading a URL.
**/
public class WebPageDownloadSimpleton extends Simpleton
    implements EventHandler {
    private static EntityConstraint undownloadedEntity = 
            new AndEntityConstraint(
                    new AttributeExistsConstraint(
                            new NameEqualsConstraint("content-location")), 
                    new AndEntityConstraint(
                            new NotEntityConstraint(
                                    new AttributeExistsConstraint(
                                            new NameEqualsConstraint(
                                                    "mime-type"))), 
                            new NotEntityConstraint(
                                    new AttributeExistsConstraint(
                                            new NameEqualsConstraint(
                                                    "content-source")))));

    ////////////////////////////////////////////////////////////////////

    /**
     * This is the default constructor. It calls the constructor that takes a
     * String with a default name.
     */
    public WebPageDownloadSimpleton() {
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * This method is where all the Simpleton's processing takes place.  It
     * downloads the contents of a url and attaches pieces of information about
     * the url to the entity along with the downloaded content.
     *
     * @param entity The entity to process.
     */
    public void process() {
        try {
            final EventConstraint isUndownloadedAndNewEntityConstraint = 
                    new AndEventConstraint(
                            new EventContainsEntityConstraint(
                                    undownloadedEntity), 
                            new EventTypeIsConstraint(AddEntityPoolEvent.class));
            Pool.getDefaultPool()
                .subscribe(this, isUndownloadedAndNewEntityConstraint);
        } catch (DataManagerException exception) {
            // FIXME: bogus runtime debug messaging
            throw new RuntimeException(exception);
        }
    }

    public void handle(DataManagerEvent event) {
        try {
            WebConnection connection = WebConnection.getConnection();

            AddEntityPoolEvent addEvent = (AddEntityPoolEvent) event;

            Entity addedEntity = addEvent.getEntity();

            Entity[] attributes = addedEntity.getAttributes(
                                          new NameEqualsConstraint(
                                                  "content-location"));

            if (attributes.length > 0) {
                String url = attributes[0].getValue().toString();

                // getContentsFromURL can throw an IOException.
                // Turn the URL into a String. Not the actual URL itself,
                // but rather the contents that it points to. Usually a
                // web page.
                String contents = connection.getContents(new URL(url));

                Entity mimeType = Entity.create("mime-type");
                Entity contentSource = Entity.create("content-source");

                mimeType.setValue(new StringEntityValue("text/html"));
                contentSource.setValue(new StringEntityValue("www"));

                addedEntity.setValue(new StringEntityValue(contents));

                addedEntity.attach(mimeType);
                addedEntity.attach(contentSource);
            }
        } catch (DataManagerException exception) {
        }
        // If there is trouble getting the contents from the URL.
         catch (IOException ioError) {
            // Signal that there was a problem with an assertion.
            Debug.writeLogMessage("Got an IOException: " + 
                                  ioError.getMessage());
        }
    }
}

/*
 * $Log: WebPageDownloadSimpleton.java,v $
 * Revision 1.29  2002/12/20 20:52:40  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.28  2002/11/21 02:58:25  mliggett
 * - EntityManager.getInstance() now loads the proper EntityManager class
 *   using reflection and the system property
 *   org.datamanager.entity_manager.class.  It defaults to
 *   VerySimpleEntityManager.
 *
 * - As a result of all the horrible things that could go wrong during
 *   reflection procedure, getInstance() now throws DataManagerException, and
 *   hence so do a few other methods now, too.
 *
 * - Added direct calls to removeEntityFromMemory() before every call to
 *   loadIntoMemory() to ensure that the hashtable never grows above the
 *   specified size.  This will be refactored in the next revision.
 *
 * - SuaveEntityManagerTester's test was exploded into four different tests.
 *   Also, it now sets and unsets the org.datamanager.entity_manager.class
 *   property before and after each test run, so that it knows it's testing
 *   the right class.
 *
 * Revision 1.27  2002/11/20 00:25:42  cjdent
 *
 *
 * EmailParametersEntityValue and EmailParameters
 *
 * Addition of SMTPHost field for the outgoing SMTP server
 * Updated the equals method in EmailParametersEntityValue to include the
 *   smtp and from address info. Was this right?
 *
 * Added an SMTP host to the EmailConfiguratorSimpleton
 *
 * SMTPSendSimpleton
 *  subscribes to the EmailParametersEnteredCollectorEvent and fires the
 *  EmailParametersRequestEvent to get SMTP host information.
 *
 *  FIXME: What happens if it is unable to get the info for some reason when
 *  it tries to make a delivery. I've set up a default, which is _not_ a good
 *  long term solution. This is integrated with the overall question of
 *  how to deal with SMTP errors.
 *
 * Rest is more bogus codebase-style
 *
 * Revision 1.26  2002/11/19 06:45:32  cjdent
 *
 * whole tree codebase style
 *
 * Revision 1.25  2002/11/18 05:09:33  xirao
 *         xi rao
 *         change the NoSuchEntityException, NoSuchIdentifierException to extends
 *         the DataManagerException. let find... method throw out the Exception.
 *
 * Revision 1.24  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.23  2002/11/18 02:38:37  mliggett
 * Added BasicEmailReplySimpleton, which currently only listens for the proper
 * event and pops up totally empty emails.  More functionality to come in just
 * a few.
 *
 * Also added this Simpleton to knownspace_config.xml and
 * EmailBundleSimpleton.
 *
 * Revision 1.22  2002/11/17 08:47:01  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 */
