// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.simpleton;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Enumeration;
import java.util.Vector;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeExistsConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventContainsEntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.ValueEqualsConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.BooleanEntityValue;
import org.datamanager.passiveentityvalue.NullEntityValue;
import org.datamanager.passiveentityvalue.StringEntityValue;

import org.datamanager.simpleton.WebConnection;

import org.datamanager.tools.Debug;


/**
This Simpleton takes the contents of the Entity and parses
all of the URLs located within it.
**/
public class WebParsingSimpleton extends Simpleton implements EventHandler {
    private static EntityConstraint unparsedMarker = new AndEntityConstraint(
                                                             new NameEqualsConstraint(
                                                                     "ContentParsed"), 
                                                             new ValueEqualsConstraint(
                                                                     BooleanEntityValue.FALSE));
    private static EntityConstraint unparsedEntity = new AttributeExistsConstraint(
                                                             unparsedMarker);

    ////////////////////////////////////////////////////////////////////

    /**
    This is the Constructor.
    **/
    public WebParsingSimpleton() {
    }

    ////////////////////////////////////////////////////////////////////

    /**
    This method subscribes the Simpleton to the default pool.
                                                                                                                                
    @param entity An Entity to process.
    **/
    public void process() {
        try {
            final EventConstraint isUnparsedAndNewEntityConstraint = 
                    new AndEventConstraint(
                            new EventTypeIsConstraint(AddEntityPoolEvent.class), 
                            new EventContainsEntityConstraint(unparsedEntity));
            Pool.getDefaultPool()
                .subscribe(this, isUnparsedAndNewEntityConstraint);
        } catch (DataManagerException exception) {
            // FIXME: bogus exception handling
            throw new RuntimeException(exception);
        }
    }

    public void handle(DataManagerEvent event) {
        try {
            WebConnection connection = WebConnection.getConnection();

            AddEntityPoolEvent addEvent = (AddEntityPoolEvent) event;

            Entity entity = addEvent.getEntity();

            // Use the parseUrlFrom() method to generate a list of Urls
            // from the contents of the entity.
            Vector urlList = connection.getURLs(entity.getValue().toString());

            Enumeration urls = urlList.elements();
            URL url;

            while (urls.hasMoreElements()) {
                try {
                    url = (URL) urls.nextElement();

                    Entity newEntity = Entity.create(url.toExternalForm());
                    Entity urlAttribute = Entity.create("content-location");

                    newEntity.setValue(NullEntityValue.getInstance());


                    // This really should be a URLEntityValue.
                    urlAttribute.setValue(
                            new StringEntityValue(url.toExternalForm()));

                    newEntity.attach(urlAttribute);

                    entity.attach(newEntity);
                    Pool.getDefaultPool().add(newEntity);
                } catch (DataManagerException exception) {
                    // No error should appear in Hydrogen.
                }
            }

            Entity[] attributes = entity.getAttributes(unparsedMarker);

            for (int i = 0; i < attributes.length; i++) {
                try {
                    attributes[i].setValue(BooleanEntityValue.TRUE);
                } catch (DataManagerException exception) {
                    // setValue() failed.
                }
            }
        } catch (MalformedURLException exception) {
            Debug.writeLogMessage(exception.getMessage());
        } catch (DataManagerException exception) {
            // Nuttin'
        }
    }
}

/*
 * $Log: WebParsingSimpleton.java,v $
 * Revision 1.48  2002/12/20 20:52:40  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.47  2002/12/20 04:40:13  cjdent
 *
 * Codebase style plus:
 *
 * - Changed back to the LsaStrategy for clustering
 * - Added an "Update Layout" button that calls updateEdges() to do
 *   similarity comparisons between the nodes on the screen. This avoids
 *   too much SVD recalculation
 *
 * - Changed the size of the entity nodes, they were a little small
 * - Added a bounding oval to the node, so when they overlap, you can tell
 *
 * It turns out that one thing that makes this entire process quite
 * expensive is that we add the WordFrequencyMapEntityValues in an
 * aggressive fashion. If we did it lazily, that might be a bit more
 * effective. The demonstartion of this problem goes like this:
 *
 *  - load in some mail from a sizable folder (50 messages or so)
 *  - wait until it is done, do an "Update Layout"
 *  - look and wonder
 *
 * shutdown and run again, without resetting. This time when you
 * load it will be blazingly fast. I'm fairly certain it is not
 * network, but I suppose it could be.
 *
 * Revision 1.46  2002/12/19 04:35:42  mliggett
 * codebase-style, plus
 *
 * Changed from using a List to a HashMap for storing mappings from documents and
 * terms to the indices for them on the matrix.
 *
 * Revision 1.45  2002/12/19 03:13:32  mliggett
 * codebase-style
 *
 * Revision 1.44  2002/12/16 10:38:41  cjdent
 *
 * Removed LSA DocumentMatrix calculation calls from
 * TouchgraphEmailDisplaySimpleton and put it in the clustering
 * strategy, via TouchgraphEntityDisplayPanel
 *
 * Revision 1.43  2002/12/10 04:30:55  xirao
 * xi rao
 *         add DocumentTermMatrixBuilderSimpletion back with code-style run
 *
 * Revision 1.42  2002/12/09 07:53:09  cjdent
 *
 * codebase-style
 *
 * Revision 1.41  2002/12/08 04:37:40  mliggett
 * Added caching, local changes to the cached values, dirty bits and write
 * back of locally modified values to CachingProxyEntity.  writeBack() is
 * currently not called anywhere but from CachingProxyEntity's finalizer,
 * but that will change in the next iteration.
 *
 * Also, writeBack() currently only writes back modified EntityValue.  If
 * the bases or attributes have changed, that's never reflected in the proxied
 * Entity.  I'll fix that in a moment, too.
 *
 * Revision 1.40  2002/12/06 12:04:44  alllee
 * Fixed triplicate download bug in EmailRetrieverSimpleton.  The problem lay in
 * the fact that three Simpletons were firing EmailParametersRequestEvent-s, and
 * the EmailParameterRequestHandlerSimpleton was responsibly dumping
 * EmailParametersEnteredCollectorEvent-s into the Pool as it is wont to do for
 * each EmailParametersRequestEvent that it handled.
 *
 * The fix involves making EmailRetrieverSimpleton cache a copy of the current
 * EmailParametersEntityValue that it is using, and comparing it using .equals(..)
 * to the event-incoming EmailParametersEntityValue.  If the event's
 * EmailParametersEntityValue is the same as the current one, it doesn't connect
 * and try to retrieve mail.  Otherwise, it does.
 *
 * Revision 1.39  2002/12/05 21:21:19  cjdent
 *
 * Updates to TouchgraphEmailDisplayPanel to use a very simple
 * method of pairwise document comparisons. Some sort of
 * completely wrong g-square meaure of variance with a bit of
 * bayesian probability is mixed in.
 *
 * THE MATH IS NOT RIGHT
 *
 * but it does do some interesting things.
 *
 * It's also not in the right place it should be in its own
 * class, perhaps out in the interfaces Allen is working on.
 *
 * I put it in here to test it out and cuz it is simple.
 *
 * The math creates a value. The closer to one it is, the more
 * similar the docs should be. I've set a very high threshold for
 * create edges.
 *
 * This needs some serious tweaking.
 *
 * In the process I bandaided the ConcurrentModificationException
 * problem by catching it. We don't care about it, so we can
 * just breeze by it. Messy.
 *
 * I'll continue working on the math in my perl prototype
 *
 * Revision 1.38  2002/12/05 07:20:01  cjdent
 *
 * Update to Touchgraph EmailDisplaySimpleton and EntityDisplayPanel
 * to support the features provided by EntityNavigateUI and
 * TouchgraphEntity Node and Edge.
 *
 * Major refactoring needed as this was done in a very
 * seat of the pants fashion.
 *
 * More in a forthcoming summary posting.
 *
 * Revision 1.37  2002/11/25 06:56:13  cjdent
 *
 * codebase style to spark a website-deploy
 *
 * Revision 1.36  2002/11/24 08:49:49  cjdent
 * codebase style
 *
 * Revision 1.35  2002/11/24 05:56:20  alllee
 * codebase-style/dead-code-elimination and the change to use
 * EmailMessageEntityHelper.EMAIL_ATTRIBUTE_(VALUE|NAME) when searching the pool
 * for email message entities.
 *
 * Revision 1.34  2002/11/24 03:57:17  mliggett
 * Many changes that finally resulted in Suave working properly.  Turns out we
 * were never persisting the objects after creation.  So EntityValues and
 * attributes were missing on restore.
 *
 * - Added toString() back to Entity.  Chris thought this might help us solve
 *   the problem.  It wasn't the problem, but I left it in.
 *
 * - Changed a few comments.
 *
 * - Completely changed the way getInstance() works, so that it would not
 *   always return a new SuaveEntityManager.
 *
 * - Added a call to persistEntity() whenever we remove something from the
 *   KernelMemory.
 *
 * - Codebase-style changed a few other things.
 *
 * Revision 1.33  2002/11/23 07:07:30  cjdent
 *
 * Toolbox changed to adjust textToPreHTML to use a
 * searchAndReplace() method to find < and > in
 * body content and change it to &lt; and &gt; before
 * it is displayed in EmailDisplaySimpleton.
 *
 * Added a test to ToolboxTester to test searchAndReplace()
 * (I made it public because it probably useful).
 *
 * Revision 1.32  2002/11/23 06:05:51  cjdent
 *
 * Made adjustments to SMTPSendSimpleton so that:
 *
 * - It only puts in an In-Reply-To header if one is available
 *   on the source entity.
 *
 * - It adds an X-Mailer vanity header
 *
 * - Sets the CC if there is one
 *
 * Revision 1.31  2002/11/23 05:44:11  cjdent
 *
 * Adds the EmailCompositionInitiatorSimpleton and makes the
 * change to BasicEmailReplysimpleton, EmailBundleSimpleton,
 * and EmailMessageEntityHelper to make it usable.
 *
 * Composition is initiate by using the "Componse New" button
 * provided by EmailCompositionInitiatorSimpleton. That
 * simpleton has an empty message Entity that it uses to
 * initiate the email Reply chain used for replies.
 *
 * BasicEmailReplySimpleton has been adjusted to notice when
 * an original message has no DateObject. Under those
 * circumstances it assume that it is creating a new clean
 * message.
 *
 * Some cleanups are requird in the SMTP send simpleton to avoid
 * some bogus headers.
 *
 * Cleanup needed badly in these simpletons.
 *
 * Revision 1.30  2002/11/23 04:33:50  cjdent
 *
 * Changed EmailConfiguratorSimpleton to use KernelProxy instead of a
 * simpleton that it creates itself for dealing with spurious threads.
 *
 * Fixed a meaningful comment in KernelProxy
 *
 * Revision 1.29  2002/11/23 03:42:36  cjdent
 *
 * Added support for a WordWrappingDocument in
 * BasicEmailMessageComponent for use in replies.
 *
 * Utilizes a WordWrappingDocument inner class
 * that extends DefaultStyledDocument.
 *
 * The wrapping is acceptable for typed text but
 * does not work with pasted text. This may
 * actually be a good thing.
 *
 * The math done for the wrapping seems a bit
 * messy.
 *
 * BasicEmailMessageComponenet is too long and
 * should be refactored.
 *
 * The rest is codebase-style.
 *
 * Revision 1.28  2002/11/21 06:16:08  cjdent
 * full bore codebase style
 *
 * Revision 1.27  2002/11/20 00:25:42  cjdent
 *
 *
 * EmailParametersEntityValue and EmailParameters
 *
 * Addition of SMTPHost field for the outgoing SMTP server
 * Updated the equals method in EmailParametersEntityValue to include the
 *   smtp and from address info. Was this right?
 *
 * Added an SMTP host to the EmailConfiguratorSimpleton
 *
 * SMTPSendSimpleton
 *  subscribes to the EmailParametersEnteredCollectorEvent and fires the
 *  EmailParametersRequestEvent to get SMTP host information.
 *
 *  FIXME: What happens if it is unable to get the info for some reason when
 *  it tries to make a delivery. I've set up a default, which is _not_ a good
 *  long term solution. This is integrated with the overall question of
 *  how to deal with SMTP errors.
 *
 * Rest is more bogus codebase-style
 *
 * Revision 1.26  2002/11/19 06:45:32  cjdent
 *
 * whole tree codebase style
 *
 * Revision 1.25  2002/11/18 05:09:33  xirao
 *         xi rao
 *         change the NoSuchEntityException, NoSuchIdentifierException to extends
 *         the DataManagerException. let find... method throw out the Exception.
 *
 * Revision 1.24  2002/11/18 04:44:39  mliggett
 * - Added the user's From: address to EmailConfiguratorSimpleton,
 *   EmailParametersEntityValue, and EmailParameters.  This now gets loaded
 *   rightly into the reply window.
 *
 * - Factored EmailParameters out of EmailRetrieverSimpleton since
 *   BasicEmailReplySimpleton will need the same sort of thing.
 *
 * - Left to do:  quote the message, build the rest of the headers, and
 *   actually send the thing out.
 *
 * - Changes in other classes are codebase-style.
 *
 * Revision 1.23  2002/11/18 02:38:37  mliggett
 * Added BasicEmailReplySimpleton, which currently only listens for the proper
 * event and pops up totally empty emails.  More functionality to come in just
 * a few.
 *
 * Also added this Simpleton to knownspace_config.xml and
 * EmailBundleSimpleton.
 *
 * Revision 1.22  2002/11/17 08:47:01  alllee
 * Retrofitted the various Simpletons to comply with the new ContainsEntityEvent
 * refactoring and finished the initial stab at making EmailDisplaySimpleton
 * handle appropriate EntityDisplayRequestEvents.  Also playing with
 * Web*Simpletons (beyond making them fit with the new
 * EventContainsEntityConstraint style) to see if I can stop them from always
 * getting modified when ant codebase-style is run.
 *
 */
