// For licensing, see: http://helium.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// To allow the use java 1.4 assertions:
//  removed assert(boolean) and assert(boolean, String).
//  changed assert(String) to writeLogMessage(String).
// Everything else remains the same mess it was before. 
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.tools;

import java.io.CharArrayWriter;
import java.io.PrintWriter;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;


/**
 * A simple debugger originally developed to implement breakpoint
 * assertion testing but now used for a single method writeLogMessage().
 * Also provides the ability watch only specific classes, or
 * packages.
 *
 *
 * Since all the method on the debugger are static, you can
 * call them with "Debug.<method>(...)".
 *
 * Every method should have its own assertions to test the state
 * of its arguments on entry into the method.
 *
 * Set DEBUG_MODE to false to turn off all assertion testing.
 * (Be absolutely sure that your code is working perfectly first!)
 * Do this with the code: Debug.setDebugger(false);
 * Test for with: Debug.isDebugging();
 *
 * Set FATAL_RESPONSE to false to turn off program death on failure
 * of any assertion.
 * Do this with the code: Debug.setFatalResponse(false);
 * Test for with: Debug.isFatalResponse();
 *
 * Set TRACE_MODE to false to turn off tracing of non-failed
 * assertions.
 * Do this with the code: Debug.setTracer(false);
 * Test for with: Debug.isTracing();
 *
 * Set MESSAGE_MORE to false to turn off message assertions.
 * Do this with the code: Debug.setMessenger(false);
 * Test for with: Debug.isMessaging();
 *
 * Set WATCH_MODE to false to turn off watching of specific
 * classes, methods, or packages.
 * Do this with the code: Debug.setWatcher(false);
 * Test for with: Debug.isWatching();
 *
 * Set IGNORE_MORE to false to turn off ignoring of specific
 * classes, methods, or packages.
 * Do this with the code: Debug.setIgnorer(false);
 * Test for with: Debug.isIgnoring();
 *
 * Use the addTarget(String) and removeTarget(String) methods
 * to target specific methods, classes, or packages.
 * Example: Debug.addTarget("org.datamanager.kernel.");
 *         This will make the debugger only report assertions from
 *         the "org.datamanager.kernel." package.
 *
 * Use the addIgnore(String) and removeIgnore(String) methods
 * to make the debugger ignore specific methods, classes, or
 * packages.
 * Example: Debug.addIgnore("org.datamanager.kernel.");
 *         This will make the debugger ignore all assertions from
 *         the "org.datamanager.kernel." package.
 *
 * Note about watching and ignoring:
 * Currently the strings that are used for watching and ignoring
 * are tested with the String.indexOf() method. This means that
 * if you pass the String, "a", to one of the add...() methods
 * you will either ignore or watch for the String "a" in a
 * method signatures. Example of method signature:
 * "org.datamanager.kernel.Entity.getValue(Entity.java:##)" is
 * the signature of the method Entity.getValue(). Notice that
 * this method signature constains the String "a" and would
 * thus be matched by a watch or ignore string. Try to be as
 * specific as possible when adding watch or ignore strings,
 * because your string might match something unexpected.
 *
 * **Also, ignoring a string will override watching for it.**
 * Ie: If you watch for "hello" and ignore "hello" with
 * IGNORE_MODE on, then the string "hello" will be ignored.
 *
 * @author Gregory Rawlins.
 * @author Matthew Farrellee
 */
public class Debug {
    ////////////////////////////////////////////////////////////////////////

    /**
     *
     * Public Methods:
     * ---------------
     * writeLogMessage(messageString)                                        
     *         fires if TRACE_MODE==true
     *
     * setWatcher(boolean)
     *         set watching mode status
     * setIgnorer(boolean)
     *         set ignoring mode status
     * setDebugger(boolean)
     *         set debugging mode status
     * setTracer(boolean)
     *         set tracing mode status
     * setFatalResponse(boolean)
     *         set fatal reponse mode status
     * setMessenger(boolean)
     *         set messaging mode status
     *
     * isWatching()
     *         report watching mode status
     * isIgnoring()
     *         report ignoring mode status
     * isDebugging()
     *         report debugging mode statuc
     * isTracing()
     *         report tracing mode status
     * isFatalResponse()
     *         report fatal response mode status
     * isMessaging()
     *         report messenging mode status
     *
     * addTarget(String)
     *         add a target the debugger will watch for
     * addIgnore(String)
     *         add a target the debugger will ignore
     *
     * removeTarget(String)
     *         remove a target the debugger was watching for
     * removeIgnore(String)
     *         remove a target the debugger was ignoring
     *
     */
    private static boolean FATAL_RESPONSE = false;
    private static boolean DEBUG_MODE = false;
    private static boolean TRACE_MODE = false;
    private static boolean MESSAGE_MODE = false;
    private static boolean WATCH_MODE = false;
    private static boolean IGNORE_MODE = false;
    private static Vector watchedItems = new Vector(5, 2);
    private static Vector ignoredItems = new Vector(5, 2);

    // number of method calls to step back in the stack to find the
    // offending method name and line number using the sun JDK
    private final static int STACK_LEVEL = 4;

    /**
     * If setMessenger() and setDebugger() have both been set to true, writes
     * the value of message.toString() onto stdout.
     */
    public synchronized final static void writeLogMessage(Object message) {
        if (DEBUG_MODE) {
            if (MESSAGE_MODE) {
                String caller = getCaller(STACK_LEVEL);

                if (watched(caller) && !ignored(caller)) {
                    printTraceCall(caller + ": " + message.toString());
                }
            }
        }
    }

    /////////////////////////////////////////////////////////////////////
    private synchronized final static void printError(String string) {
        System.err.println(string);
    }

    /////////////////////////////////////////////////////////////////////
    private synchronized final static void printTraceCall(String string) {
        System.out.println(string);
    }

    /////////////////////////////////////////////////////////////////////
    private synchronized final static void checkFatality() {
        if (FATAL_RESPONSE) {
            System.exit(1);
        }
    }

    /////////////////////////////////////////////////////////////////////
    private synchronized final static String getCaller(int callerNumber) {
        /**
         * Determine the callerNumber'th method that called this method
         * and return its name.
         *
         * NOTE: This method may not work with code compiled using a JIT
         * or with a non-JDK JVM since in either case the stack trace
         * format is unspecified in the language spec.
         * A method for disabling a JIT is to invoke java with
         * "-Djava.compiler=NONE."
         *
         * Stack trace format assumed:
         * ---------------------------
         * java.lang.Throwable
         * at <this method's name>
         * at <caller's method's name>                                (callerNumber == 1)
         * at <caller's caller's method's name>        (callerNumber == 2)
         * <and so on for higher callerNumbers>
         */
        if (callerNumber < 1) {
            return "Debugger";
        }

        //transform the caller number into the token number to search for
        int tokenNumber = (callerNumber * 2) + 3;

        //create a stream to catch the stack trace
        CharArrayWriter stream = new CharArrayWriter();

        //send the stack trace to the stream
        Throwable throwable = new Throwable();
        throwable.printStackTrace(new PrintWriter(stream));

        //parse the stack trace to find the callerNumber'th method call
        StringTokenizer stackTrace = new StringTokenizer(stream.toString());
        String token = "";

        try {
            for (int i = 0; i < tokenNumber; i++) {
                token = stackTrace.nextToken();
            }
        } catch (NoSuchElementException ignored) {
            token = "unknown";
        }

        return token;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean watched(String caller) {
        if (!WATCH_MODE) {
            return true;
        }

        Enumeration names = watchedItems.elements();
        String name;

        while (names.hasMoreElements()) {
            name = (String) names.nextElement();

            if (caller.indexOf(name) >= 0) {
                return true;
            }
        }

        return false;
    }

    /////////////////////////////////////////////////////////////////////
    private synchronized static boolean ignored(String target) {
        if (!IGNORE_MODE) {
            return false;
        }

        Enumeration names = ignoredItems.elements();
        String name;

        while (names.hasMoreElements()) {
            name = (String) names.nextElement();

            if (target.indexOf(name) >= 0) {
                return true;
            }
        }

        return false;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void addIgnore(String target) {
        ignoredItems.add(target);
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void addTarget(String target) {
        watchedItems.add(target);
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void removeIgnore(String target) {
        ignoredItems.remove(target);
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void removeTarget(String target) {
        watchedItems.remove(target);
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void setWatcher(boolean state) {
        WATCH_MODE = state;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void setDebugger(boolean state) {
        DEBUG_MODE = state;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void setTracer(boolean state) {
        TRACE_MODE = state;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void setFatalResponse(boolean state) {
        FATAL_RESPONSE = state;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static void setMessenger(boolean state) {
        MESSAGE_MODE = state;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean isWatching() {
        return WATCH_MODE;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean isDebugging() {
        return DEBUG_MODE;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean isTracing() {
        return TRACE_MODE;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean isFatalResponse() {
        return FATAL_RESPONSE;
    }

    /////////////////////////////////////////////////////////////////////
    public synchronized static boolean isMessaging() {
        return MESSAGE_MODE;
    }
}

/*
 * $Log: Debug.java,v $
 * Revision 1.8  2002/12/20 20:52:45  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.7  2002/11/16 03:28:08  cjdent
 *
 * Javadoc cleanup at the class level.
 *
 * Revision 1.6  2002/10/28 02:14:54  mliggett
 * Modified writeLogMessage to take an Object instead of a String.  It just calls
 * toString() on it, but this make it a little more convenient for the user.
 *
 */
