// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.tools;

import java.io.File;
import java.io.FileNotFoundException;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.NameEqualsIgnoringCaseConstraint;

import org.datamanager.event.LaunchBrowserWithURLEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;

import org.datamanager.tools.kernelproxy.KernelProxy;


/**
 * A mish-mash of KnownSpace convenience functions.
 *
 * @author Matt Liggett
 */
public final class Toolbox {
    public static String HELIUM_FOLDER = ".helium";
    private static final KernelProxy KERNEL_PROXY = KernelProxy.getInstance();

    /**
     * Returns the first attribute of the given Entity whose name equals the
     * given name.
     */
    public final static Entity getFirstAttributeNamed(Entity base, String name)
                                               throws DataManagerException {
        Entity[] attributes = KERNEL_PROXY.getAttributes(base, 
                                                         new NameEqualsConstraint(
                                                                 name));

        if ((attributes == null) || (attributes.length == 0)) {
            return null;
        }

        return attributes[0];
    }

    /**
     * Returns the first attribute of the given Entity whose name equals the
     * given name, ignoring case in the name.
     */
    public final static Entity getFirstAttributeNamedIgnoringCase(Entity base, 
                                                                  String name)
        throws DataManagerException {
        Entity[] attributes = KERNEL_PROXY.getAttributes(base, 
                                                         new NameEqualsIgnoringCaseConstraint(
                                                                 name));

        if ((attributes == null) || (attributes.length == 0)) {
            return null;
        }

        return attributes[0];
    }

    /**
     * Returns the String content with new pre tags inserted and all http: or
     * https: URLs manually hreffed.  This should *not* be invoked on Strings
     * that are already HTML-enated
     * @param content String to be scanned/converted for URLs
     * @return String converted into <pre></pre> styled HTML, with
     *         <a * href>..</a>
     * links inserted for all http or https URLs.
     * 
     * FIXME: hard-coded bufferOffset constants can be hidden away with an
     * insertIntoBuffer method that uses the to-be-added String's length to
     * recompute the new bufferOffset.  
     */
    public final static String textToPreHtml(String content) {
        content = fixHTMLEntities(content);

        final StringBuffer buffer = new StringBuffer(content);
        int currentBufferIndex = 0;
        int bufferOffset = 0;

        // generate java regexes to detect URL's in the body
        Pattern urlPattern = Pattern.compile(
                                     "((?:http|https)://[\\w]+(?:.[\\w]+)(?:[\\w-.,&@\\?^=%:/~+#]*[\\w-&@?^=%/~+#])?)");
        Matcher urlMatcher = urlPattern.matcher(content);

        buffer.insert(currentBufferIndex, "<pre>");
        bufferOffset += 5;

        while (urlMatcher.find()) {
            buffer.insert(urlMatcher.start() + bufferOffset, "<a href='");
            bufferOffset += 9;

            currentBufferIndex = urlMatcher.end();
            buffer.insert(currentBufferIndex + bufferOffset, 
                          "'>" + urlMatcher.group() + "</a>");

            bufferOffset += (urlMatcher.group().length() + 6);
        }

        buffer.append("</pre>");
        buffer.insert(0, "<html>");
        buffer.append("</html>");

        return buffer.toString();
    }

    /**
     * Changes < and > in a String to &lt; and &gt;.
     */
    private final static String fixHTMLEntities(String content) {
        content = searchAndReplace(content, "<", "&lt;");
        content = searchAndReplace(content, ">", "&gt;");

        return content;
    }

    /**
     * Replaces regexp in content with replacement.
     */
    public final static String searchAndReplace(String content, 
                                                final String regexp, 
                                                final String replacement) {
        Pattern pattern = Pattern.compile(regexp);
        Matcher matcher = pattern.matcher(content);
        content = matcher.replaceAll(replacement);

        return content;
    }

    /**
     * Returns the Helium directory, wherein we should place things like
     * persistence files.
     */
    public static File getHeliumDirectory() throws FileNotFoundException {
        String homeDirectoryName = System.getProperty("user.home") + 
                                   HELIUM_FOLDER;
        String currentDirectoryName = System.getProperty("user.dir") + 
                                      HELIUM_FOLDER;

        File homeDirectoryFile = new File(homeDirectoryName);
        File currentDirectoryFile = new File(currentDirectoryName);

        // FIXME: the following hairy mess attempts to find a location for
        // home first looking in the users home directory, then looking
        // in the current working directory.
        if (homeDirectoryFile.exists()) {
            if (homeDirectoryFile.isDirectory()) {
                return homeDirectoryFile;
            }
        } else if (homeDirectoryFile.mkdir()) {
            return homeDirectoryFile;
        }

        if (currentDirectoryFile.exists()) {
            if (currentDirectoryFile.isDirectory()) {
                return currentDirectoryFile;
            }
        } else if (currentDirectoryFile.mkdir()) {
            return currentDirectoryFile;
        }

        throw new FileNotFoundException(
                "Cannot find a place to put helium files.");
    }

    /**
     * Fires the LaunchBrowserURLEvent to cause the provided
     * url to be displayed in a browser.
     */
    public static void launchURL(URL url) {
        try {
            KernelProxy.getInstance()
                       .addEvent(new LaunchBrowserWithURLEvent(
                                         KernelProxy.getInstance()
                                                    .getEventGenerator(), url));
        } catch (DataManagerException exception) {
            // FIXME: we need a better mechanism for dealing with these
            throw new RuntimeException(exception);
        }
    }

    public static void launchURL(String urlString) {
        URL url;

        try {
            url = new URL(urlString);
        } catch (MalformedURLException exception) {
            // FIXME: we need a better mechanism for dealing with these
            throw new RuntimeException(exception);
        }

        launchURL(url);
    }
}

/*
 * $Log: Toolbox.java,v $
 * Revision 1.14  2002/12/20 20:52:46  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.13  2002/12/19 03:13:33  mliggett
 * codebase-style
 *
 * Revision 1.12  2002/12/18 06:07:04  cjdent
 *
 * Adds a launchURL helper method that fires the LaunchBrowserWithURLEvent
 *
 * Adjusted TextSearchSimpleton and EmailConfiguratorSimpleton to use
 * this for their Help button actions. Got rid of the Cancel buttons as
 * they are not used.
 *
 * Each Simpleton has, for the time being, a HELP_URL_STRING that for the
 * time being points to some stuff that I'm writing but is not yet
 * committed.
 *
 * Revision 1.11  2002/12/06 19:57:20  alllee
 * Changed KernelProxy getInstance() calls to use a constant KERNEL_PROXY instead
 * due to the number of Toolbox calls..
 *
 * Revision 1.10  2002/12/01 23:19:48  mliggett
 * Added getHeliumDirectory() so there is now a common way to find a folder
 * in which to drop random files (e.g. persistence files).
 *
 * Revision 1.9  2002/11/24 05:46:27  alllee
 * Missing a character in the URL handling for ampersands in query strings.
 *
 * Revision 1.8  2002/11/23 07:07:30  cjdent
 *
 * Toolbox changed to adjust textToPreHTML to use a
 * searchAndReplace() method to find < and > in
 * body content and change it to &lt; and &gt; before
 * it is displayed in EmailDisplaySimpleton.
 *
 * Added a test to ToolboxTester to test searchAndReplace()
 * (I made it public because it probably useful).
 *
 * Revision 1.7  2002/11/16 20:06:46  mliggett
 * Very very minor reformattings.
 *
 * Revision 1.6  2002/11/14 06:53:28  cjdent
 *
 * Adjusted Toolbox so that
 *
 *   It doesn't </pre><a href="url">url</a><pre> just the href. You can
 *   href in a pre.
 *
 * Adjusted EmailDisplaySimpleton so that
 *
 *   it displays both content-type text/plain and null content-type in the <pre>
 *   thingie.
 *
 *   detects HTML content type and shows that too instead of saying it is
 *   an unsupported content type
 *
 * Yes this does indeed mean that you can view your spam in all its glorious colors.
 * It does not, however, mean that you can (yet) read multipart/alternative messages
 * of the like that the emailer that Jason Baumgartner uses.
 *
 * TODO:
 *
 *   when a HyperLinkEvent is fired, hear it, get the URL and fire off
 *   a LaunchBrowserWithURLEvent
 *
 *   consider ways to display images only (JEditor pane will show any URL or
 *   String given, but as far as I can tell you can't pass Image objects and
 *   the like
 *
 *   The JEditorPane needs to do some FontMetrics or something to get to be
 *   the correct width for whatever font that is that it uses for <pre> text.
 *
 * Revision 1.5  2002/11/14 05:20:57  alllee
 * Added textToPreHtml which converts a normal text String into HTML-enabled text
 * with added hrefs for http and https URLs.  Not too much data validation is
 * occurring right now, and the code has an unpleasant aroma to it (see FIXME).
 *
 * Revision 1.4  2002/11/04 02:27:50  cjdent
 *
 * Added getFirstAttributeNamedIgnoringCase(). Same as getFirstAttributeNamed
 * except it uses the NameEqualsIgnoringCase contstraint. Used, at start,
 * by EmailDisplaySimpleton.
 *
 * Revision 1.3  2002/10/31 04:41:43  mliggett
 * Made getFirstAttributeNamed() static.
 * Made getFirstAttributeNamed() use KernelProxy so it would be useful in AWT
 * threads.
 *
 * This last one sucks.  There's no reason for other threads calling this function
 * to incur this overhead.  MattF was right:  eventually we need to scrap the whole
 * notion of DataManagerWorker threads and just keep a Thread->permissions map.  A
 * child thread can then inherit permissions from its parent, even if it's not a
 * DMW (like an AWT thread spawned from a Simpleton).
 *
 * Revision 1.2  2002/10/31 03:49:49  mliggett
 * Actually checked that the code would compile this time.
 *
 * Revision 1.1  2002/10/31 03:45:10  mliggett
 * Added as a place to house getFirstAttributeNamed(), because I figured this function would have wide-ranging applicability.
 *
 */
