// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.tools.kernelproxy;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Simpleton;

import org.datamanager.passiveentityvalue.BooleanEntityValue;


/**
 * An abstract class that can be used to interact with the kernel.
 *
 * An InterfaceWorker is a Simpleton (so it has a kernel thread)
 * that can cause events in the kernel.
 *
 * Essentially, a new InterfaceWorker is created.  Its contents must
 * be registered by the user, so that the thread is run.  The results
 * of the operation be then be retrieved through a get() method call.
 *
 * example:
 * AddEntityWorker worker = new AddEntityWorker(entity);
 * registerWorker(worker);
 * Object value = worker.get();
 *
 * @see Simpleton
 * @see KernelProxy
 *
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu
 */
public abstract class InterfaceWorker extends Simpleton {
    /** The value that is returned as a result of construct(). */
    private Object value; // see getValue(), setValue()

    /**
     * Starts a thread that will call the <code>construct</code> method
     * and then exit.
     */
    public InterfaceWorker() {
    }

    /** 
     * Gets the value produced by the worker thread, or null if it 
     * hasn't been constructed yet.
     */
    protected synchronized Object getValue() {
        return value;
    }

    /** 
     * Sets the value produced by worker thread 
     */
    private synchronized void setValue(Object object) {
        value = object;

        try {
            notifyAll();
        } catch (IllegalMonitorStateException e) {
            e.printStackTrace();
        }
    }

    /** 
     * Computes the value to be returned by the <code>get</code> method. 
     */
    protected abstract Object construct() throws DataManagerException;

    /**
     * Returns the value created by the <code>construct</code> method.  
     * Returns null if either the constructing thread or
     * the current thread was interrupted before a value was produced.
     */
    public synchronized Object get() {
        try {
            if (getValue() == null) {
                wait();
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt(); // propagate
            e.printStackTrace();

            return null;
        }

        return getValue();
    }

    /**
     * Calls construct() in the subclasses.
     */
    public void process() {
        Object object;

        try {
            object = construct();
        } catch (DataManagerException exception) {
            throw new RuntimeException(
                    "Uncaught exception in KernelProxy. FIXME!" + 
                    exception.getMessage());
        }

        if (object == null) {
            object = new Boolean(true);
        }

        setValue(object);
    }

    public Simpleton.Information getInformation() {
        return Simpleton.Information.NONE;
    }
}

/*
 *
 * $Log
 *
 */
