// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.tools.kernelproxy;

import org.datamanager.constraint.AndEntityConstraint;
import org.datamanager.constraint.AndEventConstraint;
import org.datamanager.constraint.AttributeNameContainsConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.NotEntityConstraint;
import org.datamanager.constraint.NotEventConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.EventGenerator;
import org.datamanager.kernel.EventHandler;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.kernelproxy.AttachEntityWorker;
import org.datamanager.tools.kernelproxy.CreateEntityWorker;
import org.datamanager.tools.kernelproxy.DetachEntityWorker;
import org.datamanager.tools.kernelproxy.GetAttributesWorker;
import org.datamanager.tools.kernelproxy.GetBasesWorker;
import org.datamanager.tools.kernelproxy.GetEntityValueWorker;
import org.datamanager.tools.kernelproxy.GetNameWorker;
import org.datamanager.tools.kernelproxy.SearchWorker;
import org.datamanager.tools.kernelproxy.SetEntityValueWorker;
import org.datamanager.tools.kernelproxy.SubscriptionWorker;


/**
 * KernelProxy serves as a wrapper around the functionality of the kernel.
 *  
 * @see java.lang.Thread
 * @see org.datamanager.kernel.Entity
 * @see org.datamanager.constraint.EventConstraint
 *
 * @author Gordon Murphy gpmurphy@indiana.edu
 * @author Bryan Dawson bdawson@indiana.edu
 * 
 */
public class KernelProxy {
    /** Entities with this Attribute can be ignored by Interface. */
    public static final String IGNORED_ENTITY = "IgnoredByInterface";

    /** The instance of the KernelProxy. */
    private static KernelProxy instance = new KernelProxy();

    /**
     * Creates the singleton instance of KernelProxy.
     */
    private KernelProxy() {
        /*
        EventTypeIsConstraint constraint = new EventTypeIsConstraint(
                                                   AddEntityPoolEvent.class);
        registerWorker(new SubscriptionWorker(constraint, this));
        */
    }

    /**
     * Gets the instance of the singleton KernelProxy.
     */
    public static KernelProxy getInstance() {
        return instance;
    }

    /**
     * Adds an entity to the pool.
     * @param entity the entity to add.
     */
    public void addEntity(Entity entity) {
        AddEntityWorker worker = new AddEntityWorker(entity);
        registerWorker(worker);
    }

    /**
     * Removes an entity from the pool.
     * @param entity the entity to remove.
     */
    public void removeEntity(Entity entity) {
        RemoveEntityWorker worker = new RemoveEntityWorker(entity);
        registerWorker(worker);

        Object object = worker.get();
    }

    /**
     * Searches for all Entities that match the given constraint.
     * 
     * @param constraint the constraint that entities must match
     * @return the results of the search
     */
    public String getEntityName(Entity entity) {
        GetNameWorker worker = new GetNameWorker(entity);
        registerWorker(worker);

        String name = (String) (worker.get());

        return name;
    }

    /**
     * Attaches an Entity as an attribute to another Entity.
     * 
     * @param base the base Entity to which the attribute will be attached 
     * @param attribute the attribute to attach
     */
    public void attachEntity(Entity base, Entity attribute) {
        AttachEntityWorker worker = new AttachEntityWorker(base, attribute);
        registerWorker(worker);

        Object object = worker.get();
    }

    /**
     * Detaches an Entity as an attribute from another Entity.
     *
     * @param base the base Entity to which the attribute will be detached 
     * @param attribute the attribute to detach
     */
    public void detachEntity(Entity base, Entity attribute) {
        DetachEntityWorker worker = new DetachEntityWorker(base, attribute);
        registerWorker(worker);

        Object object = worker.get();
    }

    /**
     * Creates a new Entity with the specified name.
     *
     * @param name the name of the new Entity
     * @return the new Entity
     */
    public Entity createEntity(String name) {
        CreateEntityWorker worker = new CreateEntityWorker(name);
        registerWorker(worker);

        Entity createdEntity = (Entity) (worker.get());

        return createdEntity;
    }

    /**
     * Creates a new Entity with an attached entity specifying that it is to be 
     * ignored by the interface.
     *
     * @param name the name of the new Entity
     * @return the new Entity
     */
    public Entity createAttribute(String name) {
        CreateAttributeWorker worker = new CreateAttributeWorker(name);
        registerWorker(worker);

        Entity createdEntity = (Entity) (worker.get());

        return createdEntity;
    }

    /**
     * Sets the specified entity's value to the specified value.
     *
     * @param entity the entity whose value will be changed
     * @param value the value to be added
     */
    public void setEntityValue(Entity entity, EntityValue value) {
        SetEntityValueWorker worker = new SetEntityValueWorker(entity, value);
        registerWorker(worker);

        Object object = worker.get();
    }

    /**
     * Sets the specified entity's value to the specified value.
     *
     * @param entity the entity whose value will be changed
     * @param value the value to be added
     */
    public EntityValue getEntityValue(Entity entity) {
        GetEntityValueWorker worker = new GetEntityValueWorker(entity);
        registerWorker(worker);

        EntityValue entityValue = (EntityValue) (worker.get());

        return entityValue;
    }

    /**
     * Retrieve all the bases of an entity that match the given constraint.
     *
     * @param entity the entity to find the bases of
     * @param constraint the constraint that the bases much match
     * @return the bases that match the constraint
     */
    public Entity[] getBases(Entity entity, EntityConstraint constraint) {
        GetBasesWorker worker = new GetBasesWorker(entity, constraint);
        registerWorker(worker);

        Entity[] entities = (Entity[]) (worker.get());

        return entities;
    }

    /**
     * Retrieve all the attributes of an entity that match the given constraint.
     *
     * @param entity the entity to find the attributes of
     * @param constraint the constraint that the attributes much match
     * @return the attributes that match the constraint
     */
    public Entity[] getAttributes(Entity entity, EntityConstraint constraint) {
        GetAttributesWorker worker = new GetAttributesWorker(entity, constraint);
        registerWorker(worker);

        Entity[] entities = (Entity[]) (worker.get());

        return entities;
    }

    /**
     * Add a DataManagerEvent to the pool.  
     * 
     * @param event the event to add to the pool
     */
    public void addEvent(DataManagerEvent event) {
        AddEventWorker worker = new AddEventWorker(event);
        registerWorker(worker);

        Object object = worker.get();
    }

    /**
     * Subscribe handler to the event described by the constraint.
     *
     * @param constraint the constraint describing the event
     * @param handler the EventHandler to be called when an event happens
     */
    public void subscribeToEvent(EventConstraint constraint, 
                                 EventHandler handler) {
        registerWorker(new SubscriptionWorker(constraint, handler));
    }

    /**
     * Subscribe handler to the event whose Class is class.
     *
     * @param event the Class class of the Event to be subscribe to.
     * @param handler the EventHandler to be called when an event happens
     */
    public void subscribeToEvent(Class event, EventHandler handler) {
        EventTypeIsConstraint constraint = new EventTypeIsConstraint(event);
        this.subscribeToEvent(event, handler);
    }

    /**
     * Register a Simpleton thread.  The thread is implicitly started.
     *
     * @param worker the thread to register
     */
    private void registerWorker(Simpleton worker) {
        worker.start();
    }

    /**
     * Register a Simpleton thread with the given priority.
     * The thread is implicitly started.
     *
     * @param worker the thread to register
     * @param the priority of the thread
     * @see java.lang.Thread
     */
    private void registerWorker(Simpleton worker, Simpleton.Priority priority) {
        worker.start();

        // Try to set the priority, if it does not work just
        // forget about it.
        try {
            worker.setPriority(priority);
        } catch (DataManagerException exception) {
            exception.printStackTrace();
        }
    }

    /**
     * Gets the constraint for entities that are to ignored.
     */
    public EntityConstraint getIgnoredByInterfaceConstraint() {
        EntityConstraint constraint = new NameEqualsConstraint(IGNORED_ENTITY);

        return constraint;
    }

    /**
     * Gets an EventGenerator from the Pool.
     */
    public EventGenerator getEventGenerator() throws DataManagerException {
        EventGenerator generator = null;
        generator = Pool.getDefaultPool();

        return generator;
    }
}

/*
 *
 * $Log
 *
 */
