// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import java.util.List;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.datamanager.constraint.EventConstraint;

import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;


public class EventGeneratorAssistantTester extends TestCase {
    private EventGeneratorAssistant eventGeneratorAssistant;
    private EventHandler handler;
    private EventConstraint alwaysFalseConstraint;
    private EventConstraint alwaysTrueConstraint;
    private DataManagerEvent dataManagerEvent;
    private boolean eventWasHandled;

    public EventGeneratorAssistantTester(String s) {
        super(s);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite("EventGeneratorAssistantTester");
        suite.addTest(new EventGeneratorAssistantTester("testAddEventHandler"));
        suite.addTest(new EventGeneratorAssistantTester("testFireEvent"));
        suite.addTest(
                new EventGeneratorAssistantTester("testRemoveEventHandler"));

        return suite;
    }

    protected void setUp() {
        this.eventGeneratorAssistant = new EventGeneratorAssistant();
        this.handler = new EventHandler() {
            public void handle(DataManagerEvent event) {
                eventWasHandled = true;
            }
        };


        this.alwaysFalseConstraint = new EventConstraint() {
            public boolean accepts(DataManagerEvent defendant)
                            throws DataManagerException {
                return false;
            }
        };


        this.alwaysTrueConstraint = new EventConstraint() {
            public boolean accepts(DataManagerEvent defendant)
                            throws DataManagerException {
                return true;
            }
        };


        this.dataManagerEvent = new DataManagerEvent() {
            public EventGenerator getSource() {
                return null;
            }

            public long getTime() {
                return System.currentTimeMillis();
            }

            public String toString() {
                return "DataManagerEvent instance";
            }
        };

    }

    protected void tearDown() {
        this.eventGeneratorAssistant = null;
        this.handler = null;
        this.alwaysFalseConstraint = null;
        this.alwaysTrueConstraint = null;
        this.dataManagerEvent = null;
    }

    public void testAddEventHandler() {
        try {
            eventGeneratorAssistant.addEventHandler(this.handler, 
                                                    this.alwaysFalseConstraint);
            this.assertTrue("the handler is now in the Map as a key", 
                            eventGeneratorAssistant.getEventListeners()
                                                   .containsKey(this.handler));
            this.assertTrue(
                    "the constraint is now one of the entries in the arraylist", 
                    ((List) eventGeneratorAssistant.getEventListeners()
                                                   .get(this.handler)).contains(
                            this.alwaysFalseConstraint));
            eventGeneratorAssistant.addEventHandler(this.handler, 
                                                    this.alwaysTrueConstraint);
            this.assertTrue("the first constraint should still in the list", 
                            ((List) eventGeneratorAssistant.getEventListeners()
                                                           .get(this.handler)).contains(
                                    this.alwaysFalseConstraint));
            this.assertTrue("the second constraint should be in the list", 
                            ((List) eventGeneratorAssistant.getEventListeners()
                                                           .get(this.handler)).contains(
                                    this.alwaysTrueConstraint));
        } catch (Exception e) {
            this.fail("Caught an exception trying to add an event handler to our eventGeneratorAssistant");
        }
    }

    public void testFireEvent() {
        try {
            DataManagerWorker worker = new DataManagerWorker(
                                               new SimpletonTest());
            worker.generateEvents(true);
            worker.start();

            while (worker.isAlive()) {
                Thread.currentThread().sleep(1000);
            }
        } catch (Exception e) {
            this.fail("Caught an exception trying to fire our event");
        }
    }

    public void testRemoveEventHandler() {
        try {
            eventGeneratorAssistant.addEventHandler(this.handler, 
                                                    this.alwaysFalseConstraint);
            eventGeneratorAssistant.addEventHandler(this.handler, 
                                                    this.alwaysTrueConstraint);

            this.assertTrue("the handler is now in the Map as a key", 
                            eventGeneratorAssistant.getEventListeners()
                                                   .containsKey(this.handler));
            this.assertTrue("the first constraint should be in the list", 
                            ((List) eventGeneratorAssistant.getEventListeners()
                                                           .get(this.handler)).contains(
                                    this.alwaysFalseConstraint));
            this.assertTrue("the second constraint should be in the list", 
                            ((List) eventGeneratorAssistant.getEventListeners()
                                                           .get(this.handler)).contains(
                                    this.alwaysTrueConstraint));

            eventGeneratorAssistant.removeEventHandler(this.handler, 
                                                       this.alwaysFalseConstraint);
            this.assertTrue("the handler should still be a key in the map", 
                            eventGeneratorAssistant.getEventListeners()
                                                   .containsKey(this.handler));
            this.assertTrue(
                    "the first constraint should no longer be in the list", 
                    !((List) eventGeneratorAssistant.getEventListeners()
                                                    .get(this.handler)).contains(
                             this.alwaysFalseConstraint));

            eventGeneratorAssistant.removeEventHandler(this.handler, 
                                                       this.alwaysTrueConstraint);
            this.assertTrue("the handler should still be a key in the map", 
                            !eventGeneratorAssistant.getEventListeners()
                                                    .containsKey(this.handler));
        } catch (Exception e) {
            this.fail("Caught an exception trying to add an event handler to our eventGeneratorAssistant");
        }
    }

    private boolean getEventWasHandled() {
        return eventWasHandled;
    }

    class SimpletonTest extends Simpleton {
        public void process() {
            // we added only the false always constraint, so ishandled should still be false afterwards
            try {
                eventGeneratorAssistant.addEventHandler(handler, 
                                                        alwaysFalseConstraint);
                eventGeneratorAssistant.fireEvent(dataManagerEvent);
                Thread.currentThread().sleep(2000);
                assertTrue("The event should not have been handled (false).", 
                           !getEventWasHandled());


                // we added the true always constraint so ishandled should be true afterwards
                eventGeneratorAssistant.addEventHandler(handler, 
                                                        alwaysTrueConstraint);
                eventGeneratorAssistant.fireEvent(dataManagerEvent);
                Thread.currentThread().sleep(2000);
                assertTrue("The event should have been handled (true).", 
                           getEventWasHandled());
            } catch (InterruptedException ex) {
                //do nothing
            }
        }
    }
}