// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.kernel;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.datamanager.constraint.EntityConstraint;

import org.datamanager.exception.DataManagerException;
import org.datamanager.exception.NoSuchEntityException;
import org.datamanager.exception.NoSuchIdentifierException;

import org.datamanager.passiveentityvalue.StringEntityValue;


public class JispEntityManagerTester extends TestCase {
    public final static String ENTITY_NAME = "xirao";
    public final static String CLASS_PROPERTY = "org.datamanager.kernel.entity_manager.class";
    public final static String TEST_CLASS = "org.datamanager.kernel.JispEntityManager";
    public final static String ENTITY_VALUE_STRING = "ricercar";
    String savedEntityManagerClassProperty;
    EntityManager entityManager;
    EntityProxy entityProxy;

    public JispEntityManagerTester(String string) {
        super(string);
    }

    protected void setEntityManagerProperty(String className) {
        savedEntityManagerClassProperty = System.getProperty(CLASS_PROPERTY);
        System.setProperty(CLASS_PROPERTY, className);
    }

    protected void resetEntityManagerProperty() {
        if (null == savedEntityManagerClassProperty) {
            // FIXME: This is ugly, because it requires "insider knowledge" of
            // EntityManager, but it seems to be the only way to fake resetting
            // of the property to null, since trying to actually set it to null
            // causes a NullPointerException to be thrown.
            System.setProperty(CLASS_PROPERTY, 
                               EntityManager.DEFAULT_IMPLEMENTATION_CLASS_NAME);
        } else {
            System.setProperty(CLASS_PROPERTY, savedEntityManagerClassProperty);
        }

        EntityManager.forgetInstance();
    }

    protected void initializeVariables() {
        try {
            EntityManager.forgetInstance();
            entityManager = EntityManager.getInstance();


            /*
            entityProxy = entityManager.createEntity(ENTITY_NAME);
            */
            entityProxy = (EntityProxy) Entity.create(ENTITY_NAME);
        } catch (DataManagerException exception) {
            fail(exception.getMessage());
        }
    }

    protected void setUp() {
        setEntityManagerProperty(TEST_CLASS);
        initializeVariables();
    }

    public static Test suite() {
        return new TestSuite(JispEntityManagerTester.class);
    }

    protected void tearDown() {
        resetEntityManagerProperty();
    }

    public void testGetInstance() {
        assertNotNull(entityManager);
    }

    public void testCreateEntity() throws DataManagerException {
        assertNotNull(entityProxy);
        assertEquals(ENTITY_NAME, entityProxy.getName());
    }

    public void testFindByIdentifier() throws DataManagerException {
        Identifier proxyIdentifier = entityManager.findIdentifier(entityProxy);
        Entity entity = entityManager.findEntity(proxyIdentifier);

        assertEquals(proxyIdentifier, entityManager.findIdentifier(entity));
    }

    public void testZeroAttributes() throws DataManagerException {
        assertEquals(0, entityProxy.getAttributes(EntityConstraint.NONE).length);
    }

    public void testAttach() throws DataManagerException {
        Entity attribute = Entity.create("attachee");

        entityProxy.attach(attribute);

        Entity[] attributes = entityProxy.getAttributes(EntityConstraint.NONE);
        assertEquals(1, attributes.length);
        assertEquals(entityManager.findIdentifier(attribute), 
                     entityManager.findIdentifier(attributes[0]));
    }

    public void testAttachBase() throws DataManagerException {
        Entity attribute = Entity.create(ENTITY_NAME);

        entityProxy.attach(attribute);

        Entity[] bases = attribute.getBases(EntityConstraint.NONE);
        assertEquals(1, bases.length);
        assertEquals(entityManager.findIdentifier(entityProxy), 
                     entityManager.findIdentifier(bases[0]));
    }

    public void testDetach() throws DataManagerException {
        Entity attribute = Entity.create(ENTITY_NAME);

        entityProxy.attach(attribute);
        entityProxy.detach(attribute);

        Entity[] attributes = entityProxy.getAttributes(EntityConstraint.NONE);
        assertEquals(0, attributes.length);
    }

    public void testDetachBase() throws DataManagerException {
        Entity attribute = Entity.create(ENTITY_NAME);

        entityProxy.attach(attribute);
        entityProxy.detach(attribute);

        Entity[] bases = attribute.getBases(EntityConstraint.NONE);
        assertEquals(0, bases.length);
    }

    public void testSetValue() throws DataManagerException {
        EntityValue entityValue = new StringEntityValue(ENTITY_VALUE_STRING);
        entityProxy.setValue(entityValue);

        assertEquals(entityValue, entityProxy.getValue());
    }
}

/*
 *
 * $Log: JispEntityManagerTester.java,v $
 * Revision 1.5  2002/12/20 20:52:50  mliggett
 * Adding/modifying license tag on all source files.
 *
 * Revision 1.4  2002/12/08 04:37:41  mliggett
 * Added caching, local changes to the cached values, dirty bits and write
 * back of locally modified values to CachingProxyEntity.  writeBack() is
 * currently not called anywhere but from CachingProxyEntity's finalizer,
 * but that will change in the next iteration.
 *
 * Also, writeBack() currently only writes back modified EntityValue.  If
 * the bases or attributes have changed, that's never reflected in the proxied
 * Entity.  I'll fix that in a moment, too.
 *
 * Revision 1.3  2002/12/05 11:55:08  alllee
 * Codebase-style.
 *
 * Revision 1.2  2002/12/05 03:10:37  mliggett
 * Added CachingProxyEntityManager with unit tests.  Right now it just passes
 * everything through to JispEntityManager, but this will change on the next
 * checkin.
 *
 * I also modified JispEntityManagerTester slightly so that I could make
 * CachingProxyEntityManagerTester inherit from it in order to reuse the code
 * in it.  I assume we consider this "good enough" style for unit tests
 * although we wouldn't want to do this in the rest of the code.
 *
 * Revision 1.1  2002/12/03 05:04:32  mliggett
 * First stab at pure, jisp-only storage.  Obviously needs refactoring (which
 * will shorten the longer source files substantially) and a
 * looksee at the javadoc.  Separate message to be sent fully
 * explaining things, but these entities keep NO data in main memory.  All
 * four pieces of Entity data (name, value, attributes and bases) are kept in
 * separate indexed jisp databases and the contents are loaded on demand.
 *
 * Includes several unit tests.  Caching EntityManager to be checked in
 * next.
 *
 *
 */
