// For licensing, see: http://helium.knownspace.org/license.html
package org.datamanager.persistence;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;

import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.datamanager.persistence.DataAccessObject;
import org.datamanager.persistence.DataAccessObjectFactory;
import org.datamanager.persistence.helper.Address;
import org.datamanager.persistence.helper.ArrayHolder;
import org.datamanager.persistence.helper.Book;
import org.datamanager.persistence.helper.CharArray;
import org.datamanager.persistence.helper.Film;
import org.datamanager.persistence.helper.FinalHolder;
import org.datamanager.persistence.helper.People;
import org.datamanager.persistence.helper.Referer;
import org.datamanager.persistence.helper.Student;


public class SerializableDataAccessObjectTester extends TestCase {
    protected DataAccessObject dataAccessObject;
    private File file;

    public SerializableDataAccessObjectTester(String testCaseName) {
        super(testCaseName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite("SerializableDataAccessObjectTester");

        suite.addTest(new SerializableDataAccessObjectTester("testPrimitive"));
        suite.addTest(
                new SerializableDataAccessObjectTester("testObjectReference"));
        suite.addTest(
                new SerializableDataAccessObjectTester(
                        "testHierarchyAndConstructor"));
        suite.addTest(
                new SerializableDataAccessObjectTester(
                        "testDAOwithInputOutputStream"));
        suite.addTest(new SerializableDataAccessObjectTester("testNull"));
        suite.addTest(new SerializableDataAccessObjectTester("testCycle"));
        suite.addTest(new SerializableDataAccessObjectTester("testEmptyVector"));
        suite.addTest(
                new SerializableDataAccessObjectTester("testNullLongArray"));
        suite.addTest(new SerializableDataAccessObjectTester("testFinal"));

        return suite;
    }

    public void setUp() {
        DataAccessObjectFactory daoFactory = DataAccessObjectFactory.newInstance();
        dataAccessObject = daoFactory.getDataAccessObject(
                                   DataAccessObjectFactory.SERIALIZABLE_PERSISTENCE);
        file = new File("tester.xml");
    }

    // testing for primitive type, final, static, public , private, protected, friendly
    public void testPrimitive() {
        Book javaBook = new Book();
        javaBook.setName("design pattern");
        javaBook.setPage(213);
        javaBook.setPrice(23.23);
        javaBook.setFirstChar('a');

        byte[] key = "test case".getBytes();
        javaBook.setKey(key[0]);
        javaBook.setReleaseState(true);

        short rank = 1;
        javaBook.setRank(rank);

        long readerNumber = 100000000;
        javaBook.setReaderNumber(readerNumber);

        dataAccessObject.persist(javaBook, file);

        Book rebuildJavaBook = (Book) dataAccessObject.restore(file);

        this.assertEquals(javaBook.getName(), rebuildJavaBook.getName());
        this.assertEquals(javaBook.getPage(), rebuildJavaBook.getPage());
        this.assertEquals(javaBook.getWeight(), rebuildJavaBook.getWeight(), 
                          0.1);
        this.assertEquals(javaBook.getPrice(), rebuildJavaBook.getPrice(), 0.1);
        this.assertEquals(javaBook.getFirstChar(), 
                          rebuildJavaBook.getFirstChar());
        this.assertEquals(javaBook.getKey(), rebuildJavaBook.getKey());
        this.assertEquals(javaBook.isRelease(), rebuildJavaBook.isRelease());
        this.assertEquals(javaBook.getRank(), rebuildJavaBook.getRank());
        this.assertEquals(javaBook.getReaderNumber(), 
                          rebuildJavaBook.getReaderNumber());
    }

    //Vector, array, and object reference
    public void testObjectReference() {
        Film film = new Film("java");

        dataAccessObject.persist(film, file);

        Film rebuildFilm = (Film) dataAccessObject.restore(file);
        this.assertEquals(film.getFilmName(), rebuildFilm.getFilmName());
        this.assertEquals(film.getFirstActor().getName(), 
                          rebuildFilm.getFirstActor().getName());
        this.assertEquals(film.getSecondActor().getName(), 
                          rebuildFilm.getSecondActor().getName());
        this.assertEquals(film.getThirdActor().getName(), 
                          rebuildFilm.getThirdActor().getName());
        this.assertEquals(film.getFirstVectorValue(), 
                          rebuildFilm.getFirstVectorValue());
        this.assertEquals(film.getSecondVectorValue().getName(), 
                          rebuildFilm.getSecondVectorValue().getName());
    }

    // test for hierarchy.
    public void testHierarchyAndConstructor() {
        Student student = new Student("xirao", 27, 12345);
        dataAccessObject.persist(student, file);

        Student rebuildStudent = (Student) dataAccessObject.restore(file);
        this.assertEquals(student.getName(), rebuildStudent.getName());
        this.assertEquals(student.getAge(), rebuildStudent.getAge());

        int id = rebuildStudent.getId();

        this.assertEquals(student.getRace(), rebuildStudent.getRace());
        this.assertEquals(student.getId(), rebuildStudent.getId());
    }

    // test for interface:
    //      1. public void persist(Object object, OutputStream outputStream)
    //      2. public Object restore(InputStream inputStream)
    public void testDAOwithInputOutputStream() {
        Film film = new Film("java");
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        dataAccessObject.persist(film, byteArrayOutputStream);

        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
                                                            byteArrayOutputStream.toByteArray());
        Film rebuildFilm = (Film) dataAccessObject.restore(byteArrayInputStream);

        this.assertEquals(film.getFilmName(), rebuildFilm.getFilmName());
        this.assertEquals(film.getFirstActor().getName(), 
                          rebuildFilm.getFirstActor().getName());
        this.assertEquals(film.getSecondActor().getName(), 
                          rebuildFilm.getSecondActor().getName());
        this.assertEquals(film.getThirdActor().getName(), 
                          rebuildFilm.getThirdActor().getName());
        this.assertEquals(film.getFirstVectorValue(), 
                          rebuildFilm.getFirstVectorValue());
        this.assertEquals(film.getSecondVectorValue().getName(), 
                          rebuildFilm.getSecondVectorValue().getName());
        this.assertNull(rebuildFilm.nullActor);
    }

    public void testCycle() {
        Referer originalReferer = new Referer();
        Referer restoredReferer;


        // make it refer to itself
        originalReferer.referent = originalReferer;

        dataAccessObject.persist(originalReferer, file);
        restoredReferer = (Referer) dataAccessObject.restore(file);


        // Make sure we didn't just get the same exact object back.
        assertNotSame(originalReferer, restoredReferer);


        // Make sure the restored object still properly points to itself.
        assertSame(restoredReferer, restoredReferer.referent);
    }

    public void testNull() {
        Referer originalReferer = new Referer();
        Referer restoredReferer;

        dataAccessObject.persist(originalReferer, file);
        restoredReferer = (Referer) dataAccessObject.restore(file);


        // The restored object should still point at null.
        assertSame(null, restoredReferer.referent);
    }

    public void testNullLongArray() {
        ArrayHolder originalArrayHolder = new ArrayHolder();
        ArrayHolder restoredArrayHolder = new ArrayHolder();

        originalArrayHolder.longArray = null;

        dataAccessObject.persist(originalArrayHolder, file);
        restoredArrayHolder = (ArrayHolder) dataAccessObject.restore(file);


        // The restored array should still be null.
        assertSame(null, restoredArrayHolder.longArray);
    }

    public void testEmptyVector() {
        Vector originalVector = new Vector();
        Vector restoredVector;

        dataAccessObject.persist(originalVector, file);
        restoredVector = (Vector) dataAccessObject.restore(file);

        assertEquals(0, restoredVector.size());
    }

    public void testCharArray() {
        CharArray charArray = new CharArray();
        CharArray rebuildCharArray;
        charArray.setValue();
        dataAccessObject.persist(charArray, file);
        rebuildCharArray = (CharArray) dataAccessObject.restore(file);

        char a = charArray.value[0];
        char b = rebuildCharArray.value[0];
        this.assertEquals(charArray.value[0], rebuildCharArray.value[0]);
    }

    public void testFinal() {
        FinalHolder originalFinalHolder = new FinalHolder();
        FinalHolder restoredFinalHolder;

        originalFinalHolder.stringBuffer.append(" foo");
        dataAccessObject.persist(originalFinalHolder, file);
        restoredFinalHolder = (FinalHolder) dataAccessObject.restore(file);


        /*
                Address myAddr = new Address("good");
                Address secAddr;
                dataAccessObject.persist(myAddr, file);
                secAddr = (Address) dataAccessObject.restore(file);
        */
        assertNotNull("restoredFinalHolder should not be null.", 
                      restoredFinalHolder);
        assertNotNull("restoredFinalHolder.stringBuffer should not be null.", 
                      restoredFinalHolder.stringBuffer);
        assertNotNull("originalFinalHolder.stringBuffer should not be null.", 
                      originalFinalHolder.stringBuffer);
        originalFinalHolder.stringBuffer.toString();
        restoredFinalHolder.stringBuffer.toString();
        assertEquals(originalFinalHolder.stringBuffer.toString(), 
                     restoredFinalHolder.stringBuffer.toString());
    }
}